;;*****************************************************************************
;; File: Bubbler.asm
;;       Source code for Commodore 64 version of Bubbler
;;
;; Build: To compile on Windows from command line in same directory as this
;;        file, run:
;;          make.bat -- to build version without cheat keys
;;          make.bat enhanced -- to build version with cheat keys and "don't
;;                               start on a trap door" fix.  (This defines
;;                               BUILD_ENHANCED_VERSION.)
;;
;; Notes: Converted from PDS (Programmer's Development System) to the Cadaver /
;;        Olaf variant of DASM (see http://cadaver.homeftp.net/tools.htm)
;;        downloaded from http://cadaver.homeftp.net/tools/dasm.zip.
;;
;;        (Conversion to single binary also means a compressor is now required.
;;        In make.bat I've used Exomizer 2, downloaded from
;;        http://hem.bredband.net/magli143/exo/)
;;
;;        Summary of changes made to get this to compile with DASM:
;;
;;          - Merge all files into this one, converting CR to NL, removing other
;;            control characters (except tab) and removing non-ASCII characters.
;;            Eg use the following command line in Cygwin:
;;              ls -1 X?.BUB | sort
;;              $ echo "" > merged.asm
;;              $ ls -1 X?.BUB | sort | while read ii
;;              > do cat $ii | tr -d "\000-\010\013\014\177-\377" | tr "\015" "\012" | tr "\016-\037" "\040" >> merged.asm
;;              > done
;;              $
;;          - Remove remaining non-text portions of PDS files by sight.  (The
;;            method above gets rid of most but not all of the binary data.)
;;            What needs to be removed is usually just the garbage lines before
;;            the "PDS Pc3.a :0000:  (C)  1985-7 P.D Systems" at the start of
;;            each file.
;;          - Comment out PDS instructions for uploading code from PC to C64
;;          - Change "DB" pseudop (declare byte data) to "BYTE" (Could have used
;;            "DC" or "DC.B" but these don't get syntax highlighted in
;;            SlickEdit on Windows.)
;;            Take care not to change "$DB" strings or "$xxDB" strings or DB
;;            inside HEX strings.
;;          - Change "DS.B" pseudop (declare space) "DS.B"
;;          - Change "CBM" pseudop (declare string in Commodore format) to
;;            "BYTE" and change string to screen codes of individual characters
;;            (where different from ASCII -- eg letters)
;;          - Change "DL exp" pseudop (declare low byte of data) to
;;            "BYTE <[exp]"
;;          - Change "DH exp" pseudop (declare high byte of data) to
;;            "BYTE >[exp]"
;;            (In both the above, square brackets only needed round complex
;;             expressions)
;;          - Change "DW" pseudop (declare word data) to "WORD"
;;          - Prepend ";" to lines beginning with "*"
;;          - Remove leading 0s from non-octal numerical constants
;;          - Prepend ";" to lines following ".END" in each original file
;;          - Add "\tPROCESSOR 6502" to the beginning of the file
;;          - Change () to [] in expressions following mnemonics
;;          - Change "LDA #<exp1+exp2" to "LDA #<[exp1+exp2]" and similar
;;          - Change "LDA #>exp1+exp2" to "LDA #>[exp1+exp2]" and similar
;;          - Split out: BubblerBackgrounds.asm
;;                       BubblerLoader.asm
;;                       BubblerMapEtcData.asm
;;                       BubblerScreenPrint.asm
;;                       CbmBasicTokens.asm
;;          - Fix duplicated labels: MESS
;;          - Changed lines of form "SYMBOL ORG SYMBOL+n" to "SYMBOL DS.B n"
;;          - Change "ASL A" to "ASL ; = ASL A"
;;                   "LSR A" to "LSR ; = LSR A"
;;                   "ROL A" to "ROL ; = ROL A"
;;                   "ROR A" to "ROR ; = ROR A"
;;          - Move memory map comment to start of code.  Reorder blocks of code
;;            so that each time an "ORG" pseudo-op is encountered it is at a
;;            higher address than the previous one.
;;          - Split code into two programs
;;              - one that puts graphics data etc in high RAM under kernel ROMs
;;                etc
;;              - one with the main game code and data
;;          - Change code that uses DRP_J_TABH, DRP_Z_TABH, DRP_D_TABH jump
;;            tables to behave defensively when encountering an invalid value
;;          - Corrected comments relating to SLOP_TAB (which had meaning
;;            reversed of the bit distinguishing between 30' and 45' slopes)
;;          - Comment out MAP0 - MAP5 data, which is not used (as DEF_MAP_1 -
;;            DEF_MAP_5 is used instead).
;;          - Changed "ERROR" pseudop to "ECHO" and "ERR"
;;
;; About: Bubbler was originally written for the ZX Spectrum by Ultimate.
;;        Lynsoft were contracted to port it to the Commodore 64.  The work was
;;        done between August 1987 and May 1988 by Jake Simpson and Matt Young
;;        (MY).  The game was never released, but in 2009 was tidied up by MY,
;;        with much help from Jani Tahvanainen, to see the light of day.  See
;;        http://gtw64.retro-net.de/Pages/t/Review_Thebubbler.php for more...
;;
;;        Help with reading data off MY's original 5.25-inch floppy disks was
;;        provided by Frank Gasking (C64 ones, mostly containing binaries) and
;;        Daniel Ramsay (PC ones, containing PDS source files).
;;
;; Extra Controls: - Commodore key (mapped to left Ctrl key on VICE emulator)
;;                   pauses the game.  Press any key to continue.
;;
;; "Trainer" (aka cheat) keys:
;;    f1                      -- opens the exit on the current level even if
;;                               you haven't stoppered all the bottles
;;    f3                      -- give you an extra life
;;    f3 + both shifts
;;    (Shift + f4 on WinVIOE) -- stops aliens, bullets, etc from killing you
;;                               (though falling off the landscape still does).
;;    f5                      -- tops your time back up to maximum
;;    f5 + both shifts
;;    (Shift + f6 on WinVIOE) -- gives you infinite time
;;    f7                      -- reduces you down to your last life
;; Using any combination of the two cheats that require the Shift key (infinite
;; lives / no killing by aliens) also changes the border colour to grey for the
;; remainder of the game.
;;
;;*****************************************************************************
        PROCESSOR 6502

;                             MEMORY MAP FOR C64 BUBBLER.
; $0000 - $00FF               ........       ZERO PAGE VARIABLES
; $0100 - $01FF               ........       STACK
; $0200 - $03FF               ........       SPECAL TABLE
;
; $0400 - $0FFF               ........       VARIABLE STORAGE / TABLE SPACE
;
;              ^End of tables: ENMUS
;
; $1000 -      ........       BUBBLER PROGRAM
;              ^End of code: FIN
;
;              ........       BLOCK DEFINTIONS FOR TIMER, NUMBERS & SCORE
;
;              ........       TEMP SPACE a] FOR SCROLLING MESSAGES
;                                                                          b] DOING DISSOLVE EFFECT ON GRAPHICS
;
;
; $8700 - $88FF               ........       PDS DOWNLOAD SOFTWARE
; $8900 - <$8CFF              ........       MATT'S TABLES ETC
;
;              ^End of data: EM
; EM    - <$C1FF              ........       GRAPHICS FOLLOWED BY AREA FOR SAVED BACKGROUNDS
;              ^End of saved backgrounds: Z
; $C200 - $CDFF               ........       WORKING AREA FOR CONSTRUCTING SCREEN
;                                            Also $C200 - $C5FF is temporary storage at
;                                            program load time for colour nybbles to copy
;                                            to VIC colour RAM.
;
;
; Data below loaded in by BubblerLoader.asm
;
; $CE00 - $CFFF               ........       SPRITE DATA FOR ARROWS & BUBBLER SIGN
; $D000 - $D3FF               ........       CHARACTER SET DESTINATION
; $D400 - $D7FF               ........       TITLE SCREEN STORAGE
;
; $D800 - $DBFF               ........       SPRITE DATA FOR GAUGES
;
; $DC00 - $DFFF               ........       COLOUR NYBBLES FOR HIRES SCREEN
; $E000 - $FFEF               ........       HIRES SCREEN
;
; $FFFA - $FFFF               ........       HARDWARE IRQ VECTORS
;
;              ORIGINAL UPLOAD MEMORY MAP
;
; $1000 - $13FF               ........       OPTION SCREEN (put at $D400- {RAM})
; $1400 - $17FF               ........       CHARACTER SET (put at $D000- {RAM})
; $1800 - $1BFF               ........       SPRITES FOR GAUGES (put at $D800- {RAM})
;
; $1C00 - $1FFF               ........       COLOUR NYBBLES FOR HIRES SCREEN (put at $DC00-
;
; $2000 - $208C               ........       UPLOAD PROGRAM
;
; $D800 - $DBFF               ........       COLOUR NYBBLES FOR OPTION SCREEN
;
;

;;
;; C64 colour codes
;;
COL_CODE_BLACK       EQU $00
COL_CODE_WHITE       EQU $01
COL_CODE_RED         EQU $02
COL_CODE_CYAN        EQU $03
COL_CODE_PURPLE      EQU $04
COL_CODE_GREEN       EQU $05
COL_CODE_BLUE        EQU $06
COL_CODE_YELLOW      EQU $07
COL_CODE_ORANGE      EQU $08
COL_CODE_BROWN       EQU $09
COL_CODE_PINK        EQU $0A
COL_CODE_DARK_GREY   EQU $0B
COL_CODE_GREY        EQU $0C
COL_CODE_LIGHT_GREEN EQU $0D
COL_CODE_LIGHT_BLUE  EQU $0E
COL_CODE_LIGHT_GREY  EQU $0F


;
; File X0.BUB
; Starts Here
;
;

;
;
;  PDS Pc3.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          10.26
;         By  "JAKE AND MATTHEW..."
;
;     Created on Wed the 22nd of Jul 1987
;        Last update 08:41 on 10/05/88
;

;;;
;;; 2009-07: MY: Think these were commands to the PDS upload the compiled code
;;;              from the PC to the C64.  Commented out.
;;;
;;;            SEND COMPUTER1
;;;
;;;
;;;            SKIP 1
;;;

FLIPIN         EQU $35
FLIPOUT        EQU $34
SDEF           EQU $DFF8

MSB            EQU $D010
TDEF           EQU $D7F8

CHARSTART      EQU $D3FF-184

VAR            EQU 3
MAX_V          EQU 7
SHE            EQU 8

MAX_STAGE      EQU 5 ; Number of game levels

;;;
;;; 2009-07: MY: Think these were commands to the PDS upload the compiled code
;;;              from the PC to the C64.  Commented out.
;;;
;;; UPLO       ASK "D'you want the big one ?
;;;
;;;
;;;
;;;            IF UPLO



PVAL           EQU $08 ;TOD protection value [must be 0-9]

MC             EQU 64 ;offset to second line of message scroll buffer

WINDOW         EQU $E18F ;start of scrolling window on hires screen
BUFLINE1       EQU $E000

WA             EQU $C200 ;start of work area
WA_WIDTH_BYTES EQU 24 ; Work area width in bytes
WA_WIDTH_PX    EQU 8*WA_WIDTH_BYTES ; Work area width in pixels
WA_HEIGHT_PX   EQU $80 ; Work area height
;WA_END         EQU $CE00
WA_END         EQU WA + [WA_WIDTH_BYTES * WA_HEIGHT_PX]

HIRES          EQU $E000 ;start of hires screen

RND0           EQU $DD04 ;randomish value 0-255
RND1           EQU $DD06 ;different randomish value 0-255
                         ;^^^ above two are decremented every processor clock cycle


        SEG.U VARIABLES
;
; Zero page variables
;
               ORG $02             ;start of zero page
ZPSTART
YVAL           DS.B 1
XVAL           DS.B 1
TABVAL         DS.B 1
TIME           DS.B 1
JOY            DS.B 1
NUM1           DS.B 1
NUM2           DS.B 1
PLAYER         DS.B 1
MESS           DS.B 1
STAGE          DS.B 1

;; 2009-10: MY: Changed this to support scrolling message longer than 255 characters
;;              on options screen
;MESSNO         DS.B 1
OPT_SCRN_MSG   DS.B 2


BUFF           DS.B 1
PLAYNO         DS.B 1
CHOICE         DS.B 2
;choice+1      EQU playno+2

ARROW          DS.B 1

;;;
;;; 2009-09: MY: NB: Code assumes these six variables are adjacent and in this
;;;              order.
;;;
; Keys for keyboard control
KLEFT          DS.B 1
KRIGHT         DS.B 1
KUP            DS.B 1
KDOWN          DS.B 1
KJUMP          DS.B 1
KFIRE          DS.B 1

;;;
;;; 2009-09: MY: LIVES and LIVES2 were originally two bytes each -- one for
;;;              each digit of the number of lives left.  There were some bugs
;;;              in how lives got updated once you got into double digits, so
;;;              I've taken the opportunity to change it to Binary-Coded
;;;              Decimal (BCD).
;;;
;;;              Note that the code continues to assume these variables are
;;;              next to each other in memory.
;;;
LIVES          DS.B 1 ; Player 1 lives
LIVES2         DS.B 1 ; Player 2 lives

;;;
;;; 2009-10: MY: Added this for "big cheats" that actually change the way the
;;;              game works (eg stop time decrementing rather than give you
;;;              extra time).  First byte is for player 1, second for player 2.
;;;
;;;              Bit 0 set = infinite time
;;;              Bit 1 set = aliens don't kill you (though they still push you
;;;                          out the way and falling off the lanscape is still
;;;                          fatal)
;;;
CHEAT_MODE     DS.B 2
CHEAT_INFINITE_TIME  EQU $01
CHEAT_NO_ALIEN_KILL  EQU $02

NUMST          DS.B 1

TEMP           DS.B 1

PLAYMESS       DS.B 1
EXITO          DS.B 1

VOC1           DS.B 1
VOC2           DS.B 1
VOC3           DS.B 1
PRIORITY       DS.B 1
V1LO           DS.B 1
V1HI           DS.B 1
V1PLO          DS.B 1
V1PLHI         DS.B 1

V1WV           DS.B 1
V1AD           DS.B 1
V1SR           DS.B 15
V4LO           DS.B 1
V4HI           DS.B 1
V4PLO          DS.B 1
V4PLHI         DS.B 1

V4WV           DS.B 1
V4AD           DS.B 1
V4SR           DS.B 1
FILTER         DS.B 1
FILO           DS.B 1
FILHI          DS.B 1

VOL            DS.B 1
OFFSET         DS.B 1

BOLT           DS.B 1
CORKED         DS.B 1
RESONANCE      DS.B 1

FILON          DS.B 1
COUNT          DS.B 1
LEAP           DS.B 1

LEAPCOUNT      DS.B 1

BANGCOUNT      DS.B 1
EXPL           DS.B 1

COUNTER1       DS.B 1
COUNTER2       DS.B 1
COUNTER3       DS.B 1

WAVE1          DS.B 1
WAVE2          DS.B 1
WAVE3          DS.B 1
MUS            DS.B 2

VTEMP1         DS.B 2
VTEMP2         DS.B 2
VTEMP3         DS.B 2

ZAPCOUNT       DS.B 1
ZAPDELAY       DS.B 1
MATCOUNT       DS.B 1
MAT            DS.B 1

XTRA           DS.B 1

XTRACOUNT      DS.B 1

CURPOSA        DS.B 1
CURPOSB        DS.B 1
AMOUNT         DS.B 1
;ENDJAKE

               ;Matt's zero page & constants
;
; Constants...
;
LEN            EQU 20 ;LENGTH OF BUFFER LINE IN CHARS

IXNUM          EQU $0D ;# of IX / SHADO tables {see also LSHAD_TAB}

IXSIZ          EQU $1B
SHADSIZ        EQU $10
SPECSIZ        EQU $1A


;               ORG ENDJAKE         ;zero page
XPIX           DS.B 1 ;0-15 = 0-30 pixels
YPIX           DS.B 1 ;0-7 = 0-14 pixels

YSM            DS.B 1 ;0-3 = 0-6 PIXELS
XSM            DS.B 1 ;0-3 = 0-6 PIXELS
COUNTER        DS.B 1 ;COUNTER FOR NUMBER OF BUFFER LINES
LINEC          DS.B 1 ;COUNTER FOR PIXEL LINES DOWN BUFFER LINE
PTR            DS.B 2 ;POINTER TO WORK AREA DATA

POS            DS.B 2 ;NO OF PIXELS DOWN WORK AREA

ROTATE         DS.B 1 ;TEMPORARY STORAGE DURING SMOOTH COPY
ROTATE2        DS.B 1
ENDY           DS.B 1 ;PTR TO RHS OF WORK AREA TO COPY

SRT0           DS.B 2 ;GENERAL PUPOSE ZERO PAGE STORAGE
SRT1           DS.B 2
ZPA            DS.B 2
ZPB            DS.B 2
ZPC            DS.B 2
ZPD            DS.B 2
ZPE            DS.B 2
ZPF            DS.B 2
ZPG            DS.B 2
SRA            DS.B 1
IX             DS.B 2
IY             DS.B 2
HL             DS.B 2
DATA           DS.B 2
NEWNAME        DS.B 1
STRIPX         DS.B 2
STRIPY         DS.B 2
STRIPDAT       DS.B 2
INITSTRDAT     DS.B 2
SCREENX        DS.B 2
SCREENY        DS.B 2
CELL_OFF       DS.B 2
COUNT_DOWN     DS.B 1
XTEMPOR        DS.B 3
YTEMPOR        DS.B 3
ZTEMPOR        DS.B 3
SCREENX_II     DS.B 2
SPEC_OFF       DS.B 2
CURCELLX       DS.B 1
CURCELLY       DS.B 1
FIVE_COUNT     DS.B 1
Y_MIN          DS.B 2
Y_MAX          DS.B 2
XL1            DS.B 2
YL1            DS.B 2
XL2            DS.B 2
YL2            DS.B 2
CELL_ABSX      DS.B 2
CELL_ABSY      DS.B 2
MORE_VAR1      DS.B 1
CELL_ST        DS.B 2
SPIKE          DS.B 1
SHAPE_NO       DS.B 1
SPECTRUM       DS.B 1
TABLE_ENT      DS.B 1 ; Number of entries in ENT_TAB
Z_TEMP         DS.B 1
JUMP_F         DS.B 1
DROP_NOT       DS.B 1 ; 2009-09: MY: 0 means blob is dropping?
TPUP           DS.B 1
TPFORWARD      DS.B 1
TPLEFT         DS.B 1
TPRIGHT        DS.B 1
TPBACK         DS.B 1

;;
;; 2009-10: MY: Added for double-height character printing
;;
DOUBLE_HEIGHT  DS.B 1
ZPH            DS.B 2

;; 2009-09: MY: Which background object the blob is sitting on (NB counted from
;;              1 not 0).  $FF means none.  With the help of SPECTL/SPECTH
;;              look-up table, this is an index+1 into the array of MAXSPEC
;;              SPECSIZ-byte background objects stored at SPECAL.
;;
;;              Doesn't look like this needs to be 2 bytes.  I think some of
;;              these variables are 16 bits because they were on the Spectrum
;;              version.  (Z80 has lots of 16-bit registers so that's a natural
;;              default size for counters such as this.)  In practice, 8 bits
;;              should be more than enough so we might have been able to
;;              simplify to that here.
;;
OBJ_REST       DS.B 2

;; 2009-09: MY: Which entry in IXTAB we are processing.  0 means the first entry
;;             (the blob).
SPR_NO         DS.B 1

SOME_WHERE     DS.B 1
SCROLL         DS.B 1
TRAP           DS.B 1
ERE            DS.B 1
SCR_LR         DS.B 2
SCR_UD         DS.B 2
TRAP_NUM       DS.B 1
STAGE1         DS.B 1
STAGE2         DS.B 2
BOTTLES        DS.B 3
CORK           DS.B 3
TIME2          DS.B 1
MASK           DS.B 1
DEAD_YET       DS.B 3
BULLET         DS.B 1
LAST_FIRE      DS.B 1
LOWES          DS.B 1
WHICH_3D       DS.B 1
TOP_REST       DS.B 1
MAX_SPED       DS.B 1
YELLO          DS.B 1 ;Matt's loopy variable

GDIR           DS.B 1 ; 0 = GAUGEPOINT needs to move anticlockwise to match GAUGEDIR
                      ; $80 = needs to move clockwise
GAUGEDIR       DS.B 1 ; Direction to which gauge pointer should be moving
LOPCNT         DS.B 1


ENDZP          ;END OF ZERO PAGE VARS

               IF ENDZP>$00FF
               ECHO "ZERO-PAGE OVERFLOW !!!"
               ERR
               ENDIF



;
; These variables all get initialised to 0 at program start in MAJORSETUP
;
               ORG $0200
;
; 3D coordinate directions are as follows, with the origin in the middle of the
; bottom plane:
;
;   z
;   ^
;   |    y
;   |  /
;   |/
;   \
;     \
;       x
;
MAXSPEC        EQU $1FE/SPECSIZ ;max # of SPECAL entries
VARS_TO_ZERO_START
SPECAL         ;**MEMORY TO STORE INFO ON BACKGROUND SHAPES {"SPECIAL OBJECTS"}**
;;             BYTE 0 ;NUMBER OF ENTRIES
               DS.B 1 ;NUMBER OF ENTRIES

               DS.B $1FE

               ;SPECAL entries {NOT SAME AS ON SPECTRUM VERSION}...
               ;00 $00:moving objects can't go behind this shape
               ;02 x-co hi
               ;03 x-co lo
               ;04 y-co hi
               ;05 y-co lo
               ;06 z-co (lo)
               ;0A shape number: $00 = Spare
               ;                 $01 = 45'   upwards Y slope (see SLOP_TAB)
               ;                 $02 = floor
               ;                 $03 = block
               ;                 $04 = 30' downwards X slope (see SLOP_TAB)
               ;                 $05 = 45' downwards X slope (see SLOP_TAB)
               ;                 $06 = shadow
               ;                 $07 = 30'   upwards Y slope (see SLOP_TAB)
               ;                 $08 = 30' downwards Y slope (see SLOP_TAB)
               ;                 $09 = 45' downwards Y slope (see SLOP_TAB)
               ;                 $0C = 30'   upwards X slope (see SLOP_TAB)
               ;                 $0D = 45'   upwards X slope (see SLOP_TAB)
               ;                 $0E = spare
               ;                 $0F = spare
               ;                 $10 = support pole
               ;                 $11 = bottle
               ;                 $12 = bunged bottle
               ;                 $13 = top trap door
               ;                 $14 = LH side trap door
               ;                 $15 = exit trap door
               ;                 $16 = RH side trap door
               ;0B x dim
               ;0C y dim
               ;0D z dim
               ;0E x-co + x-dim hi
               ;0F x-co + x-dim lo
               ;14 x-co + x-dim hi
               ;15 x-co + x-dim lo
               ;16 y-co + y-dim hi
               ;17 y-co + y-dim lo
               ;18 top of object for current IX shape
               ;                {= [z-co + z-dim] (lo) except for slopes}
               ;19 z-co + z-dim (lo)
END_SPE



              ORG $0400


;;
;; 2009-09: MY: Not clear why this needs 10 bytes
;;
;; First byte is direction pointer on gauge:
;;   0: pointing to "9 o'clock"  = -x, -y direction
;;   2:                            -x, 0y
;;   4:             "12 o'clock" = -x, +y
;;   6:                            0x, +y
;;   8:             "3 o'clock"  = +x, +y
;;   A:                            +x, 0y
;;   C:             "6 o'clock"  = +x, -y
;;   E:                            0x, -y
;;
;; Odd numbers (up to $F) are intermediate steps
;;
GAUGEPOINT     ;BYTE 0,0,0,0,0,0,0,0,0,0
                DS.B 10


UNITS          ;BYTE 0,0,0,0,0,0,0
                DS.B 7

               ;Tables for last position of blob

               ;0,1 - x-co lo,hi

               ;2,3 - y-co lo,hi

               ;4   - z-co (lo)
               ;5,6 - SCREENX lo,hi
               ;7,8 - SCREENY-$8000 lo,hi
               ;

ST_POS         ;BYTE 0,0,0,0,0,0,0,0,0
        DS.B 9
ST_POS_1       ;BYTE 0,0,0,0,0,0,0,0,0
        DS.B 9
ST_POS_2       ;BYTE 0,0,0,0,0,0,0,0,0
        DS.B 9

               ;table of bottles inside specal table

               ;positions correspond with positions in

               ;specal table

BOTT           DS.B $14*2


               ;table of bunged bottles for player 1

BOTT_1         DS.B $14*2


               ;table of bunged bottles for player 2

BOTT_2         DS.B $14*2

;; 2009-09: MY: These are indexed by SPR_NO.  At the beginning of DROP_THEM it
;;              looks like we loop through the first SEVEN entries in IXTAB (ie
;;              #0 - #6) to handle movement.  (Other IXTAB entries do not move
;;              or only move in Z-direction.)  So extended length of these
;;              tables from 6 to 7 bytes.
;MAXRIGHT       DS.B 6
;MAXLEFT        DS.B 6
;MAXBACK        DS.B 6
;MAXFORWARD     DS.B 6
;MAXUP          DS.B 6
MAXRIGHT       DS.B 7
MAXLEFT        DS.B 7
MAXBACK        DS.B 7
MAXFORWARD     DS.B 7
MAXUP          DS.B 7

               ; List of objects under the blob
ENT_TAB        DS.B $12

               ; Offsets into this structure:
               ;   0 - x-co + x-dim hi
               ;   1 - x-co + x-dim lo
               ;   2 - y-co + y-dim hi
               ;   3 - y-co + y-dim lo
               ;   4 - unused?
               ;   5 - z-co + z-dim (lo only)
               ;
               ; 2009-09: MY: Looks like this could be 6 bytes rather than 7
               ;              words long, but there's no harm in it being too
               ;              long.
IXTB           ;WORD 0,0,0,0,0,0,0
               DS.W 7

;
; See comment below about why we need to be near a page boundary here
;
               ALIGN $100

ZOP            ;BYTE 0,0,0,0
               DS.B 4

;*****************************************************************************
;* NB: ORDER and YORDER must NOT
;* cross page boundaries and each
;* must start at least 3 bytes
;* AFTER the begining of a page
;*
;* This is, for one thing, to simplify the code in ALOCAT
;*****************************************************************************
YORDER         ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0
               DS.B 100

ORDER          ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
               ;BYTE 0
               DS.B 113

VARS_TO_ZERO_END

;;        ORG $1A00 Moved from here
SCROLST        DS.B 132 ;132 = $84 bytes temp storage for scrolling messages

;; 2009-10: MY: DISBUF actually needs more space than SCROLST.  See comment
;;              elsewhere that sizes it at (2*$68)+2 bytes.  We have plenty of
;;              space in the work area below $0800 so no need to make it
;;              overlap with SCROLST.
;DISBUF         EQU SCROLST ;$70 bytes needed for dissolve effect on moving shapes
;               ;It is OK for above 2 to be @ same place - never used @ same time
DISBUF         DS.B (2*$68)+2

;;*****************************************************************************
        SEG CODE

        INCLUDE CbmBasicTokens.asm

        ;
        ; BASIC lines to run code below and then load & run main application
        ;
        ORG $0801
BASIC_L10
        WORD BASIC_END ; Address of next line?
        WORD 10 ; Line number
        BYTE BASIC_TKN_SYS ; SYS command
        BYTE " "
        BYTE [CODE_START/1000 + $30] ; Address (in ASCII)
        BYTE [[CODE_START%1000]/100 + $30]
        BYTE [[CODE_START%100]/10 + $30]
        BYTE [[CODE_START%10] + $30]
        BYTE $00 ; End of line
BASIC_END
        WORD $0000


               ;ORG $1B00
;
; Sprite positions for guage pointers
;
POINTABX1      BYTE 35,34,33,35,80,83,85,84,84,85,86,84,80,35
               BYTE 33,35
POINTABX2      BYTE 235,234,233,235,24,27,29,28,28,29,30,28,24,235
               BYTE 233,234
POINTABY       BYTE 198,197,196,195,195,195,195,197,214,216,217,217,218,217
               BYTE 217,216
;
; Sprite pointers for guage pointers
;
SDEFTAB        BYTE 96,97,98,99,100,101,102,103
               BYTE 104,105,106,107,108,109,110,111,112


               ;pointer direction from joystick register

DIRS           BYTE $FF,$FF,$FF,$FF,$FF,$0C,$08,$0A
               BYTE $FF,$00,$04,$02,$FF,$0E,$06,$FF

;;;
;;; 2009-09: MY: I can't believe we hard-coded the addresses stored in these
;;;              tables!  The fix here is an improvement, but really CHARTAB
;;;              should be split up into separate named blocks.
;;;

;NUMTABLO       BYTE $00,$20,$40,$60,$80,$A0,$C0,$E0,$00,$20
;NUMTABHI       BYTE $12,$12,$12,$12,$12,$12,$12,$12,$13,$13
NUMTABLO       BYTE <[CHARTAB + $0200 + (0 * $20)], <[CHARTAB + $0200 + (1 * $20)], <[CHARTAB + $0200 + (2 * $20)]
               BYTE <[CHARTAB + $0200 + (3 * $20)], <[CHARTAB + $0200 + (4 * $20)], <[CHARTAB + $0200 + (5 * $20)]
               BYTE <[CHARTAB + $0200 + (6 * $20)], <[CHARTAB + $0200 + (7 * $20)], <[CHARTAB + $0200 + (8 * $20)]
               BYTE <[CHARTAB + $0200 + (9 * $20)]
NUMTABHI       BYTE >[CHARTAB + $0200 + (0 * $20)], >[CHARTAB + $0200 + (1 * $20)], >[CHARTAB + $0200 + (2 * $20)]
               BYTE >[CHARTAB + $0200 + (3 * $20)], >[CHARTAB + $0200 + (4 * $20)], >[CHARTAB + $0200 + (5 * $20)]
               BYTE >[CHARTAB + $0200 + (6 * $20)], >[CHARTAB + $0200 + (7 * $20)], >[CHARTAB + $0200 + (8 * $20)]
               BYTE >[CHARTAB + $0200 + (9 * $20)]

;NUMTABLOR      BYTE $40,$60,$80,$A0,$C0,$E0,$00,$20,$40,$80
;NUMTABHIR      BYTE $13,$13,$13,$13,$13,$13,$14,$14,$14,$14
NUMTABLOR      BYTE <[CHARTAB + $0340 + (0 * $20)], <[CHARTAB + $0340 + (1 * $20)], <[CHARTAB + $0340 + (2 * $20)]
               BYTE <[CHARTAB + $0340 + (3 * $20)], <[CHARTAB + $0340 + (4 * $20)], <[CHARTAB + $0340 + (5 * $20)]
               BYTE <[CHARTAB + $0340 + (6 * $20)], <[CHARTAB + $0340 + (7 * $20)], <[CHARTAB + $0340 + (8 * $20)]
               BYTE <[CHARTAB + $0340 + (9 * $20)]
NUMTABHIR      BYTE >[CHARTAB + $0340 + (0 * $20)], >[CHARTAB + $0340 + (1 * $20)], >[CHARTAB + $0340 + (2 * $20)]
               BYTE >[CHARTAB + $0340 + (3 * $20)], >[CHARTAB + $0340 + (4 * $20)], >[CHARTAB + $0340 + (5 * $20)]
               BYTE >[CHARTAB + $0340 + (6 * $20)], >[CHARTAB + $0340 + (7 * $20)], >[CHARTAB + $0340 + (8 * $20)]
               BYTE >[CHARTAB + $0340 + (9 * $20)]

XPOSTAB        BYTE 6,31,6,31
YPOSTAB        BYTE 6,6,10,10

ARROWPOS       BYTE 139,167,195

;LOSCORETAB     BYTE $40,$70,$A0,$D0,$00,$30,$60,$90,$C0,$F0
;HISCORETAB     BYTE $15,$15,$15,$15,$16,$16,$16,$16,$16,$16
LOSCORETAB     BYTE <[CHARTAB + $0540 + (0 * $30)], <[CHARTAB + $0540 + (1 * $30)], <[CHARTAB + $0540 + (2 * $30)]
               BYTE <[CHARTAB + $0540 + (3 * $30)], <[CHARTAB + $0540 + (4 * $30)], <[CHARTAB + $0540 + (5 * $30)]
               BYTE <[CHARTAB + $0540 + (6 * $30)], <[CHARTAB + $0540 + (7 * $30)], <[CHARTAB + $0540 + (8 * $30)]
               BYTE <[CHARTAB + $0540 + (9 * $30)]
HISCORETAB     BYTE >[CHARTAB + $0540 + (0 * $30)], >[CHARTAB + $0540 + (1 * $30)], >[CHARTAB + $0540 + (2 * $30)]
               BYTE >[CHARTAB + $0540 + (3 * $30)], >[CHARTAB + $0540 + (4 * $30)], >[CHARTAB + $0540 + (5 * $30)]
               BYTE >[CHARTAB + $0540 + (6 * $30)], >[CHARTAB + $0540 + (7 * $30)], >[CHARTAB + $0540 + (8 * $30)]
               BYTE >[CHARTAB + $0540 + (9 * $30)]

;LOSCORETAB2    BYTE $C0,$F0,$20,$50,$80,$B0,$E0,$10,$40,$70
;HISCORETAB2    BYTE $17,$17,$18,$18,$18,$18,$18,$19,$19,$19
LOSCORETAB2    BYTE <[CHARTAB + $07C0 + (0 * $30)], <[CHARTAB + $07C0 + (1 * $30)], <[CHARTAB + $07C0 + (2 * $30)]
               BYTE <[CHARTAB + $07C0 + (3 * $30)], <[CHARTAB + $07C0 + (4 * $30)], <[CHARTAB + $07C0 + (5 * $30)]
               BYTE <[CHARTAB + $07C0 + (6 * $30)], <[CHARTAB + $07C0 + (7 * $30)], <[CHARTAB + $07C0 + (8 * $30)]
               BYTE <[CHARTAB + $07C0 + (9 * $30)]
HISCORETAB2    BYTE >[CHARTAB + $07C0 + (0 * $30)], >[CHARTAB + $07C0 + (1 * $30)], >[CHARTAB + $07C0 + (2 * $30)]
               BYTE >[CHARTAB + $07C0 + (3 * $30)], >[CHARTAB + $07C0 + (4 * $30)], >[CHARTAB + $07C0 + (5 * $30)]
               BYTE >[CHARTAB + $07C0 + (6 * $30)], >[CHARTAB + $07C0 + (7 * $30)], >[CHARTAB + $07C0 + (8 * $30)]
               BYTE >[CHARTAB + $07C0 + (9 * $30)]

XCORKTAB       BYTE 0,37

SCOREX         BYTE 0,1,3,4,6,7
SCOREZ         BYTE 31,32,34,35,37,38
SCORENO        BYTE 5,4,3,2,1,0

SCOREXZ        BYTE 16,17,19,20,22,23

ANDTAB         BYTE $00,$F0,$00,$F0,$00,$F0

;;;
;;; 2009-09: MY: DASM doesn't have the CBM macro to declare strings as arrays
;;;              of Commodore screen codes.  This is a poor-man's version that
;;;              works for most letters numbers and punctuation symbols.
;;;
;;;              Replace this:
;;;                CBM "SOME STRING"
;;;              with this:
;;;                DV.B ASCII2CBM 'S, 'O, 'M, 'E, ' , 'S, 'T, 'R, 'I, 'N, 'G
;;;
;;;              Note that spaces on the end of lines can be a problem as many
;;;              editors remove them, so they are best done as follows -- eg
;;;              replace this:
;;;                CBM "X "
;;;              with this:
;;;                DV.B ASCII2CBM 'X, " "
;;;
ASCII2CBM   EQM [.. & $3F]

MESSTAB        ;CBM ".... BUBBLER .... COPYRIGHT"
               ;CBM " ASHBY COMPUTERS AND GRAPHICS"
               ;CBM " LTD. 1987  -  ALL"
               ;CBM " GRAPHICS AND AUDIO VISUAL BY"
               ;CBM " ULTIMATE PLAY THE GAME."
               ;CBM " SOFTWARE CONVERSION BY JEMS AND MATT"
               ;CBM " OF LYNSOFT -U.K.- LTD.    "
               DV.B ASCII2CBM ' , '., '., '., ' , 'B, 'U, 'B, 'B, 'L, 'E, 'R
               DV.B ASCII2CBM ' , '., '., '.
               DV.B ASCII2CBM ' , 'C, 'O, 'P, 'Y, 'R, 'I, 'G, 'H, 'T
               DV.B ASCII2CBM ' , 'A, 'S, 'H, 'B, 'Y
               DV.B ASCII2CBM ' , 'C, 'O, 'M, 'P, 'U, 'T, 'E, 'R, 'S
               DV.B ASCII2CBM ' , 'A, 'N, 'D, ' , 'G, 'R, 'A, 'P, 'H, 'I, 'C, 'S
               DV.B ASCII2CBM ' , 'L, 'T, 'D, ' , '1, '9, '8, '7, ' , '-
               DV.B ASCII2CBM ' , 'A, 'L, 'L
               DV.B ASCII2CBM ' , 'G, 'R, 'A, 'P, 'H, 'I, 'C, 'S, ' , 'A, 'N, 'D
               DV.B ASCII2CBM ' , 'A, 'U, 'D, 'I, 'O, ' , 'V, 'I, 'S, 'U, 'A, 'L
               DV.B ASCII2CBM ' , 'B, 'Y, ' , 'U, 'L, 'T, 'I, 'M, 'A, 'T, 'E
               DV.B ASCII2CBM ' , 'P, 'L, 'A, 'Y, ' , 'T, 'H, 'E
               DV.B ASCII2CBM ' , 'G, 'A, 'M, 'E, '.
               DV.B ASCII2CBM ' , 'S, 'O, 'F, 'T, 'W, 'A, 'R, 'E
               DV.B ASCII2CBM ' , 'C, 'O, 'N, 'V, 'E, 'R, 'S, 'I, 'O, 'N
               DV.B ASCII2CBM ' , 'T, 'O, ' , 'C, '6, '4, ' , 'I, 'N
               DV.B ASCII2CBM ' , '1, '9, '8, '7, '/, '8, ' , 'B, 'Y
               DV.B ASCII2CBM ' , 'J, 'A, 'K, 'E, ' , 'S, 'I, 'M, 'P, 'S, 'O, 'N
               DV.B ASCII2CBM ' , 'A, 'N, 'D, ' , 'M, 'A, 'T, 'T
               DV.B ASCII2CBM ' , 'Y, 'O, 'U, 'N, 'G, ' , 'F, 'O, 'R
               DV.B ASCII2CBM ' , 'L, 'Y, 'N, 'S, 'O, 'F, 'T, ' , 'U, 'K
               DV.B ASCII2CBM ' , 'L, 'T, 'D, '., ' , '=, '=, '=
               DV.B ASCII2CBM ' , 'T, 'H, 'A, 'N, 'K, 'S, ' , 'T, 'O
               DV.B ASCII2CBM ' , 'F, 'R, 'A, 'N, 'K
               DV.B ASCII2CBM ' , 'G, 'A, 'S, 'K, 'I, 'N, 'G, ',
               DV.B ASCII2CBM ' , 'D, 'A, 'N, 'I, 'E, 'L
               DV.B ASCII2CBM ' , 'R, 'A, 'M, 'S, 'A, 'Y, ' , 'A, 'N, 'D
               DV.B ASCII2CBM ' , 'E, 'S, 'P, 'E, 'C, 'I, 'A, 'L, 'L, 'Y
               DV.B ASCII2CBM ' , 'J, 'A, 'N, 'I
               DV.B ASCII2CBM ' , 'T, 'A, 'H, 'V, 'A, 'N, 'A, 'I, 'N, 'E, 'N
               DV.B ASCII2CBM ' , 'F, 'O, 'R
               DV.B ASCII2CBM ' , 'H, 'E, 'L, 'P, ' , 'G, 'E, 'T, 'T, 'I, 'N, 'G
               DV.B ASCII2CBM ' , 'T, 'H, 'I, 'S, ' , 'R, 'U, 'N, 'N, 'I, 'N, 'G
               DV.B ASCII2CBM ' , 'A, 'G, 'A, 'I, 'N, ' , 'I, 'N
               DV.B ASCII2CBM ' , '2, '0, '0, '9, ' , '-, ' , 'M, '., 'Y, '.
               DV.B ASCII2CBM ' , '=, '=, '=
               DV.B ASCII2CBM ' , ' , ' , ' , ' , ' , ' , ' , " "
ENMESS

;; 2009-10: MY: Changed code to support scrolling message longer than 255 characters
;;              on options screen
;               IF #ENMESS-MESSTAB-1 > 255
;               ECHO "MESSAGE IN MESSTAB IS TOO LONG!"
;               ERR
;               ENDIF

               ; Last byte of this (player number) gets modified by the code
WELCOME        ;BYTE 16,12,1,25,5,18,32,49      ;"player 1"
               DV.B ASCII2CBM 'P, 'L, 'A, 'Y, 'E, 'R, ' , '1

STARTAB        BYTE 6,7,8,9,10,11,12,13,14,15  ; offsets in work area

TIMEOUT        ;BYTE 20,9,13,5,32,15,21,20      ;"time out"
               DV.B ASCII2CBM 'T, 'I, 'M, 'E, ' , 'O, 'U, 'T
TIMEOUTEND

GAMEOVER       ;BYTE 7,1,13,5,32,15,22,5,18   ;"game over"
               DV.B ASCII2CBM 'G, 'A, 'M, 'E, ' , 'O, 'V, 'E, 'R
GAMEOVEREND

NOTHING        ;BYTE 32,32          ;"  "
               DV.B ASCII2CBM " ", " "


               ;;
               ;; 2009-10: MY: We never got as far as implementing an end of
               ;;              game message the first time around.  The text
               ;;              here is copied from an online screen-shot/map of
               ;;              the Spectrum version.
               ;;
               ;;              $FF followed by a positive integer means insert
               ;;              that many spaces.
               ;;              $FF followed by 0 means end of message.
               ;;              $FE means toggle double-height printing
               ;;
GAME_COMPLETE_MESSAGE
               BYTE $FE, $FF, 2
               DV.B ASCII2CBM 'C, 'O, 'N, 'G, 'R, 'A, 'T, 'U, 'L, 'A, 'T, 'I, 'O, 'N, 'S
               BYTE $FF, 3
               BYTE $FE, $FF, 22
               DV.B ASCII2CBM 'Y, 'O, 'U, ' , 'H, 'A, 'V, 'E, ' , 'E, 'S, 'C, 'A, 'P, 'E, 'D
               BYTE $FF, 23
               DV.B ASCII2CBM 'F, 'R, 'O, 'M, ' , 'V, 'A, 'D, 'R, 'A, 'S, ' , 'P, 'R, 'I, 'S, 'O, 'N
               BYTE $FF, 22
               DV.B ASCII2CBM 'Y, 'O, 'U, 'R, ' , 'N, 'E, 'X, 'T, ' , 'M, 'I, 'S, 'S, 'I, 'O, 'N
               BYTE $FF, 24
               DV.B ASCII2CBM 'I, 'S, ' , 'T, 'O, ' , 'O, 'V, 'E, 'R, 'T, 'H, 'R, 'O, 'W
               BYTE $FF, 24
               DV.B ASCII2CBM 'V, 'A, 'D, 'R, 'A, ' , 'A, 'N, 'D, ' , 'F, 'R, 'E, 'E, ' , 'T, 'H, 'E
               BYTE $FF, 25
               DV.B ASCII2CBM 'I, 'R, 'K, 'O, 'N, ' , 'P, 'E, 'O, 'P, 'L, 'E
               BYTE $FF, 4
               BYTE $FF, 0

SARAH          HEX FEFDFBF7EFDFBF7F ;write values for keyboard row numbers

ROWTAB         BYTE 223,253,247,251,251,253,251,247,247,239,239,239,223
               BYTE 239,239,239,223,127,251,253,251,247,247,253,251,247,253,223,191
               BYTE 191,191,127,127,127,127,127,253,254,254,254,254,191
               BYTE 223,223,223,223,191,239,127,127,253,253,251,251,247
               BYTE 247,239,191,254,191,254,191,254,254
;ROWTAB         BYTE $DF,$FD,$F7,$FB,$FB,$FD,$FB,$F7,$F7,$EF,$EF,$EF,$DF
;               BYTE $EF,$EF,$EF,$DF,$7F,$FB,$FD,$FB,$F7,$F7,$FD,$FB,$F7,$FD,$DF,$BF
;               BYTE $BF,$BF,$7F,$7F,$7F,$7F,$7F,$FD,$FE,$FE,$FE,$FE,$BF
;               BYTE $DF,$DF,$DF,$DF,$BF,$EF,$7F,$7F,$FD,$FD,$FB,$FB,$F7
;               BYTE $F7,$EF,$BF,$FE,$BF,$FE,$BF,$FE,$FE

BITPAT         BYTE %01000000,%00000100,%00010000,%00010000
               BYTE %00000100,%01000000,%00100000,%00000100
               BYTE %00100000,%00000010,%00000100,%00100000
               BYTE %00000100,%00010000,%10000000,%01000000
               BYTE %00000010,%01000000,%00000010,%00100000
               BYTE %01000000,%01000000,%10000000,%00000010
               BYTE %10000000,%00000010,%00010000,%00100000
               BYTE %00000001,%00000100,%01000000,%00000010
               BYTE %00010000,%00000100,%10000000,%00100000
               BYTE %10000000,%00010000,%00100000,%01000000
               BYTE %00001000,%00000010,%00000001,%10000000
               BYTE %00001000,%00010000,%10000000,%00001000
               BYTE %00000001,%00001000,%00000001,%00001000
               BYTE %00000001,%00001000,%00000001,%00001000
               BYTE %00000001,%00001000,%00000001,%00100000
               BYTE %00000010,%00010000,%10000000,%00000100


UPDOWNL        BYTE <[WA+(0*$C0)-1], <[WA+(1*$C0)-1], <[WA+(2*$C0)-1], <[WA+(3*$C0)-1]
               BYTE <[WA+(4*$C0)-1], <[WA+(5*$C0)-1], <[WA+(6*$C0)-1], <[WA+(7*$C0)-1]
               BYTE <[WA+(8*$C0)-1], <[WA+(9*$C0)-1], <[WA+(10*$C0)-1], <[WA+(11*$C0)-1]
               BYTE <[WA+(12*$C0)-1], <[WA+(13*$C0)-1], <[WA+(14*$C0)-1], <[WA+(15*$C0)-1]

UPDOWNH        BYTE >[WA+(0*$C0)-1], >[WA+(1*$C0)-1], >[WA+(2*$C0)-1], >[WA+(3*$C0)-1]
               BYTE >[WA+(4*$C0)-1], >[WA+(5*$C0)-1], >[WA+(6*$C0)-1], >[WA+(7*$C0)-1]
               BYTE >[WA+(8*$C0)-1], >[WA+(9*$C0)-1], >[WA+(10*$C0)-1], >[WA+(11*$C0)-1]
               BYTE >[WA+(12*$C0)-1], >[WA+(13*$C0)-1], >[WA+(14*$C0)-1], >[WA+(15*$C0)-1]


               ; Low bytes now also calculated so this table not needed
;CHARVEC1L     BYTE <$D000, <$D008, <$D010, <$D018, <$D020, <$D028, <$D030, <$D038
;              BYTE <$D040, <$D048, <$D050, <$D058, <$D060, <$D068, <$D070, <$D078
;              BYTE <$D080, <$D088, <$D090, <$D098, <$D0A0, <$D0A8, <$D0B0, <$D0B8
;              BYTE <$D0C0, <$D0C8, <$D0D0, <$D0D8, <$D0E0, <$D0E8, <$D0F0, <$D0F8
;              BYTE <$D100, <$D108, <$D110, <$D118, <$D120, <$D128, <$D130, <$D138
;              BYTE <$D140, <$D148, <$D150, <$D158, <$D160, <$D168, <$D170, <$D178
;              BYTE <$D180, <$D188, <$D190, <$D198, <$D1A0, <$D1A8, <$D1B0, <$D1B8
;              BYTE <$D1C0, <$D1C8, <$D1D0, <$D1D8, <$D1E0, <$D1E8, <$D1F0, <$D1F8


               ;high bytes now calculated so this table not needed

;CHARVEC1H     ;DH $D000,$D008,$D010,$D018,$D020,$D028,$D030,$D038

;              ;DH $D040,$D048,$D050,$D058,$D060,$D068,$D070,$D078
;              ;DH $D080,$D088,$D090,$D098,$D0A0,$D0A8,$D0B0,$D0B8
;              ;DH $D0C0,$D0C8,$D0D0,$D0D8,$D0E0,$D0E8,$D0F0,$D0F8
;              ;DH $D100,$D108,$D110,$D118,$D120,$D128,$D130,$D138
;              ;DH $D140,$D148,$D150,$D158,$D160,$D168,$D170,$D178
;              ;DH $D180,$D188,$D190,$D198,$D1A0,$D1A8,$D1B0,$D1B8
;              ;DH $D1C0,$D1C8,$D1D0,$D1D8,$D1E0,$D1E8,$D1F0,$D1F8


PLOTYLO        BYTE        <[HIRES],  <[HIRES+320*1],  <[HIRES+320*2],  <[HIRES+320*3],  <[HIRES+320*4]
               BYTE  <[HIRES+320*5],  <[HIRES+320*6],  <[HIRES+320*7],  <[HIRES+320*8],  <[HIRES+320*9]
               BYTE <[HIRES+320*10], <[HIRES+320*11], <[HIRES+320*12], <[HIRES+320*13], <[HIRES+320*14]
               BYTE <[HIRES+320*15], <[HIRES+320*16], <[HIRES+320*17], <[HIRES+320*18], <[HIRES+320*19]
               BYTE <[HIRES+320*20], <[HIRES+320*21], <[HIRES+320*22], <[HIRES+320*23], <[HIRES+320*24]

PLOTYHI        BYTE        >[HIRES],  >[HIRES+320*1],  >[HIRES+320*2],  >[HIRES+320*3],  >[HIRES+320*4]
               BYTE  >[HIRES+320*5],  >[HIRES+320*6],  >[HIRES+320*7],  >[HIRES+320*8],  >[HIRES+320*9]
               BYTE >[HIRES+320*10], >[HIRES+320*11], >[HIRES+320*12], >[HIRES+320*13], >[HIRES+320*14]
               BYTE >[HIRES+320*15], >[HIRES+320*16], >[HIRES+320*17], >[HIRES+320*18], >[HIRES+320*19]
               BYTE >[HIRES+320*20], >[HIRES+320*21], >[HIRES+320*22], >[HIRES+320*23], >[HIRES+320*24]


TIMETABLE      BYTE $81,$91,$A1,$B1,$42,$52,$62,$72,$82,$92,$A2,$B2
               BYTE $43,$53,$63,$73,$83,$93,$A3,$B3
               BYTE $44,$54,$64,$74,$84,$94,$A4,$B4
               BYTE $45,$55,$65,$75,$85,$95,$A5,$B5
               BYTE $46,$56,$66,$76,$86,$96,$A6,$B6
               BYTE $47,$57,$67,$77,$87,$97,$A7,$B7
               BYTE $48,$58,$68,$78,$88,$98,$A8,$B8
               BYTE $49,$59,$69,$79,$89,$99,$A9,$B9
               BYTE $4A,$5A,$6A,$7A,$8A,$9A,$AA,$BA
               BYTE $4B,$5B,$6B,$7B,$8B,$9B,$AB,$BB
               BYTE $4C,$5C,$6C,$7C,$8C,$9C,$AC,$BC
               BYTE $4D,$5D,$6D,$7D,$8D,$9D,$AD,$BD


TIMETABLE2     BYTE $0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0,$0,$0,$0,$0,$01,$02,$03
               BYTE $0C,$0C,$0C,$0C,$0C,$0D,$0E,$0E


               ;3's division tables for 45 degree slope print
DIV3           HEX 000000101010202020303030404040505050606060707070
REM3           HEX 204060204060204060204060204060204060204060204060
AREM3          HEX 002040002040002040002040002040002040002040002040


SCRVECLO       BYTE <[HIRES+(15*320)+(10*8)]
               BYTE <[HIRES+(14*320)+(10*8)], <[HIRES+(13*320)+(10*8)]
               BYTE <[HIRES+(12*320)+(10*8)], <[HIRES+(11*320)+(10*8)]
               BYTE <[HIRES+(10*320)+(10*8)], <[HIRES+(9*320)+(10*8)]
               BYTE <[HIRES+(8*320)+(10*8)], <[HIRES+(7*320)+(10*8)]
               BYTE <[HIRES+(6*320)+(10*8)], <[HIRES+(5*320)+(10*8)]
               BYTE <[HIRES+(4*320)+(10*8)], <[HIRES+(3*320)+(10*8)]
               BYTE <[HIRES+(2*320)+(10*8)], <[HIRES+(1*320)+(10*8)]

SCRVECHI       BYTE >[HIRES+(15*320)+(10*8)]
               BYTE >[HIRES+(14*320)+(10*8)], >[HIRES+(13*320)+(10*8)]
               BYTE >[HIRES+(12*320)+(10*8)], >[HIRES+(11*320)+(10*8)]
               BYTE >[HIRES+(10*320)+(10*8)], >[HIRES+(9*320)+(10*8)]
               BYTE >[HIRES+(8*320)+(10*8)], >[HIRES+(7*320)+(10*8)]
               BYTE >[HIRES+(6*320)+(10*8)], >[HIRES+(5*320)+(10*8)]
               BYTE >[HIRES+(4*320)+(10*8)], >[HIRES+(3*320)+(10*8)]
               BYTE >[HIRES+(2*320)+(10*8)], >[HIRES+(1*320)+(10*8)]

;;;
;;; 2009-09: MY: Note that the code assumes that these three are next too each
;;;              other!
;;;
SCORE1         BYTE 0,0,0,0,0,0,0,0
SCORE2         BYTE 0,0,0,0,0,0,0
HISCORE        BYTE 5,4,6,0,1,0,0

; High nybble is colour of play area for each stage
SCRCOL         BYTE $70,$70,$50,$10,$50,$70


FLR1VL         BYTE <ZAZ1, <ZAZ1, <ZAZ2, <ZAZ2, <ZAZ1, <ZAZ1
FLR2VL         BYTE >ZAZ1, >ZAZ1, >ZAZ2, >ZAZ2, >ZAZ1, >ZAZ1
FLR3VL         BYTE <ZAZ1A, <ZAZ1A, <ZAZ2A, <ZAZ2A, <ZAZ1A, <ZAZ1A
FLR4VL         BYTE >ZAZ1A, >ZAZ1A, >ZAZ2A, >ZAZ2A, >ZAZ1A, >ZAZ1A


TOP1VL         BYTE <ZAZ3, <ZAZ3, <ZAZ4, <ZAZ3, <ZAZ4, <ZAZ3
TOP2VL         BYTE >ZAZ3, >ZAZ3, >ZAZ4, >ZAZ3, >ZAZ4, >ZAZ3
TOP3VL         BYTE <ZAZ3A, <ZAZ3A, <ZAZ4A, <ZAZ3A, <ZAZ4A, <ZAZ3A
TOP4VL         BYTE >ZAZ3A, >ZAZ3A, >ZAZ4A, >ZAZ3A, >ZAZ4A, >ZAZ3A


RGT1VL         BYTE <ZAZ5, <ZAZ5, <ZAZ6, <ZAZ6, <ZAZ5, <ZAZ5
RGT2VL         BYTE >ZAZ5, >ZAZ5, >ZAZ6, >ZAZ6, >ZAZ5, >ZAZ5
RGT3VL         BYTE <ZAZ5A, <ZAZ5A, <ZAZ6A, <ZAZ6A, <ZAZ5A, <ZAZ5A
RGT4VL         BYTE >ZAZ5A, >ZAZ5A, >ZAZ6A, >ZAZ6A, >ZAZ5A, >ZAZ5A


LFT1VL         BYTE <ZAZ7, <ZAZ7, <ZAZ8, <ZAZ8, <ZAZ7, <ZAZ7
LFT2VL         BYTE >ZAZ7, >ZAZ7, >ZAZ8, >ZAZ8, >ZAZ7, >ZAZ7
LFT3VL         BYTE <ZAZ7A, <ZAZ7A, <ZAZ8A, <ZAZ8A, <ZAZ7A, <ZAZ7A
LFT4VL         BYTE >ZAZ7A, >ZAZ7A, >ZAZ8A, >ZAZ8A, >ZAZ7A, >ZAZ7A


SONLO          BYTE <SERPUT, <SERSHOT, <SERJUMP, <SERBANG, <MATSONIX, <SEREXTRA
SONHI          BYTE >SERPUT, >SERSHOT, >SERJUMP, >SERBANG, >MATSONIX, >SEREXTRA


LEAPTAB        BYTE 0,2,6,12,16,19,21,24,27,29,30,31,32,33,34,35,34,33,31,30
               BYTE 29,27,24,21,19,16,14,16,12,6,2,0,0


BANGTAB        BYTE 6-VAR,9-VAR,12-VAR,14-VAR,16-VAR,18-VAR,19-VAR,20-VAR,21-VAR
               BYTE 21-VAR,20-VAR,19-VAR,18-VAR,16-VAR,14-VAR,12-VAR,9-VAR,6-VAR,3-VAR


XTRATAB        BYTE 22,16,22,33

KUSER          BYTE "  ",40,"    ",41,"    ",30,"    ",31
               ;BYTE "    ", ['J-$40], ['U-$40], ['M-$40], ['P-$40]
               ;BYTE "    ", ['F-$40], ['I-$40], ['R-$40], ['E-$40], "  "
               DV.B ASCII2CBM ' , ' , ' , ' , 'J, 'U, 'M, 'P
               DV.B ASCII2CBM ' , ' , ' , ' , 'F, 'I, 'R, 'E
               DV.B ASCII2CBM " ", " "

KUSER2         BYTE " ",66,26,65,"  ",66,24,65,"  ",66,59,65,"  ",66,47,65,"   "
               BYTE 66,64,65,"     ",66,62,65,"   "


CURTAB         BYTE 4,9,14,19,25,33

LASTKEYTAB     BYTE %00000001,%00000010,%00000100,%00001000
               BYTE %00010000,%00100000,%01000000,%10000000

;
; Key codes returned by BOARDCHK
;
KATE           BYTE 255,60,255,40,37,38,39,255
               BYTE 255,23,1,255,26,19,5,255
               BYTE 255,18,4,255,3,6,20,24
               BYTE 255,25,7,255,2,8,21,22
               BYTE 255,9,10,255,13,11,15,14
               BYTE 42,16,12,44,45,27,0,43
               BYTE 28,41,29,255,255,59,30,46
               BYTE 255,31,255,255,32,255,17,255

;
; What to display on the "define keys" screen for each key (eg want to display
; a carriage return symbol for return key etc)
;
LYNDA          BYTE  0, 1, 2, 3, 4, 5, 6, 7
               BYTE  8, 9,10,11,12,13,14,15
               BYTE 16,17,18,19,20,21,22,23
               BYTE 24,25,26,58,60,59,30,40
               BYTE 64, 0, 0, 0, 0,49,51,53
               BYTE 55,42,43,44,45,46,47,48
               BYTE 49,50,51,52,53,54,55,56
               BYTE 57, 0, 0,61,62, 0, 0, 1


FIL_TAB        BYTE 1,2,4

FIL_TAB2       BYTE 254,253,251


SPECTL         BYTE <[(0*SPECSIZ)+SPECAL+1], <[(1*SPECSIZ)+SPECAL+1], <[(2*SPECSIZ)+SPECAL+1], <[(3*SPECSIZ)+SPECAL+1]
               BYTE <[(4*SPECSIZ)+SPECAL+1], <[(5*SPECSIZ)+SPECAL+1], <[(6*SPECSIZ)+SPECAL+1], <[(7*SPECSIZ)+SPECAL+1]
               BYTE <[(8*SPECSIZ)+SPECAL+1], <[(9*SPECSIZ)+SPECAL+1], <[(10*SPECSIZ)+SPECAL+1], <[(11*SPECSIZ)+SPECAL+1]
               BYTE <[(12*SPECSIZ)+SPECAL+1], <[(13*SPECSIZ)+SPECAL+1], <[(14*SPECSIZ)+SPECAL+1], <[(15*SPECSIZ)+SPECAL+1]
               BYTE <[(16*SPECSIZ)+SPECAL+1], <[(17*SPECSIZ)+SPECAL+1], <[(18*SPECSIZ)+SPECAL+1], <[(19*SPECSIZ)+SPECAL+1]
               BYTE <[(20*SPECSIZ)+SPECAL+1], <[(21*SPECSIZ)+SPECAL+1], <[(22*SPECSIZ)+SPECAL+1], <[(23*SPECSIZ)+SPECAL+1]
               BYTE <[(24*SPECSIZ)+SPECAL+1], <[(25*SPECSIZ)+SPECAL+1], <[(26*SPECSIZ)+SPECAL+1], <[(27*SPECSIZ)+SPECAL+1]

SPECTH         BYTE >[(0*SPECSIZ)+SPECAL+1], >[(1*SPECSIZ)+SPECAL+1], >[(2*SPECSIZ)+SPECAL+1], >[(3*SPECSIZ)+SPECAL+1]
               BYTE >[(4*SPECSIZ)+SPECAL+1], >[(5*SPECSIZ)+SPECAL+1], >[(6*SPECSIZ)+SPECAL+1], >[(7*SPECSIZ)+SPECAL+1]
               BYTE >[(8*SPECSIZ)+SPECAL+1], >[(9*SPECSIZ)+SPECAL+1], >[(10*SPECSIZ)+SPECAL+1], >[(11*SPECSIZ)+SPECAL+1]
               BYTE >[(12*SPECSIZ)+SPECAL+1], >[(13*SPECSIZ)+SPECAL+1], >[(14*SPECSIZ)+SPECAL+1], >[(15*SPECSIZ)+SPECAL+1]
               BYTE >[(16*SPECSIZ)+SPECAL+1], >[(17*SPECSIZ)+SPECAL+1], >[(18*SPECSIZ)+SPECAL+1], >[(19*SPECSIZ)+SPECAL+1]
               BYTE >[(20*SPECSIZ)+SPECAL+1], >[(21*SPECSIZ)+SPECAL+1], >[(22*SPECSIZ)+SPECAL+1], >[(23*SPECSIZ)+SPECAL+1]
               BYTE >[(24*SPECSIZ)+SPECAL+1], >[(25*SPECSIZ)+SPECAL+1], >[(26*SPECSIZ)+SPECAL+1], >[(27*SPECSIZ)+SPECAL+1]


CELL_TAB_Y                    ;all maps
ZERO           BYTE $23,$FF                                     ;7c80
ONE            BYTE $23,$1C,$29,$FF                             ;7d00
TWO            BYTE $23,$1C,$29,$15,$1D,$25,$FF                 ;7d80
THREE          BYTE $23,$1C,$29,$15,$1D,$25,$0E,$16,$1E,$26,$FF ;7e00
FOUR           BYTE $1C,$29,$15,$1D,$25,$0E,$16,$1E,$26,$07,$0F ;7e80
               BYTE $17,$1F,$27,$FF
FIVE           BYTE $15,$1D,$25,$0E,$16,$1E,$26,$07,$0F,$17,$1F ;7f00
               BYTE $27,$00,$08,$10,$18,$20,$28,$FF
SIX            BYTE $0E,$16,$1E,$26,$07,$0F,$17,$1F,$27,$00,$08 ;7f80
               BYTE $10,$18,$20,$28,$01,$09,$11,$19,$21,$29,$FF
SEVEN          BYTE $07,$0F,$17,$1F,$27,$00,$08,$10,$18,$20,$28 ;8000
               BYTE $01,$09,$11,$19,$21,$29,$02,$0A,$12,$1A,$22
               BYTE $FF
EIGHT          BYTE $00,$08,$10,$18,$20,$28,$01,$09,$11,$19,$21 ;8080
               BYTE $29,$02,$0A,$12,$1A,$22,$03,$0B,$13,$1B,$FF
NINE           BYTE $01,$09,$11,$19,$21,$29,$02,$0A,$12,$1A,$22 ;8100
               BYTE $03,$0B,$13,$1B,$09,$0C,$1C,$FF
AEY            BYTE $02,$0A,$12,$1A,$22,$03,$0B,$13,$1B,$04,$0C ;8180
               BYTE $14,$05,$0D,$FF
BEE            BYTE $03,$0B,$13,$1B,$04,$0C,$14,$05,$0D,$06,$FF ;8200
CEE            BYTE $04,$0C,$14,$05,$0D,$06,$FF                 ;8280
DEE            BYTE $05,$0D,$06,$FF                             ;8300
EII            BYTE $06,$FF                                     ;8380

               ;
               ; Bits 7-9 of your "screen Y" position are used as in index
               ; into this table to get the appropriate ($FF-terminated) list
               ; (above) of map cell numbers that could be on screen.
               ;
YET_ANOTH      WORD ZERO,ZERO,ONE,TWO,THREE,FOUR,FIVE
               WORD SIX,SEVEN,EIGHT,NINE,AEY,BEE,CEE
               WORD DEE,EII

               ;
               ; Look-up table from cell number to cell x-coordinate
               ;
CELL_X_TAB     BYTE 1,2,3,4,5,6,7
               BYTE 1,2,3,4,5,6,7
               BYTE 1,2,3,4,5,6,7
               BYTE 1,2,3,4,5,6,7
               BYTE 1,2,3,4,5,6,7
               BYTE 1,2,3,4,5,6,7

               ;
               ; Look-up table from cell number to cell y-coordinate
               ;
CELL_Y_TAB     BYTE 1,1,1,1,1,1,1
               BYTE 2,2,2,2,2,2,2
               BYTE 3,3,3,3,3,3,3
               BYTE 4,4,4,4,4,4,4
               BYTE 5,5,5,5,5,5,5
               BYTE 6,6,6,6,6,6,6

               ;
               ; Look-up table from map cell number to mid-point of possible
               ; high-byte "screen X" coordinates
               ;
CELL_X2_TAB    BYTE 2,3,4,5,6,7,8
               BYTE 3,4,5,6,7,8,9
               BYTE 4,5,6,7,8,9,$A
               BYTE 5,6,7,8,9,$A,$B
               BYTE 6,7,8,9,$A,$B,$C
               BYTE 7,8,9,$A,$B,$C,$D

CELL_Y2_TLO    BYTE $00,$80,$00,$80,$00,$80,$00
               BYTE $80,$00,$80,$00,$80,$00,$80
               BYTE $00,$80,$00,$80,$00,$80,$00
               BYTE $80,$00,$80,$00,$80,$00,$80
               BYTE $00,$80,$00,$80,$00,$80,$00
               BYTE $80,$00,$80,$00,$80,$00,$80

CELL_Y2_THI    BYTE $80,$80,$81,$81,$82,$82,$83
               BYTE $7F,$80,$80,$81,$81,$82,$82
               BYTE $7F,$7F,$80,$80,$81,$81,$82
               BYTE $7E,$7F,$7F,$80,$80,$81,$81
               BYTE $7E,$7E,$7F,$7F,$80,$80,$81
               BYTE $7D,$7E,$7E,$7F,$7F,$80,$80

BOTT_NO        BYTE 0,5,5,12,12,9 ; Number of bottles on each level?

               ; Maps are made up of cells: 7 in X direction times 6 in Y
               ; direction, totalling 42.  See BubblerMapEtcData.asm for
               ; cell data.  The first byte of each cell is:
               ;   bits 0-5 = number of shapes in that cell
               ;   bit    7 = set if there are any shapes in that cell that you
               ;              can go behind
               ; The next 2 bytes (or 3 bytes if the number of shapes in the
               ; cell is greater than $10) are bit flags which are set for the
               ; shapes you can pass behind.  Bit 7 of the first byte is for
               ; shape 1, bit 6 for shape 2, etc.
               ;
               ; Beyond this, three bytes are used to store each shape:
               ;   byte 0, bits 4-7 = x coordinate
               ;           bits 0-3 = y coordinate
               ;   byte 1, bits 4-6 = z cooridnate
               ;           bit 7 + bits 0-3 = shape type
               ;   byte 2, bits 6-7 = z dimension
               ;           bits 3-5 = y dimension
               ;           bits 0-2 = x dimension
               ; EXCEPT if shape type = $1F = moving background shape then:
               ;   byte 2, bits 0-3: 0 = lift set-up position
               ;                     1 = spare
               ;                     2 = materialise pad
               ;                     3 = spike alien
               ;                     4 = gun
               ;           bit 6: extra bit for z coordinate
               ;           bit 7: set when moving background shape on screen
               ;                  (cleared when it passes off screen or when
               ;                  you die)
               ;
               ; Coordinates and dimensions are multiplied by 16 to give pixels
               ; (ie shape locations and sizes are stored to 16-pixel accuracy).
               ;
DEF_MAP_1      WORD SCR0,SCR1,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR2,SCR3,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0

DEF_MAP_2      WORD SCR20,SCR21,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR22,SCR23,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0

DEF_MAP_4      WORD SCR43,SCR42,SCR41,SCR0,SCR0,SCR0,SCR0
               WORD SCR46,SCR45,SCR44,SCR0,SCR0,SCR0,SCR0
               WORD SCR49,SCR48,SCR47,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0

DEF_MAP_3      WORD SCR33,SCR32,SCR31,SCR0,SCR0,SCR0,SCR0
               WORD SCR36,SCR35,SCR34,SCR0,SCR0,SCR0,SCR0
               WORD SCR39,SCR38,SCR37,SCR0,SCR0,SCR0,SCR0
               WORD SCR312,SCR311,SCR310,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0

DEF_MAP_5      WORD SCR57,SCR56,SCR55,SCR54,SCR53,SCR52,SCR51
               WORD SCR59,SCR58,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR510,SCR511,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR512,SCR513,SCR514,SCR515,SCR516,SCR517
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0
               WORD SCR0,SCR0,SCR0,SCR0,SCR0,SCR0,SCR0

MAP_BASE       WORD 0
               WORD DEF_MAP_1,DEF_MAP_2
               WORD DEF_MAP_4,DEF_MAP_3
               WORD DEF_MAP_5

TRAP_BASE      WORD 0
               WORD TRAP1,TRAP2
               WORD TRAP3,TRAP4
               WORD TRAP5


;****************************************
;*
;*             Lots of tables in this program !!
;*             These are graphics offsets and
;*             dimensions for the moving
;*             background shapes.
;*
;*             Shape: 0 - lift set-up pos
;*                             1 - spare
;*                             2 - materialise pad
;*                             3 - spike alien
;*                             4 - gun
;*
;****************************************

SHPTBS         HEX 06000C1010 ;shape

SHPTBD         HEX 0000000001 ;data offset

SHPTBX         HEX 0A00100B0A
SHPTBY         HEX 0A00100B0A
SHPTBZ         HEX 0A00000A10

; Table of flags showing which types of SPECAL (ie background) object the blob
; is allowed to start on.
;
; See comment at SPECAL for shape numbers, but essentially the background
; objects the blob is allowed to start on are:
;   floor ($02)
;   block ($03)
;   bottle ($11)
;   bunged bottle ($12)
;   top trap door ($13)
;   exit trap door ($15)
;
; 2009-10: MY: Personally I think it's rather annoying that you can start on a
;              top trap door as it means you fall down it straight away.  And
;              on level 2 at least this can result in instant death because
;              some traps deposit you above the void when you fall down them.
;
;              In the "enhanced" build I've changed this table to prevent top
;              trap doors being used as starting points.
;
RETAIN         BYTE 0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1
               IFNCONST BUILD_ENHANCED_VERSION
               BYTE 1
               ELSE
               BYTE 0
               ENDIF
               BYTE 0,1,0


JUMP_TAB       BYTE 2,2,2,2,2,2,2,2,2,2,4,4,4,4,2,2,3,3,3

;
; Indexes into IXTAB
;
SPRTABL        BYTE <[IXTAB], <[ALTABS], <[ALTABS+IXSIZ]
               BYTE <[SFIRE], <[SFIRE+IXSIZ], <[MYBUL], <[MYBUL+IXSIZ]
               BYTE <[BACKS], <[BACKS2], <[STATOB], <[STATOB+IXSIZ]
               BYTE <[STATOB+(IXSIZ*2)], <[STATOB+(IXSIZ*3)]

SPRTABH        BYTE >[IXTAB], >[ALTABS], >[ALTABS+IXSIZ]
               BYTE >[SFIRE], >[SFIRE+IXSIZ], >[MYBUL], >[MYBUL+IXSIZ]
               BYTE >[BACKS], >[BACKS2], >[STATOB], >[STATOB+IXSIZ]
               BYTE >[STATOB+(IXSIZ*2)], >[STATOB+(IXSIZ*3)]


;****************************************
;*
;*             Work out slopes !!
;*
;*             This is going to be fun !!
;*
;*
;*             Coding in SLOP_TAB:
;*             bits0-3 - shape number
;*
;*             bit4 - 0=45' slope; 1=30' slope
;*             bit5 - 0=sloping down with increasing x/y
;*                             1=sloping up with increasing x/y
;*
;*             bit6 - 0=slope in x direction; 1=slope in y direction
;*             bit7 - 0=slope; 1=other background object...ignore other bits
;*
;* So Shape  Bits 4-7  Meaning
;*      1        6       45'   upward y slope
;*      4        1       30' downward x slope
;*      5        0       45' downward x slope
;*      7        7       30'   upward y slope
;*      8        5       30' downward y slope
;*      9        4       45' downward y slope
;*      C        3       30'   upward x slope
;*      D        2       45'   upward x slope
;* Index into SLOP_TAB by shape number.  $FF means shape is not a slope.
;****************************************
SLOP_TAB       HEX FF61FFFF1405FF77
               HEX 5849FFFF3C2DFFFF
;               HEX FFFFFFFFFFFFFF
               ;; 2009-09: MY: Add some extra bytes so it's harder to run off
               ;;              the end of the table -- especially if index is
               ;;              masked to 5 bits.
               HEX FFFFFFFFFFFFFFFF
               HEX FFFFFFFFFFFFFFFF

;
; Look-up table used to translate X & Y velocities into index into MOVJMPL/MOVJMPH
;
MOV_POINT      BYTE 0,10,2,0,6,8,4,0,14,12,0,0,0

;
; Use GAUGEPOINT as an index into this to get see whether there is an
; X-component to the direction.  Index into X_MOTION+4 to see whether there is
; a Y-component.
;
X_MOTION       BYTE $12,$12,$12,$12,$12,$12,0,0
               BYTE $12,$12,$12,$12,$12,$12,0,0
               BYTE $12,$12,$12,$12,$12,$12

; Use GAUGEPOINT as an index into this to get see whether X-component of
; direction is positive or negative
VEL_TAB        BYTE $80,$80,$80,$80
               BYTE $80,$80,0,0
               BYTE 0,0,0,0
               BYTE 0,0,0,0

; Use GAUGEPOINT as an index into this to get see whether Y-component of
; direction is positive or negative
VEL_TAB2       BYTE $80,$80,0,0
               BYTE 0,0,0,0
               BYTE 0,0,0,0
               BYTE $80,$80,$80,$80

SPEED_TAB      BYTE 0,1,1,1,1,2,2,2,4,3



;;
;; Each of these DRP_ tables should have valid jump entries at the following
;; offsets:
;;
;;   OFFEST   SLOPE TYPE             J_TABs     Z_TABs    D_TABs
;;   ------   ----------             ------     ------    ------
;;      1     45'   upward y slope   SLOPE1_7   SLOP1_7   CHECK_X
;;      4     30' downward x slope   SLOPE4_5   SLOP4_5   CHECK_Y
;;      5     45' downward x slope   SLOPE4_5   SLOP4_5   CHECK_Y
;;      7     30'   upward y slope   SLOPE1_7   SLOP1_7   CHECK_X
;;      8     30' downward y slope   SLOPE8_9   SLOP8_9   CHECK_X
;;      9     45' downward y slope   SLOPE8_9   SLOP8_9   CHECK_X
;;      C     30'   upward x slope   SLOPEC_D   SLOPC_D   CHECK_Y
;;      D     45'   upward x slope   SLOPEC_D   SLOPC_D   CHECK_Y
;;
;; There should be $00FF at the other offsets.
;;
DRP_J_TABH     BYTE    >$00FF, >SLOPE1_7, >$00FF, >$00FF
               BYTE >SLOPE4_5, >SLOPE4_5, >$00FF, >SLOPE1_7
               BYTE >SLOPE8_9, >SLOPE8_9, >$00FF, >$00FF
               BYTE >SLOPEC_D, >SLOPEC_D, >$00FF, >$00FF

DRP_J_TABL     BYTE    <$00FF, <SLOPE1_7, <$00FF, <$00FF
               BYTE <SLOPE4_5, <SLOPE4_5, <$00FF, <SLOPE1_7
               BYTE <SLOPE8_9, <SLOPE8_9, <$00FF, <$00FF
               BYTE <SLOPEC_D, <SLOPEC_D, <$00FF, <$00FF

DRP_Z_TABH     BYTE   >$00FF, >SLOP1_7, >$00FF, >$00FF
               BYTE >SLOP4_5, >SLOP4_5, >$00FF, >SLOP1_7
               BYTE >SLOP8_9, >SLOP8_9, >$00FF, >$00FF
               BYTE >SLOPC_D, >SLOPC_D, >$00FF, >$00FF

DRP_Z_TABL     BYTE   <$00FF, <SLOP1_7, <$00FF, <$00FF
               BYTE <SLOP4_5, <SLOP4_5, <$00FF, <SLOP1_7
               BYTE <SLOP8_9, <SLOP8_9, <$00FF, <$00FF
               BYTE <SLOPC_D, <SLOPC_D, <$00FF, <$00FF

DRP_D_TABH     BYTE   >$00FF, >CHECK_X, >$00FF, >$00FF
               BYTE >CHECK_Y, >CHECK_Y, >$00FF, >CHECK_X
               BYTE >CHECK_X, >CHECK_X, >$00FF, >$00FF
               BYTE >CHECK_Y, >CHECK_Y, >$00FF, >$00FF

DRP_D_TABL     BYTE   <$00FF, <CHECK_X, <$00FF, <$00FF
               BYTE <CHECK_Y, <CHECK_Y, <$00FF, <CHECK_X
               BYTE <CHECK_X, <CHECK_X, <$00FF, <$00FF
               BYTE <CHECK_Y, <CHECK_Y, <$00FF, <$00FF


MOVJMPL        BYTE <BFOR_LEFT, <BFOR_LEFT, <BLEFT, <BLEFT
               BYTE <BACK_LEFT, <BACK_LEFT, <BBACK, <BBACK
               BYTE <BACK_RIGHT, <BACK_RIGHT, <BRIGHT, <BRIGHT
               BYTE <BFOR_RIGHT, <BFOR_RIGHT, <BFOR, <BFOR

MOVJMPH        BYTE >BFOR_LEFT, >BFOR_LEFT, >BLEFT, >BLEFT
               BYTE >BACK_LEFT, >BACK_LEFT, >BBACK, >BBACK
               BYTE >BACK_RIGHT, >BACK_RIGHT, >BRIGHT, >BRIGHT
               BYTE >BFOR_RIGHT, >BFOR_RIGHT, >BFOR, >BFOR


WOLLOP         BYTE $88,$88,$88,$88,$88,$88,$00,$00
               BYTE $08,$08,$08,$08,$08,$08,$80,$80
               BYTE $88,$88,$88,$88,$88,$88


KILL_SPR       BYTE 0,0,0,0,0,1,0,0,0,0,0,1,0,1,1,0,1,1
MYST_COLL      BYTE 2,2,3,3,0,0,4,5,6



S              EQU SHADSIZ
LSHAD_TAB      BYTE <[SHADO], <[SHADO+S], <[SHADO+2*S], <[SHADO+3*S], <[SHADO+4*S], <[SHADO+5*S], <[SHADO+6*S]
               BYTE <[SHADO+7*S], <[SHADO+8*S], <[SHADO+9*S], <[SHADO+$A*S], <[SHADO+$B*S], <[SHADO+$C*S]

HSHAD_TAB      BYTE >[SHADO], >[SHADO+S], >[SHADO+2*S], >[SHADO+3*S], >[SHADO+4*S], >[SHADO+5*S], >[SHADO+6*S]
               BYTE >[SHADO+7*S], >[SHADO+8*S], >[SHADO+9*S], >[SHADO+$A*S], >[SHADO+$B*S], >[SHADO+$C*S]


ENMUS



;
; File X1.BUB
; Starts Here
;
;

;
;
;  PDS Pc1.10 :000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          01.65
;         By  "JAKE AND MATTHEW..."
;
;     Created on Wed the 02nd of Mar 1988
;        Last update 08:41 on 10/05/88
;

;Main code starts here...
;;             ORG $1B00

               ;Code between star markers will be on loader on final version

;***********vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvfrom here

;
; This info is from http://www.c64-wiki.com/index.php/Interrupt,
; http://www.atarimax.com/jindroush.atari.org/aopc.html and
; http://www.6502.org/tutorials/register_preservation.html
;
;   $FFFA$FFFB is the vector for NMI handling; it points to 65091/$FE43
;   $FFFC$FFFD is the vector for handling cold start, pointing to 64738/$FCE2
;   $FFFE$FFFF is the vector for handling both IRQ and BRK-instructions; it points to 65352/$FF48
;
; BRK causes a non-maskable interrupt and increments the program counter by
; two. Therefore an RTI will go to the address of the BRK +3 so that BRK may
; be used to replace a three- byte instruction for debugging and the
; subsequent RTI will be correct. If BRK is used in place of a one- or
; two-byte op code, the controlling software will have to adjust the RTI
; appropriately.
;
; The B (Break) flag, bit 4 of the P (Processor status) register, is a frequent
; source of confusion on the 6502. The sole purpose of this flag is to
; distinguish a BRK from a IRQ. However, the behavior of BRK and IRQ (and how
; to distinguish between the two) can be described without even mentioning the
; B flag.
;   After the return address has been pushed onto the stack, a BRK instruction
; pushes the value of the P register ORed with $10, then transfers control to
; the BRK/IRQ interrupt handler.
;   After the return address has been pushed onto the stack, an IRQ interrupt
; pushes the value of the P register ANDed with $EF, then transfers control to
; the BRK/IRQ interrupt handler.
;   This means that the value of the P register that was pushed onto the stack
; must be used to distinguish a BRK from a IRQ, NOT the value of the P register
; upon entry to the BRK/IRQ interrupt handler
;
HARDVECS       ;WORD RT,RT,RT
               ;WORD RESET,RESET,RESET
               WORD HANDLE_NMI, HANDLE_COLD_START, HANDLE_IRQ_OR_BREAK

               ;
               ; We shouldn't get any NMIs but ignore them if we do
               ;
HANDLE_NMI     RTI

HANDLE_IRQ_OR_BREAK
               PHA
               TXA
               TSX
               PHA
               INX
               INX
               LDA $100,X ; get the status register from the stack
               AND #$10   ; mask B flag
               BNE HANDLE_BREAK
;               BEQ HANDLE_IRQ
;HANDLE_IRQ
               PLA ; Ignore IRQs too
               TAX
               PLA
               RTI

               ;
               ; It's a bug if we hit a BRK.  For release code, just reset the
               ; game.  For debugging, uncomment the following line, which will
               ; force the monitor to run.  (At least this is what happens in
               ; the VICE emulator.)
               ;
               ; At this point the stack holds:
               ;
               ;   [SP + 0:     Next free entry on stack]
               ;    SP + 1:     XR at break point
               ;    SP + 2:     AC at break point
               ;    SP + 3:     Processor flags at break point
               ;    SP + 4, +5: Address + 2 of BRK instruction
               ;    ....
               ;                GET_GOING + 2
               ;                MAINLOOP + 2
               ;
HANDLE_BREAK   ;BYTE $02 ; = JAM!


HANDLE_COLD_START
;RT             ;LDA #$35
               LDA #FLIPIN
               STA $01

               LDA #$7F
               STA $DD0D
               LDA $DD0D

               LDA #0
               STA $DD0E
               STA $DD0F

               ; Uncomment line below to drop into the monitor when we are restarting because of an interrupt
               ;BYTE $02 ; = JAM!

;        JMP CODE_START


CODE_START     SEI
               LDA #$2F
               STA $00

               ;
               ; Copy colour nybbles from their temporary location into colour
               ; RAM.
               ;
               ; We only want to run this code once as (a) the colour RAM
               ; doesn't change, and (b) what we are copying into it gets
               ; overwritten as soon as the game starts.
               ;
FIRST_TIME     LDY #0 ; smc
               BNE NOT_FIRST_TIME
               LDA #$35
               STA $01
UPL0           LDA OPTION_SCREEN_NYBBLES,Y
               STA $D800,Y
               LDA [OPTION_SCREEN_NYBBLES+$100],Y
               STA $D900,Y
               LDA [OPTION_SCREEN_NYBBLES+$200],Y
               STA $DA00,Y
               LDA [OPTION_SCREEN_NYBBLES+$300],Y
               STA $DB00,Y
               INY
               BNE UPL0
               INC FIRST_TIME+1
NOT_FIRST_TIME

               ; Set I/O area visible at $D000-$DFFF, RAM visible at
               ; $A000-$BFFF & $E000-$FFFF.
               ;LDA #$35
               LDA #FLIPIN
               STA $01
;;;
;;; 2009-09: MY: We don't use interrupts, but we do use the least-significant
;;;              bytes of the two timers as random-ish values.
;;;
               LDA #$08
               STA $DC0E ;CIA#1 TIMER A off
               LDA #$88
               STA $DC0F ;CIA#1 TIMER B off enable TOD alarm write

               LDA #PVAL

               STA $DC08 ;TOD 10ths
               STA $DC09 ;TOD sec
               STA $DC0A ;TOD min
               STA $DC0B ;TOD hr

               LDA #$7F
               STA $DC0D ;CIA#1 IRQS off
               STA $DD0D ;CIA#2 NMIS off
               LDA $DC0D ;clear CIA#1 flags
               LDA $DD0D ;clear CIA#2 flags

;               LDA #<ESMP
;               STA $FFFA ;NMI lo
;               LDA #>ESMP
;               STA $FFFB ;NMI hi
;
               LDX #$00
               STX $DD05 ;TA hi
               STX $DD07 ;TB hi
               DEX ;=$FF

               STX $DD04 ;TA lo
               STX $DD06 ;TB lo
               LDA #$11 ;TA force load, cont, start
               STA $DD0E

               LDX #$17
DDLY           DEX

               BNE DDLY
               STA $DD0F
;
;               LDA #$9F
;               STA $DD0D ;enable NMIs from all sources
;
;ZY             JMP ZY
;ESMP
               LDA #$04
               PHA

               PLP       ; Clears carry, decimal etc processor status flags

               STX $D020 ; Screen border colour
               STX $D021 ; Screen background colour

               STX $D01A ;VIC IRQs off

               STX $D019 ;clear VIC IRQ flags
               STX $DD00 ;set VIC bank

; 2009-09: MY: Check whether we need this...
               lda #$15
               sta $d018

               LDX #5
SETHVCS        LDA HARDVECS,X
               STA $FFFA,X
               DEX
               BPL SETHVCS

;               JMP RLST


;;             JMP $8700 ;re-run PDS download software

;***********^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^to here


;
; 2009.07: MY: Real start ?
;
;RLST
               JSR MAJORSETUP ;set up variables for first time use

BEGIN          JSR TITLES     ;print up title page

;ENT2
               JSR GAMEON     ;go for start music

               JSR SETUP      ;set up variables for game use e.g lives
               JSR TEST       ;clear work area

RE_START       JSR RENTERY    ;scroll stage message and play area on

MAINLOOP       JSR DROP_THEM  ;drop all objects and update positions
               JSR OKMLOOP    ;move screen and gauge
               JSR QUAAGAR    ;do guns and saucers firing

               JSR GEN_MYST   ;generate mystery bubbles
               JSR DO_TIME    ;work out timers

               LDX LAST_FIRE
               BEQ NO_DEC_FIR
               DEC LAST_FIRE
NO_DEC_FIR
;;;
;;; 2009-09: MY: Comment this out as we don't need the "exit to PDS" key!
;;;
;               LDY #34
;               JSR KEYS ; RUN/STOP
;               BEQ NOPDS
;
;               JMP RT ;return to PDS control
;
;NOPDS          INY
               LDY #35 ; Replaces INY ; COMMODORE
               JSR KEYS
               BEQ NOPAUSE    ;check for pause
               JSR PAUSE      ;goto pause
NOPAUSE
               ; Disable cheats in the "unenhanced" version, but in such a way
               ; that they can be added back in by modifying the uncompressed
               ; binary
               IFCONST BUILD_ENHANCED_VERSION
               JSR CHECK_CHEAT_KEYS
               ELSE
               JSR CHEAT_KEYS_END
               ENDIF

               LDA EXITO
               CMP #3
               BNE PLEEEASE
               JSR RETURNOKM
;;;
;;; 2009-09: MY: Added this (to exit cleanly at game completion) after
;;;              replacing the "exit to PDS" JMP RT in RETURNOKM with an RTS
;;;
               LDA STAGE
               CMP #MAX_STAGE
               BEQ BEGIN ; .:TBC:. Should this be CODE_START?

PLEEEASE       LDA #FLIPIN
               STA $01

               JSR SONIX      ;update any sounds
               JSR JUMP_UP    ;do any jumps

               JSR TRAP_ON    ;do trap drop

               JSR YOURS      ;do blob shooting

NOSON          JSR DEATH      ;is any one dead?
               CMP #2
               BCS MAINLOOP   ;no, go back and do loop again

               CMP #1
;               BEQ GOLOOP     ;yes but a new player is ready
               BEQ RE_START     ;yes but a new player is ready

               JMP BEGIN      ;yes, all so go back to title page
;GOLOOP         JMP RE_START


;******************************************************
;* Process the "cheat" keys
;******************************************************
CHECK_CHEAT_KEYS
               LDY #37 ; f1
               JSR KEYS
               BEQ NO_NEXT_ST
               ; Pressing f1 is cheat to exit on the current
               ; level even if you haven't stoppered all the bottles
               LDA #0
               STA BOTTLES
               LDA #1
               STA EXITO
               JSR EXITOPEN
NO_NEXT_ST
               LDY #38 ; f3
               JSR KEYS
               BEQ NO_EXTRA_LIFE
               ; Pressing f3 is a cheat to give you an extra life...
               JSR F_EXTRA

               ; ...but holding down both shift keys as well stops aliens,
               ; bullets, etc from killing you -- though falling off the
               ; landscape still does.  (On the WinVICE emulator it seems you
               ; have to press shift + f4 to give the right input.)
               JSR CHECK_SHFT_KYS
               BEQ NO_EXTRA_LIFE

               LDA #CHEAT_NO_ALIEN_KILL
               JSR SET_CHEAT_MODE

NO_EXTRA_LIFE
               LDY #39 ; f5
               JSR KEYS
               BEQ NO_EXTRA_TIME
               ; Pressing f5 tops up your time...
               LDA #99
               SEC
               SBC TIME
               STA TIME2

               ; ...but holding down both shift keys as well gives you
               ; infinite time.  (On the WinVICE emulator it seems you have to
               ; press shift + f6 to give the right input.)
               JSR CHECK_SHFT_KYS
               BEQ NO_EXTRA_TIME

               LDA #CHEAT_INFINITE_TIME
               JSR SET_CHEAT_MODE

NO_EXTRA_TIME
               LDY #40 ; f7
               JSR KEYS
               BEQ NO_USE_UP_LIVES
               ; Pressing f7 reduces you down to your last life - useful if
               ; you've given yourself lots of extra lives and then later want
               ; to end your game
               LDX PLAYER
               LDA #1
               STA LIVES,X
               JSR PLOT_BCDNUM

NO_USE_UP_LIVES
CHEAT_KEYS_END RTS

;
; Border colours to signify that a game-play-changing cheat is active
;
CHEAT_BORDER_COLS BYTE COL_CODE_BLACK      ; No cheat
                  BYTE COL_CODE_DARK_GREY  ; Infinite time
                  BYTE COL_CODE_GREY       ; No alien kill
                  BYTE COL_CODE_LIGHT_GREY ; Infinite time and No alien kill

;******************************************************
;* Set cheat mode bit for current player
;*
;* Parameters: Acc -- bit to set.  If 0 just sets the right border colour for
;*                    current player's cheat mode.
;*
;* Registers affected: A, X
;******************************************************
SET_CHEAT_MODE LDX PLAYER
               ORA CHEAT_MODE,X
               STA CHEAT_MODE,X
               TAX
               LDA CHEAT_BORDER_COLS,X
               STA $D020
               RTS

;******************************************************
;* Turn off the cheats that affect game play
;******************************************************
CHEATS_OFF     LDA #0
               STA $D020        ; Set border back to black.  (Non-black = cheat mode)
               STA CHEAT_MODE   ; Big cheats off for player 1
               STA CHEAT_MODE+1 ; Big cheats off for player 2
CHECK_SHFT_DN  RTS


;******************************************************
;* Test whether both shift keys were held down
;* On return, Z flag is unset if they were
;******************************************************
CHECK_SHFT_KYS LDY #36 ; LEFT SHIFT
               JSR KEYS
               BEQ CHECK_SHFT_DN
               LDY #61 ; RIGHT SHIFT
               JMP KEYS ; JSR/RTS


;****************************************
;*
;*             Works out which direction
;*             the shortest route to the
;*             desired pointer direction and
;*             positions the pointer at this
;*             point.
;*
;****************************************
MAKEDIR        LDA GAUGEPOINT
               SEC
               SBC GAUGEDIR
               BEQ RETS ;no direction needed

               AND #$0F
               CMP #8
               BCS SETBIT
               LDA #0 ;set anticlockwise

               STA GDIR
               RTS

SETBIT         LDA #128 ;set clockwise

               STA GDIR
RETS           RTS



;****************************************
;*             Updates the pointer definition
;*             on the screen. Uses 'Player'
;*             as an offset to work out which
;*             pointer to update.
;*
;*      Input: PLAYER
;*             GAUGEPOINT
;*             GAUGEDIR
;*             GDIR
;****************************************
UPDATEPOINT    LDA PLAYER
               ASL ; = ASL A

               TAX

               LDA GAUGEPOINT
               CMP GAUGEDIR
               BEQ GOUPDATE
               LDA GDIR
               BNE FORWARD

               DEC GAUGEPOINT
               JMP BACK
FORWARD        INC GAUGEPOINT

BACK           LDA GAUGEPOINT
               AND #$0F
               STA GAUGEPOINT
GOUPDATE       TAY

               LDA PLAYER
               BNE POINTER2
               LDA POINTABX1,Y

               JMP POINTER1
POINTER2       LDA POINTABX2,Y

               CMP #200
               BCS UNSETMSB
               LDA #2
               JMP SETMSB
UNSETMSB       LDA #0
SETMSB         STA MSB

               LDA POINTABX2,Y
POINTER1       STA 53248,X     ; $D000,X

               LDA POINTABY,Y
               STA 53249,X     ; $D001,X

RESETSDEF      LDA #FLIPOUT
               STA $01

               LDA SDEFTAB,Y
               LDX PLAYER
               STA SDEF,X
               INC $01 ;=$35 : I/O in
               RTS



;****************************************
;*             Plots a character anywhere on
;*             the hires screen using yval
;*             as the y coor and xval as the
;*              x coor (like text screen.)
;*             SMTAB is selfmodifying code
;*             for the table of chars it is
;*             printing from. (tabval is the
;*             position in that table.
;*             automatically incremented.)
;*
;* Inputs: XVAL - X coordinate (columns across screen) of where to plot char
;*         YVAL - Y coordinate (rows down screen) of where to plot char
;*         TABVAL
;*         SMTAB+1/+2
;*         DOUBLE_HEIGHT - 0 = normal height, 1 = print characters double height
;*
;* Invariants: Y-register is saved and restored by this routine
;****************************************
PLOT           TYA
               PHA

               LDY YVAL
               LDA PLOTYLO,Y
;               STA SMADR+1
               STA ZPH
               LDX PLOTYHI,Y

               LDA XVAL
               CMP #$20
               BCC PLOT_1
               INX
               AND #$1F

PLOT_1
;               STX SMADR+2
               STX ZPH+1

;GOPLOT
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               TAY

               ;;
               ;; 2009-10: MY: Original version of this routine could only
               ;;              cope with character offsets of 0-$1F.  This is
               ;;              fine for plotting small groups of special
               ;;              characters (big digits, bottle cork status,
               ;;              timer bar, etc).  But to reuse this routine for
               ;;              writing text (including spaces & punctuation) on
               ;;              the screen I have extended the code below to
               ;;              cope with offsets up to $3F (which means
               ;;              incrementing the MSB of where we read the
               ;;              character bitmap data from).
               ;;
               LDA TABVAL
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               BCC PLOT_NO_CARRY
               INC SMTAB+2
PLOT_NO_CARRY  TAX
PLOTIT
SMTAB          LDA CHARTAB,X ; self-modifying code
;SMADR          STA $E000,Y   ; self-modifying code
               STA (ZPH),Y
               INX
               INY

               ; We don't want to mess with what's in the AR, XR or YR, so this
               ; is a sneaky way of reading the LSB of DOUBLE_HEIGHT!
               LSR DOUBLE_HEIGHT
               BCC PLOT_NO_DOUB
               INC DOUBLE_HEIGHT
               STA (ZPH),Y
               INY

PLOT_NO_DOUB   TXA
               AND #7
               BEQ PLOT_2

               ; None of the instructions above after the LSR affect the carry
               ; flag.  So it's still set if and only if DOUBLE_HEIGHT is set.
               BCC PLOTIT
               CMP #4
               BNE PLOTIT
               ; We've plotted the top half of the double-height character.  We
               ; need to jump down 8 pixels on the hires screen for the bottom
               ; half.
               TYA
               SEC
               SBC #8
               TAY
               CLC
               LDA ZPH
               ADC #<320
               STA ZPH
               LDA ZPH+1
               ADC #>320
               STA ZPH+1
               BNE PLOTIT ; = JMP

               ; If above we incremented the MSB of where we read character bitmap
               ; data from, we need to restore it here.
PLOT_2         LDA TABVAL
               AND #$20
               BEQ PLOT_NO_FIXUP
               DEC SMTAB+2
PLOT_NO_FIXUP  PLA
               TAY

               INC TABVAL
               RTS


;****************************************
;*             Reset timer onscreen to zero
;*             time. Uses blank timer defs
;*             and PLOTS them onscreen.
;****************************************
RESETIMER      LDA #0
               STA TIME
               STA TABVAL

               LDA #16
               STA YVAL
               LDA #2
               STA XVAL
               LDA #<CHARTAB
               STA SMTAB+1
               LDA #>CHARTAB
               STA SMTAB+2

RTIMEL         JSR PLOT
               DEC TABVAL
               INC XVAL
               LDA XVAL
               CMP #14

               BNE RTIMEL
               LDA #27

               STA XVAL
RTIMEL2        JSR PLOT

               DEC TABVAL
               INC XVAL
               LDA XVAL
               CMP #39

               BNE RTIMEL2

               INC TABVAL
               INC YVAL
               LDA #2
               STA XVAL
RTIMEL3        JSR PLOT

               DEC TABVAL
               INC XVAL
               LDA XVAL
               CMP #14

               BNE RTIMEL3

               LDA #27

               STA XVAL
RTIMEL4        JSR PLOT

               DEC TABVAL
               INC XVAL
               LDA XVAL
               CMP #39

               BNE RTIMEL4

               LDA #25

               STA TABVAL
               LDA #14

               STA XVAL
               JSR PLOT
               LDA #39

               STA XVAL
               DEC TABVAL
               JSR PLOT
               DEC TABVAL
               DEC TABVAL
               DEC YVAL
               JSR PLOT
               DEC TABVAL
               LDA #14

               STA XVAL
               JMP PLOT


;****************************************
;*             Increment timers using PLAYER
;*             as an offset.
;****************************************
TIMER          LDA TIME
               CMP #99
               BNE GOTIMER

               RTS

GOTIMER        LDA PLAYER
               PHA

               LDA #0
               STA PLAYER
               JSR GODEC1A    ;increment timer one by one

               INC PLAYER
               JSR GODEC1A    ;increment timer two by one

               INC TIME

               PLA
               STA PLAYER
               RTS


;****************************************
;*             Mask off four bits on the timer
;*             definitions so that it looks
;*             nice and round.
;****************************************
MASKLINE1      LDA $E000+[320*16]+13
               AND #$7F
               STA $E000+[320*16]+13

               LDA $E000+[320*17]+11
               AND #$7F
               STA $E000+[320*17]+11

MASKLINE2      LDA $E000+[320*16]+[8*25]+13
               AND #$7F
               STA $E000+[320*16]+[8*25]+13

               LDA $E000+[320*17]+[8*25]+11
               AND #$7F
               STA $E000+[320*17]+[8*25]+11
               RTS


;****************************************
;*             Decrement timers by one using
;*             PLAYER as an offset.
;****************************************
DECTIMER1      LDA TIME

               BNE GODEC1
DECTIMER_RET   RTS

;GODEC1         DEC TIME
GODEC1         LDY PLAYER ; If relevant cheat mode is on, don't decrement time
               LDA CHEAT_MODE,Y
               AND #CHEAT_INFINITE_TIME
               BNE DECTIMER_RET
               DEC TIME

GODEC1A        LDY TIME

               LDA TIMETABLE,Y               ;load up time defintion

               AND #15

               STA XVAL       ;Bottom four bits xcoor
               LDA PLAYER
               CMP #1
               BNE CONTDEC

               LDA XVAL       ;if player two add 25 to x coor
               CLC

               ADC #25

               STA XVAL
CONTDEC        LDA TIMETABLE,Y               ;top four bits of table is definition
               AND #$F0
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               STA TABVAL     ;make into a 4 bit number
               LDA #16
               STA YVAL
               LDA #<CHARTAB  ;load up table address for PLOT
               STA SMTAB+1
               LDA #>CHARTAB
               STA SMTAB+2
               JSR PLOT

               INC YVAL
               JSR PLOT
               INC XVAL
               LDA TIMETABLE2,Y
               ASL ; = ASL A

               STA TABVAL
               DEC YVAL
               JSR PLOT
               INC YVAL
               JSR PLOT
               JMP MASKLINE1


;****************************************
;*             Second plot routine. This
;*             starts the plot half way through
;*             a character on screen for use
;*             on the lives numbers, score etc
;*             It uses a special charset of
;*             offset characters.
;***************************************
PLOTII         TYA
               PHA

               LDY YVAL
               LDA PLOTYLO,Y
               STA SMADRII+1
               STA SMADR2+1
               LDA PLOTYHI,Y
               STA SMADRII+2
               STA SMADR2+2

               LDA XVAL
               CMP #31
               BEQ GOPLOTII
               BCC GOPLOTII

               INC SMADRII+2
               INC SMADR2+2
               SEC
               SBC #32

GOPLOTII       ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               TAY

               LDA TABVAL
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               TAX
PLOTIT2
SMADR2         LDA $E000,Y   ; smc
SMAND          AND #$0F      ; smc
SMTABII        ORA CHARTAB,X ; smc
SMADRII        STA $E000,Y   ; smc
               INX
               INY
               TXA
               AND #7
               BNE PLOTIT2

               PLA
               TAY

               INC TABVAL
               RTS


;******************************************************************************
;* Plot a Binary-Coded Decimal number (supplied in the Accumulator) on the
;* screen.  Other parameters as for PLOTNUM.
;******************************************************************************
PLOT_BCDNUM    TAY
               AND #$0F
               STA NUM2
               TYA
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               STA NUM1
               ; Fall through to PLOTNUM

;******************************************************************************
;* Plot a number on the screen
;******************************************************************************
PLOTNUM        LDA XPOSTAB,X
               STA XVAL
               LDA YPOSTAB,X
               STA YVAL

               LDA #$0F
               STA SMAND+1

               LDY NUM1
               LDA NUMTABLO,Y
               STA SMTABII+1
               STA SMTAB+1

               LDA NUMTABHI,Y
               STA SMTABII+2
               STA SMTAB+2

               LDA #0
               STA TABVAL
               JSR PLOT
               INC XVAL
               JSR PLOTII
               DEC XVAL
               INC YVAL
               JSR PLOT
               INC XVAL
               JSR PLOTII
               LDA #$F0
               STA SMAND+1

               LDY NUM2
               LDA NUMTABLOR,Y
               STA SMTABII+1
               STA SMTAB+1

               LDA NUMTABHIR,Y
               STA SMTABII+2
               STA SMTAB+2

               LDA #0
               STA TABVAL
               DEC YVAL
               JSR PLOTII
               INC XVAL
               JSR PLOT
               DEC XVAL
               INC YVAL
               JSR PLOTII
               INC XVAL
               JMP PLOT


;****************************************
;*             Plot the cork on screen off or on
;****************************************
CORKPLOT       LDA XCORKTAB,Y
               STA XVAL
               LDA #8
               STA YVAL
               LDA #0
               STA TABVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               DEC XVAL
               DEC XVAL
               INC YVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               DEC XVAL
               DEC XVAL
               INC YVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               INC YVAL
               DEC XVAL
               DEC XVAL
               JSR PLOT
               INC XVAL
               JSR PLOT
               INC XVAL
               JMP PLOT


;****************************************
;*             Do routine that turns off cork
;****************************************
CORKOFF
;               LDA #$80
               LDA #<[CHARTAB + $0480]
               STA SMTAB+1
;               LDA #$14
               LDA #>[CHARTAB + $0480]
               STA SMTAB+2

               LDA #0
               STA CORK
               LDY PLAYER
               JMP CORKPLOT


;****************************************
;*             Do routine that turns on cork
;****************************************
CORKON
;               LDA #$E0
               LDA #<[CHARTAB + $04E0]
               STA SMTAB+1
;               LDA #$14
               LDA #>[CHARTAB + $04E0]
               STA SMTAB+2

               LDA #1
               STA CORK
               LDY PLAYER
               JSR CORKPLOT

               LDA #FLIPIN
               STA $01

               JMP GETCORK


;****************************************
;*             Add numbers in Units to score
;****************************************
ADDSCORE       LDX #0
               LDA PLAYER
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               TAY

INCSCORE       LDA UNITS,X

               BEQ ENDSCR
               LDA SCORE1,Y
               CLC

               ADC UNITS,X

               CMP #10

               BCC INCS1

               SEC

               SBC #10

               INC UNITS+1,X

INCS1          STA SCORE1,Y

               INY

               INX

               JMP INCSCORE
ENDSCR         LDX #0
               LDA #0
CLEARUNITS     STA UNITS,X

               INX

               CPX #7
               BNE CLEARUNITS


;****************************************
;*             Plot the score on the screen
;****************************************
PLOTSCORE      LDY #0
SCORE1X        LDA PLAYER

               CMP #3
               BNE SCORE2X

               LDA #15
               JMP EXITSCOREX

SCORE2X        ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A

EXITSCOREX     CLC

               ADC SCORENO,Y

               TAX

               LDA SCORE1,X
               TAX

               TYA

               AND #1
               CMP #1
               BEQ TYPETWO

               LDA LOSCORETAB,X
               STA SMTAB+1

               STA SMTABII+1

               LDA #<PLOTINUM
               STA SMJSR+1

               LDA #>PLOTINUM

               STA SMJSR+2

               LDA HISCORETAB,X
               JMP TYPEONE

TYPETWO        LDA #<PLOTINUM2
               STA SMJSR+1

               LDA #>PLOTINUM2

               STA SMJSR+2

               LDA LOSCORETAB2,X

               STA SMTAB+1

               STA SMTABII+1

               LDA HISCORETAB2,X

TYPEONE        STA SMTAB+2
               STA SMTABII+2

               LDA SCOREX,Y
               STA XVAL
               LDA PLAYER
               CMP #1
               BCC PLAY1

               CMP #2
               BCC PLAY4

               LDA #22

               STA YVAL
               LDA SCOREXZ,Y

               STA XVAL
               JMP PLAY2

PLAY4          LDA SCOREZ,Y

               STA XVAL
PLAY1          LDA #3

               STA YVAL
PLAY2          LDA #0

               STA TABVAL
               LDA ANDTAB,Y
               STA SMAND+1

SMJSR          JSR PLOTINUM

               INY

               CPY #6
               BEQ EXITSCORE

               JMP SCORE1X

EXITSCORE      RTS


PLOTINUM       JSR PLOT
               INC XVAL
               JSR PLOTII
               DEC XVAL
               INC YVAL
               JSR PLOT
               INC XVAL
               JSR PLOTII
               DEC XVAL
               INC YVAL
               JSR PLOT
               INC XVAL
               JMP PLOTII
PLOTINUM2      JSR PLOTII

               INC XVAL
               JSR PLOT
               DEC XVAL
               INC YVAL
               JSR PLOTII
               INC XVAL
               JSR PLOT
               DEC XVAL
               INC YVAL
               JSR PLOTII
               INC XVAL
               JMP PLOT

;;;
;;; 2009-09: MY: I have changed the default up/down keys from ';' and '/' to
;;;              'K' and 'M' as these are easier to use on a PC keyboard.
;;;
DEF_KEY_LEFT   EQU ['Z-$40]
DEF_KEY_RIGHT  EQU ['X-$40]
DEF_KEY_UP     EQU ['K-$40]
DEF_KEY_DOWN   EQU ['M-$40]
DEF_KEY_JUMP   EQU 32 ; Space
DEF_KEY_FIRE   EQU 60 ; Return

DEF_KEYS       BYTE DEF_KEY_LEFT, DEF_KEY_RIGHT, DEF_KEY_UP, DEF_KEY_DOWN
               BYTE DEF_KEY_JUMP, DEF_KEY_FIRE
DEF_KEYS_END

;****************************************
;*
;*             Used only once at the start
;*             this sets up preliminary
;*             variables like keys, no of
;*             players, joysticks etc
;*
;****************************************
MAJORSETUP
               ; Reset all zero-page variables
               ; Do this only once, at program start
        LDA #0
        LDY #ZPSTART
RESET_ZP_VARS
        STA $0,Y
        INY
        BNE RESET_ZP_VARS
               ; Default keys for keyboard control
;               LDA #26
;               STA KLEFT
;               LDA #24
;               STA KRIGHT
;               LDA #29
;               STA KUP
;               LDA #46
;               STA KDOWN
;               LDA #32
;               STA KJUMP
;               LDA #60
;               STA KFIRE

;;;
;;; 2007-09 MY: Changed this so that we can define the default keys in one place
;;;             and have them display correctly on the "define keys" screen.
;;;
               ; Default keys for keyboard control
               LDY #[DEF_KEYS_END - DEF_KEYS - 1]
WRITE_DEF_KEYS LDX DEF_KEYS,Y
               STX KLEFT,Y

               LDA LYNDA,X ; What to display for this key
               LDX CURTAB,Y ; Where to display it
               STA KUSER2-2,X

               DEY
               BPL WRITE_DEF_KEYS

               LDA #1
               STA BOTTLES
               STA CHOICE
               STA CHOICE+1

               LDA #3
               STA PLAYER
               JSR PLOTSCORE

               LDA #1
               STA STAGE
               JSR SET_ST_POS ;set start position on stage

               LDY #8
UP_ERE5        LDA ST_POS,Y
               STA ST_POS_1,Y
               STA ST_POS_2,Y
               DEY
               BPL UP_ERE5

               LDA #0
               STA PLAYER
               STA PLAYNO
               STA CURPOSA

;        LDY #0
;CLEAR_VARS
;        STA $0200,Y
;        STA $0300,Y
;        STA $0400,Y
;        STA $0500,Y
;        STA $0600,Y
;        STA $0700,Y
;        DEY
;        BNE CLEAR_VARS
               ;LDA #<SPECAL
               LDA #<VARS_TO_ZERO_START
               STA ZPA
               ;LDA #>SPECAL
               LDA #>VARS_TO_ZERO_START
               STA ZPA+1
               ;LDA #<[END_SPE - SPECAL]
               LDA #<[VARS_TO_ZERO_END - VARS_TO_ZERO_START]
               STA ZPB
               ;LDA #>[END_SPE - SPECAL]
               LDA #>[VARS_TO_ZERO_END - VARS_TO_ZERO_START]
               STA ZPB+1
               LDA #0
               JSR FILL_MEM

               LDA CURTAB
               STA CURPOSB

               JMP TEST

;*******************************************************************************
;* Fill memory
;*
;* Caller to set:
;*  ZPA - start address of memory to fill
;*  ZPB - length of memory to fill
;*  Accumulator - byte to store in memory
;*******************************************************************************
FILL_MEM       LDY #0

               LDX ZPB+1
               BEQ FILL_MEM_LAST_PG

FILL_MEM_LP1   STA (ZPA),Y
               INY
               BNE FILL_MEM_LP1

               INC ZPA+1
               DEC ZPB+1
               BNE FILL_MEM_LP1

FILL_MEM_LAST_PG
               LDX ZPB
               BEQ FILL_MEM_DONE
FILL_MEM_LP2   STA (ZPA),Y
               INY
               DEX
               BNE FILL_MEM_LP2

FILL_MEM_DONE  RTS


;****************************************
;*             Called at the start of each
;*             game. This sets up live no.,
;*             stage, scores, scrolling message
;*             sprites and much much more !!
;****************************************
SETUP          JSR HIRESON

               JSR CHEATS_OFF

               LDA #0
               STA TRAP
               STA EXITO
               STA $D01A

;               STA LIVES
;               STA LIVES2

               STA CORK
               STA CORK+1
               STA CORK+2
               LDA #1
               STA STAGE

               STA STAGE1
               STA STAGE2
               JSR SET_ST_POS
               LDY #8
UP_ERE51       LDA ST_POS,Y
               STA ST_POS_1,Y
               STA ST_POS_2,Y
               DEY
               BPL UP_ERE51
               JSR RESET_BOT

               LDY STAGE
               LDX BOTT_NO,Y
               STX BOTTLES
               STX BOTTLES+1
               STX BOTTLES+2

        ; Turn on sprites 0 & 1, and set to double-width
               LDA #3
               STA 53277           ;$D01D
               STA 53269           ;$D015

LIVENO1        LDA #3
;               STA LIVES+1
               STA LIVES

               LDA PLAYNO
               BEQ NOLIVENO2

LIVENO2        LDA #3
               JMP PLAY2YES
NOLIVENO2      LDA #0

PLAY2YES       STA LIVES2

        ; Set colours of sprites 0 & 1
               LDA #12
               STA 53287       ;$D027
               STA 53288       ;$D028

               LDA #0
               LDX #6
CLEARSCORE     STA UNITS,X
               STA SCORE1,X
               STA SCORE2,X
               STA SCROLST,X  ;FOR SCROLLING MESSAGE
               DEX
               BPL CLEARSCORE

;LEVELINIT2
               JSR SET_ST_POS      ;set start position on stage

               LDY #0
               STY PLAYER
               JSR UPDATEPOINT
               JSR PRINT_NO        ;PRINT # of bottles on screen
               JSR PLOTSCORE
               JSR CORKOFF

               INC PLAYER
               JSR UPDATEPOINT
               JSR PRINT_NO
               JSR PLOTSCORE
               JSR CORKOFF

               LDA #0
               STA PLAYER

;****************************************
;*             Called after a player dies,
;*             this sets up the 'little'
;*             things like clean screen
;*             updates live nos on screen
;*             sounds and resetimers.
;****************************************
SECSETUP       LDA PLAYER
               CLC
               ADC #49
               STA WELCOME+7

               LDX #0
               STX PLAYMESS
               STX TIME2
               STX BULLET

               LDA LIVES
;               STA NUM1
;               LDA LIVES+1
;               STA NUM2
;               JSR PLOTNUM
               JSR PLOT_BCDNUM

               LDX #1
               LDA LIVES2
;               STA NUM1
;               LDA LIVES2+1
;               STA NUM2
;               JSR PLOTNUM
               JSR PLOT_BCDNUM

               JSR PRINT_NO
               LDA #0
               STA LAST_FIRE

               STA JUMP_F
               STA GDIR
               STA GAUGEDIR
               STA PLAYMESS
               STA VOC1
               STA VOC2
               STA VOC3
               STA PRIORITY
               STA YPIX
               STA XPIX
               STA IXTAB+1 ;data offset for blob

               STA GAUGEPOINT
               LDA #IXTAB&255
               STA IX
               LDA #IXTAB/256
               STA IX+1

               LDA #FLIPOUT
               STA $01

               LDA #96
               STA SDEF
               STA SDEF+1

               INC $01 ;=$35 : I/O in

               LDA #0
               JSR SET_CHEAT_MODE ; Set border colour for current player's cheat mode

               LDA POINTABX1
               STA 53248     ; $D000 - Sprite #0 X-coordinate
               LDA POINTABX2
               STA 53250     ; $D002 - Sprite #1 X-coordinate
               LDA POINTABY
               STA 53249     ; $D001 - Sprite #0 Y-coordinate
               STA 53251     ; $D003 - Sprite #1 Y-coordinate

               LDA #0
               STA MSB

               JSR COLCHANGE

               JSR SONSETUP
               LDA #0
               STA TIME
               JMP SETUPSCROLL



;****************************************
;*             Set up the hires screen so
;*             it is displayed.
;****************************************
HIRESON        LDA #59
               STA 53265 ; $D011

               LDA #120
               STA 53272 ; $D018

               RTS



;****************************************
;*             Set up the title screen so it
;*             is displayed.
;****************************************
TITLES         LDA $D011
               AND #$DF
               STA $D011 ;bitmap off

               LDA #%01010100
               STA 53272 ; $D018

               LDA #1+2+4+8+16+32
               STA 53269 ; $D015

               JSR SETCOLOURS ;set up the colours on the player 1/2 option.
               LDA #1
               STA 53287 ; $D027 - Sprite #0 color (only bits #0-#3)
               STA 53288 ; $D028 - Sprite #1 color (only bits #0-#3)
               STA 53289 ; $D029 - Sprite #2 color (only bits #0-#3)
               STA 53290 ; $D02A - Sprite #3 color (only bits #0-#3)
               STA 53291 ; $D02B - Sprite #4 color (only bits #0-#3)
               STA 53292 ; $D02C - Sprite #5 color (only bits #0-#3)

               LDA #57
               STA 53249 ; $D001 - Sprite #0 Y-coordinate
               STA 53251 ; $D003 - Sprite #1 Y-coordinate
               STA 53253 ; $D005 - Sprite #2 Y-coordinate
               STA 53255 ; $D007 - Sprite #3 Y-coordinate

               LDA #0
               STA BUFF
               STA ARROW      ;set up the arrows.

               STA MSB

               STA 53277 ; $D01D - Sprite double width register. Bit #x: 1 = Sprite #x is stretched to double width.

;; 2009-10: MY: Changed this to support scrolling message longer than 255 characters
;;              on options screen
               LDA #<ENMESS
               STA OPT_SCRN_MSG
               LDA #>ENMESS
               STA OPT_SCRN_MSG + 1
               LDA #255
;               STA MESSNO
               STA AMOUNT
               LDA #144
               STA 53248 ; $D000 - Sprite #0 X-coordinate

               LDA #168
               STA 53250 ; $D002 - Sprite #1 X-coordinate

               LDA #192
               STA 53252 ; $D004 - Sprite #2 X-coordinate

               LDA #216
               STA 53254 ; $D006 - Sprite #3 X-coordinate

               LDY CHOICE
               LDA ARROWPOS,Y
               STA 53257 ; $D009 - Sprite #4 Y-coordinate

               LDY CHOICE+1
               LDA ARROWPOS,Y
               STA 53259 ; $D00B - Sprite #5 Y-coordinate

               LDA #110
               STA 53256 ; $D008 - Sprite #4 X-coordinate

               LDA #237
               STA 53258 ; $D00A - Sprite #5 X-coordinate

               JSR STARTMUSIC


               LDA #FLIPOUT
               STA $01

               LDX #56        ;set up bubbler sign.

               STX TDEF
               INX

               STX TDEF+1
               INX

               STX TDEF+2
               INX

               STX TDEF+3
               INX

               STX TDEF+4
               INX

               STX TDEF+5
               LDY #5
               LDX #0
PRINTSCORE                                   ;print up the scores from the last game

               LDA SCORE1,X   ;on the title screen.
               CLC

               ADC #48

               STA $D000+1107,Y
               LDA SCORE2,X
               CLC

               ADC #48

               STA $D000+1135,Y
               INX

               DEY

               BPL PRINTSCORE
               LDY #126-21    ;plot user chars on the window at bot of screen

DEFINECHAR     TYA

               STA 1904+12+$D000-108,Y

               INY

               CPY #127
               BNE DEFINECHAR
               LDA #0
               TAY

CLEARCHARS                                   ;clear the definitions so window is clear
               STA CHARSTART,Y

               INY
               CPY #184
               BNE CLEARCHARS
               JSR READNEXT   ;read down the first char into user chars
DOTHAT         LDA #FLIPOUT
               STA $01

               JSR SCROLLTITS ;scroll the window            message

               LDA #FLIPIN
               STA $01

               JSR DOMUSIC    ;perform the music.

               JSR MOVEARROWS ;move the arrows

               LDA CHOICE
               ORA CHOICE+1
               AND #2
               CMP #2
               BEQ DONTDOTHIS
               JSR DETECT     ;do the keyboard scan
               BEQ CHECK_JOY

               JMP SEE_IF_XIT
CHECK_JOY      LDA #$FF
               STA $DC00

               LDA $DC00
               AND #$10
               BNE DONTDOTHAT
SEE_IF_XIT     LDA CHOICE     ;check to see if the user option is
               ORA CHOICE+1   ;not being used before exiting

               AND #2
               CMP #2
               BEQ DONTDOTHAT
               RTS

DONTDOTHIS     JSR DOUSERKEY

DONTDOTHAT     JSR USERKEYS   ;do the user definable keys
               LDA #FLIPIN
               STA $01

DOTHIS         LDX $D012      ;check raster for top of screen

               CPX #1
               BNE DOTHIS
               JMP DOTHAT


;****************************************
;*             Read the next char into the
;*             user definable chars for
;*             scrolling into the window.
;****************************************
READNEXT       LDX OPT_SCRN_MSG
               LDY OPT_SCRN_MSG + 1
               INX
               BNE READ_NEXT_1
               INY
READ_NEXT_1    CPX #<ENMESS
               TYA
               SBC #>ENMESS
               BCC READ_NEXT_2
               LDX #<MESSTAB
               LDY #>MESSTAB
READ_NEXT_2
               STX OPT_SCRN_MSG
               STY OPT_SCRN_MSG + 1

               ; Multiply by 8 to get offset from $D000
               ;
               ; Only want/expect character values in range 0-$3F, so don't have to
               ; carry bits 6 or 7 when we multiply.
               LDX #$D0
               LDY #0
               LDA (OPT_SCRN_MSG),Y
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               BCC NO_CARRY
               INX
NO_CARRY
               STA SMGET+1
               STX SMGET+2

; READNEXT       LDX MESSNO     ;contains the number of the char being read.
;
;                CPX #ENMESS-MESSTAB-1         ;length of message
;                BNE GOREAD
;                LDX #255
; GOREAD         INX
;
;                STX MESSNO
;
;                ; Multiply by 8 to get offset from $D000
;                ;
;                ; Only want/expect character values in range 0-$3F, so don't have to
;                ; carry bits 6 or 7 when we multiply.
;                LDY #$D0
;                LDA MESSTAB,X
;                ASL ; = ASL A
;                ASL ; = ASL A
;                ASL ; = ASL A
;                BCC NO_CARRY
;                INY
; NO_CARRY
;                STA SMGET+1
;                STY SMGET+2
;
;                ; This lookup version replaced by multiply version above
;                ; LDY MESSTAB,X
;                ; LDA CHARVEC1L,Y
;                ; STA SMGET+1
;                ; LDA #$D0
;                ; CPY #$20
;                ; ADC #0 ;AR=$D1 if carry flag is set by ^
;                ; STA SMGET+2

               LDY #7
SMGET          LDA $D000,Y ;smc
               STA $D3F8,Y
               DEY
               BPL SMGET

               RTS


;****************************************
;* Scroll the window message one pixel at a time
;****************************************
SCROLLTITS     LDA BUFF
               BNE GOANDOIT
               LDA #8
               STA BUFF
               JSR READNEXT
               LDA #1
GOANDOIT       AND #1
               BEQ NOTNOW
               JSR ROTCOLORS

NOTNOW         DEC BUFF
               LDX #0
GOSCROLLT      LDA CHARSTART+8,X
               ASL ; = ASL A
               ROL CHARSTART,X
               INX
               CPX #184
               BNE GOSCROLLT

               RTS


;****************************************
;*
;****************************************
SETCOLOURS     LDA #FLIPIN
               STA $01

               LDY #0
               LDA #2
STORECOL       STA 1184+$D400+18,Y
               STA 1264+$D400+18,Y

               INY

               CPY #20

               BNE STORECOL
               LDX #10
               LDA PLAYNO
               CMP #1
               BNE COLORB
               LDX #90

COLORB         LDY #10

COLORSET       TYA
               STA 1184+$D400+17,X

               DEX

               DEY

               BNE COLORSET
               RTS


;****************************************
;*             rotate the colours on player
;****************************************
ROTCOLORS      LDA #FLIPIN
               STA $01

               LDA PLAYNO
               CMP #1
               BEQ ROTB
               DEC 1184+$D400+20
               DEC 1184+$D400+21
               DEC 1184+$D400+22
               DEC 1184+$D400+23
               DEC 1184+$D400+24
               DEC 1184+$D400+25
               DEC 1184+$D400+26
               DEC 1184+$D400+18
               DEC 1184+$D400+19
               JMP EXRTC

ROTB           DEC 1264+$D400+20
               DEC 1264+$D400+21
               DEC 1264+$D400+22
               DEC 1264+$D400+23
               DEC 1264+$D400+24
               DEC 1264+$D400+25
               DEC 1264+$D400+26
               DEC 1264+$D400+18
               DEC 1264+$D400+19

EXRTC          LDA #FLIPOUT
               STA $01
               RTS

;****************************************
;*             Scroll the STAGE message
;****************************************
SETUPSCROLL    LDA #0
               STA MESS
               LDY #$84
CLEARSPACE     STA SCROLST-1,Y
               DEY
               BNE CLEARSPACE

               LDA STAGE
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               STA NUMST

               LDX #0

               LDA #FLIPOUT
               STA $01

READCHARS      LDA $D000+[19*8],X            ;load 's'
               STA SCROLST+[9*4],Y

               LDA $D000+[19*8]+4,X
               STA SCROLST+[9*4]+MC,Y
               LDA $D000+[20*8],X            ;load 't'
               STA SCROLST+[10*4],Y
               LDA $D000+[20*8]+4,X
               STA SCROLST+[10*4]+MC,Y

               LDA $D000+[1*8],X                            ;load 'a'
               STA SCROLST+[11*4],Y
               LDA $D000+[1*8]+4,X

               STA SCROLST+[11*4]+MC,Y

               LDA $D000+[7*8],X                            ;load 'g'
               STA SCROLST+[12*4],Y
               LDA $D000+[7*8]+4,X

               STA SCROLST+[12*4]+MC,Y

               LDA $D000+[5*8],X                            ;load 'e'
               STA SCROLST+[13*4],Y
               LDA $D000+[5*8]+4,X

               STA SCROLST+[13*4]+MC,Y

               TXA

               PHA

               LDX NUMST
               LDA $D000+[48*8],X            ;load 'num in x'
               STA SCROLST+[15*4],Y
               LDA $D000+[48*8]+4,X
               STA SCROLST+[15*4]+MC,Y

               PLA

               TAX

               INC NUMST

               INX

               INY

               CPY #4
               BNE READCHARS
               ; Fall through...

;****************************************
;*             Display window on screen
;****************************************
DISPCHARS      LDA #FLIPIN
               STA $01

               LDY #31

               LDX #62

GODISP         LDA SCROLST,Y
               STA $E000+[320*16]+[8*16],X
               STA $E000+[320*16]+[8*16]+1,X

               LDA SCROLST+MC,Y
               STA $E000+[320*17]+[8*16],X
               STA $E000+[320*17]+[8*16]+1,X

               DEX
               DEX

               DEY

               BPL GODISP
               RTS


;****************************************
;*             Do the scroll of the window
;****************************************
ACTUALSCROLL   LDA TIME
               CMP #20

               BCS STAGEM
               JMP DELA
STAGEM         LDA MESS
               CMP #128
               BNE DOSCROLL
               JMP DELA
DOSCROLL       INC MESS
               LDX #$40
GOSCROLL       LDA SCROLST+4-$40,X

               ASL ; = ASL A

               ROL SCROLST-$40,X ;top
               LDA SCROLST+MC+4-$40,X
               ASL ; = ASL A

               ROL SCROLST+MC-$40,X ;bottom
               INX

               BPL GOSCROLL
               JMP DISPCHARS


;****************************************
;*             Little delay
;****************************************

DELA           LDY #7
DELA1          LDX #0

DELA2          INX
               BNE DELA2

               DEY
               BNE DELA1

               RTS


;****************************************
;*             Update the high score if necessary
;****************************************
HIUPDATE       LDA PLAYER
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               CLC
               ADC #5
               TAY

GOHI           LDA SCORE1,Y
               CMP HISCORE,Y

               BEQ NEXTCOMP
               BCC NOTHIGHER

               BCS JHIGHER

NEXTCOMP       DEY

               BPL GOHI
NOTHIGHER      RTS

JHIGHER        LDA PLAYER
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               CLC
               ADC #5
               TAY

MOVEHIGH       LDA SCORE1,Y
               STA HISCORE,Y

               DEY

               BPL MOVEHIGH
               LDY #3
               STY PLAYER
               JSR PLOTSCORE

               LDY #0
               STY PLAYER
               RTS



;****************************************
;*             Check for one specific key
;*
;* Table below is from
;* www.atarimagazines.com/compute/issue39/242_1_VIC-20_64_TRANSLATIONS_READING_THE_KEYBOARD.php
;* Row heading is value to store in $DC00 for row select.
;* Column heading is bit that will be cleared in output read from $DC01 if
;* corresponding key in selected row is pressed.
;*
;*          $7F        $BF      $DF       $EF         $F7     $FB      $FD     $FE
;*  $7F     RUN/STOP   Q        COMMODORE SPACE       2       CTRL     <-      1
;*  $BF     /          UP ARROW =         RIGHT SHIFT HOME    ;        *       
;*  $DF     ,          @        :         .           -       L        P       +
;*  $EF     N          O        K         M           0       J        l       9
;*  $F7     V          U        H         B           8       G        Y       7
;*  $FB     X          T        F         C           6       D        R       5
;*  $FD     LEFT SHIFT E        S         Z           4       A        W       3
;*  $FE     ^CURSORv   f5       f3        f1          f7      <CURSOR> RETURN  DEL
;*
;* Y-reg tells which key to check for
;   YR   ROWTAB  BITPAT     KEY
;   0     $DF    %01000000   @
;   1     $FD    %00000100   A
;   2     $F7    %00010000   B
;   3     $FB    %00010000   C
;   4     $FB    %00000100   D
;   5     $FD    %01000000   E
;   6     $FB    %00100000   F
;   7     $F7    %00000100   G
;   8     $F7    %00100000   H
;   9     $EF    %00000010   I
;   10    $EF    %00000100   J
;   11    $EF    %00100000   K
;   12    $DF    %00000100   L
;   13    $EF    %00010000   M
;   14    $EF    %10000000   N
;   15    $EF    %01000000   O
;   16    $DF    %00000010   P
;   17    $7F    %01000000   Q
;   18    $FB    %00000010   R
;   19    $FD    %00100000   S
;   20    $FB    %01000000   T
;   21    $F7    %01000000   U
;   22    $F7    %10000000   V
;   23    $FD    %00000010   W
;   24    $FB    %10000000   X
;   25    $F7    %00000010   Y
;   26    $FD    %00010000   Z
;   27    $DF    %00100000   :
;   28    $BF    %00000001   
;   29    $BF    %00000100   ;
;   30    $BF    %01000000   UP ARROW
;   31    $7F    %00000010   <- (LEFT ARROW)
;   32    $7F    %00010000   SPACE
;   33    $7F    %00000100   CTRL
;   34    $7F    %10000000   RUN/STOP
;   35    $7F    %00100000   COMMODORE
;   36    $FD    %10000000   LEFT SHIFT
;   37    $FE    %00010000   f1
;   38    $FE    %00100000   f3
;   39    $FE    %01000000   f5
;   40    $FE    %00001000   f7
;   41    $BF    %00000010   *
;   42    $DF    %00000001   +
;   43    $DF    %10000000   ,
;   44    $DF    %00001000   -
;   45    $DF    %00010000   .
;   46    $BF    %10000000   /
;   47    $EF    %00001000   0
;   48    $7F    %00000001   1
;   49    $7F    %00001000   2
;   50    $FD    %00000001   3
;   51    $FD    %00001000   4
;   52    $FB    %00000001   5
;   53    $FB    %00001000   6
;   54    $F7    %00000001   7
;   55    $F7    %00001000   8
;   56    $EF    %00000001   9
;   57    $BF    %00001000   HOME
;   58    $FE    %00000001   DEL
;   59    $BF    %00100000   =
;   60    $FE    %00000010   RETURN
;   61    $BF    %00010000   RIGHT SHIFT
;   62    $FE    %10000000   ^CURSORv
;   63    $FE    %00000100   <CURSOR>
;****************************************
KEYS           LDA ROWTAB,Y
               STA $DC00

               LDA $DC01
               EOR #$FF
               AND BITPAT,Y
               RTS



;****************************************
;*             Scan keys to see if one is pressed
;****************************************
DETECT         LDY #47 ; 0
               JSR KEYS
               BEQ NOT0
NOTB           RTS

NOT0           LDY #1 ; A
               JSR KEYS
               BEQ NOTA
               JMP ONEPL

NOTA           INY ; = 2 ; B
               JSR KEYS
               BEQ NOTB
               JMP TWOPL

ONEPL          LDA PLAYNO
               BEQ EXSPL ;already one player

               LDA #0
               BEQ LOVESEXY ;=jmp
TWOPL          LDA PLAYNO

               BNE EXSPL ;already two player

               LDA #1
LOVESEXY       STA PLAYNO
               JSR SETCOLOURS
EXSPL          LDA #0

               RTS



;****************************************
;*             Move the arrows on the title screen
;****************************************
MOVEARROWS     LDA ARROW
               BEQ CHECKEYS
               TAY

               AND #128
               BNE ARROW2M

               LDX #0
GOARROW        TYA
               AND #64

               BNE ARROW1UP
               TXA
               TAY
               ASL ; = ASL A
               TAX
               INC 53257,X ; $D009
               JMP CHECKARROW1

ARROW1UP       TXA
               TAY
               ASL ; = ASL A
               TAX
               DEC 53257,X ; $D009
               DEC 53257,X ; $D009

CHECKARROW1
               LDA 53257,X ; $D009
               PHA
               TYA
               TAX
               PLA

               LDY CHOICE,X
               CMP ARROWPOS,Y
               BEQ STOPARROW
               RTS

STOPARROW      LDA #0
               STA ARROW
               RTS

ARROW2M        LDX #1
               JMP GOARROW

CHECKEYS       LDY #48 ; 1
               JSR KEYS
               BNE ARROW1
               INY ; = 49 ; 2
               JSR KEYS
               BNE ARROW2
               RTS

ARROW1         LDX #0
DOSETARROW     LDY CHOICE,X
               CPY #2
               BEQ RESETCHOICE

TRYAGAIN       INC CHOICE,X
               INY

               LDA ARROW

               ORA #1
               STA ARROW

               RTS

RESETCHOICE
               LDA #255
               STA CHOICE,X
               TAY ;=255

               LDA ARROW

               ORA #64

               STA ARROW

               JMP TRYAGAIN
ARROW2         LDX #1
               LDA #128
               STA ARROW

               JMP DOSETARROW


;****************************************
;*             Scan direction keys for game
;*
;* Returns a 4-bit value in the accumulator (bits 4-7 clear) meaning:
;*  bit 0: 0 = up pressed
;*  bit 1: 0 = down pressed
;*  bit 2: 0 = left pressed
;*  bit 3: 0 = right pressed
;****************************************
KEYSCAN        LDA #$0F
               STA SRA

               LDX #3
KEY_2          LDA KTAB,X
               STA SELF_1+1
SELF_1         LDY KUP ;smc
               JSR KEYS
               BEQ HERE
               LDA SRA
               AND AND_TAB,X
               STA SRA

HERE           DEX
               BPL KEY_2

               LDA SRA
               RTS


KTAB           BYTE KLEFT,KRIGHT,KUP,KDOWN

AND_TAB        BYTE 11,7,14,13; %1011, %0111, %1110, %1101


;*****************************************************************************
;* Death routines
;*
;* Returns: Accumulator = 3 = not dead
;*****************************************************************************
DEATH          LDA #0 ; Set object we are looking at to the first one -- ie the blob
               STA TABLE_ENT

               LDA TIME
               CMP #2
               BCC DEATHYES ; Time ran out

               LDA DEAD_YET
               BNE DEATHYES2 ; We are already dying

               LDA IXTAB+$10 ;bit 6 set if shadow is spike
               AND #$40
               BEQ NO_BLOB_D ; There is no spike so we are not above the void

               LDA IXTAB+$6 ;z-co (lo) of blob
               BEQ DEATHYES2 ; We reached level 0 in empty space
               BMI DEATHYES1 ; We are below level 0 in empty space

NO_BLOB_D      LDA #3 ; We must still be alive!
               RTS

DEATHYES1      CMP #$FF-$18
               BCC NO_BLOB_D

               INC IXTAB+6 ;z-co (lo) of blob
               INC IXTAB+6 ;z-co (lo) of blob
DEATHYES2      INC TABLE_ENT
DEATHYES       JSR KILL
               LDA PLAYER
               CMP #1
               BEQ KILL2 ;branch if "player 2"

               JSR RESET_BOT

               LDA STAGE
               STA STAGE1
               LDA CORK
               STA CORK+1
               LDA BOTTLES
               STA BOTTLES+1

               SEC
;               LDA LIVES+1
;               SBC #1               ;0/11111111111
;               STA LIVES+1
;               LDA LIVES
;               SBC #0
;               STA LIVES
;               ORA LIVES+1
               ; Decimal flag only affects arithmetic functions, not increments
               ; decrements, so we can't just do SED / DEC LIVES / CLD.
               LDA LIVES
               SED
               SBC #1
               CLD
               STA LIVES

               BNE NOGAMEOVER
               JSR GAMEOVER1

               JMP NOTIMEOUT

NOGAMEOVER     JSR SETIMEOUT

NOTIMEOUT      LDA LIVES2
;               ORA LIVES2+1
               BEQ ORALL

               INC PLAYER

               ;swap positions

               LDY #8
UP_ERE         LDA ST_POS,Y
               STA ST_POS_1,Y
               LDA ST_POS_2,Y
               STA ST_POS,Y
               DEY
               BPL UP_ERE

               LDA BOTTLES+2
               STA BOTTLES
               LDA STAGE2
               STA STAGE
               LDA CORK+2
               STA CORK

ORALL          JSR SECSETUP
               JSR RESET_STOP
               LDY #0
               LDA LIVES
;               ORA LIVES+1
               ORA LIVES2
;               ORA LIVES2+1
               BEQ COMPLETEOVER

               JSR RESETIMER

               LDA #1
               RTS

COMPLETEOVER   JSR GAMEOFF

               LDA #0
               RTS

GAMEOVER1      JSR RESET_BUNG

               JSR SETGAMEOVER

               JMP HIUPDATE


               ;routine for "player 2"

KILL2          JSR RESET_BOT
               LDA STAGE
               STA STAGE2
               LDA CORK
               STA CORK+2
               LDA BOTTLES
               STA BOTTLES+2

               LDY #8
UP_ERE3        LDA ST_POS,Y
               STA ST_POS_2,Y
               DEY
               BPL UP_ERE3

               SEC
;               LDA LIVES2+1
;               SBC #1
;               STA LIVES2+1
;               LDA LIVES2
;               SBC #0
;               STA LIVES2
;               ORA LIVES2+1
               LDA LIVES2
               SED
               SBC #1
               CLD
               STA LIVES2
               BNE NOGAMEOVER2

               JSR GAMEOVER1
               JMP NOTIMEOUT2
NOGAMEOVER2
               JSR SETIMEOUT

NOTIMEOUT2     LDA LIVES
;               ORA LIVES+1
               BEQ ORALL

               DEC PLAYER

               LDY #8
UP_ERE4        LDA ST_POS_1,Y
               STA ST_POS,Y
               DEY
               BPL UP_ERE4

               LDA BOTTLES+1
               STA BOTTLES
               LDA CORK+1
               STA CORK
               LDA STAGE1
               STA STAGE
               JMP ORALL

;*****************************************************************************
;*
;*
;*
;*****************************************************************************
PLAYMESSM      LDA #FLIPOUT
               STA $01

               LDA YPIX
               BEQ MORECHARS

               LDX #8
DLAY2          LDY #0
DLAY1          INY
               BNE DLAY1
               DEX
               BNE DLAY2

               DEC YPIX
               DEC YPIX
               JMP PWTBUF ;bank ROMs in then do WKTOBUF
MORECHARS      LDA #4

               STA YPIX
               JSR SHIFTDOWN

               LDA PLAYMESS
               CMP #2
               BCC NOCLEARTOP
               RTS

NOCLEARTOP     INC PLAYMESS
               LDY #0
PLOP           LDX STARTAB,Y
               LDA WELCOME,Y
               STA TEMP
               TYA
               PHA

               ; Multiply by 8 to get offset from $D000
               ; Input is <$40 so don't need to worry about carrying the top two bits
               LDY #$D0
               LDA TEMP
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               BCC NO_CARRY_1
               INY
NO_CARRY_1     STA LOADA+1
               STY LOADA+2

               ; This lookup version replaced by multiply version above
               ; LDY TEMP
               ; LDA CHARVEC1L,Y
               ; STA LOADA+1
               ; LDA #$D0
               ; CPY #$20
               ; ADC #0 ;AR=$D1 if carry flag is set by ^
               ; STA LOADA+2

               LDA PLAYMESS
               CMP #1
               BNE NOADC4
               LDA LOADA+1
               CLC
               ADC #$04
               STA LOADA+1

NOADC4         LDY #0
PLOP2
LOADA          LDA $D000,Y ;smc
               STA WA,X  ; top of screen
               STA WA+WA_WIDTH_BYTES,X
               INY
               TXA
               CLC
               ADC #[2*WA_WIDTH_BYTES]
               TAX
               CPY #4
               BNE PLOP2

               PLA
               TAY
               INY
               CPY #8
               BNE PLOP
               JMP PWTBUF ;bank ROMs in then do WKTOBUF


;****************************************
;*             Time out/Game over message
;****************************************
ENDPLAYMESS    LDA #FLIPOUT
               STA $01

               LDA YPIX
               CMP #4
               ;beq morechar ;BCS below will branch if >=4

               BCS MORECHAR
               INC YPIX
               INC YPIX
               LDX #7
DLAY3          LDY #0

DLAY4          INY
               BNE DLAY4

               DEX
               BNE DLAY3

               JMP PWTBUF ;bank ROMs in then do WKTOBUF
MORECHAR       LDA #0
               STA YPIX
               JSR SHIFTUP

               LDA PLAYMESS
               CMP #2
               BCC NOCLEARBOT
               RTS

NOCLEARBOT     JSR CLEARBOT
               INC PLAYMESS
               LDY #0
PLOD           LDX STARTAB,Y
GETFROM        LDA WELCOME,Y ; smc -- message to display
               STA TEMP
               TYA
               PHA

               ; Multiply by 8 to get offset from $D000
               ; Input is <$40 so don't need to worry about carrying the top two bits
               LDY #$D0
               LDA TEMP
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               BCC NO_CARRY_2
               INY
NO_CARRY_2     STA LOADA1+1
               STY LOADA1+2

               ; This lookup version replaced by multiply version above
               ; LDY TEMP
               ; LDA CHARVEC1L,Y
               ; STA LOADA1+1
               ; LDA #$D0
               ; CPY #$20
               ; ADC #0 ;AR=$D1 if carry flag is set by ^
               ; STA LOADA1+2

               LDA PLAYMESS
               CMP #2
               BNE NOINC4
               LDA LOADA1+1
               CLC
               ADC #$04
               STA LOADA1+1
NOINC4         LDY #0
PLOD2
LOADA1         LDA $D000,Y ;smc
               STA WA+[14*WA_WIDTH_BYTES*8],X  ; bottom of screen
               STA WA+[14*WA_WIDTH_BYTES*8]+WA_WIDTH_BYTES,X
               INY
               TXA
               CLC
               ADC #[2*WA_WIDTH_BYTES]
               TAX
               CPY #4
               BNE PLOD2

               PLA
               TAY
               INY
HOWMANY        CPY #8     ;self-modifying code -- number of characters in message
               BNE PLOD
               JMP PWTBUF ;bank ROMs in then do WKTOBUF

CLEARBOT       LDY #$C0
               LDA #$0

CLEARBOT1      STA WA+[14*WA_WIDTH_BYTES*8]-1,Y
               DEY
               BNE CLEARBOT1

               JMP PWTBUF ;bank ROMs in then do WKTOBUF


SETIMEOUT      LDA #0
               STA PLAYMESS
               JSR SAVPOS
               JSR ALOCAT
               JSR MOVSHPRT
               JSR WKTOBUF

               LDA #<TIMEOUT
               STA GETFROM+1
               LDA #>TIMEOUT
               STA GETFROM+2

               LDA #[TIMEOUTEND-TIMEOUT]
               STA HOWMANY+1

               LDA TABLE_ENT
               BEQ ISOKAY

               LDA #<NOTHING
               STA GETFROM+1
               LDA #>NOTHING
               STA GETFROM+2

               LDA #1
               STA HOWMANY+1

ISOKAY         LDA #$32
               STA YELLO ;Matt's loopy variable
DEATHLOOP      JSR ENDPLAYMESS
               DEC YELLO
               BNE DEATHLOOP

               RTS

;******************************************************************************
;*
;******************************************************************************
SETGAMEOVER    LDA #0
               STA PLAYMESS
               JSR SAVPOS
               JSR ALOCAT
               JSR MOVSHPRT
               JSR WKTOBUF

               LDA #<GAMEOVER
               STA GETFROM+1
               LDA #>GAMEOVER
               STA GETFROM+2

               LDA #[GAMEOVEREND-GAMEOVER]
               STA HOWMANY+1

               LDA #$32
               STA YELLO ;Matt's loopy variable
DIETWO         JSR ENDPLAYMESS
               DEC YELLO
               BNE DIETWO

               RTS



COLCHANGE      LDA #FLIPOUT
               STA $01

               LDY STAGE
               LDA TOP1VL,Y
               STA TOPMD1+1
               LDA TOP2VL,Y
               STA TOPMD2+1
               LDA TOP3VL,Y
               STA TOPMD3+1
               LDA TOP4VL,Y
               STA TOPMD4+1
               LDA FLR1VL,Y
               STA FLRMD1+1
               LDA FLR2VL,Y
               STA FLRMD2+1
               LDA FLR3VL,Y
               STA FLRMD3+1
               LDA FLR4VL,Y
               STA FLRMD4+1
               LDA LFT1VL,Y
               STA LFTMD1+1
               LDA LFT2VL,Y
               STA LFTMD2+1
               LDA LFT3VL,Y
               STA LFTMD3+1
               LDA LFT4VL,Y
               STA LFTMD4+1
               LDA RGT1VL,Y
               STA RGTMD1+1
               LDA RGT2VL,Y
               STA RGTMD2+1
               LDA RGT3VL,Y
               STA RGTMD3+1
               LDA RGT4VL,Y
               STA RGTMD4+1

               LDA SCRCOL,Y
               JSR SET_PLAY_AREA_COLOUR

               LDA #FLIPIN
               STA $01

               LDA STAGE
               ASL ; = ASL A
               TAY

               LDA MAP_BASE,Y
               STA MAP_ADD1+1
               STA MAP_ADD2+1
               LDA MAP_BASE+1,Y
               STA MAP_ADD1+2
               STA MAP_ADD2+2

               LDA TRAP_BASE,Y
               STA CHEC_AGAIN+1
               STA LDA_AGAIN+1
               STA LDX_AGAIN+1
               LDA TRAP_BASE+1,Y
               STA CHEC_AGAIN+2
               STA LDA_AGAIN+2
               STA LDX_AGAIN+2

               RTS

;****************************************
;*             Set colour for play area on hires screen
;*
;*             AR = colour to set
;****************************************
SET_PLAY_AREA_COLOUR
               LDY #19
COLOOP         STA $D800+1074,Y
               STA $D800+1114,Y
               STA $D800+1154,Y
               STA $D800+1194,Y
               STA $D800+1234,Y
               STA $D800+1274,Y
               STA $D800+1314,Y
               STA $D800+1354,Y
               STA $D800+1394,Y
               STA $D800+1434,Y
               STA $D800+1474,Y
               STA $D800+1514,Y
               STA $D800+1554,Y
               STA $D800+1594,Y
               DEY
               BPL COLOOP
               RTS

;****************************************
;*             Delay Loop
;****************************************
DELAY          LDX #20
               LDY #0
DELAY1         INY

               BNE DELAY1
               DEX

               BNE DELAY1
               RTS


;****************************************
;*             Dead !! Do sound and explode blob
;****************************************
KILL           LDA #0
               STA SFIRE

               STA SFIRE+IXSIZ

               JSR DEATHSONIX
               LDA #7
               STA IXTAB ;shape
               LDA #0
               STA IXTAB+1 ;data offset
               LDY #12

KILLOOP        TYA
               PHA

               JSR WOOSH

               JSR SAVPOS
               JSR ALOCAT
               JSR MOVSHPRT
               JSR WKTOBUF

               JSR PUTPOS
               JSR WOOSH

               JSR DELAY

               PLA
               TAY

               AND #1
               BNE NOALT

               INC IXTAB+1 ;data offset
NOALT          DEY
               BPL KILLOOP

               JSR WKTOBUF

               JSR DELAY

               LDA #0
               STA IXTAB+1 ;data offset
               STA IXTAB ;shape
               RTS


;****************************************
;*             clear screen etc and remake timer
;****************************************
RENTERY        JSR RESETIMER
               JSR TIMESOUND  ;set up timer increment sound

               LDY #0
LOOP           TYA
               PHA

               JSR TIMER      ;increment timer on screen

               LDA #FLIPIN
               STA $01
               JSR ZAP

               JSR ACTUALSCROLL              ;scroll 'stage' message in window
NOTREADY       LDX $D012
               CPX #1
               BNE NOTREADY   ;check for raster at top of screen

               JSR ACTUALSCROLL
ALMSRDY        LDX $D012 ;raster
               CPX #1
               BNE ALMSRDY

               PLA
               TAY

               INY

               CPY #99

               BNE LOOP
               JSR TEST ;clear work area and display

               LDY #0
               STY $D404

               STY $D40B

               STY $D412

               LDA #6
               STA YELLO ;Matt's loopy variable
DISPLAYER      JSR PLAYMESSM ;scroll down 'player' message
               DEC YELLO

               BNE DISPLAYER

               LDA #0
               STA XPIX
               ;lda #0

               STA YPIX
               ;lda #0

               STA IXTAB ;shape              ;set blob to not existing
               LDA IXTAB+$10 ;bit 7 set if resting on object

               ORA #%10000000
               STA IXTAB+$10 ;set blob resting on object

               LDA #$FF
               STA IXTAB+$11 ;no shadow
               LDA #<IXTAB
               STA IX

               LDA #>IXTAB
               STA IX+1

               LDA #<SHADO
               STA IY

               LDA #>SHADO
               STA IY+1
               JSR TRSHAD ;set up shadow

               ;set up blob start positon
               LDA ST_POS
               STA IXTAB+$03 ;x-co lo
               LDA ST_POS+1
               STA IXTAB+$02 ;x-co hi
               LDA ST_POS+2
               STA IXTAB+$05 ;y-co lo
               LDA ST_POS+3
               STA IXTAB+$04 ;y-co hi
               LDA ST_POS+4
               STA IXTAB+$06 ;z-co (lo)
               AND #0
               STA IXTAB+$16 ;x-co acc

               STA IXTAB+$17 ;y-co acc

               STA IXTAB+$18 ;z-co acc

               STA IXTAB+$12 ;x-co vel

               STA IXTAB+$13 ;y-co vel

               STA IXTAB+$14 ;z-co vel

               ;set up screen start position
               LDA ST_POS+5
               STA SCREENX

               LDA ST_POS+6
               STA SCREENX+1

               LDA ST_POS+7 ;lo
               CLC
               ADC #$80
               STA SCREENY

               LDA ST_POS+8 ;hi
               ADC #$00
               STA SCREENY+1

               ;set up very clever patch routine to eliminate unwanted moving
               ;shapes when scrolling down...
               LDA #<PATCH
               STA DOACTPR+1
               LDA #>PATCH
               STA DOACTPR+2

               LDY #0
               STY MYBUL

               STY MYBUL+IXSIZ

               STY SFIRE

               STY SFIRE+IXSIZ

               STY BACKS

               STY BACKS2
               STY ALTABS
               STY ALTABS+IXSIZ
MAP_DOWN_L     STY YELLO ;Matt's loopy variable
               JSR SCRL_START ;scroll screen down four pixels
               JSR SAVPOS ;save backgrounds (moving shapes) and set up sort table
               JSR ALOCAT ;do sort into YORDER table

               JSR MOVSHPRT ;print sorted shapes

               JSR WKTOBUF ;copy work area to screen

               JSR PUTPOS ;restore saved backgrounds to work area
                              ;^^^leave in as stops lift buggering

               LDY YELLO

               INY

               CPY #$20
               BCC MAP_DOWN_L
               INC IXTAB ;shape ;=1

               ;unpatch the "very clever" routine...

               LDA #<BRITT
               STA DOACTPR+1
               LDA #>BRITT
               STA DOACTPR+2

               JMP REZZ


PATCH          LDA YELLO ;how many 1/2 chars screen has moved down
               ASL ; = ASL A

               ASL ; = ASL A ;*4=# of pixels

               LDY #$0F
               CMP (IX),Y ;on-screen y-co
               BCC SUNCITY

               ;Do print this object
               JMP BRITT

               ;Don't print this object
SUNCITY        PLA ;pull return address off stack
               PLA
               JMP MVSHPRL


;
; File X2.BUB
; Starts Here
;
;

;
;
;  PDS Pc3.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          10.26
;         By  "JAKE AND MATTHEW..."
;
;     Created on Wed the 22nd of Jul 1987
;        Last update 08:41 on 10/05/88
;

; To set up sound effects, just call the relevant setup
; routines and make sure that JSR SONIX is in the main loop
; and it will do all the rest itself. A list of routine names
; and when they are called follows



; 1) GETCORK   when the blob obtains a cork
; 2) SHOOT                    when the blob shoots

; 3) JUMPSON   when the blob jumps

; 4) BANG                     when something explodes

; 5) MATERIAL  when something materialises
; 6) BURST                    when a bubble bursts

; 7) EXTRAL    when an extra life is obtained

; 8) CORKING   when a bottle is corked



;**************************************

;*             Set up sound register so that
;*             when a sound is first produced,
;*             it is not affected by the last

;*             sound done.
;****************************************

SONSETUP       LDA #FLIPIN
               STA $01

               LDA #15
               STA VOL

               STA $D418

               LDA #4
               STA CORKED
               STA BOLT
               STA EXPL
               STA LEAP
               STA MAT

               STA XTRA
               LDA #0
               STA FILO
               STA FILHI

               STA FILTER
               STA RESONANCE

               STA FILON

               STA $D415
               STA $D416
               STA $D417
               RTS


;****************************************
;*             Works out, given a priority,
;*             if a voice is availble for the
;*             sound effect and if it is, it
;*             sets up the registers of that
;*             voice for the effect (From
;*             v4hi onwards.) It also puts
;*             the address of the effect
;*             service routine in the SONIX
;*             routine. Handles filters etc
;****************************************
WHICHVOICE     LDA #FLIPIN
               STA $01

               LDA VOC1
               BEQ VOICE1
               LDA VOC2
               BEQ VOICE2
               LDA VOC3
               BEQ VOICE3     ;if any voices free, then use them

               LDA VOC1
               CMP PRIORITY
               ;beq voice1 - bcs (below) will branch if >=

               BCS VOICE1
               LDA VOC2
               CMP PRIORITY
               ;beq voice2 - bcs (below) will branch if >=

               BCS VOICE2
               LDA VOC3
               CMP PRIORITY
               ;beq voice3 - bcs (below) will branch if >=

               BCS VOICE3     ;if any voices are of same priority, use them
               LDY #100
               RTS

VOICE1         LDY #0
               LDA PRIORITY
               STA VOC1
               LDA SONLO,X

               STA SMJSR1+1
               LDA SONHI,X

               STA SMJSR1+2
               LDA #0
               STA PRIORITY   ;set up service routine, set voice1 to used
               JMP COMSOUND   ;and give an offset of zero.

VOICE2         LDY #7
               LDA PRIORITY
               STA VOC2
               LDA SONLO,X

               STA SMJSR2+1
               LDA SONHI,X

               STA SMJSR2+2
               LDA #1
               STA PRIORITY   ;set up service routine, set voice2 to used
               JMP COMSOUND   ;and give an offset of seven.
VOICE3         LDY #14

               LDA PRIORITY
               STA VOC3
               LDA SONLO,X

               STA SMJSR3+1
               LDA SONHI,X

               STA SMJSR3+2
               LDA #2         ;set up service routine, set voice3 to used
               STA PRIORITY   ;and give an offset of fourteen.

;****************************************
;*             Set up voice, using offset
;*             from psuedo registers v4lo
;*             onwards
;****************************************
COMSOUND       STY OFFSET
               LDX #0
COMMENCE       LDA V4LO,X
               STA V1LO,Y
               STA $D400,Y

               INY

               INX

               CPX #7
               BNE COMMENCE
               LDY PRIORITY
               LDA FILON      ;work out filtering

               BEQ TURN_OFF
               LDA FILTER
               ORA FIL_TAB,Y

               ORA RESONANCE

               STA FILTER
               LDA FILO
               STA $D415

               LDA FILHI

               STA $D416

               JMP TURNED_ON

TURN_OFF       LDA FILTER
               AND FIL_TAB2,Y
               ORA RESONANCE

               STA FILTER
               LDA VOL

               AND #$0F
               STA VOL

TURNED_ON      LDA FILTER

               STA $D417

               LDA VOL

               STA $D418

               LDY OFFSET
               LDA #0         ;set sound going

               STA $D404,Y

               LDA V4WV
               STA $D404,Y

               RTS



;****************************************

;*             Sound service routine. This
;*             goes through each voice and
;*             jsrs the relevant sound routine
;*             checking first to see if one

;*             has been set up.

;****************************************

SONIX          LDA #FLIPIN
               STA $01

               LDA VOC1
               BEQ NOVOICE1
               LDY #0
SMJSR1         JSR SERPUT     ;selfmodifying jsr for voice1
NOVOICE1

               LDA VOC2
               BEQ NOVOICE2
               LDY #7
SMJSR2         JSR SERPUT     ;selfmodifying jsr for voice2
NOVOICE2

               LDA VOC3
               BEQ NOVOICE3
               LDY #14

SMJSR3         JMP SERPUT     ;selfmodifying jmp for voice3
NOVOICE3

               RTS



;****************************************

;*             Sound effect for getting a

;*             cork. Called from CORKON.
;****************************************

GETCORK        LDA CORKED

               CMP #4
               BEQ CONTPUT

               TAY                           ;effect already occuring so reset voice.

               LDA #0
               STA VOC1,Y
CONTPUT        LDA #65

               STA V4WV
               LDA #0
               STA V4HI
               STA V4LO
               STA V4SR
               LDA #10

               STA V4AD
               STA V4PLO

               STA V4PLHI
               LDA #3
               STA PRIORITY
               LDA #1
               STA FILON

               LDA #15+16
               STA VOL

               LDA #$63
               STA FILHI

               LDA #$F0
               STA RESONANCE

               LDX #0
               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOCORKED
               LDY PRIORITY
               STY CORKED
NOCORKED       RTS



;****************************************

;*             Service routine for Corking
;*             sound effect.
;****************************************

SERPUT         LDA V1HI,Y
               CMP #40

               ;beq endcorked - bcs (below) will branch if >=
               BCS ENDCORKED

               ;clc
               ADC #5
               STA $D401,Y

               STA V1HI,Y
               LDA V1PLHI,Y
               SEC

               SBC #2
               STA V4PLHI,Y
               STA $D403,Y

               RTS

ENDCORKED      LDY CORKED     ;reset voice
               LDA #0
               STA VOC1,Y
               LDA #4
               STA CORKED
               RTS


;****************************************
;*             Shooting sound effect. Uses
;*             BOLT as a counter in the
;*             service routine.
;****************************************
SHOOT          LDA BOLT
               CMP #4
               BEQ CONTSHOT
               TAY                           ;effect already occuring so reset voice.

               LDA #0
               STA VOC1,Y
CONTSHOT       LDA #129

               STA V4WV
               LDA #52

               STA V4HI
               STA V4LO
               LDA #$0C
               STA V4SR
               LDA #$1A
               STA V4AD
               LDA #1
               STA PRIORITY
               ;lda #1

               STA FILON

               TAX ;=1 ;{for WHICHVOICE subroutine}
               LDA #15+16
               STA VOL

               LDA #$93
               STA FILHI

               LDA #$40
               STA RESONANCE

               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOSHOT
               LDY PRIORITY
               STY BOLT
NOSHOT         RTS


;****************************************
;*             Service routine for shooting
;*             effect.
;****************************************
SERSHOT        LDA V1HI,Y
               CMP #37

               BEQ ENDSHOT

               BCC ENDSHOT

               ;sec
               SBC #5
               STA $D401,Y

               STA V1HI,Y
               RTS

ENDSHOT        LDY BOLT       ;reset voice
               LDA #0
               STA VOC1,Y
               LDA #4
               STA BOLT
               RTS


;****************************************
;*             Jumping sound effect.
;****************************************
JUMPSON        LDA LEAP
               CMP #4
               BEQ CONTJUMP
               TAY                           ;effect already occuring so reset voice
               LDA #0
               STA VOC1,Y
CONTJUMP       LDA #17
               STA V4WV
               LDA #3
               STA V4HI
               STA V4LO
               LDA #$0E
               STA V4SR
               LDA #$0F
               STA V4AD
               LDA #2
               STA PRIORITY
               LDA #0
               STA LEAPCOUNT

               STA FILON

               LDA VOL

               AND #$F0
               ORA #15

               STA VOL

               LDA #$F0
               STA RESONANCE

               LDX #2
               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOJUMP
               LDY PRIORITY
               STY LEAP
NOJUMP         RTS



;*******************************************
;*             Service routine for Jump effect
;*             Uses LEAPCOUNT as a counter &
;*             offset to LEAPTAB which contains
;*             freq values for effect.
;*******************************************
SERJUMP        LDX LEAPCOUNT
               CPX #$20
               BEQ ENDJUMP

               LDA LEAPTAB,X

               STA $D401,Y

               STA V1HI,Y
               INC LEAPCOUNT

               RTS

ENDJUMP        LDA #0

               STA V1HI,Y
               STA $D404,Y

               LDY LEAP
               STA VOC1,Y
               LDA #4
               STA LEAP
               RTS


;****************************************
;*             Explosion sound for exploding
;*             aliens. Heavy use of filters.
;****************************************
BANG           LDA EXPL
               CMP #4
               BEQ CONTBANG
               TAY                           ;effect already occuring so reset voice.

               LDA #0
               STA VOC1,Y
CONTBANG       LDA #129+2

               STA V4WV
               LDA #3
               STA V4HI
               STA V4LO
               LDA #$0C
               STA V4SR
               LDA #$1A
               STA V4AD
               LDX #0
               STX BANGCOUNT

               INX ;=1

               STX FILON

               INX ;=2

               STX PRIORITY
               INX ;=3 ;{for WHICHVOICE subroutine}
               LDA #15+16
               STA VOL

               LDA #$68
               STA FILHI

               LDA #$F0
               STA RESONANCE

               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOBANG
               LDY PRIORITY
               STY EXPL
NOBANG         RTS


;****************************************
;*             Service the explosion. Use
;*             BANGCOUNT for counter and
;*             offset to BANGTAB which houses
;*             freq valus for effect.
;****************************************
SERBANG        LDX BANGCOUNT
               CPX #19
               BEQ ENBYTEANG

               LDA BANGTAB,X

               STA $D401,Y

               STA V1HI,Y
               INC BANGCOUNT

               RTS

ENBYTEANG      LDY EXPL       ;reset voice
               LDA #0

               STA VOC1,Y
               LDA #4
               STA EXPL
               RTS


;**************************************
;*             materialization sound
;**************************************
MATERIAL       LDA MAT
               CMP #4
               BEQ CONTMAT

               TAY                           ;effect already occuring so reset voice
               LDA #0
               STA VOC1,Y
CONTMAT        LDA #65

               STA V4WV
               LDA #30

               STA V4HI
               STA V4LO
               LDA #$08
               STA V4SR
               LDA #$28
               STA V4AD
               LDA #1
               STA PRIORITY
               STA MATCOUNT
               STA FILON

               LDA #$63
               STA FILHI

               LDA #8
               STA V4PLHI
               LDA #15+16
               STA VOL

               LDA #$F0
               STA RESONANCE

               LDX #4
               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOMAT

               LDY PRIORITY
               STY MAT

NOMAT          RTS


;****************************************
;*             Actually perform the
;*             materialiaztion sound
;****************************************
MATSONIX       LDA MATCOUNT

               CMP #13

               BEQ ENDMAT
               AND #1
               BNE OTHEREND
               LDA MATCOUNT
               ASL ; = ASL A

               CLC

               ADC V1HI,Y
BACKIN         STA $D401,Y

               STA V1HI,Y
               LDA #0
               STA $D404,Y

               LDA V1WV,Y
               STA $D404,Y

               INC MATCOUNT
               RTS

OTHEREND       ASL ; = ASL A

               STA MUS

               LDA V1HI,Y
               SEC

               SBC MUS

               JMP BACKIN
ENDMAT         LDA #0
               STA V1HI,Y
               STA $D404,Y

               LDY MAT

               STA VOC1,Y
               LDA #4
               STA MAT

               RTS



;****************************************

;*             When a bubble bursts, this

;*             happens
;****************************************

BURST          LDA #129
               STA V4WV
               LDA #2
               STA V4HI
               STA V4LO
               STA PRIORITY
               LDA #$0C
               STA V4SR
               LDA #$19
               STA V4AD
               LDA #0
               STA FILON

               LDA VOL

               AND #$F0
               ORA #15

               STA VOL

               LDA #$68
               STA FILHI

               LDA #$F0
               STA RESONANCE

               LDX #3
               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOBURST

               LDY PRIORITY
               LDA #0
               STA VOC1,Y
NOBURST        RTS


;****************************************
;*             Guess what, you got an extra life !!
;****************************************
EXTRAL         LDA XTRA
               CMP #4
               BEQ CONTEXTRA

               TAY                           ;effect already occuring so reset voice
               LDA #0
               STA VOC1,Y
CONTEXTRA      LDA #33+4

               STA V4WV
               STA V4HI
               STA V4LO
               LDA #$0C
               STA V4SR
               LDA #$1C
               STA V4AD
               LDA #2
               STA PRIORITY
               LDA #0
               STA FILON

               STA XTRACOUNT

               LDA VOL

               AND #$F0
               ORA #15

               STA VOL

               LDA #$F0
               STA RESONANCE

               LDX #5
               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOEXTRA

               LDY PRIORITY
               STY XTRA
NOEXTRA        RTS


;****************************************
;*             This services the extra sound
;****************************************
SEREXTRA       LDX XTRACOUNT
               CPX #4
               BEQ ENDEXTRA
               LDA XTRATAB,X

               STA $D401,Y

               STA V1HI,Y
               INC XTRACOUNT

               RTS

ENDEXTRA       LDA #0
               STA V1HI,Y
               STA $D404,Y

               LDY XTRA
               STA VOC1,Y
               LDA #4
               STA XTRA
               RTS


;****************************************
;*             You've just corked a bottle
;****************************************
CORKING        LDA #129
               STA V4WV
               LDA #$B6
               STA V4LO
               LDA #$00
               STA V4SR
               LDA #$84
               STA V4AD
               LDX #0
               STX FILON

               INX ;=1

               STX V4HI
               INX ;=2

               STX PRIORITY
               INX ;=3 ;{for WHICHVOICE subroutine}
               LDA VOL

               AND #$F0
               ORA #15

               STA VOL

               LDA #$68
               STA FILHI

               LDA #$F0
               STA RESONANCE

               JSR WHICHVOICE ;give effect a voice

               CPY #100
               BEQ NOCORKING

               LDY PRIORITY
               LDA #0
               STA VOC1,Y
NOCORKING      RTS


;****************************************
;*             Check each voice for end of
;*             note and if necesary, set up
;*             a new note for that voice.
;****************************************
DOMUSIC
;              RTS ;temp
               LDA COUNTER1
               BEQ NEXTV1
RETV2          DEC COUNTER1   ;voice1 dealt with
               LDA COUNTER1
               CMP #254
               BEQ STARTMUSIC ;restart the music

               LDA COUNTER2
               BEQ NEXTV2
RETV3          DEC COUNTER2   ;voice2 dealt with
               LDA COUNTER3
               BEQ NEXTV3     ;deal with voice 3

RETV4          DEC COUNTER3

               LDX #0
               RTS                           ;go back and wait
STARTMUSIC     JMP SETUPMUSIC ;reset music variables for begining of tune


;****************************************
;*             Get the values from the table
;*             and store them in the voices
;*             and increment offsets.
;****************************************
DONOTE         PHA

               LDY #0
               LDA (MUS),Y
               BNE CONTNOTE

               PLA
               JSR ADDONE
               JMP SETCOUNT

CONTNOTE       STA $D401,X

               JSR ADDONE
               LDA (MUS),Y
               STA $D400,X

               JSR ADDONE
               LDA #0
               STA $D404,X

               PLA

               STA $D404,X

SETCOUNT       LDA (MUS),Y
               PHA
               JSR ADDONE
               PLA

               RTS


;****************************************
;*             Add one to the offsets.
;****************************************
ADDONE         INC MUS
               BNE ADD1OK
               INC MUS+1
ADD1OK         RTS


;****************************************
;*             Set up note on voice 1,2 & 3
;****************************************
NEXTV1         LDA VTEMP1
               STA MUS
               LDA VTEMP1+1
               STA MUS+1

               LDA WAVE1

               LDX #0
               JSR DONOTE
               STA COUNTER1

               LDA MUS
               STA VTEMP1
               LDA MUS+1
               STA VTEMP1+1
               JMP RETV2

NEXTV2         LDA VTEMP2
               STA MUS
               LDA VTEMP2+1
               STA MUS+1

               LDA WAVE2

               LDX #7
               JSR DONOTE
               STA COUNTER2

               LDA MUS
               STA VTEMP2
               LDA MUS+1
               STA VTEMP2+1
               JMP RETV3

NEXTV3         LDA VTEMP3
               STA MUS
               LDA VTEMP3+1
               STA MUS+1

               LDA WAVE3

               LDX #14

               JSR DONOTE
               STA COUNTER3

               LDA MUS
               STA VTEMP3
               LDA MUS+1
               STA VTEMP3+1
               JMP RETV4


;****************************************
;*             Set up the music variables
;*             and voices.
;****************************************
SETUPMUSIC     LDA #15

               STA $D418

               LDA #$59
               STA $D405      ;adsr

               STA $D40C

               STA $D413

               JSR RESET_TWO

               STX $D417

               LDA #8
               STA $D403

               STA $D40A

               STA $D411

               LDA #15

               STA $D402

               STA $D409

               STA $D410

               LDA #$45
               STA WAVE1

               LDA #$47
               STA WAVE2

               LDA #33

               STA WAVE3

               LDA #<MUSIC1

               STA VTEMP1
               LDA #>MUSIC1

               STA VTEMP1+1
               LDA #<MUSIC2

               STA VTEMP2
               LDA #>MUSIC2

               STA VTEMP2+1
               LDA #<MUSIC3

               STA VTEMP3
               LDA #>MUSIC3

               STA VTEMP3+1
               LDX #255
               RTS


;****************************************
;*             Set up sound when setting up
;*             timer.
;****************************************
TIMESOUND      LDA #$0F
               STA $D418

               LDA #0
               STA $D404

               STA $D40B

               STA $D412

               STA $D415

               STA $D416

               STA $D417

               LDA #$DD
               STA $D405

               STA $D40C

               STA $D413

               LDA #$0C
               STA $D406

               STA $D40D

               STA $D414

               LDA TIME
               STA ZAPCOUNT
               CLC

               ADC #8
               STA ZAPDELAY
               STA $D401

               STA $D40F

               STA $D408

               LDA #0
               STA $D400

               STA $D403

               STA ZAPCOUNT
               LDA #4
               STA $D407
               STA $D40A

               LDA #8
               STA $D411
               STA $D40E

               LDA #65+4
               STA $D404
               STA $D40B
               STA $D412

               RTS


;****************************************
;*             Actually perform the timer
;*             increment sound.
;****************************************
ZAP            INC ZAPCOUNT
               INC ZAPDELAY
               LDA ZAPDELAY
               STA $D401
               AND #$03
               ASL ; = ASL A

               ASL ; = ASL A

               ORA #$07
               STA $D403

               STA $D40A

               STA $D411

               LDA ZAPCOUNT
               STA $D400

               CLC

               ADC #4
               STA $D407

               STA $D408

               CLC

               ADC #4
               STA $D40E

               STA $D40F

               RTS


;****************************************
;*             Set up the blob explosion sound
;****************************************
DEATHSONIX     LDA #$0F
               STA $D418

               LDA #0
               STA COUNT

               STA $D404
               STA $D40B
               STA $D412
               STA $D415
               STA $D416
               STA $D417

               LDA #$2C       ;adsr
               STA $D405
               STA $D40C
               STA $D413

               LDA #$0E
               STA $D406
               STA $D40D
               STA $D414

               LDA #0
               STA $D400
               STA $D403

               LDX #2
               STX $D401

               INX
               STX $D40F

               INX
               STX $D408

               LDA #4
               STA $D407
               STA $D40A

               LDA #8
               STA $D411
               STA $D40E

               LDA #$47
               STA $D412

               LDA #$81
               STA $D40B

               STA $D404
               RTS


;****************************************
;*             Perform death sound
;****************************************
WOOSH          INC LOPCNT
               LDA LOPCNT
               AND #1
               BEQ VOLON

               LDA #3
VOLON          STA $D417
               RTS


;****************************************
;*             Decrement the timer by one and
;*             add 111 to score each time.
;****************************************
DECTIMER       LDA #1
               STA UNITS
               STA UNITS+1
               STA UNITS+2
               STA UNITS+3
               JSR ADDSCORE   ;add 1111 to score

               JSR SETUPTUNE  ;set up end of stage tune

WAITABIT       LDA $D012

               CMP #1
               BNE WAITABIT
               JSR DOMUSIC

               CPX #255

               BNE WAITABIT

               JSR TIMESOUND

DECTIMER2      LDA TIME
               BNE GODTIMER
               RTS

GODTIMER       JSR ZAPTWO     ;do sound
               JSR GODEC1A    ;increment timer by one

               DEC TIME
               LDA #1
               STA UNITS
               STA UNITS+1
               JSR ADDSCORE   ;add 11 to score

               JSR DELAY

               LDA #FLIPIN
               STA $01

WAIT           LDA $D012

               CMP #1
               BEQ DECTIMER2

               JMP WAIT


;****************************************
;*             Actually perform the timer
;*             decrement sound.
;****************************************
ZAPTWO         DEC ZAPCOUNT
               DEC ZAPDELAY
               LDA ZAPDELAY
               STA $D401
               AND #$02
               ASL ; = ASL A

               ASL ; = ASL A

               ORA #$07
               STA $D403

               STA $D40A

               STA $D411

               LDA ZAPCOUNT
               STA $D400

               CLC

               ADC #4
               STA $D407

               STA $D408

               CLC

               ADC #4
               STA $D40E

               STA $D40F

               RTS


;****************************************
;*             Set up tune for end of stage.
;****************************************
SETUPTUNE      LDA #15
               STA $D418

               LDA #$39
               STA $D405      ;adsr

               STA $D40C

               STA $D413

               LDA #$75
               STA $D406

               STA $D40D

               STA $D414

               LDA #7
               STA $D403

               STA $D40A

               STA $D411

               LDA #150
               STA $D402

               STA $D409

               STA $D410

               JSR RESET_COU

               STX $D417

               LDA #65

               STA WAVE1

               STA WAVE2

               STA WAVE3

               LDA #<TUNE1
               STA VTEMP1
               LDA #>TUNE1
               STA VTEMP1+1
               LDA #<TUNE2
               STA VTEMP2
               LDA #>TUNE2
               STA VTEMP2+1
               LDA #<TUNE3
               STA VTEMP3
               LDA #>TUNE3
               STA VTEMP3+1
               RTS


;****************************************
;*             Set up the music which is
;*             played when the game starts
;****************************************
SETUPSONG      LDA #15
               STA $D418

               LDA #$9A
               STA $D405      ;adsr

               STA $D40C

               STA $D413

               LDA #8
               STA $D403

               STA $D40A

               STA $D411

               LDA #150
               STA $D402

               STA $D409

               STA $D410

               JSR RESET_TWO

               LDA #65

               STA WAVE1

               LDA #65

               STA WAVE2

               LDA #33

               STA WAVE3

               LDA #<SONG1
               STA VTEMP1
               LDA #>SONG1
               STA VTEMP1+1
               LDA #<SONG2
               STA VTEMP2
               LDA #>SONG2
               STA VTEMP2+1
               LDA #<SONG3
               STA VTEMP3
               LDA #>SONG3
               STA VTEMP3+1
               RTS


;****************************************
;*             Set up the music which is
;*             played when the game is over
;****************************************
SETUPMELODY    LDA #15+16
               STA $D418

               LDA #$89
               STA $D405      ;adsr

               LDA #$0A
               STA $D40C

               STA $D413

               LDA #8
               STA $D403

               STA $D40A

               STA $D411

               LDA #150
               STA $D402

               STA $D409

               STA $D410

               JSR RESET_TWO

               LDA #$86
               STA $D417

               LDA #$A5
               STA $D415

               STA $D416

               LDA #65

               STA WAVE1

               STA WAVE2

               STA WAVE3

               LDA #<MELODY1
               STA VTEMP1
               LDA #>MELODY1
               STA VTEMP1+1
               LDA #<MELODY2
               STA VTEMP2
               LDA #>MELODY2
               STA VTEMP2+1
               LDA #<MELODY3
               STA VTEMP3
               LDA #>MELODY3
               STA VTEMP3+1
               RTS


RESET_TWO      LDX #0

               LDA #$06
               STA $D406

               STA $D40D

               STA $D414

RESET_COU      LDA #$00

               STA $D415

               STA $D416

               STA $D404

               STA $D40B

               STA $D412

               STA COUNTER1
               STA COUNTER2
               STA COUNTER3
               RTS


;****************************************
;*             Play the start of game music
;****************************************
GAMEON         JSR SETUPSONG  ;set up game on tune
WAITABITMORE   LDA $D012      ;wait for raster
               CMP #1
               BNE WAITABITMORE
               JSR DOMUSIC    ;play the music
               CPX #255
               BNE WAITABITMORE
               RTS


;****************************************
;*             Play music for finish or death
;****************************************
GAMEOFF        JSR SETUPMELODY               ;set up game over music
JUSTAJIFFY     LDA $D012      ;wait for raster
               CMP #1
               BNE JUSTAJIFFY
               JSR DOMUSIC    ;play the music

               CPX #255
               BNE JUSTAJIFFY

               JMP CHEATS_OFF ; = JSR + RTS


;****************************************
;*             Clear the two lines of defineable
;*             keys.
;****************************************
CLEARUSER      LDY #35
               LDA #32

CLEARU         STA $D000+1386,Y              ;store ascii 32 on both lines
               STA $D000+1426,Y
               DEY

               BPL CLEARU
               RTS


;*****************************************
;*             User defineable keys routine
;****************************************
USERKEYS       INC COUNT

               LDA #FLIPOUT
               STA $01

               LDA CHOICE
               ORA CHOICE+1
               AND #2
               CMP #2
               BNE CLEARUSER

PRINTUP        LDY #35
PRINTUSER      LDA KUSER,Y    ;print up the key definitions on screen
               STA $D000+1386,Y
               LDA KUSER2,Y
               STA $D000+1426,Y
               DEY

               BPL PRINTUSER

               LDA #FLIPIN
               STA $01

               LDY CURPOSB

               LDA COUNT
               AND #2
               BEQ CUROFF
CURON                                        ;flash cursor on user key
               LDA #1
               JMP CONTKEYS
CUROFF         LDA #6
CONTKEYS       STA $D400+1424,Y
               LDA AMOUNT     ;is it time for key check
               BMI THE_REST
               DEC AMOUNT
               RTS

THE_REST       LDY #63        ;check keyboard for cursor keys
               JSR KEYS
               BEQ STAYPUT

               LDA #8
               STA AMOUNT
               LDA #6
               LDY CURPOSB

               STA $D400+1424,Y              ;set flashing cursor to blue

               LDY #61 ; RIGHT SHIFT
               JSR KEYS       ;check for shift keys
               BNE USERLEFT
               LDY #36 ; LEFT SHIFT
               JSR KEYS
               BNE USERLEFT
USERIGHT       INC CURPOSA    ;move the cursor right
               LDY CURPOSA    ;by incrementing the cursor position
               CPY #6
               BNE NORESETCURL

               LDY #0
               STY CURPOSA

               JMP NORESETCURL

USERLEFT       DEC CURPOSA    ;move the cursor left

               LDY CURPOSA    ;by decrementing the cursor position
               CPY #255
               BNE NORESETCURL

               LDY #5
               STY CURPOSA

NORESETCURL
               LDA CURTAB,Y
               STA CURPOSB    ;store the new cursor position
STAYPUT        RTS


;****************************************
;*             Check the keyboard to see if
;*             a key has been pressed and if
;*             it has, is it legal.
;*             If it is, the key num is
;*             inside A else A holds 255
;****************************************
BOARDCHK       LDY #7
JANE           LDA SARAH,Y

               STA $DC00

               LDA $DC01
               CMP #255
               BNE CHECKEY    ;read keys from keyboard.

               DEY
               BPL JANE
               RTS

CHECKEY                                      ;one has been pressed.
               LDX #255
               STA V4HI
DEBRA          INX
               LDA V4HI
               EOR LASTKEYTAB,X              ;work out which bit pattern it is
               AND LASTKEYTAB,X
               BEQ DEBRA

               TYA
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A

               STX V4LO
               CLC

               ADC V4LO       ;add number of bit pat (1 to 7) to column
               TAY

               LDA KATE,Y     ;get corresponding key from table
               RTS


;****************************************
;*             Actually work out which key
;*             is pressed and stick it in
;*             the appropriate user definition.
;****************************************
DOUSERKEY      LDA #FLIPIN
               STA $01

               JSR BOARDCHK   ;check board to see if key is hit
               CMP #255
               BEQ NOKEYHIT
               LDY CURPOSA    ;one has, so store key value at key variable
               STA KLEFT,Y

               LDY CURPOSB

               TAX

               LDA LYNDA,X

               STA KUSER2-2,Y ;and store key on screen

NOKEYHIT       RTS


;****************************************
;*             Move work area two chars to
;*             the right & clear left column
;****************************************
SHIFTRIGHT     LDX #15

HR1            LDA UPDOWNL,X
               STA LOADE+1
               CLC
               ADC #2
               STA STOREE+1
               LDA UPDOWNH,X
               STA LOADE+2
               ADC #0
               STA STOREE+2
               LDY #$BE
LOADE          LDA WA,Y ;smc
STOREE         STA WA,Y ;smc

               DEY

               BNE LOADE

               DEX

               BPL HR1

               LDA #<WA
               STA PTR

               LDA #>WA
               STA PTR+1

               LDX #WA_HEIGHT_PX ;pixels high of work area
CLRRGTL2       LDY #0
               TYA
               STA (PTR),Y
               INY
               STA (PTR),Y

               LDA PTR
               CLC
               ADC #$18
               STA PTR
               BCC CLRRGTL
               INC PTR+1
CLRRGTL
               DEX
               BNE CLRRGTL2
               RTS


;****************************************
;*             Move work area two chars to
;*             the left & clear right column
;****************************************
SHIFTLEFT      LDX #15
HL1            LDA UPDOWNL,X
               STA STOREG+1
               CLC

               ADC #2
               STA LOADG+1

               LDA UPDOWNH,X

               STA STOREG+2
               ADC #0
               STA LOADG+2

               LDY #1
LOADG          LDA WA,Y ;smc
STOREG         STA WA,Y ;smc

               INY

               CPY #191
               BCC LOADG

               DEX

               BPL HL1

               LDA #<WA
               STA PTR

               LDA #>WA
               STA PTR+1

               LDX #WA_HEIGHT_PX ;pixels high of work area
               LDY #$16
CLRLFTL2       LDA #0
               STA (PTR),Y
               INY
               STA (PTR),Y
               DEY

               LDA PTR
               CLC
               ADC #$18
               STA PTR
               BCC CLRLFTL
               INC PTR+1
CLRLFTL
               DEX
               BNE CLRLFTL2
               RTS


;****************************************
;*             Move the work area down one
;*             char & clear top char line
;****************************************
SHIFTDOWN      LDX #14
SHDNL          LDA UPDOWNL,X
               STA LOADDM+1
               LDA UPDOWNH,X
               STA LOADDM+2

               LDA UPDOWNL+1,X
               STA STOREDM+1
               LDA UPDOWNH+1,X
               STA STOREDM+2

               LDY #$C0
LOADDM         LDA WA-1,Y ;smc

STOREDM        STA WA-1,Y ;smc
               DEY

               BNE LOADDM
               DEX

               BPL SHDNL

               TYA

               LDY #$C0
CLRDLP         STA WA-1,Y
               DEY

               BNE CLRDLP
               RTS


;*****************************************
;*             Move the work area up one
;*             line & clear bottom char line
;*****************************************
SHIFTUP        LDX #0

SHUPL          LDA UPDOWNL+1,X
               STA LOADUM+1
               LDA UPDOWNH+1,X

               STA LOADUM+2
               LDA UPDOWNL,X

               STA STOREUM+1

               LDA UPDOWNH,X

               STA STOREUM+2

               LDY #$C0
LOADUM         LDA WA-1,Y ;smc

STOREUM        STA WA-1,Y ;smc
               DEY

               BNE LOADUM
               INX

               CPX #15

               BCC SHUPL

               TYA

               LDY #$C0
CLRULP         STA WA+(15*$C0)-1,Y

               DEY

               BNE CLRULP
               RTS

;INSERT


;
; File X3.BUB
; Starts Here
;
;

;
;
;  PDS Pc1.10 :000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          01.52
;         By  "JAKE AND MATTHEW..."
;
;     Created on Fri the 04th of Mar 1988
;        Last update 08:41 on 10/05/88
;

;              ORG INSERT

;****************************************
;*             Set up the SPECAL table with
;*             the objects that are currently
;*             on screen.
;****************************************
EXIT_CELL      RTS
SET_SPECAL     JSR CLEAR_IX
               LDA #0
               STA SPECAL ;number of "SPECAL" objects
SET_CELL       LDA SCREENX
               CLC
               ADC #$C0
               STA SCREENX_II
               LDA SCREENX+1
               ADC #0
               STA SCREENX_II+1

               LDA SCREENY    ;work out screen minimum and maximum y values
               CLC                           ;by adding + subtracting $40
               ADC #$40
               STA Y_MAX
               LDA SCREENY+1
               ADC #0
               STA Y_MAX+1

               LDA SCREENY
               SEC
               SBC #$40
               STA Y_MIN
               LDA SCREENY+1
               SBC #0
               STA Y_MIN+1

               LDA SCREENY+1  ;work out which cell(s) you are over
               SEC
               SBC #$7C
               ASL ; = ASL A
               TAY

               LDA SCREENY
               AND #128
               BEQ DONT_INC1

               INY

DONT_INC1      TYA
               ASL ; = ASL A
               TAY

               LDA YET_ANOTH,Y
               STA CELL_ST
               LDA YET_ANOTH+1,Y
               STA CELL_ST+1

               ;STA LOOP_FIVE
DONT_INC       LDA #255
               STA FIVE_COUNT

LOOP_AGAIN     INC FIVE_COUNT
               LDY FIVE_COUNT
               LDA (CELL_ST),Y

               BMI EXIT_CELL ;Branch if reached last cell (dummy value=$ff)
               TAY

               LDA CELL_X2_TAB,Y             ;Check cell not to left or right of screen...
               CLC
               ADC #2
               CMP SCREENX+1
               BCC LOOP_AGAIN

               SEC
               SBC #3
               CMP SCREENX_II+1
               BCS LOOP_AGAIN

               ;
               ; Current cell is on screen
               ;
               LDA CELL_X_TAB,Y
               STA CURCELLX
               LDA CELL_Y_TAB,Y
               STA CURCELLY
               TYA
               ASL ; = ASL A
               TAY

MAP_ADD1       LDA DEF_MAP_1,Y ; smc
               STA CELL_OFF
               INY

MAP_ADD2       LDA DEF_MAP_1,Y
               STA CELL_OFF+1

               LDY #0
               LDA (CELL_OFF),Y
               AND #$3F ; Number of shapes in this cell
               BEQ LOOP_AGAIN
               STA COUNT_DOWN

               ; Read the flags to say which shapes in this cell can be gone
               ; behind.
               TAX

               INY ; = 1
               LDA (CELL_OFF),Y
               STA ROTATE

               INY ; = 2
               LDA (CELL_OFF),Y
               STA ROTATE2

               CPX #$10
               BCC NOT_10

               INY
               LDA (CELL_OFF),Y
               STA ENDY
NOT_10         INY

RE_ENT         LDX #0
               ASL ENDY
               ROL ROTATE2

               ROL ROTATE
               BCC CNGOBHD

               INX

CNGOBHD        STX COUNTER ;flag...0:moving shapes cant go behind this object

               INY

               ; Read shape number (ie type)
               LDA #$FF
               STA SHAPE_NO
               LDA (CELL_OFF),Y
               DEY

               AND #%10001111
               CMP #%10001111
               BNE NO_SET_LESS_ONE

               ; It's $1F, a moving background object
               ; Read what type it is
               INY
               INY
               LDA (CELL_OFF),Y
               AND #$0F
               STA SHAPE_NO
               DEY
               DEY

NO_SET_LESS_ONE
               LDA (CELL_OFF),Y              ;work out the x coor of the object

               AND #%11110000 ;and store it in xtempor then add its
               STA XTEMPOR    ;x dimensions to the x and store it in

               INY                           ;xtempor+1
               INY
               LDA SHAPE_NO
               BMI ITSABLOCK

               LDA #$08
               CLC

               JMP ITSASPRITE
ITSABLOCK      LDA (CELL_OFF),Y

               AND #%00000111
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A

ITSASPRITE     ADC #$10
               ADC XTEMPOR
               STA XTEMPOR+1

               LDA #0
               ADC #0
               STA XTEMPOR+2

               DEY                           ;same again for the y except store in
               DEY                           ;in ytempor(+1)
               LDA (CELL_OFF),Y
               AND #%00001111
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               STA YTEMPOR

               INY
               INY

               LDA SHAPE_NO
               BMI ITSABLOK2

               LDA #$08
               CLC

               JMP ITSASPRIT2
ITSABLOK2      LDA (CELL_OFF),Y

               AND #%00111000
               ASL ; = ASL A

ITSASPRIT2     ADC #$10
               ADC YTEMPOR
               STA YTEMPOR+1

               LDA #0
               ADC #0
               STA YTEMPOR+2

               DEY                           ;same again for the z except store in
                                             ;in ztempor(+1)

               LDA (CELL_OFF),Y
               AND #%01110000
               STA ZTEMPOR

               LDA SHAPE_NO
               BMI ITSABLOK3

               INY

               LDA (CELL_OFF),Y
               DEY

               AND #%01000000
               ASL ; = ASL A

               CLC

               ADC ZTEMPOR

               STA ZTEMPOR

               LDA SHAPE_NO
               BNE NOT_LIFT
               LDA #$70
               JMP ROUND_NOT

NOT_LIFT       LDA #$8

ROUND_NOT      INY
               JMP ITSASPRIT3
ITSABLOK3      LDA (CELL_OFF),Y              ;Correct the z coor so that if it is 0
               AND #%10001111 ;then set it to $10 else leave it alone.

               CMP #6
               BEQ RE_DO_Z

               CMP #2
               BNE NO_RE_DO_Z
RE_DO_Z        LDA ZTEMPOR
               STA ZTEMPOR+1

               INY

               JMP CONT_CELL

NO_RE_DO_Z     INY

               LDA (CELL_OFF),Y
               AND #%11000000
               LSR ; = LSR A

               LSR ; = LSR A

ITSASPRIT3     CLC
               ADC #$10
               CLC
               ADC ZTEMPOR
               STA ZTEMPOR+1

CHECK_UP_DO
CONT_CELL      JSR CONVERT ;get 2d x/y edges of shape

;****************************************
;*
;*             The compare routine...
;*
;*             "IT'S TORTURE
;*              BUT I'M ALMOST THERE"
;*
;****************************************

               LDA SCREENX
               CMP XL2
               LDA SCREENX+1
               SBC XL2+1
               BCS LESS_ONE ;branch if shape is off left of screen

               LDA XL1
               CMP SCREENX_II
               LDA XL1+1
               SBC SCREENX_II+1
               BCS LESS_ONE ;branch if shape is off right of screen

               JSR CHECK_Y1
LESS_ONE       INY

               DEC COUNT_DOWN ;# of shapes left in this cell to check
               BEQ LOOPAGAIN1
               JMP RE_ENT
LOOPAGAIN1     JMP LOOP_AGAIN


;****************************************
;*             Check the y coordinate of the
;*             shape to see if it is within
;*             the screen. If so then fall
;*             thru to ITS_IN and return.
;****************************************
CHECK_Y1       LDA YL1
               CMP Y_MAX
               LDA YL1+1
               SBC Y_MAX+1
               BCS NOT_IN ;branch if shape is off bot of screen

               LDA Y_MIN
               CMP YL2
               LDA Y_MIN+1
               SBC YL2+1
               BCS NOT_IN ;branch if shape if off top of screen

;*********************************
;*             Put current shape
;*             into specal table
;*********************************
ITS_IN         LDA SHAPE_NO
               BMI YES_SO_FAR
               JMP ITS_IX
YES_SO_FAR     LDA SPECAL
               CMP #MAXSPEC ;max # of SPECAL entries
               BCC IT_IS

               INC 53280 ; $D020 ; overflow!!!...flash border
NOT_IN         RTS

IT_IS          TAX
               LDA SPECTL,X
               STA SPEC_OFF
               LDA SPECTH,X
               STA SPEC_OFF+1
               TYA

               TAX

               LDY #0
               LDA COUNTER ;can shape be gone behind ?

               STA (SPEC_OFF),Y
               LDY #2
               LDA CURCELLX
               STA (SPEC_OFF),Y ;x-co hi

               LDA XTEMPOR
               INY ;=3
               STA (SPEC_OFF),Y ;x-co lo

               INY ;=4
               LDA CURCELLY
               STA (SPEC_OFF),Y ;y-co hi

               LDA YTEMPOR
               INY ;=5
               STA (SPEC_OFF),Y ;y-co lo

               INY ;=6
               LDA ZTEMPOR
               STA (SPEC_OFF),Y ;z-co (lo)

               TXA
               TAY
               DEY

               LDA (CELL_OFF),Y
               AND #%10001111
               BPL NO_HI_SET

               AND #$1F
               ORA #$10
NO_HI_SET      LDY #$0A

               STA (SPEC_OFF),Y ;shape number
               TXA

               TAY

               DEY

               STY SOME_WHERE
               LDA (CELL_OFF),Y
               PHA

               INY

               LDA (CELL_OFF),Y
               PHA

               AND #%00000111 ;x dimension
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A

               CLC
               ADC #$10
               LDY #$0B
               STA (SPEC_OFF),Y ;x-dim

               INY ;=$C
               PLA

               PHA

               AND #%00111000 ;y dimension

               ASL ; = ASL A

               ;clc
               ADC #$10
               STA (SPEC_OFF),Y ;y-dim

               INY ;=$D
               PLA

               AND #%11000000 ;z dimension

               LSR ; = LSR A

               LSR ; = LSR A

               ;clc
               ADC #$10
               STA (SPEC_OFF),Y ;z-dim

               PLA

               AND #%10001111
               CMP #2
               BEQ Z_NOT_RITE
               CMP #6
               BNE Z_RITE
Z_NOT_RITE     LDA #0
               STA (SPEC_OFF),Y ;z-dim

Z_RITE         LDY #$14
               CLC

               LDA XTEMPOR+2  ;set up x,y and z + dim into specal tab

               ADC CURCELLX
               STA (SPEC_OFF),Y ;[x-co + x-dim] hi

               INY ;=$15

               LDA XTEMPOR+1

               STA (SPEC_OFF),Y ;[x-co + x-dim] lo

               INY ;=$16

               CLC

               LDA YTEMPOR+2
               ADC CURCELLY
               STA (SPEC_OFF),Y ;[y-co + y-dim] hi

               INY ;=$17

               LDA YTEMPOR+1
               STA (SPEC_OFF),Y ;[y-co + y-dim] lo

               LDY #$19
               LDA ZTEMPOR+1

               STA (SPEC_OFF),Y ;[z-co + z-dim] (lo)

               TXA
               PHA

               LDY #$0A       ;check to see if object is bottle.
               LDA (SPEC_OFF),Y              ;if so then put its memory origin into
               CMP #$11       ;BOTT in same place as in specal table
               BNE NOT_BOT

               LDA SPECAL
               ASL ; = ASL A
               TAX

               DEC SOME_WHERE
               CLC
               LDA SOME_WHERE
               ADC CELL_OFF
               STA BOTT,X
               LDA CELL_OFF+1
               ADC #0
               STA BOTT+1,X

NOT_BOT        PLA
               TAY

               INC SPECAL
               RTS


;****************************************
;*             Put object/sprite into IX table
;****************************************
ITS_IX         TYA
               PHA

               LDA SHAPE_NO
               BEQ IS_LIFT

               LDA #<STATOB
               STA SPEC_OFF
               LDA #>STATOB
               STA SPEC_OFF+1
               LDY #5
LOOP_YES       LDX #0
               LDA (SPEC_OFF,X) ;shape
               BEQ BACK_IN

               DEY
               BEQ NO_FREE

               CLC
               LDA SPEC_OFF
               ADC #IXSIZ
               STA SPEC_OFF
               BCC LOOP_YES
               INC SPEC_OFF+1
               JMP LOOP_YES

IS_LIFT        LDA #<BACKS ;lift1
               STA SPEC_OFF
               LDA #>BACKS
               STA SPEC_OFF+1

               LDA BACKS ;shape
               BEQ BACK_IN

               LDA #<BACKS2 ;lift2
               STA SPEC_OFF
               LDA #>BACKS2
               STA SPEC_OFF+1

BACK_IN        LDX SHAPE_NO
               LDA SHPTBS,X
               LDY #0
               STA (SPEC_OFF),Y ;shape
               INY ;=1
               TXA ;lda SHAPE_NO
               BEQ NODEF

               LDA SHPTBD,X
               STA (SPEC_OFF),Y ;data offset

NODEF          INY ;=2
               LDA CURCELLX
               STA (SPEC_OFF),Y ;x-co hi

               LDA XTEMPOR
               INY ;=3
               STA (SPEC_OFF),Y ;x-co lo

               INY ;=4
               LDA CURCELLY
               STA (SPEC_OFF),Y ;y-co hi

               LDA YTEMPOR
               INY ;=5
               STA (SPEC_OFF),Y ;y-co lo

               INY ;=6
               TXA ;lda SHAPE_NO
               BEQ NO_ZEED

               LDA ZTEMPOR
               STA (SPEC_OFF),Y ;z-co (lo)

GO_BACK        LDA SHPTBX,X
               LDY #$B
               STA (SPEC_OFF),Y ;x-dim

               LDA SHPTBY,X
               INY ;=$C
               STA (SPEC_OFF),Y ;y-dim

               LDA SHPTBZ,X
               INY ;=$D
               STA (SPEC_OFF),Y ;z-dim

               LDA SPEC_OFF
               STA IX
               LDA SPEC_OFF+1
               STA IX+1
               JSR DIMS

NO_FREE        PLA
               TAY

               RTS


NO_ZEED        LDA LOPCNT

               AND #$7F
               STA (SPEC_OFF),Y ;z-co (lo)

               JMP GO_BACK


;***************************************
;*             Convert 3d specal coors
;*             into screen x and y co-or
;*             XL1 & YL1 bottom left of block
;*             XR2 & YR2 top right of block
;****************************************
CONVERT        PHA
               TYA
               PHA
               TXA
               PHA

               LDY FIVE_COUNT
               LDA (CELL_ST),Y
               TAY

               LDA #0         ;set up cell x and y base address from cell no
               STA CELL_ABSX  ;using tables
               LDA CELL_X2_TAB,Y
               STA CELL_ABSX+1

               LDA CELL_Y2_TLO,Y
               STA CELL_ABSY

               LDA CELL_Y2_THI,Y
               STA CELL_ABSY+1

               ;work out XL1 by adding cell coor to lo x & y

               LDA XTEMPOR
               CLC
               ADC YTEMPOR
               STA XL1

               LDA CELL_ABSX+1
               ADC #0
               STA XL1+1

               ;work out XL2 by adding cell coor to hi x & y

               LDA XTEMPOR+1
               CLC
               ADC YTEMPOR+1
               STA XL2
               LDA XTEMPOR+2
               ADC YTEMPOR+2
               CLC
               ADC CELL_ABSX+1
               STA XL2+1

               SEC                           ;work out yl1 (Y pos of top of shape)
               LDA CELL_ABSY
               SBC ZTEMPOR+1
               STA YL1
               LDA CELL_ABSY+1
               SBC #0
               STA YL1+1

               LDA YTEMPOR+2 ;hi
               LSR ; = LSR A ;/2
               STA ZPA+1 ;temp
               LDA YTEMPOR+1 ;lo
               ROR ; = ROR A
               STA ZPA ;temp

               SEC
               LDA YL1
               SBC ZPA
               STA YL1

               LDA YL1+1
               SBC ZPA+1
               STA YL1+1

               LDA XTEMPOR
               LSR ; = LSR A
               CLC
               ADC YL1
               STA YL1

               LDA YL1+1
               ADC #0
               STA YL1+1

               SEC                           ;work out yl2 (Y pos of bot of shape)
               LDA CELL_ABSY
               SBC ZTEMPOR
               STA YL2
               LDA CELL_ABSY+1
               SBC #0
               STA YL2+1

               LDA YTEMPOR
               LSR ; = LSR A
               STA ZPA ;temp

               LDA YL2
               SEC
               SBC ZPA
               STA YL2

               LDA YL2+1
               SBC #0
               STA YL2+1

               LDA XTEMPOR+2
               LSR ; = LSR A ;/2
               STA ZPA+1 ;temp

               LDA XTEMPOR+1
               ROR ; = ROR A
               CLC
               ADC YL2
               STA YL2
               LDA YL2+1
               ADC ZPA+1
               STA YL2+1

               PLA
               TAX
               PLA
               TAY
               PLA

               RTS


;****************************************
;*             Scroll the screen down at the
;*             start of the wave
;****************************************
SCRL_START     DEC YPIX
               DEC YPIX
               BPL ANOTUP
               PHA

               JSR SHIFTDOWN

               LDA #<WA
               STA INITSTRDAT
               LDA #>WA
               STA INITSTRDAT+1

               LDA SCREENY
               SEC
               SBC #8
               STA SCREENY

               BCS ALZ474
               DEC SCREENY+1
               SEC
ALZ474         SBC #$40
               STA STRIPY

               LDA SCREENY+1
               SBC #0
               STA STRIPY+1

               LDA SCREENX
               STA STRIPX
               CLC
               ADC #$C0
               STA ZPD

               LDA SCREENX+1
               STA STRIPX+1
               ADC #0
               STA ZPD+1

               JSR SET_SPECAL
               JSR PRHSTRIP
               LDA #3
ANOTUP1        STA YPIX

               PLA

ANOTUP         RTS


;****************************************
;*             Update all defintions
;****************************************
WOBBLE         INC LOPCNT
               LDA IXTAB+1 ;blob data offset

               AND #6
               BNE NO_BLOB

               LDA LOPCNT
               AND #1
               STA IXTAB+1 ;blob data offset

NO_BLOB        LDA LOPCNT

               AND #1
               STA BACKS+1 ;lift#1 data offset              ;rotate lift definition

               STA BACKS2+1 ;lift#2 data offset
               LDA LOPCNT
               AND #$7F
               STA BACKS+6 ;lift#1 z-co (lo)

               STA BACKS2+6 ;lift#2 z-co (lo)
               ;animate materialisation pads

               LDX #0
CKAGN          LDA STATOB,X ;shape
               CMP #$0C ;materialisation pad

               BNE RETU_PAD
               INC STATOB+1,X ;data offset

               LDA STATOB+1,X ;data offset

               CMP #3
               BCC RETU_PAD
               LDA #0
               STA STATOB+1,X ;data offset

RETU_PAD       TXA

               CLC

               ADC #IXSIZ
               TAX

               CPX #(IXSIZ*4)
               BCC CKAGN



               LDA LOPCNT
               AND #3
               STA TEMP_IX

               LDA SFIRE

               CMP #$0E
               BNE SFIRE_TO
               LDA SFIRE+1

               AND $FF-3

               ORA TEMP_IX

               STA SFIRE+1

SFIRE_TO       LDA SFIRE+IXSIZ

               CMP #$0E
               BNE SFIRE_THR

               LDA SFIRE+IXSIZ+1

               AND #$FF-3
               ORA TEMP_IX

               STA SFIRE+IXSIZ+1

SFIRE_THR      JMP ALIEN_ANI


;****************************************
;*             Clear all sprite defintions
;*             for background objects in the IX block
;****************************************
CLEAR_IX       LDA #0
               STA BACKS ;lift1
               STA BACKS2 ;lift2

               STA STATOB ;shape
               STA STATOB+IXSIZ ;shape
               STA STATOB+[IXSIZ*2] ;shape
               STA STATOB+[IXSIZ*3] ;shape

               LDY #$12*2     ;clear bottle table
               ;lda #0
CLEAR_BOTT     STA BOTT,Y
               DEY
               BPL CLEAR_BOTT
               RTS


;****************************************
;*             Rezz the bubble up on screen
;****************************************
REZZ           LDA LOPCNT ;need to save LOPCNT to preserve lift position
               PHA

               JSR MATERIAL
               LDA #$10
               STA LOPCNT ;clears bits 0&1

RZBB           STA YELLO

               ORA #$03
               STA IXTAB+$09 ;chars across shape/dissolve

RZBS           JSR SAVPOS
               JSR ALOCAT
               JSR MOVSHPRT
               JSR WKTOBUF

               JSR PUTPOS
               JSR SONIX

               JSR DELA
               INC LOPCNT
               LDA LOPCNT
               AND #3
               BNE RZBS

               LDA YELLO

               ASL ; = ASL A

               BCC RZBB
               LDA #$03 ;turn off dissolve on blob

               STA IXTAB+$09 ;chars across shape/dissolve
               PLA

               STA LOPCNT ;restore LOPCNT
               RTS


;****************************************
;*             Work out if blob (or whatever)
;*             is on top of an object and if
;*             so then dont decrement z
;****************************************
DROP           LDA #$FF ;dummy value
               LDY #$12
PAY_ME         STA ENT_TAB-1,Y
               DEY
               BNE PAY_ME

               LDY #$11
               STA (IX),Y ;object height above shadow. $FF = no shadow.

               LDA #128
               DEY ;=$10
               STA (IX),Y ;bit 7 set if resting on an object

               ;set up blob x,y & z coor + dims into IXTB
               LDY #$B
               LDA (IX),Y ;x-dim

               LDY #$3
               CLC
               ADC (IX),Y ;x-co lo
               STA IXTB+1

               DEY ;=$2
               LDA (IX),Y ;x-co hi
               ADC #0
               STA IXTB

               LDY #$C
               LDA (IX),Y ;y-dim

               LDY #$5
               CLC
               ADC (IX),Y ;y-co lo
               STA IXTB+3

               DEY ;=$4
               LDA (IX),Y ;y-co hi
               ADC #0
               STA IXTB+2

               LDY #$0D
               LDA (IX),Y ;z-dim
               LDY #$6
               CLC
               ADC (IX),Y ;z-co (lo)
               STA IXTB+5

               LDA SPECAL ; Number of background objects in SPECAL table
               STA SPECTRUM
               LDY #$FF
               STY OBJ_REST
               INY ;=0

               STY ERE

               STY TABLE_ENT

DROP1          LDY SPECTRUM   ;work out which block to check
               LDA SPECTL-1,Y
               STA SPEC_OFF
               LDA SPECTH-1,Y
               STA SPEC_OFF+1

               LDY #$19
               LDA (SPEC_OFF),Y ;SPECAL [z-co + z-dim] (lo)
               DEY ;=$18
               STA (SPEC_OFF),Y ;top of SPECAL object for current IX shape

               LDY #$A
               LDA (SPEC_OFF),Y ;shape number
               CMP #6
               BNE HERO
KANSAS         JMP BACK_HERE ;ignore shadows (shape #6)

HERO           CMP #$14
               BCC OKIDOKI

               CMP #$15
               BNE KANSAS ;can't sit on side trap doors (shapes # $14,$16)

OKIDOKI
               AND #$1F ;; 2009-09: MY: Added this to make sure we can't run off the end of SLOP_TAB
               TAY
               LDA SLOP_TAB,Y
               BMI NOTSLOPE ;branch if not a slope

               PHA

               AND #$20 ; This bit set means sloping up with increasing x/y
               BNE MRIGHTTO

               ; Sloping down with increasing x/y
               PLA
               PHA

               LDY #3 ;check x-co
               AND #$40 ; 0=slope in x direction; 1=slope in y direction
               BEQ MGOBAX

               ;find how far along slope IX shape is...
               LDY #5 ;check y-co
MGOBAX         LDA (IX),Y ;IX y-co lo / x-co lo
               SEC

               SBC (SPEC_OFF),Y ;SPECAL y-co lo / x-co lo
               TAX ;temp

               DEY ;=4 / 2
               LDA (IX),Y ;IX y-co hi / x-co hi
               SBC (SPEC_OFF),Y ;SPECAL y-co hi / x-co hi
               JMP DABIZNIZ

               ; Sloping up with increasing x/y
MRIGHTTO       PLA
               PHA

               LDY #$15
               AND #$40 ; 0=slope in x direction; 1=slope in y direction
               BEQ MRERIGHT

               ;find how far along slope IX shape is...
               LDY #$17
MRERIGHT       LDA (SPEC_OFF),Y ;SPECAL [y-co + y-dim] lo / [x-co + x-dim] lo
               SEC
               SBC IXTB-$14,Y ;IX [y-co + y-dim] lo / [x-co + x-dim] lo
               TAX

               DEY ;=$16 / $14
               LDA (SPEC_OFF),Y ;SPECAL [y-co + y-dim] hi / [x-co + x-dim] hi
               SBC IXTB-$14,Y ;IX [y-co + y-dim] hi / [x-co + x-dim] hi
DABIZNIZ       BCC MTOP2

               BEQ COME_HERE

MBOT           PLA

               LDY #$06
               LDA (SPEC_OFF),Y ;SPECAL z-co (lo)
               LDY #$18
               JMP TOPSTR

MTOP2          LDX #0

COME_HERE      STX SOME_WHERE

               PLA
               AND #$10 ; 0=45' slope; 1=30' slope
               BEQ NO_LSRA

               LSR SOME_WHERE ;dist up is half dist along for 30' slopes

NO_LSRA        LDY #$19

               LDA (SPEC_OFF),Y ;SPECAL [z-co + z-dim] (lo)
               SEC

               SBC SOME_WHERE
               BCS HRSBTWD

               AND #0
HRSBTWD        LDY #$06

               CMP (SPEC_OFF),Y ;SPECAL z-co (lo)
               BCS HOTWATER
               LDA (SPEC_OFF),Y ;SPECAL z-co (lo)
HOTWATER       LDY #$18
TOPSTR         STA (SPEC_OFF),Y ;top of SPECAL object for current IX shape

               ;check right most x coor
NOTSLOPE       LDY #3
               LDA (IX),Y ;IX x-co lo
               LDY #$15
               CMP (SPEC_OFF),Y ;SPECAL [x-co + x-dim] lo
               LDY #2
               LDA (IX),Y ;IX x-co hi
               LDY #$14
               SBC (SPEC_OFF),Y ;SPECAL [x-co + x-dim] hi
               BCS BACK_HERE ;not overlapping on x so loop to next bgnd shape

               ;check left most x coor
               LDY #3
               LDA (SPEC_OFF),Y ;SPECAL x-co lo
               CMP IXTB+1 ;IX [x-co + x-dim] lo
               DEY ;=2

               LDA (SPEC_OFF),Y ;SPECAL x-co hi
               SBC IXTB ;IX [x-co + x-dim] hi
               BCS BACK_HERE ;not overlapping on x so loop to next bgnd shape

               ;check back most y coor
               LDY #$17
               LDA (SPEC_OFF),Y ;SPECAL [y-co + y-dim] lo
               LDY #5
               CMP (IX),Y ;IX y-co lo
               LDY #$16
               LDA (SPEC_OFF),Y ;SPECAL [y-co + y-dim] hi
               LDY #4
               SBC (IX),Y ;IX y-co hi
               BCC BACK_HERE ;not overlapping on y so loop to next bgnd shape

               ;check front most y coor
               LDY #5
               LDA (SPEC_OFF),Y ;SPECAL y-co lo
               CMP IXTB+3 ;IX [y co + y-dim] lo
               DEY ;=4
               LDA (SPEC_OFF),Y ;SPECAL y-co hi
               SBC IXTB+2 ;IX [y co + y-dim] hi
               BCS BACK_HERE ;not overlapping on y so loop to next bgnd shape

               INC TABLE_ENT

               INC ERE

               LDY TABLE_ENT
               LDA SPECTRUM
               STA ENT_TAB-1,Y

BACK_HERE      DEC SPECTRUM
               BEQ KOUYATE

               JMP DROP1      ;loop back to check x & y of next object

KOUYATE        LDA TABLE_ENT
               BNE ONSTHN     ;check to see if table is empty
               JMP DEC_ZED    ;no objects under blob
ONSTHN         STA Z_TEMP
DROP2          LDY TABLE_ENT  ;loop and check table to see if
               LDX ENT_TAB-1,Y               ;any objects have same z. If so then
               LDA SPECTL-1,X ;set up object in tables but then check next
               STA SPEC_OFF   ;object
               LDA SPECTH-1,X
               STA SPEC_OFF+1

               LDY #$A
               LDA (SPEC_OFF),Y ;SPECAL shape number
               AND #$1F ;; 2009-09: MY: Added this to make sure we can't run off the end of SLOP_TAB
               TAY

               LDA SLOP_TAB,Y
               BPL SLOP_CHEC2 ;branch if a slope

               LDY #6
               LDA (IX),Y ;IX z-co (lo)
               LDY #$18
               SEC

               SBC (SPEC_OFF),Y ;top of SPECAL object for current IX shape

               BEQ NO_DEC_ZED2

               CMP #$01
               BNE NO_WAY3

NO_DEC_ZED2
               LDA SPR_NO
               BNE NOT_IMPORT
               LDY #$0A
               LDA (SPEC_OFF),Y ;shape number
               CMP #$13 ;top trap door

               BEQ TRAP_STUFF
               CMP #$15 ;exit
               BEQ TRAP_STUFF
               CMP #$11 ;uncorked bottle

               BNE EXIT_C1

TRAP_STUFF     LDY #$18
               LDA (SPEC_OFF),Y
               LDY #6
               CMP (IX),Y ;IX z-co (lo)
               BNE EXIT_C2

               JMP TRAP_CHEC

SLOP_CHEC2     JMP SLOP_CHEC

EXIT_C2        CLC
               ADC #1
               CMP (IX),Y ;IX z-co (lo)
               BNE EXIT_C1

               JMP TRAP_CHEC

EXIT_C1        LDA EXITO      ;if exit closed sign up, remove it

               CMP #2
               BNE NOT_IMPORT
               LDA #0
               STA EXITO

               JSR EXITOPEN
NOT_IMPORT     LDY TABLE_ENT
               LDA ENT_TAB-1,Y
               STA OBJ_REST
               LDY #$18
               LDA (SPEC_OFF),Y ;top of SPECAL object for current IX shape
               STA TOP_REST
NO_WAY3        JMP NO_WAY     ;loop to check next object under IX shape

SLOP_CHEC      LDY #$18
               LDA (SPEC_OFF),Y ;top of SPECAL object for current IX shape

               LDY #6
               CMP (IX),Y ;IX z-co (lo)
               BCC NO_WAY
               TAX

               LDA (SPEC_OFF),Y ;SPECAL z-co (lo)
               CMP IXTB+5 ;IX [z-co + z-dim] (lo)
               BCS NO_WAY

               TXA
               STA (IX),Y ;IX z-co (lo)

EXIT_CHEC      LDY SPR_NO
               BNE DINK

               LDY #$01       ;make blob round
               LDA #$02
               STA (IX),Y ;data offset

DINK           JMP EXIT_C1

NO_WAY         DEC TABLE_ENT
               BEQ NODROP2

               JMP DROP2      ;loop back and check next object

NODROP2        LDA OBJ_REST
               BMI NO_OBJ     ;branch if not sitting on object

               JMP NOFALL     ;turn off shadow

NO_OBJ         AND #0
               STA SOME_WHERE ;clear flag for object under blob
               LDA Z_TEMP
               STA TABLE_ENT

               LDA #$90
               STA Z_TEMP     ;set shad_height to $90
DROP3          LDY TABLE_ENT  ;loop and check table to see if
               LDX ENT_TAB-1,Y               ;any objects are below the blob
               LDA SPECTL-1,X ;if so, then check distance
               STA SPEC_OFF   ;between blob and object. Least distance
               LDA SPECTH-1,X ;means closest object
               STA SPEC_OFF+1

               LDY #$6
               LDA (IX),Y ;z-co (lo)
               SEC
               LDY #$18
               SBC (SPEC_OFF),Y ;top of SPECAL object for current IX shape
               BCC NO_SUB_ZED ;branch if IX shape is below SPECAL shape

               CMP Z_TEMP
               BCS NO_SUB_ZED ;branch if a previously checked object is closer

               STA Z_TEMP
               LDY TABLE_ENT

               LDA ENT_TAB-1,Y
               STA OBJ_REST
               LDY #$0A
               LDA (SPEC_OFF),Y ;shape number
               STA SOME_WHERE
NO_SUB_ZED     DEC TABLE_ENT

               LDA TABLE_ENT

               BNE DROP3      ;loop back and check next object
               LDA SOME_WHERE ;object under blob (0=no object)

               BEQ DEC_ZED

               ;above an object
               ;first check to see if it is a slope
               LDY #$0A       ;if only 3 pixels up, then set no shadow

               LDA (SPEC_OFF),Y
               AND #$1F ;; 2009-09: MY: Added this to make sure we can't run off the end of SLOP_TAB
               TAY

               LDA SLOP_TAB,Y
               BMI CONT_SHAD

               LDY #6
               LDA (IX),Y ;IX z-co (lo)
               LDY #$18
               SBC (SPEC_OFF),Y
               CMP #3
               BCS CONT_SHAD

               LDA #2
               STA DROP_NOT
               LDA #$FF
               LDY #$11
               STA (IX),Y ;object height above its shadow -- $FF = no shadow
               AND #0
               JMP DO_SHAD

CONT_SHAD      LDA Z_TEMP

               LDY #$11
               STA (IX),Y ;object height above its shadow
               AND #0 ;clear spike bit

               JMP DO_SHAD

               ;above nothing/shadow
DEC_ZED        LDA #$10 ;dummy z value to make sure shadow (spike) is displayed

               LDY #$11
               STA (IX),Y ;object height above its shadow
               LDA #64        ;set spike def

DO_SHAD        DEY ;=$10
               STA (IX),Y ;bit7 set: resting on object ;bit6 set: shadow is spike
               LDA JUMP_F
               BNE ABOUT

               LDY #$1A
               LDA (IX),Y ;bit1 set if gravity not to act on object
               BNE ABOUT_ERE

               JSR SET_DROP_V
ABOUT_ERE      LDY SPR_NO
               BNE DONK

               LDA #2
               LDY #1         ;make blob round
               STA (IX),Y ;data offset

ABOUT          LDA DROP_NOT
               CMP #2
               BNE ABOUT2
               DEC DROP_NOT
               JMP DONTDOZIS

ABOUT2         LDA #0
               STA DROP_NOT
DONK           JMP DONTDOZIS

NOFALL         LDY #$14
               LDA (IX),Y ;velocity/direction in Z

               AND #$80
               BEQ UH_UH

               LDA #0
               STA (IX),Y
UH_UH          LDA SPR_NO     ;if not blob then jump round next bit
               BEQ TOOFAR
               JMP DONTDOZIS

TOOFAR         LDA JUMP_F
               BNE NO_STOP_IT
               LDA DROP_NOT
               BNE NO_PROB

               INC DROP_NOT
               LDA #3
               JMP YES_PROB
NO_PROB        LDA #0

YES_PROB       STA IXTAB+1 ;BLOB data offset

NO_STOP_IT     LDY OBJ_REST
               LDA SPECTL-1,Y
               STA SPEC_OFF
               LDA SPECTH-1,Y
               STA SPEC_OFF+1
               LDY #$0A
               LDA (SPEC_OFF),Y ;shape number
               AND #$1F ;; 2009-09: MY: Added this to make sure we can't run off the end of SLOP_TAB
               TAY

               LDA SLOP_TAB,Y
               BMI NOT_A_SLOP
               LDA #2
               STA IXTAB+1 ;BLOB data offset (ie animation frame)

NOT_A_SLOP     LDY #$0A           ;check to see if object of rest is
               LDA (SPEC_OFF),Y   ;one we want the blob to start on
               TAY

               LDA RETAIN,Y
               BNE FOREVER

               JMP DONTDOZIS  ;basically, no. Non. Nein. No fucking way.

               ;Pull out current blob x/y coords
               ;& stick them into TEMP_IX for transfer to ST_POS
FOREVER        LDA IXTAB+3 ;BLOB x-co lo
               AND #$F0 ;centre it
               TAX ;temp

               LDY #3
               CMP (SPEC_OFF),Y ;SPECAL x-co lo
               LDA IXTAB+2 ;BLOB x-co hi
               DEY ;=2
               SBC (SPEC_OFF),Y ;SPECAL x-co hi
               BCS FIXJK1

               LDA (SPEC_OFF),Y ;SPECAL x-co hi
               STA TEMP_IX+1 ;SAVE x-co hi
               INY ;=3
               LDA (SPEC_OFF),Y ;SPECAL x-co lo
               STA TEMP_IX ;SAVE x-co lo
               JMP FIXJK2

FIXJK1         TXA ;temp ^ ;centred BLOB x-co lo ;lda IXTAB+3:and #$F0
               STA TEMP_IX ;SAVE x-co lo
               LDA IXTAB+2 ;BLOB x-co hi
               STA TEMP_IX+1 ;SAVE x-co hi

FIXJK2         LDA IXTAB+5 ;BLOB y-co lo
               AND #$F0 ;centre it
               TAX ;temp

               LDY #5
               CMP (SPEC_OFF),Y ;SPECAL y-co lo
               LDA IXTAB+4 ;BLOB y-co hi

               DEY ;=4

               SBC (SPEC_OFF),Y ;SPECAL y-co hi
               BCS FIXJK3

               LDA (SPEC_OFF),Y ;SPECAL y-co hi
               STA TEMP_IX+3 ;SAVE y-co hi

               INY ;=5

               LDA (SPEC_OFF),Y ;SPECAL y-co lo
               STA TEMP_IX+2 ;SAVE y-co lo

               JMP FIXJK4

FIXJK3         TXA ;temp ^ ;centred BLOB y-co lo ;lda IXTAB+5:and #$F0
               STA TEMP_IX+2 ;SAVE y-co lo
               LDA IXTAB+4 ;BLOB y-co hi
               STA TEMP_IX+3 ;SAVE y-co hi

               ;Check not saving a position inside spike...
FIXJK4         LDA #4
               STA YELLO ;counter - number of static objects

               LDA #<STATOB
               STA ZPA
               LDA #>STATOB
               STA ZPA+1

ANAGAIN        LDY #0
               LDA (ZPA),Y ;shape
               CMP #$10 ;spike alien/gun

               BNE SUBWAY
               INY ;=1

               LDA (ZPA),Y ;print shape flag

               BMI SUBWAY


               INY ;=2

               LDA (ZPA),Y ;x-co hi
               EOR TEMP_IX+1 ;SAVE x-co hi

               BNE SUBWAY
               INY ;=3

               ORA (ZPA),Y ;lda ;x-co lo

               EOR TEMP_IX ;SAVE x-co lo

               BNE SUBWAY
               INY ;=4

               ORA (ZPA),Y ;lda ;y-co hi

               EOR TEMP_IX+3 ;SAVE y-co hi

               BNE SUBWAY
               INY ;=5

               ORA (ZPA),Y ;lda ;y-co lo

               EOR TEMP_IX+2 ;SAVE y-co lo

               BEQ DONTDOZIS

SUBWAY         LDA #IXSIZ
               CLC
               ADC ZPA
               STA ZPA
               BCC STREAM
               INC ZPA+1
STREAM         DEC YELLO ;counter
               BNE ANAGAIN

               ;copy blob x/y co-ords in TEMP_IX to ST_POS table
               LDX #3
BILGE          LDA TEMP_IX,X
               STA ST_POS,X
               DEX

               BPL BILGE

               ;save blob z-co and screen co-ords to ST_POS table
               LDA IXTAB+6 ;z-co (lo)
               STA ST_POS+4
               LDA SCREENX

               STA ST_POS+5
               LDA SCREENX+1

               STA ST_POS+6
               LDA SCREENY

               STA ST_POS+7
               LDA SCREENY+1

               STA ST_POS+8


DONTDOZIS      JSR CHECKXY ;check in which x/y directions blob can move

               LDY SPR_NO
               CPY #3
               BCS NO_SHADOW ; Only the blob and the two aliens have shadows

               LDA LSHAD_TAB,Y
               STA IY
               LDA HSHAD_TAB,Y
               STA IY+1
               JMP TRSHAD
NO_SHADOW      RTS


;****************************************
;*             Deal with jump
;****************************************
JUMP_UP        LDA JUMP_F
               BNE DO_UP

               LDY PLAYER
               LDA CHOICE,Y
               BNE JOY_TEST
               LDY KJUMP
               JSR KEYS
               BNE YES_JUMP
NO_JUMP        RTS
JOY_TEST       LDA #FLIPIN
               STA $01

               LDY #32
               JSR KEYS
               BEQ NO_JUMP

YES_JUMP       LDA DROP_NOT
               BEQ NO_JUMP

               LDA #0
               STA IXTAB+$14 ;BLOB velocity/direction in Z

               LDA #3
               STA IXTAB+1 ;BLOB data offset

               LDA #$12
               STA JUMP_F
               JMP JUMPSON



DO_UP          LDY JUMP_F

               LDA JUMP_TAB,Y
               STA IXTAB+1 ;BLOB data offset

               LDA IXTAB+$14 ;BLOB velocity/direction in Z

               CMP #$07
               BEQ JUMPED
               BCS JUMPED
               INC IXTAB+$14 ;BLOB velocity/direction in Z

JUMPED         DEC JUMP_F
               RTS


;****************************************
;*             Temporary routine to check zeds
;*             of all moving objects
;****************************************
DROP_THEM      LDY #6         ; Objects 0-6 are: Blob, 2x Aliens, 2x Alien bullets, 2x Blob bullets
               STY SPR_NO
RE_DROP        LDA SPRTABL,Y  ;blob
               STA IX
               LDA SPRTABH,Y
               STA IX+1

               LDY #0
               LDA (IX),Y ;shape
               BEQ NEXT_SPRIT

               LDY SPR_NO
               BEQ NOT_ALIE

               ;; 2009-09: MY: Comment next line out (along with JSR CREATE_ALI
               ;;              below) to get rid of aliens.  Useful for
               ;;              testing!
               JSR ALIEN

               LDY #0
               LDA (IX),Y ;shape
               CMP #7 ; 7 = explosion
               BEQ NEXT_SPRIT
NOT_ALIE       JSR DROP
               LDA #FLIPIN
               STA $01

               JSR GET_GOING

               LDY SPR_NO
               JSR ON_OFF
NEXT_SPRIT     DEC SPR_NO
               LDY SPR_NO
               BPL RE_DROP

               ;; 2009-09: MY: Comment next line out (along with JSR ALIEN
               ;;              above) to get rid of aliens.  Useful for
               ;;              testing!
               JSR CREATE_ALI
               JMP CHECK_IX




;****************************************
;*             Checks for how far IX shape can
;*             move in each direction in the
;*             x-y plane
;****************************************
CHECKXY        LDA #$10       ;set max movement for each direction to default

               LDX SPR_NO
               STA MAXRIGHT,X
               STA MAXLEFT,X

               STA MAXFORWARD,X
               STA MAXBACK,X

               STA MAXUP,X

               LDA SPECAL ;number of objects in SPECAL table

               STA SPECTRUM
MCHKLP         LDY SPECTRUM
               LDA SPECTL-1,Y
               STA SPEC_OFF
               LDA SPECTH-1,Y
               STA SPEC_OFF+1

               ;is SPECAL object above IX shape ?
               LDY #$06
               LDA (SPEC_OFF),Y ;SPECAL z-co (lo)
               CMP IXTB+5 ;IX [z-co + z-dim] (lo)
               BCC SAHARA1

               LDY #$12
               LDA SPECTRUM
GOBI           CMP ENT_TAB-1,Y

               BEQ DUMDEDUM
               DEY

               BNE GOBI
               JMP MCNXSP ;loop

DUMDEDUM       LDY #$0A
               LDA (SPEC_OFF),Y ;shape number
               CMP #$14 ;LH side trap door
               BEQ GOBI2

               CMP #$16 ;RH side trap door
               BEQ GOBI2

               LDY #$06
               LDA (SPEC_OFF),Y ;work out max height that blob can jump
               SEC
               SBC IXTB+5 ;IX [z-co + z-dim] (lo)
               STA MAXUP,X

GOBI2          JMP MCNXSP ;loop

               ;is IX shape above SPECAL object ?
SAHARA1        ;ldy #$06
               LDA (IX),Y ;IX z-co (lo)
               CLC
               ADC #1
               LDY #$18
               CMP (SPEC_OFF),Y ;top of object for current IX shape
               BCC SAHARA2

               JMP MCNXSP ;loop

SAHARA2        ORA #$FF
               STA TPRIGHT
               STA TPLEFT
               STA TPFORWARD
               STA TPBACK

               ;find out how far away +ve x direction [RIGHT]
               LDY #$03
               LDA (SPEC_OFF),Y ;SPECAL x-co lo
               SEC
               SBC IXTB+1 ;IX [x-co + x-dim] lo
               TAX ;temp

               DEY ;=$02

               LDA (SPEC_OFF),Y ;SPECAL x-co hi
               SBC IXTB ;IX [x-co + x-dim] hi
               BCC OUTRNG1 ;branch if on wrong side
               BNE RUMBLE1 ;branch if >$100 pixels away
               TXA

               BPL OWL1 ;branch if not going to piss up sign

RUMBLE1        LDA #$7F ;"a large distance"

OWL1           STA TPRIGHT

OUTRNG1
               ;find out how far away +ve y direction [FORWARD]
               LDY #$05
               LDA (SPEC_OFF),Y ;SPECAL y-co lo
               SEC
               SBC IXTB+3 ;IX [y-co + y-dim] lo
               TAX ;temp

               DEY ;=$04
               LDA (SPEC_OFF),Y ;SPECAL y-co hi
               SBC IXTB+2 ;IX [y-co + y-dim] hi
               BCC OUTRNG2 ;branch if on wrong side
               BNE RUMBLE2 ;branch if >$100 pixels away
               TXA

               BPL OWL2
RUMBLE2        LDA #$7F

OWL2           STA TPFORWARD

OUTRNG2
               ;find out how far away in -ve x direction [LEFT]
               LDY #$03
               LDA (IX),Y ;IX x-co lo
               LDY #$15
               SEC

               SBC (SPEC_OFF),Y ;SPECAL [x-co + x-dim] lo
               TAX ;temp

               LDY #$02
               LDA (IX),Y ;IX x-co hi
               LDY #$14
               SBC (SPEC_OFF),Y ;SPECAL [x-co + x-dim] hi
               BCC OUTRNG3 ;branch if on wrong side
               BNE RUMBLE3 ;branch if >$100 pixels away
               TXA

               BPL OWL3
RUMBLE3        LDA #$7F

OWL3           STA TPLEFT
OUTRNG3
               ;find out how far away in -ve y direction [BACK]
               LDY #$05
               LDA (IX),Y ;IX y-co lo
               LDY #$17
               SEC

               SBC (SPEC_OFF),Y ;SPECAL [y-co + y-dim] lo
               TAX ;temp

               LDY #$04
               LDA (IX),Y ;IX y-co hi
               LDY #$16
               SBC (SPEC_OFF),Y ;SPECAL [y-co + y-dim] hi
               BCC OUTRNG4 ;branch if on wrong side
               BNE RUMBLE4 ;branch if >$100 pixels away
               TXA

               BPL OWL4
RUMBLE4        LDA #$7F

OWL4           STA TPBACK
OUTRNG4
               LDX SPR_NO
               LDA TPFORWARD

               BPL WSX0
               LDA TPBACK
               BPL WSX1
               LDA TPLEFT
               BPL WSX2
               LDA TPRIGHT

               CMP MAXRIGHT,X
               BCS MCNXSP
               STA MAXRIGHT,X
               JMP MCNXSP

WSX2           CMP MAXLEFT,X
               BCS MCNXSP

               STA MAXLEFT,X
               JMP MCNXSP

WSX1           LDA TPLEFT
               AND TPRIGHT
               BPL MCNXSP

               LDA TPBACK
               CMP MAXBACK,X
               BCS MCNXSP

               STA MAXBACK,X
               JMP MCNXSP

WSX0           LDA TPLEFT
               AND TPRIGHT
               BPL MCNXSP

               LDA TPFORWARD
               CMP MAXFORWARD,X
               BCS MCNXSP

               STA MAXFORWARD,X

MCNXSP         DEC SPECTRUM
               BEQ MFINCHK

               JMP MCHKLP ;loop to check next object in SPECAL table

MFINCHK        RTS


;****************************************
;*             Shooting !!!!!!
;****************************************
YOURS          LDA #FLIPIN
               STA $01

               LDY PLAYER
               LDA CHOICE,Y
               BNE JOY_TES1

               LDY KFIRE
               JSR KEYS
               BNE YES_BULL ;branch if fire key pressed
NODOBUL        RTS
JOY_TES1       LDA #$FF
               STA $DC00

               LDA $DC00 ;read joystick
               AND #$10 ;fire button

               BNE NODOBUL ;branch if fire not pressed

YES_BULL       LDA LAST_FIRE
               BNE NODOBUL ;branch if fire not released last time round

               LDA MYBUL
               BEQ YES_BANG ;branch if 1st player-bullet IX block free

               LDA MYBUL+IXSIZ
               BNE NODOBUL ;branch if two player-bullets already exist

               LDA #IXSIZ
YES_BANG       CLC
               ADC #<MYBUL
               STA IX
               LDA #>MYBUL
               ADC #0
               STA IX+1
               LDY #0
               LDA #5
               STA (IX),Y ;shape
               INY ;=1

               LDA #0
               STA (IX),Y ;data offset
               LDA IXTAB+3 ;BLOB x-co lo

               CLC

               ADC #4
               LDY #3
               STA (IX),Y ;x-co lo

               LDA IXTAB+2 ;BLOB x-co hi

               ADC #0
               DEY ;=2

               STA (IX),Y ;x-co hi

               LDA IXTAB+5 ;BLOB y-co lo

               CLC

               ADC #4
               LDY #5
               STA (IX),Y ;y-co lo

               LDA IXTAB+4 ;BLOB y-co hi

               ADC #0
               DEY ;=4

               STA (IX),Y ;y-co hi

               LDY #6
               LDA IXTAB+6 ;BLOB z-co (lo)

               STA (IX),Y ;z-co (lo)

               LDY #9
               LDA #1
               STA (IX),Y ;chars across of bullet
               LDY #$12
               LDA IXTAB+$12

               AND #$80
               STA (IX),Y ;direction in X
               INY ;=$13

               LDA IXTAB+$13

               AND #$80
               STA (IX),Y ;direction in Y
               DEY ;=$12

               LDA IXTAB+$12

               AND #$7F
               BEQ NO_VEL_1
               LDA #MAX_V+1
               ORA (IX),Y
               STA (IX),Y ;set velocity in X

NO_VEL_1       INY ;=$13

               LDA IXTAB+$13

               AND #$7F
               BEQ NO_VEL_2
               LDA #MAX_V+1
               ORA (IX),Y
               STA (IX),Y ;set velocity in Y

NO_VEL_2       DEY ;=$12

               LDA (IX),Y
               AND #$7F
               STA TEMP_IX

               INY ;=$13

               LDA (IX),Y
               AND #$7F
               ORA TEMP_IX

               BNE GOT_DIR

               LDX GAUGEPOINT
               LDA WOLLOP,X
               DEY ;=$12
               STA (IX),Y ;set X velocity
               INY ;=$13
               LDA WOLLOP+4,X
               STA (IX),Y ;set Y velocity
GOT_DIR        JSR SET_SPEED
               LDA #5
               STA LAST_FIRE

               JMP SHOOT



;****************************************

;*             Turn bullets/aliens off if they are
;*             outside the screen

;****************************************

ON_OFF         JSR ALI_DEAD
               CPY #0
               BNE YES_OFF

               RTS

YES_OFF        LDY #1

               LDA (IX),Y ;data offset - bit7 set if shape not to be printed

               AND #$80
               BNE OFF_WIV_IT
               RTS

OFF_WIV_IT     LDA #0
               TAY

               STA (IX),Y ;set shape=0 (free)
NOZE           RTS



SET_STOP       LDY SPR_NO
               CPY #3
               BCC NOTONOFF
               CPY #7
               BCS NOZE
               LDA MAXLEFT,Y

               BEQ OFF_WIV_IT
               LDA MAXRIGHT,Y
               BEQ OFF_WIV_IT
               LDA MAXBACK,Y

               BEQ OFF_WIV_IT
               LDA MAXFORWARD,Y
               BEQ OFF_WIV_IT
NOTONOFF       RTS



;****************************************
;*
;*             Pause. Print up 'pause' and wait
;*             for a key press. Update any
;*             messages already in window
;*
;****************************************
PAUSE          LDA #0

               LDY #$84
CLEARSPACE2    STA SCROLST-1,Y
               DEY
               BNE CLEARSPACE2

               LDA #FLIPOUT
               STA $01

               LDX #0
READPAUSE      LDA $D000+[16*8],X            ;load 'p'
               STA SCROLST+[9*4],Y
               LDA $D000+[16*8]+4,X
               STA SCROLST+[9*4]+MC,Y

               LDA $D000+[1*8],X             ;load 'a'
               STA SCROLST+[10*4],Y
               LDA $D000+[1*8]+4,X
               STA SCROLST+[10*4]+MC,Y

               LDA $D000+[21*8],X            ;load 'u'
               STA SCROLST+[11*4],Y
               LDA $D000+[21*8]+4,X
               STA SCROLST+[11*4]+MC,Y

               LDA $D000+[19*8],X            ;load 's'
               STA SCROLST+[12*4],Y
               LDA $D000+[19*8]+4,X
               STA SCROLST+[12*4]+MC,Y

               LDA $D000+[5*8],X             ;load 'e'
               STA SCROLST+[13*4],Y
               LDA $D000+[5*8]+4,X
               STA SCROLST+[13*4]+MC,Y

               LDA $D000+[4*8],X             ;load 'd'
               STA SCROLST+[14*4],Y
               LDA $D000+[4*8]+4,X
               STA SCROLST+[14*4]+MC,Y

               INX

               INY

               CPY #4
               BNE READPAUSE

               INC $01 ;=$35 : I/O in

               JSR DISPCHARS

               LDA #64

               STA YELLO ;Matt's loopy variable
PAUSELOOP      JSR DOSCROLL

LOOPINSIDE     LDA $D012

               CMP #1
               BNE LOOPINSIDE
               DEC YELLO

               BNE PAUSELOOP

               LDY #28

               JSR DELA1

               ;Wait for any key to be pressed

               AND #$00
               STA $DC00

NOWHERE        LDA $DC01
               EOR #$FF
               BEQ NOWHERE

               ORA #$FF
               STA $DC00



               LDA #64

               STA YELLO ;Matt's loopy variable
PAUSELOOP2     JSR DOSCROLL
LOOPINSID2     LDA $D012 ;raster

               CMP #1
               BNE LOOPINSID2
               DEC YELLO

               BNE PAUSELOOP2
               LDA EXITO

               BEQ RETUS

               JMP EXITOPEN
RETUS          RTS

;
; File X4.BUB
; Starts Here
;
;

;
;
;  PDS Pc1.10 :000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          01.61
;         By  "JAKE AND MATTHEW..."
;
;     Created on Thu the 03rd of Mar 1988
;        Last update 08:41 on 10/05/88
;

;****************************************
;*             Print 'Exit Open' in window
;****************************************
EXITOPEN       LDA #FLIPOUT
               STA $01

               LDY #7
READEXIT       LDA $D000+[5*8],Y             ;load 'e'
               STA $E000+[320*16]+[18*8],Y

               LDA $D000+[24*8],Y            ;load 'x'
               STA $E000+[320*16]+[19*8],Y

               LDA $D000+[9*8],Y             ;load 'i'
               STA $E000+[320*16]+[20*8],Y

               LDA $D000+[20*8],Y            ;load 't'
               STA $E000+[320*16]+[21*8],Y

               DEY

               BPL READEXIT
               LDY #7
               LDA EXITO

               CMP #1
               BEQ OPENEXIT
               CMP #2
               BEQ CLOSEDEXIT
               JMP CLEAREXIT

OPENEXIT       LDA $D000+[15*8],Y            ;load 'o'
               STA $E000+[320*17]+[18*8],Y

               LDA $D000+[16*8],Y            ;load 'p'
               STA $E000+[320*17]+[19*8],Y

               LDA $D000+[5*8],Y             ;load 'e'
               STA $E000+[320*17]+[20*8],Y

               LDA $D000+[14*8],Y            ;load 'n'
               STA $E000+[320*17]+[21*8],Y

               DEY

               BPL OPENEXIT
               INC $01 ;=$35 : I/O in
               RTS


;****************************************
;*             Print 'Exit Closed' in window
;****************************************
CLOSEDEXIT     LDA $D000+[3*8],Y             ;load 'c'
               STA $E000+[320*17]+[17*8],Y

               LDA $D000+[12*8],Y            ;load 'l'
               STA $E000+[320*17]+[18*8],Y

               LDA $D000+[15*8],Y            ;load 'o'
               STA $E000+[320*17]+[19*8],Y

               LDA $D000+[19*8],Y            ;load 's'
               STA $E000+[320*17]+[20*8],Y

               LDA $D000+[5*8],Y             ;load 'e'
               STA $E000+[320*17]+[21*8],Y

               LDA $D000+[4*8],Y             ;load 'd'
               STA $E000+[320*17]+[22*8],Y

               DEY

               BPL CLOSEDEXIT
               INC $01 ;=$35 : I/O in
               RTS



CLEAREXIT      LDA #0

               LDY #$84
CLEARSPACE3
               STA SCROLST-1,Y

               DEY

               BNE CLEARSPACE3

               STY EXITO

               JMP DISPCHARS





;RESET         SEI
;       LDA #$37  ; Page in ROMS & I/O
;              STA $01
;              JMP $FCE2 ; Cold reset



;****************************************
;*             Update x, y and z of sprites
;****************************************
GET_GOING      JSR SET_STOP

               LDY #$14
               LDA (IX),Y
               AND #$7F
               BEQ NO_UP_DO
               JSR UP_DOWN

NO_UP_DO       LDA #0
               STA MUS

               LDY SPR_NO
               BEQ BLOB_IT

               CPY #1
               BEQ NOPE1

               CPY #2
               BEQ NOPE1

               JMP DO_GO

BLOB_IT        LDY PLAYER
               LDA CHOICE,Y
               BNE JOY2

               JSR KEYSCAN
               ;BMI NOPE1 ;; 2009-09: MY: Commented out as KEYSCAN never returns -ve value
               JMP JOY3

JOY2           LDA #$FF
               STA $DC00
               LDA $DC00 ;joystick port 2

               ;; 2009-09: MY: Moved label as KEYSCAN already stored its value in SRA
;JOY3           STA SRA

;; 2009-09: MY: Uncomment this line to hard-code a key-press for testing
;  bit 0: 0 = up pressed
;  bit 1: 0 = down pressed
;  bit 2: 0 = left pressed
;  bit 3: 0 = right pressed
;               LDA #%00001110

               STA SRA
JOY3
               AND #$0F
               TAX
               LDA DIRS,X ;table of ptr positions from port value
               BMI NOPE1

               STA GAUGEDIR
               STA MUS

               JSR CALL_ACCEL
NOPE1          JSR CALL_DECEL

               JSR MAKEDIR

               JSR UPDATEPOINT

DO_GO          JSR SET_SPEED

;
; Work out which direction to move the blob in.  Build a 4-bit index into the
; MOV_POINT table:
;
;   bits 0-1 = %00: no X velocity, %01: +ve X velocity, %10: -ve X velocity
;   bits 2-3 = %00: no Y velocity, %01: +ve Y velocity, %11: -ve Y velocity
;
               LDX #0
               LDY #$12
               STX MASK
               LDA (IX),Y ; velocity/direction in X - bit 7: 1=left/0=right;   bits 0-6: speed
               AND #$7F
               BEQ HMM1

               INC MASK
               INX

               LDA (IX),Y
               BPL HMM1
               INC MASK
               INX

HMM1           INY ; = $13
               LDA (IX),Y ; velocity/direction in Y - bit 7: 1=forward/0=back; bits 0-6: speed
               AND #$7F
               BEQ HMM2
               LDA MASK
               CLC
               ADC #4
               TAX

               LDA (IX),Y
               BPL HMM2
               LDA MASK
               CLC
               ADC #8
               TAX

HMM2           CPX #0
               BEQ NO_MOVEM
               LDA MOV_POINT,X
               TAX
;;
;; 2009-09: MY: Removed following unreachable code and jump past it!
;;
;               JMP DO_GO_2
;
;               LDA GAUGEPOINT,Y
;               TAX
;DO_GO_2
               LDA MOVJMPL,X
               STA JUMP_ALT+1
               LDA MOVJMPH,X
               STA JUMP_ALT+2
               LDX SPR_NO
JUMP_ALT       JMP BBACK ;smc
NO_MOVEM       RTS


;****************************************
;* Called if there is direction input from keyboard/joystick.  The input will
;* be stored in GAUGEDIR.  (If GAUGEPOINT -- the current gauge direction -- is
;* different then the gauge gradually swings round until it is pointing in the
;* direction being steered by the keyboard/joystick.  The MAKEDIR subroutine,
;* called after this subroutine, works out which direction to move the gauge,
;* and stores that direction in GDIR.  Then UPDATEPOINT, the next subroutine
;* called, actually updates GAUGEPOINT.)
;*
;*             Pick up gauge dir and add one
;*             to the directions, checking first
;*             that the opposite directions are
;*             zero.
;****************************************
CALL_ACCEL     LDY GAUGEPOINT

               ;; 2009-09: MY: Delete these?
               TYA
               TAX

               LDA X_MOTION,Y ; Is there any X component to the current gauge direction?
               BEQ DO_YCCEL
               ;; 2009-09: MY: Delete this?
               TAY

               LDA IXTAB+$12 ; Blob velocity/direction in X
               AND #$7F
               BEQ GO_XCCEL

               LDX GAUGEPOINT
               ;STX MUS+1 ;; 2009-09: MY: Seems no point to this so commented out

               LDA IXTAB+$12
               AND #$80
               EOR VEL_TAB,X

               BMI DO_YCCEL
               LDA IXTAB+$12

               AND #$7F
;GO_XCCEL
               CMP #MAX_V
               BEQ DONE_XCCEL
GO_XCCEL
               CLC
               ADC #1
DONE_XCCEL     ORA VEL_TAB,X
               STA IXTAB+$12

DO_YCCEL       LDY GAUGEPOINT
               LDA X_MOTION+4,Y ; Is there any Y component to the current gauge direction?
               BEQ DO_NO_CEL

               ;; 2009-09: MY: Delete these?
               TAY
               INY

               LDA IXTAB+$13 ; Blob velocity/direction in Y
               AND #$7F      ; Mask off direction to get speed
               BEQ GO_YCCEL
               LDX GAUGEPOINT
               LDA IXTAB+$13
               AND #$80       ; Get Y direction from bit 7: 1=forward/0=back
               EOR VEL_TAB2,X ; See if it's the same as the Y direction of the pointer
               BMI DO_NO_CEL  ; If it's not, there's no acceleration to do

               LDA IXTAB+$13
               AND #$7F
;GO_YCCEL
               CMP #MAX_V
               BEQ DONE_YCCEL
GO_YCCEL
               CLC
               ADC #1
DONE_YCCEL     ORA VEL_TAB2,X
               STA IXTAB+$13

DO_NO_CEL      RTS


;****************************************
;*             Call the velocity routines
;****************************************
CALL_DECEL     LDA MUS
               BNE SPEC_DECEL

               LDA DROP_NOT
               BEQ DROPPING
               LDY OBJ_REST
               LDA SPECTL-1,Y
               STA SPEC_OFF
               LDA SPECTH-1,Y
               STA SPEC_OFF+1
               LDY #$0A
               LDA (SPEC_OFF),Y
               AND #$1F ;; 2009-09: MY: Added this to make sure we can't run off the end of SLOP_TAB
               TAY

               LDA SLOP_TAB,Y
               BMI DROPPING

               ; 2009-09: MY: These next two instructions aren't needed, as we
               ;              already have the shape number in the Y-reg
               AND #$0F
               TAY

               LDA DRP_J_TABH,Y
;               BNE DRP_J_OK ; 2009-09: MY: If we don't have a valid jump address, don't try to jump there!
;               RTS
;DRP_J_OK
               STA LEAP_OFF+2
               LDA DRP_J_TABL,Y
               STA LEAP_OFF+1
LEAP_OFF       JMP SLOP4_5 ; smc

DROPPING       JSR NOPE2
               JMP NOPE3A

SPEC_DECEL     LDA DROP_NOT ; If we're dropping we don't have to check whether
               BEQ NO_SPEC  ; we're sitting on a slope.

               LDY OBJ_REST
               LDA SPECTL-1,Y
               STA SPEC_OFF
               LDA SPECTH-1,Y
               STA SPEC_OFF+1
               LDY #$0A
               LDA (SPEC_OFF),Y
               AND #$1F ;; 2009-09: MY: Added this to make sure we can't run off the end of SLOP_TAB
               TAY

               LDA SLOP_TAB,Y
               BMI NO_SPEC

               ; We're sitting on a slope, so look up what type

               ; 2009-09: MY: These next two instructions are harmless but not
               ;              needed, as we already have the shape number in
               ;              the Y-reg, and it must be less than $F to be a
               ;              slope.
               ;AND #$0F
               ;TAY

               LDA DRP_D_TABH,Y
;               BNE DRP_D_OK ; 2009-09: MY: If we don't have a valid jump address, don't try to jump there!
;               RTS
;DRP_D_OK
               STA LEAP_OFF2+2
               LDA DRP_D_TABL,Y
               STA LEAP_OFF2+1

               LDA DRP_Z_TABH,Y
;               BNE DRP_Z_OK ; 2009-09: MY: If we don't have a valid jump address, don't try to jump there!
;               RTS
;DRP_Z_OK
               STA SORT_VEL_X+2
               STA SORT_VEL_Y+2
               LDA DRP_Z_TABL,Y
               STA SORT_VEL_X+1
               STA SORT_VEL_Y+1

               LDX GAUGEPOINT

               ; 2009-09: MY: We were assuming here that the low-byte address
               ;              of CHECK_X is different from that of CHECK_Y.
               ;              Rather hokey, but I guess almost bound to work.
               ;              Nonetheless it's more obvious to get the slope
               ;              direction bit from SLOP_TAB.  This also allows us
               ;              to put the LDA/STA instructions above into a more
               ;              natural order.
               ;CMP #<CHECK_X
               ;BEQ SLOPE_ONY
               LDA SLOP_TAB,Y ; bit6 - 0=slope in x direction; 1=slope in y direction
               AND #%01000000
               BNE SLOPE_ONY

               ; We're sitting on an X-direction slope
               ;
               ; GAUGEPOINT at 6, 7, E or F means little or no X-component to the gauge direction
               CPX #6
               BEQ SORT_VEL_X
               CPX #7
               BEQ SORT_VEL_X
               CPX #14
               BEQ SORT_VEL_X
               CPX #15
               BEQ SORT_VEL_X

               JSR CHECK_X
               JMP LEAP_OFF2

               ;; 2009-09: MY: AFAIKT SORT_VEL_X and SORT_VEL_Y do the same
               ;;              thing (the JSR addresses are modified to the
               ;;              same thing at the same time).  So they could be merged.
SORT_VEL_X     JSR SLOP1_7 ; smc from DRP_Z_TABH/L: SLOP1_7 / SLOP4_5 / SLOP8_9 / SLOPC_D

               JMP LEAP_OFF2

SORT_VEL_Y     JSR SLOP1_7 ; smc from DRP_Z_TABH/L: SLOP1_7 / SLOP4_5 / SLOP8_9 / SLOPC_D
               JMP LEAP_OFF2

               ; We're sitting on a Y-direction slope
               ;
               ; GAUGEPOINT at 2, 3, A or B means little or no Y-component to the gauge direction
SLOPE_ONY      CPX #2
               BEQ SORT_VEL_Y
               CPX #3
               BEQ SORT_VEL_Y
               CPX #10
               BEQ SORT_VEL_Y
               CPX #11
               BEQ SORT_VEL_Y

               JSR CHECK_Y
LEAP_OFF2      JMP SLOP4_5 ; smc from DRP_D_TABH/DRP_D_TABL so = JMP CHECK_X or JMP CHECK_Y

               ; We're not sitting on a slope
NO_SPEC        JSR CHECK_Y
               ; JSR CHECK_X  Fall through to this instead!
               ; RTS

;******************************************************************************
;* Decrement "sprite"'s X speed if its X velocity is in opposite direction from
;* X-component of gauge direction or there is no X-component of gauge direction
;******************************************************************************
CHECK_X        LDY #$12
               LDA (IX),Y
               AND #$7F
               BEQ NO_DEC_X ; Done if current X-speed is 0

               LDX GAUGEPOINT
               LDA (IX),Y
               AND #$80
               STA MASK
               ; GAUGEPOINT at 6 or E means no X-component to the gauge direction
               CPX #6
               BEQ WINK
               CPX #14
               BEQ WINK

               EOR VEL_TAB,X
               BPL NO_DEC_X
WINK           LDA (IX),Y
               AND #$7F
               SEC
               SBC #1
               ORA MASK
               STA (IX),Y
NO_DEC_X       RTS


;******************************************************************************
;* Decrement "sprite"'s Y speed if its Y velocity is in opposite direction from
;* Y-component of gauge direction or there is no Y-component of gauge direction
;******************************************************************************
CHECK_Y        LDY #$13
               LDA (IX),Y
               AND #$7F
               BEQ NO_DEC_Y ; Done if current Y-speed is 0
               LDA (IX),Y
               AND #$80
               STA MASK
               LDX GAUGEPOINT ; 2009-09: MY: Added this
               ; GAUGEPOINT at 2 or A means no Y-component to the gauge direction
               CPX #2
               BEQ WONK
               CPX #10
               BEQ WONK

               EOR VEL_TAB2,X
               BPL NO_DEC_Y
WONK           LDA (IX),Y
               AND #$7F
               SEC

               SBC #1
               ORA MASK
               STA (IX),Y
NO_DEC_Y       RTS



;***************************************
;*             Set vel for dropping
;***************************************
SET_DROP_V     LDY #$14
               LDA (IX),Y ;velocity/direction in Z

               BMI ALREADY_DO ;branch if going down
               BNE STOP_UP

               LDA #$80 ;set to DOWN, zero velocity
               STA (IX),Y
ALREADY_DO     AND #$7F
               CMP #MAX_V
               BEQ FAST_ENUFF
               CLC

               ADC #1
               ORA #$80
               STA (IX),Y
FAST_ENUFF     RTS

STOP_UP        SEC
               SBC #1
               STA (IX),Y
               RTS


;****************************************
;*             Set up the accumulators
;****************************************
SET_SPEED      LDY #$12
               LDA (IX),Y ; velocity/direction in X - bit 7: 1=left/0=right;   bits 0-6: speed
               AND #$7F
               TAY
               LDA SPEED_TAB,Y
               LDY #$16
               STA (IX),Y ;x-co accum

               LDY #$13
               LDA (IX),Y ; velocity/direction in Y - bit 7: 1=forward/0=back; bits 0-6: speed
               AND #$7F
               TAY
               LDA SPEED_TAB,Y
               LDY #$17
               STA (IX),Y ;y-co accum

               LDY #$14
               LDA (IX),Y ; velocity/direction in Z - bit 7: 1=down/0=up;      bits 0-6: speed
               AND #$7F
               TAY
               LDA SPEED_TAB,Y
               LDY #$18
               STA (IX),Y ;z-co accum
               RTS


;******************************************************************************
;* Adjust speed on slopes
;*
;* 2009-09: MY: AFAIKT These routines:
;*               - slow us down by 1 in the direction perpendicular to the slope
;*               - slow us down by 2 in the slope direction if we are travelling up it
;*               - speed us up by 1 in the slope direction otherwise
;******************************************************************************
SLOPE1_7       JSR NOPE2 ; Slow down X speed by 1, unless it is already 0 (so eg 4 -> 3, -4 -> -3, etc)
               JMP SLOP1_7

SLOPE4_5       JSR SLOP4_5
               JMP NOPE3A ; Slow down Y speed by 1, unless it is already 0

SLOPE8_9       JSR NOPE2 ; Slow down X speed by 1, unless it is already 0
               JMP SLOP8_9

SLOPEC_D       JSR SLOPC_D
               JMP NOPE3A ; Slow down Y speed by 1, unless it is already 0


;******************************************************************************
;* Acceleration/deceleration handler for slopes 4 & 5 (downwards X slopes):
;*      4 = 30' downwards X slope
;*      5 = 45' downwards X slope
;*
;* 2009-09: MY: It seems we don't distinguish between the two steepness of the
;*              slopes for determining how much it slows down/speeds up things
;*              travelling on it.  Presumably players don't notice this...
;******************************************************************************
SLOP4_5        LDY #$12
               LDA (IX),Y     ;l/r velocity (bit 7 is direction)
               AND #$7F
               BEQ RIGHT4_5
               LDA (IX),Y
               BMI LEFT4_5

               ; Velocity is already positive in X direction (or zero), so
               ; assuming we haven't reached maximum velocity, being on the
               ; slope speeds us up by one unit in positive X direction.
               CMP #MAX_V
               BEQ GO_R
               BCC RIGHT4_5
               LDA #MAX_V
               JMP YEAH2

RIGHT4_5       TAX
               INX
               TXA
YEAH2          STA (IX),Y

GO_R           RTS

               ; Velocity is negative in X direction, so being on the slope
               ; slows us down by two units until we reach 0.
LEFT4_5        AND #$7F
               TAX
               DEX
               BEQ ENUFF1
               DEX

ENUFF1         TXA
               ORA #$80
               STA (IX),Y
NOWT1_7        RTS


;******************************************************************************
;* Acceleration/deceleration handler for slopes C & D (upwards X slopes):
;*      C = 30' upwards X slope
;*      D = 45' upwards X slope
;******************************************************************************
SLOPC_D        LDY #$12

               LDA (IX),Y     ;l/r velocity
               AND #$7F
               BEQ LEFTC_D

               LDA (IX),Y
               BPL RIGHTC_D
               AND #$7F
               CMP #MAX_V
               BEQ GO_L
               BCC LEFTC_D

               LDA #$80+MAX_V
               JMP YEAH1

LEFTC_D        TAX
               INX

               TXA

               ORA #$80
YEAH1          STA (IX),Y

GO_L           RTS

RIGHTC_D       TAX
               DEX
               BEQ ENUFF2
               DEX
ENUFF2         TXA

               STA (IX),Y
               RTS


;******************************************************************************
;* Acceleration/deceleration handler for slopes 1 & 7 (upwards Y slopes):
;*      1 = 45' upwards Y slope
;*      7 = 30' upwards Y slope
;******************************************************************************
SLOP1_7        LDY #$13

               LDA (IX),Y     ;f/b velocity
               AND #$7F
               BEQ FOR1_7
               LDA (IX),Y
               BPL BACK1_7

               AND #$7F
               CMP #MAX_V
               BEQ GO_FB1_7
               BCC FOR1_7
               LDA #$80+MAX_V
               JMP YEAH4

FOR1_7         TAX
               INX
               TXA

               ORA #$80
YEAH4          STA (IX),Y

GO_FB1_7       RTS

BACK1_7        TAX
               DEX

               BEQ ENUFF3
               DEX

ENUFF3         TXA

               STA (IX),Y
               RTS


;******************************************************************************
;* Acceleration/deceleration handler for slopes 8 & 9 (downwards Y slopes):
;*      8 = 30' downwards Y slope
;*      9 = 45' downwards Y slope
;******************************************************************************
SLOP8_9        LDY #$13

               LDA (IX),Y     ;f/b velocity
               AND #$7F
               BEQ BACK8_9

               LDA (IX),Y
               BMI FOR8_9
               CMP #MAX_V
               BEQ GO_FB8_9
               BCC BACK8_9

               LDA #MAX_V
               JMP YEAH3

BACK8_9        TAX
               INX
               TXA
YEAH3          STA (IX),Y

GO_FB8_9       RTS

FOR8_9         AND #$7F
               TAX
               DEX
               BEQ ENUFF4
               DEX

ENUFF4         TXA
               ORA #$80
               STA (IX),Y
               RTS


;******************************************************************************
;* Slow down X speed by 1, unless it is already 0 (so eg 4 -> 3, -4 -> -3, etc)
;******************************************************************************
NOPE2          LDY #$12

               LDA (IX),Y     ;l/r velocity
               AND #$7F
               BEQ NOWT

               LDA (IX),Y
               BMI LEFT_Y
               TAX
               DEX
               TXA
               STA (IX),Y
               RTS

LEFT_Y         AND #$7F
               TAX
               DEX
               TXA
               ORA #$80
               STA (IX),Y
NOWT           RTS


;******************************************************************************
;* Slow down Y speed by 1, unless it is already 0 (so eg 4 -> 3, -4 -> -3, etc)
;******************************************************************************
NOPE3A         LDY #$13
               LDA (IX),Y     ;f/b velocity
               AND #$7F
               BEQ NOWT ; = RTS
               LDA (IX),Y
               BMI FOR_Y

               TAX
               DEX
               TXA
               STA (IX),Y
               RTS

FOR_Y          AND #$7F
               TAX
               DEX
               TXA
               ORA #$80
               STA (IX),Y
               RTS


;****************************************
;*             Set up trap doors/exit/bottle
;****************************************
TRAP_CHEC      LDY #3

               LDA (SPEC_OFF),Y              ;are we at least 8 pixels onto the trap/exit

               SEC

               SBC (IX),Y ;IX x-co lo
               BCC OTHER_WAY

               CMP #8
               BCC WITHIN
VOLCANO        JMP EXIT_C1
OTHER_WAY      CLC
               ADC #8
               BCC VOLCANO

WITHIN         LDY #5
               LDA (SPEC_OFF),Y
               SEC

               SBC (IX),Y
               BCC OTHER_II
               CMP #8
               BCC INSIDE
               JMP EXIT_C1

OTHER_II       CLC

               ADC #8
               BCS INSIDE
               JMP EXIT_C1

INSIDE         LDY #$0A
               LDA (SPEC_OFF),Y
               CMP #$15
               BEQ EXIT_STUFF ;if exit then jump to exit routine

               CMP #$11
               BEQ BOTTLE_STUFF
               LDA #$08
               STA TRAP
               LDA #0
               TAY

               STA (IX),Y ;blob shape
               TAX

PAIN           STX TRAP_NUM
               LDY #2
CHEC_AGAIN     LDA TRAP2,X    ;compare trap door coors with table to see
               CMP (SPEC_OFF),Y              ;which trap door it is

               BNE YET_AGAIN

               INY

               INX

               CPY #7
               BNE CHEC_AGAIN
               LDY #2
LDA_AGAIN      LDA TRAP2,X    ;pick up new blob coors (3/d)
               STA (IX),Y
               INY

               INX

               CPY #7
               BNE LDA_AGAIN

               LDX #$03
               LDA #$0D
               CLC

               ADC TRAP_NUM
               TAY

LDX_AGAIN      LDA TRAP2,Y    ;pick up new screen coors (2/d)
               STA SCR_LR,X
               DEY

               DEX

               BPL LDX_AGAIN

               LDA #0
               STA DROP_NOT
               STA IXTAB+$12

               STA IXTAB+$13

               STA IXTAB+$14

               JMP CORKING

NO_WAY_HOSE
               RTS

YET_AGAIN      LDA TRAP_NUM   ;check next trap door

               CLC

               ADC #$0E
               TAX

               JMP PAIN

;****************************************
;*             Deal with the exits
;****************************************
EXIT_STUFF     LDA BOTTLES    ;Exit open ?
               BEQ GO_NEXT

               LDA #2         ;Print up Exit closed
               STA EXITO

               JSR EXITOPEN
               JMP NOT_IMPORT
GO_NEXT        LDA #0         ;Clear exit open sign

               STA EXITO

               JSR EXITOPEN
               LDA #3         ;Set up flag for mainloop to catch

               STA EXITO      ;and realise we're on the exit
               DEC IXTAB

               RTS

;****************************************
;*             Deal with the bottles
;****************************************
NO_CORK        JMP EXIT_C1
BOTTLE_STUFF   LDA CORK
               BEQ NO_CORK

               LDA #FLIPIN
               STA $01

               JSR CORKING ;sound
               LDY TABLE_ENT

               LDA ENT_TAB-1,Y

               ASL ; = ASL A

               TAY

               LDA BOTT-2,Y
               STA CELL_OFF
               LDA BOTT-1,Y
               STA CELL_OFF+1
               LDA #0
               STA BOTT-2,Y
               STA BOTT-1,Y
               STA CORK

               LDY STAGE
               LDA BOTT_NO,Y
               SEC
               SBC BOTTLES
               ASL ; = ASL A

               LDX PLAYER
               BEQ PLAY_UNE
               CLC
               ADC #$14*2
PLAY_UNE       TAY

               LDA CELL_OFF
               STA BOTT_1,Y
               INY

               LDA CELL_OFF+1
               STA BOTT_1,Y
               LDY #1
               LDA (CELL_OFF),Y
               AND #%01110000
               ORA #$82
               STA (CELL_OFF),Y
               DEC BOTTLES

               LDX PLAYER
               DEC BOTTLES+1,X

               JSR PRINT_NO
               LDA #2
               STA UNITS+2

               LDA #1
               STA UNITS+1

               STA UNITS

               JSR ADDSCORE


               ;Plot the cork on the screen
               ;

               ;set STRIPX = (x(3d)+y(3d))

               ;set STRIPY = $8000 -z -z-dim +x(3d)/2 -y(3d)/2

               LDY #$3
               LDA (SPEC_OFF),Y ;x-co lo
               STA ZPA
               DEY ;=$2
               LDA (SPEC_OFF),Y ;x-co hi
               STA ZPA+1

               LDY #$5
               LDA (SPEC_OFF),Y ;y-co lo
               STA ZPB
               DEY ;=$4
               LDA (SPEC_OFF),Y ;y-co hi
               STA ZPB+1

               ;work out on-screen x-co
               LDA ZPA ;x-co lo temp
               CLC
               ADC ZPB ;y-co lo temp
               STA STRIPX
               LDA ZPA+1 ;x-co hi temp
               ADC ZPB+1 ;y-co hi temp
               STA STRIPX+1

               LDA STRIPX
               SEC
               SBC SCREENX
               STA ZPG ;x-pos on screen ;no hi byte

               ;work out on-screen y-co
               LDA #$00
               SEC
               LDY #$19
               SBC (SPEC_OFF),Y ;z-co + z-dim (lo)
               STA ZPC

               LDA #$80
               SBC #0
               TAX ;hi

               LDA ZPA ;x-co lo temp
               LSR ZPA+1 ;x-co hi temp       ;divide by 2
               ROR ; = ROR A
               CLC
               ADC ZPC
               TAY ;lo
               TXA ;hi
               ADC ZPA+1
               TAX ;hi

               LSR ZPB+1 ;y-co hi temp       ;divide by 2
               ROR ZPB ;y-co lo temp

               TYA ;lo

               SEC

               SBC ZPB

               STA STRIPY
               TXA ;hi

               SBC ZPB+1

               STA STRIPY+1

               LDA STRIPY
               SEC
               SBC SCREENY
               CLC
               ADC #$40
               STA ZPG+1 ;y-pos on screen ;no hi byte

               JSR TRNPOS ;work out position in work area
               LDA ZPG

               STA INITSTRDAT
               LDA ZPG+1
               STA INITSTRDAT+1

               LDA STRIPY ;do bottom of strip
               CLC
               ADC #$18
               STA ZPD

               LDA STRIPY+1
               ADC #0
               STA ZPD+1

               JSR SET_SPECAL ;set up SPECAL table with objects on screen

               JSR PRVSTRIP ;print L/H vertical strip of bottle
               LDA INITSTRDAT ;move data ptr across
               CLC

               ADC #$02
               STA INITSTRDAT
               BCC BLDJK1
               INC INITSTRDAT+1
               CLC

BLDJK1         LDA STRIPX ;move x-co across
               ADC #$10
               STA STRIPX
               BCC BLDJK2
               INC STRIPX+1
BLDJK2         JSR PRVSTRIP ;print R/H vertical strip of bottle

;make time up to 99
               LDA #99

               SEC

               SBC TIME
               STA TIME2

               LDA BOTTLES

               BNE NO_OPEN

               LDA #1
               STA EXITO

               JSR EXITOPEN
NO_OPEN        JMP CORKOFF


TEMP_IX        DS.B $11,0


;***************************************
;*             Deal with trap
;****************************************
TRAP_ON        LDA TRAP       ;is trap activated
               BNE TRAP_GO

               RTS

TRAP_GO        LDA SCROLL     ;wait until finished scrolling
               BEQ EXIT_TRAP

               RTS

EXIT_TRAP      DEC TRAP       ;wait for samll delay

               LDA TRAP
               CMP #1
               BEQ RESET_TRAP
               RTS

RESET_TRAP     INC IXTAB      ;blob shape
               DEC TRAP       ;reset blob
               LDA #0
               STA DROP_NOT
               LDA #6
               STA UNITS+1

               LDA #1
               STA UNITS

               JSR ADDSCORE
               JMP CORKON


;****************************************
;*             Sort out timers. if Time2 is
;*             not = 0 then inc time else
;*             dec time (if lopcnt = 7)
;****************************************
DO_TIME        LDA TIME2
               BEQ DEC_IT
               DEC TIME2

               INC TIME
               JMP GODEC1A

DEC_IT         LDA LOPCNT
               AND #7
               BNE NO_TIME

               JMP DECTIMER1

NO_TIME        RTS


;****************************************
;*             Check the blob against all other
;*             IX objects that are on.
;****************************************
CHECK_IX       LDY #12
               STY SPR_NO
               LDA #0
               STA DEAD_YET
               STA LOWES

               LDA #<IXTAB
               STA IY
               LDA #>IXTAB
               STA IY+1
               JSR RE_CHECK

               INC LOWES

               LDA #0
               STA DEAD_YET+1
               LDA ALTABS ;shape
               BEQ NOT_DEAD1

               CMP #7 ;explosion

               BEQ NOT_DEAD1

               LDY #6
               STY SPR_NO
               LDA #<ALTABS
               STA IY
               LDA #>ALTABS
               STA IY+1
               JSR RE_CHECK
               LDA DEAD_YET+1
               BEQ NOT_DEAD1

               JSR ALI_SCR

               JSR BANG
               LDA #7 ;explosion

               LDY #0
               STA (IY),Y ;shape
               TYA
               INY
               STA (IY),Y ;data offset

NOT_DEAD1      INC LOWES
               LDA ALTABS+IXSIZ
               BEQ NOT_DEAD2

               CMP #7 ;explosion

               BEQ NOT_DEAD2

               LDY #6
               STY SPR_NO
               LDA #0
               STA DEAD_YET+2
               LDA #<[ALTABS+IXSIZ]
               STA IY
               LDA #>[ALTABS+IXSIZ]
               STA IY+1
               JSR RE_CHECK
               LDA DEAD_YET+2
               BEQ NOT_DEAD2

               JSR ALI_SCR

               JSR BANG
               LDA #7 ;explosion

               LDY #0
               STA (IY),Y ;shape
               TYA
               INY
               STA (IY),Y ;data offset

NOT_DEAD2      RTS


RE_CHECK       LDA SPRTABL,Y  ;blob
               STA IX
               LDA SPRTABH,Y
               STA IX+1

               LDY #0
               STY WHICH_3D
               LDA (IX),Y ;IX shape
               BEQ NEXT_ONE
               CMP #$0E
               BNE NO_WORRY
               LDA (IY),Y ;IY shape
               CMP #$9
               BEQ NEXT_ONE

NO_WORRY       LDY LOWES
               BNE DANK

               CMP #5
               BEQ NEXT_ONE
DANK           JSR SPRITE_3D
               TAX

               ;
               ; Extra code for the cheat that stops aliens killing us:
               ;  - first check it's the blob's collision we're looking at
               ;  - if it is, check the cheat is activated for the current
               ;    player
               ;  - if so, skip over the bit that might flag us as dying
               ;
               LDA LOWES
               BNE NOT_BLOB
               LDY PLAYER ; If relevant cheat mode is on, don't decrement time
               LDA CHEAT_MODE,Y
               AND #CHEAT_NO_ALIEN_KILL
               BNE NEXT_ONE
NOT_BLOB

               LDY #$09
               LDA (IX),Y ;top 4 bits - dissolve info
               AND #$F0
               BNE NEXT_ONE
               TXA

               LDX LOWES
               STA DEAD_YET,X
               AND #$FF
               BNE EXIT_4D

NEXT_ONE       DEC SPR_NO
               LDY SPR_NO
               CPY LOWES

               BNE RE_CHECK
EXIT_3D        RTS


EXIT_4D        LDY #0
               LDA (IX),Y ;shape
               CMP #$0E
               BEQ EXIT_5D

               CMP #5
               BNE EXIT_3D

EXIT_5D        TYA ;=0
               STA (IX),Y ;shape

               RTS


ALI_SCR        LDA #3
               STA UNITS
               LDA #5
               STA UNITS+1
               LDA #1
               STA UNITS+2
               JMP ADDSCORE


;****************************************
;*             Actual check the sprite and
;*             see if it is within the confines
;*             of the blob. If it is then pick
;*             up a number from a table and move
;*             the blob so it is on the edge of the
;*             object.
;****************************************
SPRITE_3D      LDY #0
               LDA (IX),Y ;IX shape
               CMP #$0C ;materialisation pad

               BEQ NOT_WITH

               ;this routine assumes max difference between high bytes is 1
               LDY #3
               SEC

               LDA (IY),Y ;IY x-co lo
               SBC (IX),Y
               TAX

               DEY ;=2

               LDA (IY),Y ;IY x-co hi
               SBC (IX),Y
               TXA

               BCS OTHER_SIDE
               ;clc
               ADC #$0C ;blob x-dim
               BCS WHY_EDGE
NOT_WITH       LDA #0
               RTS

OTHER_SIDE     STA ZPA ;temp

               LDY #$0B
               LDA (IX),Y ;IX x-dim
               ;sec
               SBC ZPA ;temp^
               BCC NOT_WITH
               ORA #$80
WHY_EDGE       STA IXTB       ;distance inside the x

               AND #$7F
               STA ZOP

               ;this routine assumes max difference between high bytes is 1
               LDY #5
               SEC

               LDA (IY),Y ;IY y-co lo
               SBC (IX),Y
               TAX

               DEY ;=4

               LDA (IY),Y ;IY y-co hi
               SBC (IX),Y
               TXA

               BCS OTHER_WHY

               ;clc
               ADC #$0C
               BCS GO_4_IT

               LDA #0
               RTS

OTHER_WHY      STA ZPA ;temp
               LDY #$0C
               LDA (IX),Y ;IX y-dim
               ;sec
               SBC ZPA ;temp^
               BCC NOT_WITH
               ORA #$80
GO_4_IT        STA IXTB+1     ;distance within y
               AND #$7F
               STA ZOP+1

               LDY #6         ;do zed
               SEC

               LDA (IY),Y ;IY z-co (lo)
               SBC (IX),Y ;IX z-co (lo)
               BCS OTHER_ZED

               ;clc
               LDY #$0D
               ADC (IY),Y ;IY z-dim
               BCS FCKTHS
               LDA #0
               RTS


SPR_SHAD       LDA WHICH_3D
               BNE NOT_WITH
               LDY #$0D
               LDA (IX),Y ;IX z-dim
               LDY #$06
               CLC

               ADC (IX),Y ;IX z-co (lo)
               STA Z_TEMP
               LDA (IY),Y ;IY z-co (lo)
               SEC

               SBC Z_TEMP
               LDY #$11
               CMP (IY),Y ;IY object height above its shadow

               BCS NOT_WITH   ;GO_NOT_W
               ;clc
               ADC #1
               STA (IY),Y ;IY object height above its shadow

               JSR SPR_SHADO

GO_NOT_W       JMP NOT_WITH


OTHER_ZED      STA ZPA ;temp
               LDY #$0D
               LDA (IX),Y ;IX z-dim
               ;sec
               SBC ZPA ;temp

               BCC SPR_SHAD ;branch if IY shape is above top of IX shape

               LDA #$84

FCKTHS         STA IXTB+2     ;distance within z

               AND #$7F
               STA ZOP+2

PING           LDY WHICH_3D
               BEQ PING_2
NBUMP          LDA #1
               RTS

PING_2         LDY #0
               LDA (IX),Y ;shape

               TAY

               CPY #$0F ;mystery bubble
               BEQ MYSTERY

               LDA KILL_SPR,Y
               BEQ NO_FEAT

               RTS

MYSTERY        LDY #1

               LDA (IX),Y ;data offset

               TAY

               LDA MYST_COLL,Y

               BEQ NBUMP

               ;Make the decision whether to update
               ;x, y or z of blob to that of the IX shape
               CMP #1
               BEQ NO_FEAT

               JMP DEAL_FEAT

NO_FEAT        LDA ZOP+2
               CMP ZOP

               BCS SET_EX
               LDY #2
TRY_WHY        CMP ZOP+1
               BCC DONE_IT

               LDY #1
DONE_IT        LDA LO_JMP,Y

               STA JUMP1+1

               LDA HI_JMP,Y
               STA JUMP1+2

JUMP1          JMP LAST_Z ;smc
SET_EX         LDA ZOP

               LDY #0
               JMP TRY_WHY



LO_JMP         BYTE <LAST_X, <LAST_Y, <LAST_Z

HI_JMP         BYTE >LAST_X, >LAST_Y, >LAST_Z



LAST_Z         LDA IXTB+2     ;reposition blob to zed of IX shape
               BPL SPR_BOT

               ;put IY shape on top of IX shape
NO_IGNOR       LDY #$0D
               LDA (IX),Y ;IX z-dim
               CLC

               LDY #6
               ADC (IX),Y ;IX z-co (lo)
               ADC #1
               STA (IY),Y ;IY z-co (lo)
               LDA #0
               LDY #1
               STA (IY),Y ;data offset

               LDY #$19
               LDA SPR_NO
               STA (IY),Y ;SPR_NO of shape that IY shape is resting on

               LDA #$FF ;set no shadow

               LDY #$11
               STA (IY),Y ;object height above its shadow
               JSR SPR_SHADO

               LDA #1
               STA DROP_NOT
               LDA #0
               RTS



SPR_BOT        LDA IXTB+2

               CMP #4
               BCS NO_IGNOR
               LDY #$06
               LDA (IX),Y ;IX z-co (lo)
               SEC

               LDY #$0D
               SBC (IY),Y ;IY z-dim
               SBC #1
               BEQ NO_ZERO

               BCC NO_ZERO

               CMP TOP_REST
               BEQ STOP_DOW
               BCC STOP_DOW
SET_ZED        LDY #$06

               STA (IY),Y ;IY z-co (lo)
               LDA #0
               RTS



NO_ZERO        LDA #1

               JMP SET_ZED



STOP_DOW       LDA #0
               LDY #$14
               STA (IX),Y ;IX velocity/direction in Z
               LDA TOP_REST
               JMP SET_ZED


LAST_X         LDA IXTB       ;deal with x position, reposition blob on x
               BPL SPR_LEF

               LDY #$0B
               LDA (IX),Y ;IX x-dim
               CLC

               LDY #3
               ADC #1
               ADC (IX),Y ;IX x-co lo
               STA (IY),Y ;IY x-co lo
               DEY ;=2

               LDA (IX),Y ;IX x-co hi
               ADC #0
               STA (IY),Y ;IY x-co hi
               LDA #0
               RTS



SPR_LEF        LDY #$03

               LDA (IX),Y ;IX x-co lo
               SEC

               LDY #$0B
               SBC (IX),Y ;IX x-dim
               LDY #$03
               STA (IY),Y ;IY x-co lo
               DEY ;=2

               LDA (IX),Y ;IX x-co hi
               SBC #0
               STA (IY),Y ;IY x-co hi
               LDA #0
               RTS


LAST_Y         LDA IXTB+1     ;deal with the y value, reposition blob on y

               BPL SPR_FOR

               LDY #$0C
               LDA (IX),Y ;IX y-dim
               CLC

               LDY #5
               ADC #1
               ADC (IX),Y ;IX y-co lo
               STA (IY),Y ;IY y-co lo
               DEY ;=4

               LDA (IX),Y ;IX y-co hi
               ADC #0
               STA (IY),Y ;IY y-co hi
               LDA #0
               RTS



SPR_FOR        LDY #$05

               LDA (IX),Y ;IX y-co lo
               SEC

               LDY #$0C
               SBC (IX),Y ;IX y-dim
               LDY #$05
               STA (IY),Y ;IY y-co lo
               DEY ;=4

               LDA (IX),Y ;IX y-co hi
               SBC #0
               STA (IY),Y ;IY y-co hi
               LDA #0
               RTS


               ;Set up shadow for shape pointed to by IY
SPR_SHADO      LDA IX
               PHA
               LDA IX+1
               PHA

               LDA IY
               STA IX
               LDA IY+1
               STA IX+1
               ;set up IY to point to SHADO table for this shape

               LDY #$15
               LDA (IX),Y ;IX table #
               TAY

               LDA LSHAD_TAB,Y

               STA IY
               LDA HSHAD_TAB,Y

               STA IY+1
               ;set up shadow for shape pointed to by IX

               JSR TRSHAD
               LDA IX
               STA IY
               LDA IX+1
               STA IY+1

               PLA
               STA IX+1
               PLA
               STA IX
               RTS



ALI_DEAD       TYA
               PHA

               DEY
               CPY #2
               BCC YEAS_ALI
EXT_ALI        PLA
               TAY
               RTS

YEAS_ALI       LDY #0
               LDA (IX),Y ;shape
               BEQ EXT_ALI

               LDY #$06
               LDA (IX),Y ;z-co (lo)
               CMP #$FD
               BCC EXT_ALI

               JSR BANG
               LDA #7 ;explosion

               LDY #0
               STA (IX),Y ;shape

               TYA ;=0

               INY ;=1

               STA (IX),Y ;data offset

               LDY #$06
               STA (IX),Y ;z-co (lo)

               LDY #$14
               STA (IX),Y ;velocity/direction in Z

               PLA
               TAY
               RTS


;****************************************
;*             Deal with the features like extra
;*             lives, 1000 points, extra time and
;*             mystery
;****************************************
DEAL_FEAT      PHA
               LDY #0
               TYA
               STA (IX),Y ;shape

               PLA

               CMP #2
               BEQ F_MYST
               CMP #3
               BEQ F_MYST
               CMP #4
               BEQ F_CLOCK

               CMP #5
               BEQ F_EXTRA

F_POINTS       LDA #1
               STA UNITS
               STA UNITS+1
               STA UNITS+2
               STA UNITS+3
               JSR ADDSCORE
               LDA #0
               RTS

F_CLOCK        LDA #99
               SEC

               SBC TIME
               STA TIME2

               LDA #0
               RTS

F_MYST         LDA #0
               RTS

F_EXTRA        LDX PLAYER

;               TXA
;               ASL ; = ASL A
;               TAY
;               CLC
;               LDA LIVES+1,Y
;               ADC #1
;               STA LIVES+1,Y
;               STA NUM2
;               LDA LIVES,Y
;               ADC #0
;               STA LIVES,Y
;               STA NUM1
;               JSR PLOTNUM

               LDA LIVES,X
               CMP #$99 ; = BCD for #99
               BEQ REACHED_MAX_LIVES
               SED
               ADC #1
               CLD
               STA LIVES,X
               JSR PLOT_BCDNUM
REACHED_MAX_LIVES
               LDA #0
               RTS


;****************************************
;*             Let them baddy guns rip !!
;****************************************
QUAAGAR        LDX #2         ;work out which IX object is gun
               LDA RND0
               AND #1
               BEQ NEXT_OBJE

               LDA #<STATOB
               STA IY
               LDA #>STATOB
               STA IY+1
               LDA STATOB
               CMP #$10
               BNE NEXT_OBJE

               LDA STATOB+1
               CMP #1
               BEQ THIS_ONE
NEXT_OBJE      LDA RND1

               AND #2
               BEQ NEXT_OBJD

               LDA #<[STATOB+IXSIZ]
               STA IY
               LDA #>[STATOB+IXSIZ]
               STA IY+1
               LDA STATOB+IXSIZ
               CMP #$10
               BNE NEXT_OBJD

               LDA STATOB+1+IXSIZ
               CMP #1
               BEQ THIS_ONE
NEXT_OBJD      LDA #<[STATOB+[IXSIZ*2]]

               STA IY
               LDA #>[STATOB+[IXSIZ*2]]
               STA IY+1
               LDA STATOB+[IXSIZ*2]
               CMP #$10
               BNE NEXT_OBJC

               LDA STATOB+1+[IXSIZ*2]
               CMP #1
               BEQ THIS_ONE
NEXT_OBJC      LDA #<[STATOB+[IXSIZ*3]]

               STA IY
               LDA #>[STATOB+[IXSIZ*3]]
               STA IY+1
               LDA STATOB+[IXSIZ*3]
               CMP #$10
               BNE NO_OBJECTS
               LDA STATOB+1+[IXSIZ*3]
               CMP #1
               BEQ THIS_ONE
NO_OBJECTS     LDA ALTABS
               CMP #9
               BNE NO_SAUCE
               LDA #<ALTABS
               STA IY
               LDA #>ALTABS
               STA IY+1
               JMP THIS_ONE
NO_SAUCE       LDA ALTABS+IXSIZ
               CMP #9
               BNE NO_SAUCES

               LDA #<[ALTABS+IXSIZ]
               STA IY
               LDA #>[ALTABS+IXSIZ]
               STA IY+1
               JMP THIS_ONE
NO_SAUCES      RTS
THIS_ONE       LDY #2
               LDA #<SFIRE

               STA IX
               LDA #>SFIRE

               STA IX+1
               LDA SFIRE

               BEQ YES_FIRE
               LDA #<[SFIRE+IXSIZ]

               STA IX
               LDA #>[SFIRE+IXSIZ]

               STA IX+1
               LDA SFIRE+IXSIZ

               BEQ YES_FIRE
               RTS

YES_FIRE       LDA LOPCNT
               EOR RND0
               AND #$93
               BEQ DEF_YES_FI
               RTS

DEF_YES_FI     LDA (IY),Y
               STA (IX),Y
               INY

               LDA (IY),Y
               ADC #2
               STA (IX),Y
               INY

               LDA (IY),Y
               STA (IX),Y
               INY

               LDA (IY),Y
               ADC #4
               STA (IX),Y
               INY

               LDA (IY),Y
               ADC #6
               STA (IX),Y
               INY

               LDA #3
               STA MAX_SPED
               JSR WORK_QUAD

               LDX #0
               LDA LOPCNT
               AND #1
               BEQ NO_VEL
               LDX #7
NO_VEL         TXA

               DEY

               ORA (IX),Y
               STA (IX),Y
               LDX #0
               LDA RND1
               AND #2
               BEQ NO_VEL_A
               LDX #7
NO_VEL_A       TXA

               INY
               ORA (IX),Y
               STA (IX),Y
               AND #7
               DEY

               ORA (IX),Y
               AND #$7F
               BNE NO_VEL_B
               LDA (IX),Y
               ORA #7
               STA (IX),Y
NO_VEL_B       LDA #$0E
               LDY #0
               STA (IX),Y
               TYA

               INY

               STA (IX),Y
               RTS



WORK_QUAD      JSR QUADRANT

               PHA

               TXA

               AND #1
               LSR ; = LSR A ;

               ROR ; = ROR A ;move bit0 to bit7
               LDY #$12
               STA (IX),Y ;velocity/direction in X

               TXA

               AND #2
               LSR ; = LSR A ;

               LSR ; = LSR A ;

               ROR ; = ROR A ;move bit1 to bit7
               INY ;=$13

               STA (IX),Y ;velocity/direction in Y

               PLA

               RTS



;****************************************
;*             Work out which quadrant the object
;*             with the IX coors is from the blob
;*
;*             Places the direction inside the first
;*             two bits of X:
;*                            bit0 - 1=-X 0=+X
;*                            bit1 - 1=-Y 0=+Y
;*
;*             Accumulator shows whether object
;*             has same co-ords as blob:
;*                            bit0 - 0 = x-coords same
;*                            bit1 - 0 = y-coords same
;****************************************
QUADRANT       LDX #1
               TXA

               PHA

               LDY #2
               LDA (IX),Y ;IX x-co hi
               CMP IXTAB+2 ;BLOB x-co hi

               BCC LEFT_Q
ANOTH_1        INY ;=3
               LDA (IX),Y ;IX x-co lo
               CMP IXTAB+3 ;BLOB x-co lo

               BEQ SOMETHING

               BCS ANOTH_2

LEFT_Q         DEX

ANOTH_2        PLA
               ORA #2
               PHA

               LDY #4
               LDA (IX),Y ;IX y-co hi
               CMP IXTAB+4 ;BLOB y-co hi

               BEQ ANOTH_3

               BCC FRONT_Q

               JMP BACK_Q
ANOTH_3        INY ;=5
               LDA (IX),Y ;IX y-co lo
               CMP IXTAB+5 ;BLOB y-co lo

               BEQ SOMETHING2
               BCC FRONT_Q

BACK_Q         TXA

               ORA #2
               TAX

FRONT_Q        PLA
               RTS



SOMETHING      PLA
               LDA #0
               PHA

               JMP LEFT_Q


SOMETHING2 PLA

               AND #255-2
               PHA

               JMP BACK_Q


;****************************************
;*             Aliens
;*
;*             Four major routines. One moves the
;*             aliens, if created.(ALIEN)
;*             One creates new aliens (CREATE)
;*             One turns aliens off if off screen (ON_OFF)
;*             One checks to see if alien is dead and if so, destroys it
;****************************************


;****************************************
;*             Work out alien movement routines
;****************************************
ALIEN          LDY SPR_NO

               BEQ NO_YOOH

               CPY #5
               BCS NO_YOOH

               JMP YOOH
NO_YOOH        RTS
YOOH           LDY #0
               LDA (IX),Y ;shape

               BEQ NO_YOOH

               CMP #5
               BEQ NO_YOOH

               CMP #$07
               BNE YES_ALIE2  ;is it explosion
               JMP EXPLOD
YES_ALIE2      LDY #$09       ;is it being rezzed up ?
               LDA (IX),Y
               PHA

               AND #$0F
               TAX

               PLA

               AND #$F0
               BEQ CH_DI_ALI1
               TAX

               LDA LOPCNT
               AND #3
               BNE ZIP_IT
               TXA

               ASL ; = ASL A

               STA (IX),Y
ZIP_IT         RTS

CH_DI_ALI1     JMP CH_DI_ALI

;make saucers rise up and then circle
SAUCERS        LDY #6

               LDA (IX),Y
               SEC

               SBC #4+$05
               CMP IXTAB+6 ;BLOB z-co (lo)

               BEQ CIRCLE1

               BCS SAUC_DO

SAUC_UP        TAX
               LDA IXTAB+$19

               CMP SPR_NO
               BEQ CIRCLE1

               TXA

               ADC #5+$05
               STA (IX),Y
               JMP CIRCLE1

SAUC_DO        LDA #0

               LDY #$1A
               STA (IX),Y ;bit0 set if gravity not to act on object
               JMP CIRCLE
CIRCLE1        LDA #1

               LDY #$1A
               STA (IX),Y
               LDA #0
               LDY #$14
               STA (IX),Y
CIRCLE         LDA #3
               STA MAX_SPED
               LDA LOPCNT
               AND #7
               BEQ PURSUE
               JMP DO_VELS


;****************************************
;*             Move pryamids so they follow you.
;****************************************
PURSUE         JSR WORK_QUAD  ;work out direction

SET_PUR        TAX
               AND #1
               BEQ NO_EXXX

               LDY #$12
               LDA (IX),Y
               ORA MAX_SPED
               STA (IX),Y ;velocity/direction in X

NO_EXXX        TXA
               AND #2
               BEQ NO_WHYY

               LDY #$13
               LDA (IX),Y
               ORA MAX_SPED
               STA (IX),Y ;velocity/direction in Y

NO_WHYY        RTS


;mystery bubble
MBUBB          LDA RND0

               AND #%1111100

               BNE NO_POP_MYS
               JSR GEN_RAN

               LDY #$1A
               LDA #0
               STA (IX),Y ;bit0 set if gravity not to act on object
NO_POP_MYS     RTS

;ordinary bubble
BUBB3          LDY #1

               LDA (IX),Y ;data offset

               CMP #6
               BEQ NEXT_STEP

               AND #4
               BEQ NO_WHYY

               LDA #3
               STA MAX_SPED
               LDY #$12
               LDA (IX),Y
               AND #$07
               INY

               ORA (IX),Y
               BEQ SET_BUBB_DIR
               LDA LOPCNT
               EOR RND1
               AND #$74
               BEQ POP_BUBB
               JMP DO_VELS

POP_BUBB       LDY #1
               LDA #6
               STA (IX),Y ;data offset

               RTS

NEXT_STEP      LDY #$1A

               LDA #0
               STA (IX),Y ;bit0 set if gravity not to act on this object

               JSR A_DEFIN

               LDY #0
               STA (IX),Y ;shape

               TYA

               INY ;=1

               STA (IX),Y ;data offset

STOP_DEF       RTS

SET_BUBB_DIR

               LDA #3
               STA MAX_SPED
               JSR GEN_RD
               LDY #$14
               LDA #2
               STA (IX),Y
               RTS


;move aliens

CH_DI_ALI      LDA #SHE

               STA MAX_SPED
               LDY #0
               LDA (IX),Y ;shape

               CMP #$11
               BEQ PURSUE1

               CMP #9
               BEQ SAUCERS1
               CMP #8
               BEQ BUBB3

               CMP #15

               BEQ MBUBB

               CMP #$0B
               BNE BALLS

; do crabs

               LDY #$14
               LDA (IX),Y
               AND #$7F
               LDY #$1A
               ORA (IX),Y ;bit0 set if gravity not to act on object
               BEQ NEW_QUAD
               LDY #$14
               LDA (IX),Y
               BEQ CRAB_DOWN

               AND #$80
               BEQ CRAB_UP

CRAB_DOWN      LDA (IX),Y

               ORA #$80
               STA (IX),Y
               LDA #$0

               LDY #$1A
               STA (IX),Y ;bit0 set if gravity not to act on object
NO_CRAB_UP     LDA #2
               STA MAX_SPED
               JMP DO_VELS

SAUCERS1       JMP SAUCERS

PURSUE1        JMP PURSUE


CRAB_UP        LDA LOPCNT

               AND #1
               BEQ NO_CRAB_UP
               LDA (IX),Y
               TAX

               DEX

               TXA

               STA (IX),Y
               JMP NO_CRAB_UP
NEW_QUAD       LDA RND0
               AND #$5

               BEQ CONT_PUR
               RTS

CONT_PUR       JSR PURSUE
               LDA #8
               LDY #$14
               STA (IX),Y
               LDA #1
               LDY #$1A
               STA (IX),Y ;bit0 set if gravity not to act on object
               RTS



; do rotating balls
BALLS          LDY #$1A

               LDA #0
               STA (IX),Y ;bit0 set if gravity not to act on object
               LDY #$12       ;update DIRECTION
               LDA (IX),Y
               AND #SHE+%111

               INY

               ORA (IX),Y
               AND #$7F
               BEQ DEAL_BOTH

               LDA #SHE
               STA MAX_SPED
DO_VELS        LDY #$12                      ;keep velocities up to speed

               LDA (IX),Y
               TAX

               AND #SHE+%111

               BEQ LEAVE_1

               TXA

               AND #$80
               ORA MAX_SPED
               STA (IX),Y
LEAVE_1        INY
               LDA (IX),Y
               TAX

               AND #SHE+%111

               BEQ LEAVE_2

               TXA

               AND #$80
               ORA MAX_SPED
               STA (IX),Y
LEAVE_2        RTS
;create direction for rotating balls

DEAL_BOTH      LDA #SHE

               STA MAX_SPED
               JMP GEN_RD
NO_ALI_FR      RTS


;****************************************
;*             Create aliens when a mat pad
;*             that has nothing already on it
;*             has been found.
;****************************************
CREATE_ALI     LDA RND1
               AND #7
               BNE NEXT_ALI
               LDA ALTABS     ;do alien one
               BNE NEXT_ALI
               LDA #<[ALTABS+IXSIZ]
               STA IX
               LDA #>[ALTABS+IXSIZ]
               STA IX+1
               LDA #<ALTABS
               STA SPEC_OFF
               LDA #>ALTABS
               STA SPEC_OFF+1
               JSR CREATE

NEXT_ALI       LDA LOPCNT     ;do alien 2
               AND #7
               BNE NO_ALI_FR

               LDA ALTABS+IXSIZ
               BNE NO_ALI_FR

               LDA #<[ALTABS+IXSIZ]
               STA SPEC_OFF
               LDA #>[ALTABS+IXSIZ]
               STA SPEC_OFF+1
               LDA #<ALTABS
               STA IX
               LDA #>ALTABS
               STA IX+1

;****************************************
;*             Create aliens by first checking to see
;*             if there is a free bottle and, if not then
;*             check all static objects to see if they
;*             are mat pads and are free. If so then create
;*             one of the two sprites
;****************************************
CREATE         JSR BUBBLES
               BNE NO_ALI_FR

               LDA #1
               STA WHICH_3D

               ;check static objects to see if there is a mat pad
               ;and if so then is there another alien on it
               LDA #4
               STA YELLO ;counter - number of static objects

               LDA #<STATOB
               STA IY
               LDA #>STATOB
               STA IY+1
PSDOF1         LDY #0
               LDA (IY),Y ;static "moving" shape
               CMP #$0C ;materialisation pad
               BNE NTGTPAD

               LDA (IX),Y ;shape
               BEQ GOT_PAD

               JSR SPRITE_3D
               BEQ GOT_PAD

NTGTPAD        LDA #IXSIZ
               CLC
               ADC IY
               STA IY
               BCC PSDOF2
               INC IY+1
PSDOF2         DEC YELLO ;counter
               BNE PSDOF1
               RTS


;****************************************
;*             Got a pad, now define alien
;****************************************
GOT_PAD        JSR A_DEFIN
               LDY #0
               STA (SPEC_OFF),Y
               TYA

               INY

               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               DEC WHICH_3D
               LDA SPEC_OFF
               STA IX
               LDA SPEC_OFF+1
               STA IX+1
               LDA #<IXTAB

               STA IY
               LDA #>IXTAB

               STA IY+1
               JSR SPRITE_3D  ;move blob if overlapping alien

NOT_ON_TOP     LDY #$09
               LDA #$40
               STA (SPEC_OFF),Y              ;set up position and rezz
               LDY #$12
               LDA #0
               STA (SPEC_OFF),Y
               INY

               STA (SPEC_OFF),Y
               LDA #1
               LDY #$1A
               STA (SPEC_OFF),Y ;bit0 set if gravity not to act on object
               JMP MATERIAL
;^ Heavy routine. Possibly the most complex routine I have to write.

               ;Lucky you ! [Matt]



EXPLOD         INY                           ;blow up alien

               LDA (IX),Y
               TAX

               AND #$80
               BNE OFF_A

               TXA

               AND #$7F
               CMP #6
               BEQ OFF_A

               TAX

               INX

               TXA

               STA (IX),Y
               RTS

ZIP_ZAP
OFF_A          LDA #0

               TAY ;=0

               STA (IX),Y ;shape

               LDY #$12
               STA (IX),Y
               INY

               STA (IX),Y
               LDA #$1A
               STA (IX),Y ;bit0 set if gravity not to act on object
               RTS



;****************************************
;*             Animate aliens
;****************************************
BUBBS1         LDA ALTABS+1
               CMP #6
               BEQ NO_ANI
               AND #4
               BEQ GENRAT1

               LDA LOPCNT
               AND #4
               LSR ; = LSR A

               LSR ; = LSR A

               ORA #4
               STA ALTABS+1
               JMP NO_ANI
GENRAT1        INC ALTABS+1

               JMP NO_ANI
ALIEN_ANI      LDX ALTABS+1

               LDA ALTABS     ;animate rotating balls
               CMP #$0A
               BEQ BALLS1
               CMP #8
               BEQ BUBBS1
               CMP #$11
               BNE NO_ANI
PYRA1          LDA LOPCNT

               AND #3
               STA ALTABS+1
               JMP NO_ANI
BALLS1         TXA            ;alien 1
               CMP #4
               BEQ TOPS1

               INC ALTABS+1
               JMP NO_ANI
TOPS1          LDA #0

               STA ALTABS+1
NO_ANI         LDX ALTABS+IXSIZ+1
               LDA ALTABS+IXSIZ              ;alien 2

               CMP #$0A
               BEQ BALLS2
               CMP #8
               BEQ BUBBS2
               CMP #$11
               BNE DEF_NO_ANI
PYRA2          LDA LOPCNT

               ADC #2
               AND #3
               STA ALTABS+IXSIZ+1
               RTS

BALLS2         TXA

               CMP #4
               BEQ TOPS2

               INC ALTABS+IXSIZ+1
               RTS

TOPS2          LDA #0

               STA ALTABS+IXSIZ+1
DEF_NO_ANI     RTS



BUBBS2         LDA ALTABS+1+IXSIZ
               CMP #6
               BEQ DEF_NO_ANI
               AND #4
               BEQ GENRAT2

               LDA LOPCNT
               AND #4
               LSR ; = LSR A

               LSR ; = LSR A

               ORA #4
               STA ALTABS+1+IXSIZ
               RTS

GENRAT2        INC ALTABS+1+IXSIZ

OUT_OF_IT      RTS


;****************************************
;*             Check for bottles inside
;*             SPECAL table and if an
;*             unopened one is found then
;*             then generate a bubble
;****************************************
BUBBLES        LDX SPECAL

               LDY #$0A
BUBB_CHEC      LDA SPECTL-1,X

               STA IY
               LDA SPECTH-1,X
               STA IY+1
               LDA (IY),Y
               CMP #$11
               BEQ YES_FOUND

               DEX

               BPL BUBB_CHEC

               LDA #0
               RTS

YES_FOUND      LDA RND0

               EOR LOPCNT
               AND #4
               BEQ YES_GEN

               LDA #0
               RTS

YES_GEN        LDY #2

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               STA (SPEC_OFF),Y
               INY

               LDA (IY),Y
               CLC

               ADC #4
               STA (SPEC_OFF),Y
               LDY #$18
               LDA (IY),Y
               LDY #$06
               STA (SPEC_OFF),Y
               LDY #0
               LDA #8
               STA (SPEC_OFF),Y
               TYA

               INY

               STA (SPEC_OFF),Y
               LDY #$12
               LDA #0
               STA (SPEC_OFF),Y
               INY

               STA (SPEC_OFF),Y
               INY

               LDA #1
               STA (SPEC_OFF),Y
               LDY #$1A
               LDA #1
               STA (SPEC_OFF),Y ;bit0 set if gravity not to act on object
               RTS


;
; File X5.BUB
; Starts Here
;
;

;
;
;  PDS Pc3.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          10.26
;         By  "JAKE AND MATTHEW..."
;
;     Created on Wed the 22nd of Jul 1987
;        Last update 08:41 on 10/05/88
;

;****************************************
;*             Work out which defintion an alien
;*             could be given.
;****************************************

A_DEFIN        LDY #$0A

               LDA RND1
               AND #1
               BEQ GOT_ALI

               INY

GOT_ALI        LDA STAGE
               CMP #2
               BCC NO_PYRAS
               LDA RND0
               AND #$30
               CMP #$30
               BNE NO_SAUC

               LDY #$11
NO_SAUC        LDA STAGE
               CMP #3
               BCC NO_PYRAS
               LDA RND1
               EOR LOPCNT
               AND #$3A
               BNE NO_PYRAS
               LDY #9
NO_PYRAS       TYA

               RTS


;****************************************
;*             Generate bombs,clocks,points,extra life and mystery
;****************************************
GEN_RAN        LDA RND0

               LDY #1
               AND #128+64+32
               BNE YES_RAN

               LDA #0
               RTS

YES_RAN        CMP #128

               BEQ GEN_CLOCK

               CMP #64

               BEQ GEN_POINTS
               CMP #32

               BEQ GEN_EXTRA

GEN_BOMB       LDA #$4

               JMP STORE_GEN

GEN_CLOCK      LDA #$6
               JMP STORE_GEN

GEN_POINTS     LDA #$8

               JMP STORE_GEN

GEN_EXTRA      LDA #$7
STORE_GEN      STA (IX),Y

               DEY

               LDA #15

               STA (IX),Y
               LDY #$1A
               LDA #0
               STA (IX),Y ;bit0 set if gravity not to act on object
               LDA #1
               RTS


;***************************************
;*             Generate random direction
;***************************************
GEN_RD         LDA RND1
               ADC ALTABS+3
               EOR LOPCNT
               PHA

               AND #$80
               LDY #$12
               STA (IX),Y
               PLA

               PHA

               AND #3
               TAX

               PLA

               AND #$40
               ASL ; = ASL A

               INY

               STA (IX),Y
               TXA

               AND #1
               BEQ NO_SET
               DEY

               LDA (IX),Y
               ORA MAX_SPED
               STA (IX),Y
               INY

NO_SET         TXA

               AND #2
               BEQ NO_SET_2
               LDA (IX),Y
               ORA MAX_SPED
               STA (IX),Y
NO_SET_2       RTS



;****************************************
;*             Generate mystery bubbles inside
;*             of sfire sprites and let them
;*             drift ready to become bombs or
;*             whatever
;****************************************
HICKS          RTS
GEN_MYST       LDA SFIRE+1

               AND #$80
               BEQ RACHAEL

               LDA #<SFIRE

               STA IX
               LDA #>SFIRE

               STA IX+1
               JSR SO_FAR
RACHAEL        LDA SFIRE+IXSIZ+1
               AND #$80
               BEQ HICKS

               LDA #<[SFIRE+IXSIZ]

               STA IX
               LDA #>[SFIRE+IXSIZ]

               STA IX+1
SO_FAR         LDA RND0
               AND #1
               BEQ NYMPHO
               RTS

NYMPHO         LDY #0
               LDA #$0F
               STA (IX),Y
               TYA

               INY

               STA (IX),Y
               LDY #$1A
               LDA #1
               STA (IX),Y ;bit0 set if gravity not to act on object
               LDY #$9

               LDA #$10
               STA (IX),Y
               LDY #2
               LDA IXTAB+2

               STA (IX),Y
               INY

               LDA IXTAB+3

               STA (IX),Y
               INY

               LDA IXTAB+4

               STA (IX),Y
               INY

               LDA IXTAB+5

               STA (IX),Y
               INY

               LDA IXTAB+6

               ADC #$30
               STA (IX),Y
               LDA #1
               STA MAX_SPED
               JMP GEN_RD


;****************************************
;*             Movement routines
;****************************************
BACK_LEFT      JSR BLEFT
               ; JMP BBACK - Fall through instead

BBACK          LDA MAXFORWARD,X
               BEQ NOPE
               BMI NOPE

               LDY #$17
               CMP (IX),Y ;y-co accum
               BCC NTOKA1
               LDA (IX),Y
               CLC

NTOKA1         LDY #5
               ADC (IX),Y ;y-co lo
               STA (IX),Y ;y-co lo

               DEY ;=4
               LDA (IX),Y ;y-co hi
               ADC #0
               STA (IX),Y ;y-co hi

NOPE           RTS



BACK_RIGHT     JSR BRIGHT
               JMP BBACK



BFOR_LEFT      JSR BLEFT
               JMP BFOR


BFOR_RIGHT     JSR BRIGHT


BFOR           LDA MAXBACK,X

               BEQ NOPE
               BMI NOPE
               LDY #$17
               CMP (IX),Y ;y-co accum
               BCC NTOKA2
               LDA (IX),Y
NTOKA2         STA SRA ;temp storage

               LDY #5
               SEC

               LDA (IX),Y ;y-co lo

               SBC SRA ;^temp storage
               STA (IX),Y ;y-co lo

               DEY ;=4

               LDA (IX),Y ;y-co hi

               SBC #0
               STA (IX),Y ;y-co hi

               RTS



BLEFT          LDA MAXLEFT,X
               BEQ NOPE4

               BMI NOPE4

               LDY #$16
               CMP (IX),Y ;x-co accum
               BCC NTOKA3
               LDA (IX),Y
NTOKA3         STA SRA ;temp storage

               LDY #3
               SEC

               LDA (IX),Y ;x-co lo

               SBC SRA ;^temp storage
               STA (IX),Y ;x-co lo

               DEY ;=2

               LDA (IX),Y ;x-co hi

               SBC #0
               STA (IX),Y ;x-co hi

NOPE4          RTS


BRIGHT         LDA MAXRIGHT,X
               BEQ NOPE4

               BMI NOPE4

               LDY #$16
               CMP (IX),Y ;x-co accum
               BCC NTOKA4
               LDA (IX),Y
               CLC

NTOKA4         LDY #3
               ADC (IX),Y ;x-co lo

               STA (IX),Y ;x-co lo

               DEY ;=2

               LDA (IX),Y ;x-co hi

               ADC #0
               STA (IX),Y ;x-co hi

               RTS



BUP            LDA MAXUP,X
               BEQ NOPE4

               BMI NOPE4

               LDY #$18
               CMP (IX),Y ;z-co accum
               BCC NTOKA7
               LDA (IX),Y
               CLC

NTOKA7         LDY #6
               ADC (IX),Y ;z-co lo

               STA (IX),Y ;z-co lo

               RTS



BDOWN          LDY #6

               LDA (IX),Y ;z-co (lo)

               LDY #$18
               SEC

               SBC (IX),Y ;z-co accum
               LDY #6
               STA (IX),Y ;z-co lo

               RTS



;****************************************
;*             do up or down routines
;****************************************
UP_DOWN        LDX SPR_NO

               LDY #$14
               LDA (IX),Y
               AND #$80
               BEQ GO_UP

               JMP BDOWN

GO_UP          JMP BUP


;****************************************
;*             Reset one stage of the map so
;*             that all bottles are uncorked
;*             using STAGE as area pointer
;****************************************
RESET_BOT      LDA STAGE
               ASL ; = ASL A

               TAY

               LDA MAP_BASE,Y
               STA POINT+1

               STA POINT2+1
               LDA MAP_BASE+1,Y
               STA POINT+2

               STA POINT2+2
               LDY #41

NEXT_SCR       TYA

               TAX

               ASL ; = ASL A

               TAY

POINT          LDA POINT,Y
               STA IY
               INY

POINT2         LDA POINT,Y

               STA IY+1
               TXA

               PHA

               LDY #0
               LDA (IY),Y
               AND #$3F
               BEQ GO_NEXT_SC
               STA COUNT_DOWN
               INY

               INY

               INY

               CMP #$10
               BCC NOT_10_SC

               INY

NOT_10_SC      INY
               LDA (IY),Y
               TAX

               AND #%10001111
               CMP #$82
               BNE CBM_64
               TXA

               AND #%01110000
               ORA #$81
               STA (IY),Y
CBM_64         INY

               INY

               DEC COUNT_DOWN
               LDA COUNT_DOWN
               BNE NOT_10_SC

GO_NEXT_SC     PLA

               TAY

               DEY

               BPL NEXT_SCR
               RTS



;****************************************
;*             Reset bunged bottle table for
;*             player x
;****************************************
RESET_BUNG     LDA #$14*2
               LDY PLAYER
               BEQ PLAY_R1

               ASL ; = ASL A

PLAY_R1        TAY
               DEY

               LDX #($14*2)-1
               LDA #0
RE_BUNG        STA BOTT_1,Y

               DEY

               DEX

               BPL RE_BUNG

               RTS



;****************************************
;*             Reset stage x map data to those
;*             that have been bunged
;****************************************
RESET_STOP     LDY #0
               LDX #0
               LDA PLAYER
               BEQ RE_ST

               LDY #$14*2
RE_ST          LDA BOTT_1,Y

               STA IY
               INY

               INX

               LDA BOTT_1,Y
               BEQ NOT_SURE
               STA IY+1
               INX

               INY

               TYA

               PHA

               LDY #1
               LDA (IY),Y
               AND #%01110000
               ORA #$82
               STA (IY),Y
               PLA

               TAY

               CPX #$14
               BNE RE_ST

NOT_SURE       RTS



;****************************************
;*             Print number of bottles on screen
;*             according to player x
;****************************************
PRINT_NO       LDA #0
               STA NUM1
        LDX PLAYER          ;0=player1. 1=player2
               LDY PLAYER
               INX

               INX

               LDA BOTTLES+1,Y

               SEC

               SBC #10

               BCC LESS_10

               INC NUM1
               JMP NO_LESS

LESS_10        LDA BOTTLES+1,Y
NO_LESS        STA NUM2

               JMP PLOTNUM



;****************************************
;*             START OF MATT'S ROUTINES
;****************************************
;TEST PLOT ROUTINES
TEST           LDA #<WA            ;fill work area and display
               STA ZPA
               LDA #>WA
               STA ZPA+1
               LDA #<[WA_END - WA]
               STA ZPB
               LDA #>[WA_END - WA]
               STA ZPB+1
               LDA #0
               JSR FILL_MEM
;              LDX #$0C ;fill work area and display
;              LDA #>WA
;              STA TESTSM+2
;              LDA #$00
;              TAY
;
;TESTSM        STA WA,Y ; smc
;              INY
;
;              BNE TESTSM
;              INC TESTSM+2
;              DEX
;
;              BNE TESTSM


               ;test scroll routines

TESTSCRL       LDA #0
               STA XPIX
               LDA #0
               STA YPIX


;...SCROLL ROUTINES...


PWTBUF
;       LDA #FLIPIN ;set ROMs in ; Do not enable this!
;              STA $01

WKTOBUF        LDA #$0E
               STA COUNTER

               LDA YPIX
               ASL ; = ASL A ;*2=PIXELS
               STA POS

               LDA XPIX
               LSR ; = LSR A
               LSR ; = LSR A ;/4 = bytes
               STA SRT1
               LDA XPIX
               AND #$03
               BNE SSCRL3

               LDA #>BUFLINE1
               STA SCAM+2
SCAC           LDA #8
               STA LINEC

               LDY COUNTER
               LDA SCRVECHI,Y
               STA SCAM+2
               LDA SCRVECLO,Y
               STA SCAM+1
SCAB           LDY POS ;how many pixel lines down (0-$7F)
               LDA WKEDL,Y ;get addr of LHS of work area pixel line
               CLC
               ADC SRT1
               STA PTR
               LDA WKEDH,Y
               ADC #0
               STA PTR+1

               LDY #LEN-1
SCAA           LDA (PTR),Y

               LDX EIGHTTAB,Y
SCAM           STA BUFLINE1,X
               DEY
               BPL SCAA

               INC SCAM+1
               INC POS
               DEC LINEC
               BNE SCAB

               INC SCAM+2
               DEC COUNTER
               BNE SCAC

               RTS ;JMP FINSCRL


SSCRL3         CMP #$02
               BNE SSCRL4

               LDA #>BUFLINE1
               STA SCCM+2
SCCC           LDA #8
               STA LINEC

               LDY COUNTER

               LDA SCRVECHI,Y
               STA SCCM+2
               LDA SCRVECLO,Y
               STA SCCM+1
SCCB           LDY POS

               LDA WKEDL,Y
               CLC
               ADC SRT1
               STA PTR

               LDA WKEDH,Y

               ADC #0
               STA PTR+1

               LDY #LEN
               LDA (PTR),Y

               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A

               STA ROTATE
               DEY

SCCA           LDA (PTR),Y

               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A

               ORA ROTATE
               LDX EIGHTTAB,Y
SCCM           STA BUFLINE1,X
               LDA (PTR),Y

               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A

               STA ROTATE
               DEY

               BPL SCCA
               INC SCCM+1
               INC POS

               DEC LINEC

               BNE SCCB
               INC SCCM+2
               DEC COUNTER

               BNE SCCC
               RTS ;JMP FINSCRL

SSCRL4         CMP #$03
               BNE SSCRL5

               LDA #>BUFLINE1
               STA SCDM+2
SCDC           LDA #8
               STA LINEC

               LDY COUNTER

               LDA SCRVECHI,Y
               STA SCDM+2
               LDA SCRVECLO,Y
               STA SCDM+1
SCBYTE         LDY POS

               LDA WKEDL,Y
               CLC
               ADC SRT1
               STA PTR

               LDA WKEDH,Y
               ADC #0
               STA PTR+1

               LDY #LEN-1
SCDA           LDA (PTR),Y

               LSR ; = LSR A

               STA ROTATE
               INY

               LDA (PTR),Y

               ROR ; = ROR A

               LSR ROTATE
               ROR ; = ROR A

               LDX EIGHTTAB-1,Y
SCDM           STA BUFLINE1,X
               DEY
               DEY
               BPL SCDA

               INC SCDM+1
               INC POS
               DEC LINEC
               BNE SCBYTE

               INC SCDM+2
               DEC COUNTER
               BNE SCDC

               RTS ;JMP FINSCRL

SSCRL5         LDA #>BUFLINE1
               STA SCBM+2
SCBC           LDA #8
               STA LINEC

               LDY COUNTER

               LDA SCRVECHI,Y
               STA SCBM+2
               LDA SCRVECLO,Y
               STA SCBM+1
SCBB           LDY POS

               LDA WKEDL,Y

               CLC

               ADC SRT1
               STA PTR

               LDA WKEDH,Y

               ADC #0
               STA PTR+1

               LDY #LEN
               LDA (PTR),Y

               ASL ; = ASL A

               STA ROTATE
               DEY

SCBA           LDA (PTR),Y

               ROL ; = ROL A

               ASL ROTATE
               ROL ; = ROL A

               LDX EIGHTTAB,Y
SCBM           STA BUFLINE1,X
               LDA (PTR),Y

               ASL ; = ASL A

               STA ROTATE
               DEY

               BPL SCBA
               INC SCBM+1
               INC POS

               DEC LINEC

               BNE SCBB
               INC SCBM+2
               DEC COUNTER

               BNE SCBC

FINSCRL        RTS


;*********************************************
;*             This is IT... THE SORT ROUTINE !
;*********************************************
BJUMP1         ;and #0
               TAY
               STA (ZPG),Y ;mark end of YORDER table

               RTS ;JMP SRTEND

BJUMP2         JMP SRTPRN


ALOCAT         LDA #<YORDER ;LO BYTE
               STA ZPG
               LDA #>YORDER ;HI BYTE
               STA ZPG+1

SORT2          LDA #<[ORDER-3]
               STA ZPA
               LDA #>[ORDER-3]
               STA ZPA+1

SORT0          LDY #0

SORT0Y         LDA ZPA
               CLC
               ADC #3
               STA ZPA ;no page boundary cross

               LDA (ZPA),Y

               BMI SORT0 ;bit7 set=shape already done
               BEQ BJUMP1 ;branch to exit if end of table reached

               INY

               LDA (ZPA),Y ;get IX BLOCK location
               STA ZPB

               INY

               LDA (ZPA),Y
               STA ZPB+1

               LDA ZPA
               STA ZPC

               LDA ZPA+1
               STA ZPC+1

SORT1          LDY #0

SORT1Y         LDA ZPC
               CLC
               ADC #3
               STA ZPC ;no page boundary cross

               LDA (ZPC),Y
               BMI SORT1Y ;bit7 set=shape already done

               BEQ BJUMP2 ;no shapes found behind IX shape so put in YORDER

               CMP #3
               BNE NOTBGND

               CMP (ZPA),Y
               BEQ SORT1Y ;dont bother comparing 2 background objects

NOTBGND        INY
               LDA (ZPC),Y
               STA ZPD

               INY
               LDA (ZPC),Y
               STA ZPD+1

               LDA ZPD
               CMP ZPB
               BNE EXX

               LDA ZPD+1
               CMP ZPB+1
               BEQ SORT1 ;comparing shape with itself!...get next one

EXX            AND #0 ;ptr in priority table: PTAB
               STA SRA

               LDY #$6
               LDA (ZPB),Y ;IX z-co (lo)
               STA ZPE

               LDA (ZPD),Y ;IY z-co (lo)
               STA ZPF

               LDY #$D
               LDA (ZPD),Y ;IY zdim
               CLC
               ADC ZPF ;no carry (no z hi byte)
               SEC
               SBC #1
               CMP ZPE ;(IYzpos+IYzdim-1)-(IXzpos)
               BCC SORTY0

               LDA (ZPB),Y ;IX zdim
               CLC
               ADC ZPE ;no carry (no z hi byte)
               SEC
               SBC #1
               CMP ZPF ;(IXzpos+IXzdim-1)-(IYzpos)

               BCS SORTZ0
SORTZ1         INC SRA

SORTZ0         INC SRA

SORTY0         LDY #$5

               LDA (ZPB),Y ;IX y-co lo
               STA ZPE

               LDA (ZPD),Y ;IY y-co lo
               STA ZPF

               DEY ;=$4
               LDA (ZPB),Y ;IX y-co hi
               STA ZPE+1

               LDA (ZPD),Y ;IY y-co hi
               STA ZPF+1

               TAX

               LDY #$C
               LDA (ZPD),Y ;IY ydim
               SEC
               SBC #1
               CLC
               ADC ZPF
               BCC YOKA
               INX
YOKA           CMP ZPE

               TXA

               SBC ZPE+1 ;(IYypos+IYydim-1)-(IXypos)

               BCC SORTX0
               LDX ZPE+1

               LDA (ZPB),Y ;IX ydim
               SEC

               SBC #1
               CLC

               ADC ZPE

               BCC YOKB
               INX

YOKB           CMP ZPF

               TXA

               SBC ZPF+1 ;(IXypos+IXydim-1)-(IYypos)

               LDA SRA

               BCS SORTY1
               ADC #3
SORTY1         CLC

               ADC #3
               STA SRA

SORTX0         LDY #$3

               LDA (ZPB),Y ;IX x-co lo

               STA ZPE

               LDA (ZPD),Y ;IY x-co lo

               STA ZPF

               DEY ;=$2
               LDA (ZPB),Y ;IX x-co hi

               STA ZPE+1

               LDA (ZPD),Y ;IY x-co hi

               STA ZPF+1

               TAX

               LDY #$B

               LDA (ZPD),Y ;IY xdim
               SEC

               SBC #1
               CLC

               ADC ZPF

               BCC XOKA
               INX

XOKA           CMP ZPE

               TXA

               SBC ZPE+1 ;(IYxpos+IYxdim-1)-(IXxpos)

               BCC SORTX2
               LDX ZPE+1

               LDA (ZPB),Y ;IX xdim
               SEC

               SBC #1
               CLC

               ADC ZPE

               BCC XOKB
               INX

XOKB           CMP ZPF

               TXA

               SBC ZPF+1 ;(IXxpos+IXxdim-1)-(IYxpos)

               LDA SRA

               BCS SORTX1
               ADC #9
SORTX1         CLC

               ADC #9
               STA SRA

SORTX2         LDX SRA

               LDA PTABLO,X
               STA SORTJUMP+1
               LDA PTABHI,X
               STA SORTJUMP+2
SORTJUMP       JMP $0000 ;smc


EJECT          EQU SORT1
HIGHER         EQU SORT1



PTABLO         BYTE <EJECT, <EJECT, <EJECT
               BYTE <LOWER, <LOWER, <EJECT
               BYTE <LOWER, <LOWER, <EJECT

               BYTE <EJECT, <HIGHER, <HIGHER
               BYTE <LOWER, <OUCH, <HIGHER
               BYTE <LOWER, <LOWER, <EJECT

               BYTE <EJECT, <HIGHER, <HIGHER
               BYTE <EJECT, <HIGHER, <HIGHER
               BYTE <EJECT, <EJECT, <EJECT


PTABHI         BYTE >EJECT, >EJECT, >EJECT
               BYTE >LOWER, >LOWER, >EJECT
               BYTE >LOWER, >LOWER, >EJECT

               BYTE >EJECT, >HIGHER, >HIGHER
               BYTE >LOWER, >OUCH, >HIGHER
               BYTE >LOWER, >LOWER, >EJECT

               BYTE >EJECT, >HIGHER, >HIGHER
               BYTE >EJECT, >HIGHER, >HIGHER
               BYTE >EJECT, >EJECT, >EJECT


OUCH           LDY #0
               LDA (ZPA),Y ;IX SHAPE NUMBER
               CMP #3 ;BACKGROUND BLOCK
               BNE OUXTR

OUXIT          JMP SORT1
OUXTR          LDA (ZPC),Y ;IY SHAPE NUMBER

               CMP #3 ;BACKGROUND BLOCK
               BNE OUXIT ;BEQ LOWER


LOWER          LDX #0

FID0           LDA LOWEST,X
               CMP #$FF
               BEQ FID4 ;LAST SHAPE IN TABLE

               CMP ZPD+1 ;IYPTR HI

               BNE FID8
               LDA LOWEST+1,X
               CMP ZPD ;IYPTR LO

               BEQ FID2
FID8           INX

               INX

               BNE FID0 ;JMP

FID4           LDA ZPD+1 ;IYPTR HI

               STA LOWEST,X
               LDA ZPD ;IYPTR LO

               STA LOWEST+1,X
               LDA #$FF ;END OF TABLE
               STA LOWEST+2,X
TRY            LDA ZPD
               STA ZPB

               LDA ZPD+1

               STA ZPB+1 ;IY TO IX

               LDA ZPC

               STA ZPA

               LDA ZPC+1

               STA ZPA+1 ;DO ORDER TABLE POSNS...QUICKER THAN LOOP



               LDA #<ORDER

               STA ZPC

               LDA #>ORDER

               STA ZPC+1

               JMP SORT1



FID2           LDA ZPC

               STA ZPA

               LDA ZPC+1

               STA ZPA+1

SRTPRN

SRTPR0         LDY #0
               LDA (ZPA),Y

               STA (ZPG),Y

               ORA #$80 ;'PRINTED' FLAG
               STA (ZPA),Y

               INY

               LDA (ZPA),Y

               STA (ZPG),Y

               INY

               LDA (ZPA),Y

               STA (ZPG),Y ;COPY DETAILS FROM ORDER TABLE TO YORDER TABLE
               LDA ZPG

               CLC

               ADC #$03
               STA ZPG ;NO CARRY TO HI BYTE
               LDA #$FF
               STA LOWEST ;RESET LOWEST TABLE
               JMP SORT2



LOWEST         BYTE $FF,$FF,$FF,$FF
               BYTE $FF,$FF,$FF,$FF
               BYTE $FF,$FF,$FF,$FF
               BYTE $FF,$FF,$FF,$FF


;****************************************

;*             End of SORT ROUTINE
;****************************************



;*********************************************
;*             Convert 3D X,Y,Z to ON SCREEN X,Y
;*
;*             x(2d)=(x(3d)+y(3d))-SCREENX
;*             y(2d)=$8040-z+x(3d)/2-y(3d)/2+(xdim/2)-SCREENY
;*
;*             Send:
;*             IX block address in IX,IX+1
;*              3D x,y,z co-ords
;*              3D x,y,z dimensions
;*              2D chars/pixels dimensions
;*
;*********************************************
THREED         LDY #$3

               LDA (IX),Y ;x-co lo

               STA ZPA

               DEY ;=$2
               LDA (IX),Y ;x-co hi

               STA ZPA+1

               LDY #$5

               LDA (IX),Y ;y-co lo

               STA ZPB

               DEY ;=$4
               LDA (IX),Y ;y-co hi

               STA ZPB+1

               ;work out on-screen x-co
               LDA ZPA ;x-co lo temp

               CLC

               ADC ZPB ;y-co lo temp

               TAX ;sta ZPC
               LDA ZPA+1 ;x-co hi temp

               ADC ZPB+1 ;y-co hi temp

               STA ZPC+1

               TXA ;lda ZPC
               SEC

               SBC SCREENX

               TAX

               LDA ZPC+1

               SBC SCREENX+1

               ;bcc jpnoton
               BEQ THREEDOK0

JPNOTON        JMP NOTON ;not on screen so set "don't print" flag

THREEDOK0      TXA
               CMP #$B4 ;=$B8-$04
               BCS JPNOTON ;branch if off right of screen
               ;clc
               ADC #$04 ;fiddle factor!
               LDY #$E

               STA (IX),Y ;write ON-SCREEN X CO-ORD
               STA ZPC

               LDY #$09

               LDA (IX),Y ;chars across of shape - max 4

               AND #$07
               ASL ; = ASL A

               ASL ; = ASL A

               ASL ; = ASL A ;*8;carry clear

               ADC ZPC

               CMP #$C0
               BCS JPNOTON ;branch if off right of screen


;              CMP #$10

;              BCC JPNOTON ;branch if in left border


               ;work out on-screen y-co
               LDA #$40
               SEC

               LDY #$6

               SBC (IX),Y ;z-co (lo)

               STA ZPC

               LDA #$80
               SBC #0
               TAX ;hi



               LDA ZPA ;x-co lo temp

               LSR ZPA+1 ;x-co hi temp       ;divide by 2
               ROR ; = ROR A

               CLC

               ADC ZPC

               TAY ;lo

               TXA ;hi

               ADC ZPA+1

               TAX ;hi

               LSR ZPB+1 ;y-co hi temp       ;divide by 2
               ROR ZPB ;y-co lo temp

               TYA ;lo

               SEC

               SBC ZPB

               STA ZPC

               TXA ;hi

               SBC ZPB+1

               TAX ;hi

               LDY #$B

               LDA (IX),Y ;x dim

               LSR ; = LSR A ;/2

               CLC

               ADC ZPC

               BCC THREEDOK2

               INX ;hi

THREEDOK2      SEC
               SBC SCREENY

               BMI NOTON ;branch if not on screen
               TAY ;lo

               TXA ;hi

               SBC SCREENY+1

               ;bcc noton
               BNE NOTON ;branch if not on screen
               TYA ;lo

               LDY #$F

               STA (IX),Y ;WRITE ON-SCREEN Y CO-ORD
               STA ZPB



JIBBL          LDY #$01 ;clear 'dont print' flag
               LDA (IX),Y
               AND #$7F
               STA (IX),Y


               LDY #$0A
               LDA (IX),Y ;pixels up of shape - max $1A
               AND #$1F
               STA ZPA

               LDA ZPB

               SEC

               SBC ZPA

               BCC EX3D
               CLC

               ADC #1
               BPL EX3D


NOTON          LDY #$1 ;set 'dont print' flag

               LDA (IX),Y
               ORA #$80
               STA (IX),Y


EX3D           RTS



;*************************************************
;*             CREATE MATERIALIZE EFFECT ON GRAPHIC
;*************************************************
DISSR          LSR ; = LSR A
               LSR ; = LSR A ;GIVES DISSOVE INFO *$10,/$4 =*$4

               STA ZPA ;TEMP

               LDA LOPCNT
               AND #3
               ORA ZPA

               TAX

               AND #1
               TAY

               LDA SELFMOD1,Y
               STA DISMOD1

               LDA SELFMOD2,Y
               STA DISMOD2

               LDA DIZZER-4,X
               STA ROTATE ;MASK
               LDY #0
               LDA (DATA),Y ;# OF BYTES ACROSS DATA-MIN:1,MAX:3
               STA DISBUF
               TAX

               INY

               LDA (DATA),Y ;# OF PIXELS DOWN DATA

               STA DISBUF+1
               STA ZPA ;TEMP

               INY

               DEX

               BEQ DIZZA

               CLC

DIZZE1         ADC ZPA

               DEX

               BNE DIZZE1
DIZZA          TAX
DIZZE2         LDA (DATA),Y ;MASK
               STA DISBUF,Y
               INY

               LDA (DATA),Y ;DATA
               AND ROTATE ;DISSOLVE MASK

               STA DISBUF,Y
               INY

               LDA ROTATE
DISMOD1        LSR ; = LSR A
DISMOD2        ROR ROTATE ;NB ROTATE MUST BE ON ZERO PAGE

               DEX

               BNE DIZZE2
               LDA #<DISBUF ;LO BYTE

               STA DATA
               LDA #>DISBUF ;HI BYTE

               STA DATA+1
               RTS



SELFMOD1       LSR ; = LSR A

               ASL ; = ASL A

SELFMOD2       BYTE $66 ;ROR zp
               BYTE $26 ;ROL zp


DIZZER         BYTE $00,$01,$00,$10
               BYTE $08,$40,$10,$02
               BYTE $09,$A0,$12,$44
               BYTE $89,$25,$62,$58
               BYTE $4D,$D2,$35,$6A
               BYTE $AD,$73,$5B,$B6
               BYTE $AF,$ED,$77,$BE
               BYTE $EF,$FD,$BE,$F7


;*****************************************************
;*             Moving shape print routine
;*
;*             DATA points to graphic data
;*             Set up following IX vars before calling:
;*
;*              2d shape dimensions
;*
;*              Dissolve info
;*
;*              Transfer address
;*              2d ON-SCREEN X,Y coords
;*
;*****************************************************
BRITT          JSR GETDATA

        ;
        ; 2009.07 MY: Uncomment the line below to stripe screen border to prove
        ; this routine is getting called
        ;
;        LDY $01
;        LDA #$35
;        STA $01
;        INC $D020
;        STY $01

               LDY #$09
               LDA (IX),Y ;top 4 bits: dissolve
               AND #$F0
               BEQ BRIT1

               JSR DISSR

BRIT1          LDY #$07
               LDA (IX),Y ;transfer address lo
               STA PTR

               INY ;=$08
               LDA (IX),Y ;transfer address hi
               STA PTR+1

               LDY #0
               LDA (DATA),Y ;# of chars across of shape - max 4
               STA COUNTER

               INY
               LDA (DATA),Y ;# of pixels up of shape - max $1A
               STA LINEC

               LDY #$09
               LDA (IX),Y ;dissolve/chars across
               AND #$F0
               ORA COUNTER ;chars across
               STA (IX),Y

               INY ;=$0A
               LDA (IX),Y ;spike shape/pixels up shape
               AND #$C0
               ORA LINEC ;pixels up of shape
               STA (IX),Y

               LDY #$0F       ;check if shape is partly off top of screen
               LDA (IX),Y ;on-screen y-co
               CMP LINEC ;pixels up of shape
               BCS DRFEELGOOD
               ;clc
               ADC #1
               STA LINEC

DRFEELGOOD     LDY #$0E
               LDA (IX),Y ;on-screen x-co
               AND #$06 ;gives horizontal pixel offset (2 pixel accuracy)
               BNE BRIT2

               DEC COUNTER

               LDA DATA
               STA BRSAA+1
               STA BRSAB+1

               LDA DATA+1
               STA BRSAA+2
               STA BRSAB+2

               LDX #2
BRA2           LDY COUNTER

BRSAA          LDA $FFFF,X ;smc ;MASK
               INX

               EOR #$FF
               AND (PTR),Y

BRSAB          ORA $FFFF,X ;smc ;DATA
               INX

               STA (PTR),Y

               DEY

               BPL BRSAA

               SEC
               LDA PTR
               SBC #WW
               STA PTR

               BCS BRA1
               DEC PTR+1

BRA1           DEC LINEC

               BNE BRA2
               RTS ;JMP FINBRIT


BRIT2          CMP #$04
               BNE BRIT3

               LDA DATA
               STA BRSCA+1
               STA BRSCB+1
               STA BRSCC+1
               STA BRSCD+1

               LDA DATA+1
               STA BRSCA+2
               STA BRSCB+2
               STA BRSCC+2
               STA BRSCD+2

               LDX #2
BRC2           LDY COUNTER

               AND #0
               STA ROTATE
               STA ROTATE2

BRSCA          LDA $FFFF,X ;smc MASK
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A

               STA ZPA ;TEMP

BRSCB          LDA $FFFF,X ;smc MASK
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A

               ORA ROTATE
               EOR #$FF
               AND (PTR),Y

               STA ZPB ;TEMP

               LDA ZPA

               STA ROTATE
               INX

BRSCC          LDA $FFFF,X ;smc DATA
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A
               STA ZPA ;TEMP

BRSCD          LDA $FFFF,X ;smc DATA
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ASL ; = ASL A
               ORA ROTATE2
               ORA ZPB
               STA (PTR),Y

               LDA ZPA
               STA ROTATE2

               INX

               DEY

               BNE BRSCA

               LDA ROTATE
               EOR #$FF
               AND (PTR),Y

               ORA ROTATE2

               STA (PTR),Y

               SEC
               LDA PTR
               SBC #WW
               STA PTR
               BCS BRC1
               DEC PTR+1
BRC1           DEC LINEC

               BNE BRC2
               RTS ;JMP FINBRIT


BRIT3          CMP #$06
               BNE BRIT4

               LDA DATA
               STA BRSDA+1
               STA BRSBYTE+1
               STA BRSDC+1
               STA BRSDD+1

               LDA DATA+1
               STA BRSDA+2
               STA BRSBYTE+2
               STA BRSDC+2
               STA BRSDD+2

               LDX #2
BRD2           LDY COUNTER

               AND #0
               STA ROTATE
               STA ROTATE2

BRSDA          LDA $FFFF,X ;smc MASK
               ASL ; = ASL A

               ROL ; = ROL A

               ROL ; = ROL A

               AND #$03
               STA ZPA ;TEMP

BRSBYTE        LDA $FFFF,X ;smc MASK
               ASL ; = ASL A

               ASL ; = ASL A

               ORA ROTATE
               EOR #$FF
               AND (PTR),Y

               STA ZPB ;TEMP

               LDA ZPA

               STA ROTATE
               INX

BRSDC          LDA $FFFF,X ;smc DATA
               ASL ; = ASL A

               ROL ; = ROL A

               ROL ; = ROL A

               AND #$03
               STA ZPA ;TEMP

BRSDD          LDA $FFFF,X ;smc DATA
               ASL ; = ASL A

               ASL ; = ASL A

               ORA ROTATE2

               ORA ZPB

               STA (PTR),Y

               LDA ZPA

               STA ROTATE2

               INX

               DEY

               BNE BRSDA

               LDA ROTATE
               EOR #$FF
               AND (PTR),Y

               ORA ROTATE2

               STA (PTR),Y

               SEC

               LDA PTR

               SBC #WW

               STA PTR

               BCS BRD1
               DEC PTR+1

BRD1           DEC LINEC

               BNE BRD2
               RTS ;JMP FINBRIT


BRIT4          LDA DATA

               STA BRSBA+1

               STA BRSBB+1

               STA BRSBC+1

               STA BRSBD+1

               LDA DATA+1
               STA BRSBA+2

               STA BRSBB+2

               STA BRSBC+2

               STA BRSBD+2

               LDX #2
BRB2           LDY COUNTER

               AND #0
               STA ROTATE
               STA ROTATE2

BRSBA          LDA $FFFF,X ;smc MASK
               LSR ; = LSR A

               LSR ; = LSR A

               STA ZPA ;TEMP

BRSBB          LDA $FFFF,X ;smc MASK
               LSR ; = LSR A

               ROR ; = ROR A

               ROR ; = ROR A

               AND #$C0
               ORA ROTATE
               EOR #$FF
               AND (PTR),Y

               STA ZPB ;TEMP

               LDA ZPA

               STA ROTATE
               INX

BRSBC          LDA $FFFF,X ;smc DATA
               LSR ; = LSR A

               LSR ; = LSR A

               STA ZPA ;TEMP

BRSBD          LDA $FFFF,X ;smc DATA
               LSR ; = LSR A

               ROR ; = ROR A

               ROR ; = ROR A

               AND #$C0
               ORA ROTATE2

               ORA ZPB

               STA (PTR),Y

               LDA ZPA

               STA ROTATE2

               INX

               DEY

               BNE BRSBA

               LDA ROTATE
               EOR #$FF
               AND (PTR),Y

               ORA ROTATE2

               STA (PTR),Y

               SEC

               LDA PTR

               SBC #WW

               STA PTR

               BCS BRB1
               DEC PTR+1

BRB1           DEC LINEC

               BNE BRB2

               RTS


               ;Get address of graphic data to DATA,DATA+1
GETDATA        LDY #0
               LDA (IX),Y ;shape

;               TAX
;               LDA DATTABLO-1,X
;               STA ZPA ;ptr to addresses of diff. frames of this shape
;               LDA DATTABHI-1,X
;               STA ZPA+1

               ASL ; = ASL A
               TAX
               LDA DATTAB-2,X
               STA ZPA ;ptr to addresses of diff. frames of this shape
               LDA DATTAB-1,X
               STA ZPA+1

               INY ;=1

               LDA (IX),Y ;data offset (which frame of animation)
               AND #$0F
               ASL ; = ASL A

               TAY

               LDA (ZPA),Y

               STA DATA
               INY

               LDA (ZPA),Y

               STA DATA+1
               RTS



;DATTABLO       ;**DATA TABLE (LO) OF SHAPES**
;               BYTE <MAJOR, <SHADOW, <SHADOW, <SHADOW, <BULLIT, <LIFTG, <EXPLOF, <BUBBLE
;               BYTE <SAUCE, <BALLY, <CRABG, <MATTER, <BULEX, <SAUZE, <MYST, <SPIKY, <PYRAM
;
;DATTABHI       ;**DATA TABLE (HI) OF SHAPES**
;               BYTE >MAJOR, >SHADOW, >SHADOW, >SHADOW, >BULLIT, >LIFTG, >EXPLOF, >BUBBLE
;               BYTE >SAUCE, >BALLY, >CRABG, >MATTER, >BULEX, >SAUZE, >MYST, >SPIKY, >PYRAM

;
; This table maps moving shape number (as stored in byte 0 of each IXTAB entry
; to the table of animation frames for that shape.  See comment at IXTAB for
; what the shapes are.
;
DATTAB         WORD MAJOR
               WORD SHADOW
               WORD SHADOW
               WORD SHADOW
               WORD BULLIT
               WORD LIFTG
               WORD EXPLOF
               WORD BUBBLE
               WORD SAUCE
               WORD BALLY
               WORD CRABG
               WORD MATTER
               WORD BULEX
               WORD SAUZE
               WORD MYST
               WORD SPIKY
               WORD PYRAM


MAJOR          WORD MAN0,MAN1,MAN2,MAN3,MAN4
LIFTG          WORD LIFT0,LIFT1
EXPLOF         WORD EXP1,EXP2,EXP3,EXP4,EXP5,EXP6,EXP7 ; Explosion
SPIKY          WORD KALIEN,SGUN0
SAUCE          WORD SAUCER
BUBBLE         WORD BUB2,BUB3,BUB4,BUB5,BUB7,BUB8,BUB6
PYRAM          WORD PY0,PY1,PY2,PY3,PY2,PY1,PY0
BULLIT         WORD BULL1,BULL1
SHADOW         WORD SHAD1,SHAD2,SHAD3,PIN3,PIN2,PIN1
CRABG          WORD CRAB0,CRAB1
BALLY          WORD BAZZ1,BAZZ2,BAZZ3,BAZZ4,BAZZ5,BAZZ4,BAZZ3,BAZZ2,BAZZ1
MATTER         WORD TELE0,TELE1,TELE2 ; Materialisation
BULEX          WORD BULLY0,BULLY1
MYST           WORD MYST1,MYST2,BUB6,BUB6,BOMB0,BOMB1,CLOCK,EXTRA,PTS
SAUZE          WORD ZIRE0,ZIRE1,ZIRE2,ZIRE3


;****************************************
;*             Loop through YORDER table
;*             and print sorted shapes
;*
;*             HL: ptr in YORDER table
;*
;****************************************

FINMSPR        RTS

MOVSHPRT       LDX #<[YORDER-3]
               LDY #>[YORDER-3]
               STX HL
               STY HL+1


MVSHPRL        LDA HL

               CLC

               ADC #3
               STA HL ;no carry to hi byte

               LDY #0
               LDA (HL),Y ;shape type
               BEQ FINMSPR ;no more shapes in YORDER table

               CMP #$03 ;background shapes

               BEQ MVSHPRL ;loop to next shape

               TAX ;temp storage

               INY ;get address of IX BLOCK from table to IX

               LDA (HL),Y
               STA IX
               INY

               LDA (HL),Y
               STA IX+1


DOACTPR        JSR BRITT ;smc ;actually print moving shape in work area

               ;***NB for scroll-on this is JSR PATCH***



               ;Print vertical strips of background objects in front



               LDA HL
               STA IY
               LDA HL+1
               STA IY+1


PRBOIFRT       LDA IY
               CLC

               ADC #3
               STA IY ;no carry to hi byte

               LDY #0
               LDA (IY),Y
               BEQ MVSHPRL ;end of table reached

               CMP #$03
               BNE PRBOIFRT ;branch if NOT background object



               ;first shape found...

               ;set up x pos of LHS of strip

               LDY #$0E
               LDA (IX),Y ;ON-SCREEN x coord

               STA ROTATE2 ;temp

               CLC

               ADC SCREENX

               AND #$F0
               STA STRIPX
               AND #0
               ADC SCREENX+1

               STA STRIPX+1


               ;set up y pos of bot of strip

               INY ;=$0F

               LDA (IX),Y ;ON-SCREEN y coord

               STA SRA ;temp

               CLC

               ADC #$08 ;rounding up to nearest 8 pixels ;no carry

               AND #$F8
               ADC SCREENY

               TAX ;temp

               AND #0
               ADC SCREENY+1

               STA ZPD+1

               TXA

               SEC

               SBC #$40
               BCS NODC1

               DEC ZPD+1

NODC1          STA ZPD


               ;set up pos of top of strip

               LDY #$0A
               LDA (IX),Y ;bits 0-5: pixels up of shape
               AND #$3F
               STA COUNTER ;temp

               LDA SRA ;ON-SCREEN y coord (temp^)
               SEC

               SBC COUNTER

               AND #$F8 ;rounding down to nearest 8 pixels

               TAX ;temp

               CLC

               ADC SCREENY

               STA STRIPY
               AND #0
               ADC SCREENY+1

               STA STRIPY+1
               LDA STRIPY
               SEC

               SBC #$40
               BCS NODC2

               DEC STRIPY+1
NODC2          STA STRIPY



               ;work out position in work area of TOP of strip

               LDA ROTATE2 ;ON-SCREEN x coord (temp^)
               AND #$F0
               LSR ; = LSR A

               LSR ; = LSR A

               LSR ; = LSR A ;/8=chars

               CLC

               ADC WKEDL,X

               STA INITSTRDAT
               AND #0
               ADC WKEDH,X

               STA INITSTRDAT+1


               ;work out how many strips across
               DEY ;=$09

               LDA (IX),Y ;bits 0-3:chars across shape

               AND #$0F
               TAX ;temp

               LDA ROTATE2

               AND #$0F
               BEQ ISATLOC

               INX

               AND #$08
               BEQ ISATLOC

               INX

ISATLOC        INX ;round up
               TXA

               LSR ; = LSR A ;/2=# of vertical strips across to print
               STA ROTATE2 ;counter


               ;get address of background shape to ZPG

               LDY #1
               LDA (IY),Y
               STA ZPG

               INY

               LDA (IY),Y
               STA ZPG+1



               ;print one shape pointed to by (ZPG)
PRVSTLP        LDY #$A
               LDA (ZPG),Y ;shape #
               TAX

               LDA OBJPTOFF,X
               STA COUNTER



OPRPLVL        LDX COUNTER
               LDY O1,X
               BEQ CHKXTRS

               LDA INITSTRDAT
               STA STRIPDAT
               LDA INITSTRDAT+1
               STA STRIPDAT+1

               LDA PLANESJL,Y
               STA OPLANESJSR+1
               LDA PLANESJH,Y
               STA OPLANESJSR+2

OPLANESJSR     JSR $0000 ;print a strip of current plane

               INC COUNTER

               BNE OPRPLVL ;jmp




               ;check for printing extra strips
CHKXTRS        DEC ROTATE2 ;counter

               BNE DOANOTHER

               JMP PRBOIFRT


               ;update relevant variables and loop to print
DOANOTHER      LDA INITSTRDAT

               CLC

               ADC #$02
               STA INITSTRDAT
               BCC NHBINC1

               INC INITSTRDAT+1
               CLC

NHBINC1        LDA STRIPX

               ADC #$10
               STA STRIPX
               BCC PRVSTLP

               INC STRIPX+1
               JMP PRVSTLP


;
; File X6.BUB
; Starts Here
;
;

;
;
;  PDS Pc1.10 :000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          01.00
;         By  "JAKE AND MATTHEW..."
;
;     Created on Wed the 30th of Mar 1988
;        Last update 08:41 on 10/05/88
;

;*******************************************
;*             SAVE DATA FROM MOVING OBJECTS
;*             AND SET UP SORT TABLE {ORDER}
;*
;*             IX points to IX blocks

;*             IY points to shadows

;*             HL points to data save area
;*
;*             X reg - ptr in ORDER table

;*
;*             Data needed in IX / SHADOW  blocks...
;*              shape number & data offset (objects only)

;*              3d xyz position

;*              3d xyz dimensions (objects only)
;*              2d shape dimensions (objects only)
;*
;*******************************************


SAVPOS         LDX #0 ;PTR IN ORDER TABLE
               STX SAVNUM ;ZERO NO OF SAVED BACKGROUNDS
               ;Put SPECAL (background) objects in ORDER table

               LDA SPECAL
               BEQ PUZAZ ;no objects in SPECAL table

               STA COUNTER

               LDA #<[SPECAL+1]
               STA IY
               LDA #>[SPECAL+1]
               STA IY+1
               LDY #$00
ADDF           LDA (IY),Y ;0:moving shapes cant go behind so dont need to sort

               BEQ ADDG
               LDA #$03 ;SHAPE NUMBER FOR SORT

               STA ORDER,X

               INX

               LDA IY
               STA ORDER,X

               INX

               LDA IY+1
               STA ORDER,X

               INX

ADDG           LDA IY
               CLC

               ADC #SPECSIZ
               STA IY
               BCC ADDG1

               INC IY+1
ADDG1          DEC COUNTER
               BNE ADDF
PUZAZ          LDA #<LASTIX

               STA IX
               LDA #>LASTIX
               STA IX+1
               LDA #<LASTSHD

               STA IY
               LDA #>LASTSHD

               STA IY+1
               LDA #<[SAVNUM+1]
               STA HL
               LDA #>[SAVNUM+1]
               STA HL+1
               LDA #IXNUM ;# OF IX TABLES
               STA SRA

SAVUN          LDY #0

               LDA (IX),Y ;SHAPE

               BEQ NUSUD1 ;IF 0 (NO SHAPE) THEN LOOP FOR NEXT SHAPE
               ;SAVE SHADOW FIRST
SAVUN1         LDY #0
               LDA (IY),Y ;SHADOW FOR THIS SHAPE ON ?
               BEQ STELLA
               ;save IX
               LDA IX
               PHA
               LDA IX+1
               PHA

               LDA IY
               STA IX
               LDA IY+1
               STA IX+1
               STX ROTATE
               JSR THREED ;get ON-SCREEN X,Y coords to IX BLOCK and
               LDX ROTATE
               LDY #1
               LDA (IX),Y ;data offset

               BMI NOSHD ;branch if not on screen
               ;stx rotate

               JSR PUTTRNSH ;get address in work area to IX BLOCK and ZPG
               ;ldx rotate

               JSR SAVSHD
               ;restore IX

NOSHD          PLA
               STA IX+1
               PLA

               STA IX
               ;NOW SAVE IX SHAPE
STELLA         STX ROTATE
               JSR DIMS
               LDX ROTATE
               JSR SAVPOZ
               ;Loop for next IX shape...
NUSUD1         LDA IX
               SEC

               SBC #IXSIZ
               STA IX
               BCS NSD1A

               DEC IX+1
NSD1A          LDA IY

               SEC

               SBC #SHADSIZ
               STA IY
               BCS NSD1B

               DEC IY+1
NSD1B          DEC SRA
               BEQ NSD1C

               JMP SAVUN

NSD1C          AND #0

               STA ORDER,X ;END OF ORDER TABLE

               STA ORDER+1,X

               RTS


;****************************************
;*             PUT SHAPE IN ORDER TABLE FOR
;*             SORTING AND SAVE DATA BEHIND
;*             WHERE IT WILL BE PRINTED
;*             IN THE WORK AREA
;****************************************
               ;put in sort table (ORDER) but dont save background

LISSY          LDA #$01 ;'SHAPE NUMBER'

               STA ORDER,X

               INX

               LDA IX ;PUT IX BLOCK ADDRESS IN ORDER TABLE

               STA ORDER,X

               INX

               LDA IX+1
               STA ORDER,X

               INX

EXSVPS         RTS



SAVPOZ         JSR PUTTRN ;GET ADDRESS IN WORK AREA TO IX BLOCK
SAVSHD         LDY #1
               LDA (IX),Y ;data offset/don't print flag
               BMI EXSVPS ;bit 7 set means shape off screen - dont print

               ;moving background shapes don't move over landscape

               ;so they dont need to have their backgrounds saved
               DEY ;=0

               LDA (IX),Y ;shape

               CMP #$0C ;MATERIALISATION

               BEQ LISSY

               CMP #$10 ;SPIKY ;gun/spikes

               BEQ LISSY

               INC SAVNUM ;one more saved background

               DEY ;=0

               LDA (IX),Y ;shape

               CMP #$02 ;SHADOW
               BNE KAYYA

               STA ORDER,X ;'shape type'=$02 {shadow}
               LDA IY
               STA ORDER+1,X

               LDA IY+1
               JMP KAKKA

KAYYA          LDA #$01 ;'shape type' {object}
               STA ORDER,X

               LDA IX
               STA ORDER+1,X

               LDA IX+1
KAKKA          STA ORDER+2,X
               INX

               INX

               INX

               ;SAVE BACKGROUND DATA

TESTENT        LDY #$07

               LDA (IX),Y ;TRANSFER ADDRESS LO

               STA SAVBSM1+1

               STA SAVBSM3+1

               INY ;=$08

               LDA (IX),Y ;TRANS ADD HI
               STA SAVBSM2+1

               STA SAVBSM3+2

               LDY #$09
               LDA (IX),Y ;chars across of shape - max 4

               AND #$07
               STA LINEC

               LDY #$0E
               LDA (IX),Y ;on-screen x-co
               AND #$06
               BNE SAVU1 ;if pixel offset <>0 then xtra char across needed

               DEC LINEC

SAVU1          LDY #$0A

               LDA (IX),Y ;pixels up of shape - max $1A
               AND #$1F
               ;check if shape if partly off top of screen

               LDY #$0F
               CMP (IX),Y ;on-screen y-co
               BCC SUKOP

               LDA (IX),Y ;on-screen y-co
               CLC

               ADC #1
SUKOP          STA COUNTER
               LDY #0 ;SAVE TRANSFER ADDR TO SAVNUM BLOCK
SAVBSM1        LDA #$FF ;smc
               STA (HL),Y
SAVBSM2        LDA #$FF ;smc
               INY

               STA (HL),Y
               LDA LINEC

               INY

               STA (HL),Y
               LDA COUNTER

               INY

               STA (HL),Y
               INY

               STX ROTATE ;TEMP SAVE

SAVU3          LDX LINEC
SAVBSM3        LDA $FFFF,X ;smc

               STA (HL),Y
               INY

               DEX

               BPL SAVBSM3

               LDA SAVBSM3+1 ;MOVE UP ONE LINE

               SEC

               SBC #WW

               STA SAVBSM3+1

               BCS SAVU2

               DEC SAVBSM3+2

SAVU2          DEC COUNTER
               BNE SAVU3

               TYA

               CLC

               ADC HL
               STA HL
               BCC SAVU4

               INC HL+1
SAVU4          LDX ROTATE ;RESTORE X
               RTS


;***********************************************
;*             PUT BACK DATA FROM MOVING OBJECTS
;***********************************************
PUTPOS         LDA SAVNUM
               BNE PUTPS1
               RTS ;NO BACKGROUNDS TO RESTORE
PUTPS1         STA NEWNAME

               LDA #<[SAVNUM+1]
               STA HL
               LDA #>[SAVNUM+1]
               STA HL+1
PUTPO1         LDY #0
               LDA (HL),Y ;TRANS ADDR LO

               STA PUTPSM1+1

               INY

               LDA (HL),Y ;TRANS ADDR HI

               STA PUTPSM1+2

               INY

               LDA (HL),Y ;CHARS ACROSS SHAPE
               STA LINEC

               INY

               LDA (HL),Y ;PIXELS UP SHAPE

               STA COUNTER

               INY

PUTPL2         LDX LINEC

PUTPL1         LDA (HL),Y
PUTPSM1        STA $FFFF,X ;smc

               INY

               DEX

               BPL PUTPL1
               LDA PUTPSM1+1 ;MOVE UP ONE LINE

               SEC

               SBC #WW

               STA PUTPSM1+1

               BCS PUTPL3
               DEC PUTPSM1+2

PUTPL3         DEC COUNTER

               BNE PUTPL2
               TYA

               CLC

               ADC HL
               STA HL
               BCC PUTPL4
               INC HL+1
PUTPL4         DEC NEWNAME

               BNE PUTPO1
               RTS


;***********************************************
;*             Get ON-SCREEN X,Y co-ords
;*             Work out transfer position
;*             Store both above in current IX block
;***********************************************
PUTTRN         STX ROTATE ;temp save

               JSR THREED ;get ON-SCREEN X,Y coords
PUTTRNSH       LDY #$0E
               LDA (IX),Y ;ON-SCREEN X POS

               STA ZPG

               INY ;=$0F

               LDA (IX),Y ;ON-SCREEN Y POS

               BPL SEEI6

               LDA #$7F
SEEI6          STA ZPG+1
               JSR TRNPOS
               LDX ROTATE
               LDY #$07
               LDA ZPG

               STA (IX),Y ;TRANSFER ADDR LO
               INY ;=$08

               LDA ZPG+1

               STA (IX),Y ;TRANSFER ADDR HI
               RTS


;****************************************
;*             X,Y TO TRANSFER POSITION
;*
;*             Send: on-screen x-co in ZPG
;*                            on-screen y-co in ZPG+1
;*
;*             Receive: address in work area in ZPG,ZPG+1
;*
;****************************************
TRNPOS         LDA ZPG+1 ;Y POS
               ASL ; = ASL A
               ASL ; = ASL A ;*4
               STA ZPA ;TEMP

               AND #0
               ROL ; = ROL A
               ASL ZPA
               ROL ; = ROL A ;*8
               STA ZPA+1

               LDA ZPA
               ASL ; = ASL A
               TAY

               LDA ZPA+1
               ROL ; = ROL A
               TAX
               TYA

               ;clc
               ADC ZPA
               STA ZPA

               TXA
               ADC ZPA+1
               TAX

               LDA ZPG ;X POS
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A ;/8
               CLC
               ADC ZPA
               STA ZPG ;TRANSFER ADDRESS LO
               BCC TRNP1
               INX
TRNP1          TXA
               CLC
               ADC #>WA ;lo byte=0
               STA ZPG+1 ;TRANSFER ADDRESS HI
               RTS


;****************************************
;*             SET UP SHADOW
;*             IX points to current IX block
;*             IY points to current SHADO table
;****************************************
DIDNT          LDY #0
               TYA
               STA (IY),Y ;clear shape type - NO SHADOW
               RTS


TRSHAD         LDY #$11
               LDA (IX),Y ;object height above its shadow
               BEQ DIDNT

               CMP #$FF
               BEQ DIDNT

               LDX #0 ;work out shadow size
               CMP #$17
               BCC SIZSH

               INX ;=1

               CMP #$25
               BCC SIZSH

               INX ;=2

SIZSH          LDY #$10

               ORA #$FF
               STA ROTATE ;flag:0=spike shadow

               LDA (IX),Y ;bit6 set if shadow is spike

               AND #$40
               BNE SIZSH1
               TXA

               JMP NORSH



               ;Spike
SIZSH1         INC ROTATE ;=0
               LDY #$06
               LDA (IX),Y ;z-co (lo) {=object height above spike}
               CMP #$05
               INX

               TXA

               BCS TOOH1 ;branch if object high enough to have shadow
               ;Impaled !
               LDY #$0A
               LDA (IX),Y ;BITS 6&7=SPIKE SHAPE
               AND #$C0
               BNE TOOH2

               LDA (IX),Y
               ORA #$40
               STA (IX),Y
               ;JSR SPKIM....SOUND.....!!!!!??????????

               JMP TOOH3

TOOH2          LDA LOPCNT

               AND #$03
               BNE TOOH3

               LDA (IX),Y ;BITS 6&7=SPIKE SHAPE
               CLC

               ADC #$40
               BCC TOOH4

               JSR EXPSET
               LDY #$10 ;RESET SHADOW TYPE

               LDA (IX),Y
               AND #$BF
               STA (IX),Y
               LDY #$0A ;RESET SPIKE SHAPE

               LDA (IX),Y
               AND #$3F
               STA (IX),Y
               JMP DIDNT ;exit

TOOH4          STA (IX),Y

TOOH3          LDY #$0A

               LDA (IX),Y
               ROL ; = ROL A

               ROL ; = ROL A

               ROL ; = ROL A

               EOR #$FF
               CLC

               ADC #1 ;2'S COMPLEMENT
               AND #$03
TOOH1          CLC
               ADC #$02
TOOH5          LDX #$02

NORSH          LDY #$01

               STA (IY),Y ;data offset (set shadow size)

               TXA

               CLC

               ADC #$FC
               EOR #$FF
               ASL ; = ASL A ;gives shadow radius in pixels (2/4/6)
               STA SRA ;shadow radius
               ASL ; = ASL A ;shadow diameter
               LDY #$0B
               STA (IY),Y ;shadow x-dim
               ;ldy #$0B

               LDA (IX),Y ;shape x-dim

               LSR ; = LSR A ;/2

               CLC

               LDY #$03
               ADC (IX),Y ;shape x-co lo

               TAX

               DEY ;=$02

               AND #0
               ADC (IX),Y ;shape x-co hi

               STA ZPA+1 ;temp

               TXA

               SEC

               SBC SRA ;shadow radius
               INY ;=$03

               STA (IY),Y ;shadow x-co lo
               LDA ZPA+1 ;^temp
               SBC #0
               DEY ;=$02

               STA (IY),Y ;shadow x-co hi
               LDY #$0C
               LDA (IX),Y ;shape y-dim

               LSR ; = LSR A ;/2

               CLC

               LDY #$05
               ADC (IX),Y ;shape y-co lo

               TAX

               DEY ;=$04

               AND #0
               ADC (IX),Y ;shape y-co hi

               STA ZPA+1 ;temp

               TXA

               SEC

               SBC SRA ;shadow radius
               INY ;=$05

               STA (IY),Y ;shadow y-co lo
               LDA ZPA+1 ;^temp
               SBC #0
               DEY ;=$04

               STA (IY),Y ;shadow y-co hi
               LDA ROTATE ;0=spike

               BEQ NITBO

               LDY #$06
               LDA (IX),Y ;shape z-co (lo)

               SEC

               LDY #$11
               SBC (IX),Y ;object height above its shadow
               BCS NITBO

               LDA #$00
NITBO          LDY #$06

               STA (IY),Y ;shadow z-co (lo)
               AND #0
               LDY #$0A
               STA (IY),Y ;CLEAR SPIKE SHAPE (TOP 2 BITS)
               LDY #0
               LDA #$02
               STA (IY),Y ;set shape=shadow
               LDA IX
               PHA
               LDA IX+1
               PHA

               LDA IY
               STA IX
               LDA IY+1
               STA IX+1
               JSR DIMS ;SET UP DIMENSIONS IN IX BLOCK

               PLA
               STA IX+1
               PLA
               STA IX
DIMRET         RTS



;****************************************

;*             Get chars across and pixels up

;*             of shape to IX block

;*             setup: shape

;*                             data offset
;****************************************

DIMS           JSR GETDATA

               LDY #$09
               LDA (IX),Y
               AND #$F0 ;preserve disolve bits

               LDX #0
               ORA (DATA,X) ;OR in width of shape in chars

               STA (IX),Y
               INY ;=$0A

               LDA (IX),Y
               AND #$C0 ;preserve spike shape bits

               LDY #$01
               ORA (DATA),Y ;OR IN PIXELS UP SHAPE

               LDY #$0A
               STA (IX),Y
               RTS



EXPSET         LDY #$00
               LDA #$07
               STA (IX),Y ;SHAPE=AN EXPLOSION...

               TYA ;=$00

               INY ;=1

               STA (IX),Y ;FRAME: EXP1

               JSR DIMS
               ;JSR EXPSO....SOUND...!!!!!

               JMP THREED


;********************************************************************
;*             PRINT BACKGROUND OBJECTS AS PLANES @ EDGES OF WORK AREA
;********************************************************************
OKMLOOP        JSR WOBBLE ;update all sprite defintions and lift size

               JSR SAVPOS ;save backgrounds (moving shapes) and set up sort table
               JSR ALOCAT ;do sort into YORDER table

               JSR MOVSHPRT ;print sorted shapes

               JSR WKTOBUF ;copy work area to screen

               JSR PUTPOS ;restore saved backgrounds to work area
               ;LDA #$35
               LDA #FLIPIN
               STA $01

               LDA SRA

               ORA #$0F
               LDX IXTAB+$0F ;on-screen y
               CPX #$28
               BCS TSC1
               AND #$FE ;up
TSC1           CPX #$58
               BCC TSC2
               AND #$FD
TSC2           LDX IXTAB+$0E ;on-screen x
               CPX #$38
               BCS TSC3
               AND #$FB
TSC3           CPX #$88
               BCC TSC4
               AND #$F7
TSC4           PHA

               LDA TRAP
               BEQ NO_TRAP

               PLA

               LDA #%11111111
               PHA

               LDA SCR_LR+1
               CMP SCREENX+1

               BEQ CONT_LR

               BCS RIGHT_LR
LEFT_LR        PLA
               LDA #%00001011
               PHA

               JMP NO_LR

RIGHT_LR       PLA

               LDA #%00000111
               PHA

               JMP NO_LR

CONT_LR        LDA SCR_LR

               CMP SCREENX

               BEQ NO_LR

               BCS RIGHT_LR
               JMP LEFT_LR

NO_LR          LDA SCR_UD+1

               CMP SCREENY+1

               BEQ CONT_UD

               BCS UP_UD

DOWN_UD        PLA
               AND #%11111100
               ORA #%00000010
               PHA

               JMP NO_UD

UP_UD          PLA
               AND #%11111100
               ORA #%00000001
               PHA

               JMP NO_UD

CONT_UD        LDA SCR_UD

               CMP SCREENY

               BEQ NO_UD

               BCS UP_UD

               JMP DOWN_UD

NO_UD
NO_TRAP        PLA
               LDX #0
               STX SCROLL
               LSR ; = LSR A

               BCS NOTUP

               INC SCROLL
               DEC YPIX
               DEC YPIX
               BPL NOTUP

               PHA

ISUP1          JSR SHIFTDOWN
               LDA #<WA
               STA INITSTRDAT
               LDA #>WA
               STA INITSTRDAT+1
               LDA SCREENY

               SEC

               SBC #8
               STA SCREENY

               BCS LZ474

               DEC SCREENY+1

               SEC

LZ474          SBC #$40

               STA STRIPY
               LDA SCREENY+1

               SBC #0
               STA STRIPY+1
               LDA SCREENX

               STA STRIPX
               CLC

               ADC #$C0
               STA ZPD

               LDA SCREENX+1

               STA STRIPX+1
               ADC #0
               STA ZPD+1

               JSR SET_SPECAL
               JSR PRHSTRIP
               LDA #4
NOTUP1         STA YPIX
               PLA

NOTUP          LSR ; = LSR A
               BCS NOTDOWN

               INC SCROLL
               INC YPIX
               INC YPIX       ;delete
               LDX YPIX
               CPX #8
               BCC NOTDOWN

               PHA

ISDOWN         JSR SHIFTUP

               LDA #<[WA+[15*8*WA_WIDTH_BYTES]]
               STA INITSTRDAT
               LDA #>[WA+[15*8*WA_WIDTH_BYTES]]
               STA INITSTRDAT+1
               LDA SCREENY

               CLC

               ADC #8
               STA SCREENY

               BCC LZ475

               INC SCREENY+1

               CLC

LZ475          ADC #$38

               STA STRIPY
               LDA SCREENY+1

               ADC #0
               STA STRIPY+1
               LDA SCREENX

               STA STRIPX
               CLC

               ADC #$C0
               STA ZPD

               LDA SCREENX+1

               STA STRIPX+1
               ADC #0
               STA ZPD+1

               JSR SET_SPECAL
               JSR PRHSTRIP
               LDA #4
NOTDOWN1       STA YPIX
               PLA

NOTDOWN        LSR ; = LSR A
               BCS NOTLEFT

               INC SCROLL
               DEC XPIX
               DEC XPIX       ;delete
               BPL NOTLEFT

               PHA

ISLEFT         JSR SHIFTRIGHT
               LDA #<WA
               STA INITSTRDAT
               LDA #>WA
               STA INITSTRDAT+1
               LDA SCREENX

               SEC

               SBC #$10
               STA SCREENX

               BCS LZ477

               DEC SCREENX+1

LZ477          STA STRIPX

               LDA SCREENX+1

               STA STRIPX+1
               JSR DOVSTRCDS

               JSR SET_SPECAL
               JSR PRVSTRIP
               LDA #8
NOTLEFT1       STA XPIX
               PLA

NOTLEFT        LSR ; = LSR A
               BCS NOTRIGHT
               INC SCROLL
               INC XPIX
               INC XPIX       ;delete
               LDX XPIX
               CPX #$10
               BCC NOTRIGHT
               PHA

ISRIGHT        JSR SHIFTLEFT
               LDA #<[WA+$16]
               STA INITSTRDAT
               LDA #>[WA+$16]
               STA INITSTRDAT+1
               LDA SCREENX

               CLC

               ADC #$10
               STA SCREENX

               BCC LZ479

               INC SCREENX+1

LZ479          CLC
               ADC #$B0
               STA STRIPX
               LDA SCREENX+1

               ADC #0
               STA STRIPX+1
               JSR DOVSTRCDS

               JSR SET_SPECAL
               JSR PRVSTRIP
               LDA #8
NOTRIGHT1      STA XPIX

               PLA

NOTRIGHT       LSR ; = LSR A
               RTS


;****************************************
;*             set up next stage
;****************************************
RETURNOKM      LDA STAGE
               CMP #MAX_STAGE
               BEQ EXITOKM

               JSR DECTIMER
               JSR RESET_BOT

               JSR RESET_BUNG
               INC STAGE

               LDA #1
               STA TABLE_ENT

               JSR SETIMEOUT

               LDY STAGE
               LDX BOTT_NO,Y
               STX BOTTLES

               LDA PLAYER
               BEQ ITS_ONE_AG
               STX BOTTLES+2

               JMP ITS_NOT_1

ITS_ONE_AG     STX BOTTLES+1

ITS_NOT_1      JSR SECSETUP

               JSR SET_ST_POS ;set start pos on stage
               JSR RENTERY

               LDA #0
               STA EXITO

               RTS

;;;
;;; 2009-10: MY: Changed this to display the "end of game" message and get back
;;;              to title page cleanly.
;;;
EXITOKM        LDA #FLIPOUT
               STA $01

               ; Set play area to white
               LDA #$10
               JSR SET_PLAY_AREA_COLOUR

               LDA #10
               STA XVAL
               LDA #1
               STA YVAL

               LDA #<$D000
               STA SMTAB+1
               LDA #>$D000
               STA SMTAB+2

               LDA #<GAME_COMPLETE_MESSAGE
               STA ZPA
               LDA #>GAME_COMPLETE_MESSAGE
               STA ZPA+1
               LDY #0
               STY SRA
WRITE_MESS_1   LDA (ZPA),Y ; Next character in the message
               ; $FF -- or rather, anything >$80, means read next byte:
               ;   0 = end of message
               ;   >0 = insert this many spaces
               ; $FE means toggle double-height printing on/off
               BPL WRITE_MESS_4
               ; We've got either $FF or $FE
               ; Flip the bits to turn this into 0 or 1 respectively
               ; Then we can use the 1 to toggle the value in DOUBLE_HEIGHT
               EOR #$FF
               BEQ WRITE_MESS_7
               EOR DOUBLE_HEIGHT
               STA DOUBLE_HEIGHT
               JMP WRITE_MESS_6

WRITE_MESS_7   INY
               BNE WRITE_MESS_5
               INC ZPA+1
WRITE_MESS_5   LDA (ZPA),Y
               BEQ WRITE_MESS_FIN
               STA SRA

WRITE_MESS_SPC LDA #$20 ; space

WRITE_MESS_4   STA TABVAL

               JSR PLOT

               LDX XVAL
               INX
               CPX #30
               BCC WRITE_MESS_3
               LDX #10
               INC YVAL
               ; If we're in double-height mode we need to jump an extra row
               ; down when we reach the last column
               LDA DOUBLE_HEIGHT
               BEQ WRITE_MESS_3
               INC YVAL

WRITE_MESS_3   STX XVAL

               ; Check whether we're doing repeating spaces
               LDX SRA
               BEQ WRITE_MESS_6
               DEC SRA
               BNE WRITE_MESS_SPC

WRITE_MESS_6   INY
               BNE WRITE_MESS_1
               INC ZPA+1
               BNE WRITE_MESS_1 ; = JMP

WRITE_MESS_FIN LDA #FLIPIN
               STA $01

               JSR GAMEOFF
;               JMP RT ;return to PDS control
               RTS ; from RETURNOKM


;****************************************
;*             Set up ST_POS to start pos
;*             on each stage
;*
;*      Input: STAGE
;*
;*      Output: ST_POS
;****************************************
SET_ST_POS     LDY STAGE

               LDA BLOBOXL,Y
               STA ST_POS
               LDA BLOBOXH,Y
               STA ST_POS+1

               LDA BLOBOYL,Y
               STA ST_POS+2
               LDA BLOBOYH,Y
               STA ST_POS+3

               LDA BLOBOZL,Y
               STA ST_POS+4

               LDA MAP_ORG_XL,Y
               STA ST_POS+5
               LDA MAP_ORG_XH,Y
               STA ST_POS+6

               LDA MAP_ORG_YL,Y
               STA ST_POS+7
               LDA MAP_ORG_YH,Y
               STA ST_POS+8

               RTS


;****************************************
;*             Origin for map to scroll down
;****************************************
MAP_ORG_XL     BYTE 0,$A0,$B0,$B0,$F0,$A0
MAP_ORG_XH     BYTE 0,$03,$03,$04,$05,$08
MAP_ORG_YL     BYTE 0,$C0,$E0,$F0,$70,$40
MAP_ORG_YH     BYTE 0,$80,$7E,$7F,$7F,$83


;****************************************
;*             Origin for blob -- indexed by stage
;****************************************
BLOBOXL        HEX 00C10080E0E0
BLOBOXH        HEX 000201020207

BLOBOYL        HEX 0021F0705010
BLOBOYH        HEX 000102020301

BLOBOZL        HEX 001040106000



DOVSTRCDS      LDA SCREENY
               SEC
               SBC #$40
               STA STRIPY
               LDA SCREENY+1
               SBC #0
               STA STRIPY+1

               LDA SCREENY
               CLC
               ADC #$40
               STA ZPD
               LDA SCREENY+1
               ADC #0
               STA ZPD+1

               RTS


               ;LIST OF PLANES (MAX 3 FOR EACH OBJECT)
               ;USED AS OFFSET TO JUMP TABLES FOR PRINTING
               ;0=NO MORE PLANES FOR THIS SHAPE
OBJPLANES
O1             BYTE 9,$D,0

O2             BYTE 2,0 ;floor

O3             BYTE 4,5,1,0
O4             BYTE 6,$A,0

O5             BYTE 8,$B,0

O6             BYTE 3,0 ;shadow
O7             BYTE 7,$C,0

O8             BYTE $10,$E,4,0

O9             BYTE $12,4,0
OC             BYTE 5,$11,$F,0

OD             BYTE 5,$13,0
O10            BYTE $17,0 ;support pole

O11            BYTE $14,$16,0 ;bottle

O12            BYTE $15,$16,0 ;bunged bottle
O13            BYTE $1A,0 ;top trap door
O14            BYTE $18,0 ;LH side trap door
O15            BYTE $1B,0 ;exit trap door

O16            BYTE $19,0 ;RH side trap door


OBJPTOFF       BYTE 0,0,O2-O1,O3-O1,O4-O1,O5-O1,O6-O1,O7-O1
               BYTE O8-O1,O9-O1,0,0,OC-O1,OD-O1,0,0,O10-O1

               BYTE O11-O1,O12-O1,O13-O1,O14-O1,O15-O1,O16-O1


;*******************************************************************
;*             Print a vertical strip (2 bytes=16 pixels wide)
;*             SETUP:Background objects to be printed in SPECAL table
;*
;*                            Strip origin x,y (16 bits each) in STRIPX,STRIPY
;*                            STRIPY+strip height in ZPD (16 bits)
;*                            Ptr to top of strip in work area: INITSTRDAT
;*******************************************************************
PRVSTRIP       LDA #(SPECAL+1)&255

               STA ZPG

               LDA #(SPECAL+1)/256

               STA ZPG+1

               LDA SPECAL
               BEQ EXPRVSTRIP
               STA SRA

PRVSTL         LDY #$A

               LDA (ZPG),Y ;shape #
               TAX

               LDA OBJPTOFF,X
               STA COUNTER

PRPLVL         LDX COUNTER

               LDY O1,X
               BEQ LPVOBJS

               LDA INITSTRDAT
               STA STRIPDAT
               LDA INITSTRDAT+1
               STA STRIPDAT+1
               LDA PLANESJL,Y
               STA PLANESJSR+1

               LDA PLANESJH,Y
               STA PLANESJSR+2

PLANESJSR      JSR $0000 ;print a strip of current plane
               INC COUNTER

               BNE PRPLVL ;jmp

LPVOBJS        LDA ZPG ;next SPECAL object
               CLC

               ADC #SPECSIZ
               STA ZPG

               BCC LZ4

               INC ZPG+1

LZ4            DEC SRA
               BNE PRVSTL
EXPRVSTRIP     RTS ;printed all planes of all objs in strip


PLANESJL       BYTE <0, <PL1, <PL2, <PL3, <PL4, <PL5, <PL6, <PL7, <PL8, <PL9, <PLA, <PLB, <PLC
               BYTE <PLD, <PLE, <PLF, <PL10, <PL11, <PL12, <PL13, <PL14, <PL15, <PL16, <PL17
               BYTE <PL18, <PL19, <PL1A, <PL1B
PLANESJH       BYTE >0, >PL1, >PL2, >PL3, >PL4, >PL5, >PL6, >PL7, >PL8, >PL9, >PLA, >PLB, >PLC
               BYTE >PLD, >PLE, >PLF, >PL10, >PL11, >PL12, >PL13, >PL14, >PL15, >PL16, >PL17
               BYTE >PL18, >PL19, >PL1A, >PL1B


PL1            JSR SETDATP1
               JMP TOPPLANE
PL2            JSR SETDATP2
               JMP TOPPLANE
PL3            JSR SETDATP3
               JMP TOPPLANE
PL4            JSR SETDATP4
               JMP SIDSQP1
PL5            JSR SETDATP5
               JMP SIDSQP2
PL6            JSR SETDATP6
               JMP SLOPFC1
PL7            JSR SETDATP7
               JMP SLOPFC2
PL8            JSR SETDATP8
               JMP FSLPFC1
PL9            JSR SETDATP9
               JMP FSLPFC2
PLA            JSR SETDATPA
               JMP TRISL1
PLB            JSR SETDATPB
               JMP TRISL2
PLC            JSR SETDATPC
               JMP RTRISL1
PLD            JSR SETDATPD
               JMP RTRISL2
PLE            JSR SETDATPE
               JMP SMLSSFC
PLF            JSR SETDATPF
               JMP RSMLSSF
PL10           JSR SETDATPCC
               JMP FCNGTR1
PL11           JSR SETDATPAA
               JMP FCNGTR2
PL12           JSR SETDATP12
               JMP FCNGTR3
PL13           JSR SETDATP13
               JMP FCNGTR4
PL14           JSR SETDATP14
               JMP PTPBOTL
PL15           JSR SETDATP15
               JMP PTPBOTL
PL16           JSR SETDATP16
               JMP PMBYTEOTL
PL17           JSR SETDATP17
               JMP PRVPOLE
PL18           JSR SETDATP18
               JMP PRSDTDR
PL19           JSR SETDATP19
               JMP RPSDTDR
PL1A           JSR SETDATP1A
               JMP PTPTRDR
PL1B           JSR SETDATP1B
               JMP PTPTRDR

SETDATP1
TOPMD1         LDA #<ZAZ3 ;smc
               STA ZPF
TOPMD2         LDA #>ZAZ3 ;smc
TOPMD3         LDX #<ZAZ3A ;smc
TOPMD4         LDY #>ZAZ3A ;smc
SETDSTR        STA ZPF+1
               STX ZPE
               STY ZPE+1
               RTS


SETDATP2
FLRMD1         LDA #<ZAZ1 ;smc
               STA ZPF
FLRMD2         LDA #>ZAZ1 ;smc
FLRMD3         LDX #<ZAZ1A ;smc
FLRMD4         LDY #>ZAZ1A ;smc
               JMP SETDSTR


SETDATP3       LDA #<SHDOW
               STA ZPF
               LDA #>SHDOW
               LDX #<SHDOWA
               LDY #>SHDOWA
               JMP SETDSTR


SETDATP4
LFTMD1         LDA #<ZAZ7 ;smc
               STA ZPF
LFTMD2         LDA #>ZAZ7 ;smc
LFTMD3         LDX #<ZAZ7A ;smc
LFTMD4         LDY #>ZAZ7A ;smc
               JMP SETDSTR


SETDATP5
RGTMD1         LDA #<ZAZ5 ;smc
               STA ZPF
RGTMD2         LDA #>ZAZ5 ;smc
RGTMD3         LDX #<ZAZ5A ;smc
RGTMD4         LDY #>ZAZ5A ;smc
               JMP SETDSTR


SETDATP6       LDA #<THIRT
               STA ZPF
               LDA #>THIRT
               LDX #<THIRTA
               LDY #>THIRTA
               JMP SETDSTR


SETDATP7       LDA #<RTHIRT
               STA ZPF
               LDA #>RTHIRT
               LDX #<RTHIRTA
               LDY #>RTHIRTA
               JMP SETDSTR


SETDATP7A      LDA #<RTHIRTA
               STA ZPF
               LDA #>RTHIRTA
               LDX #<RTHIRT
               LDY #>RTHIRT
               JMP SETDSTR


SETDATP8       LDA #<FORT
               STA ZPF
               LDA #>FORT
               LDX #<FORTA
               LDY #>FORTA
               JMP SETDSTR


SETDATP9       LDA #<RFORT
               STA ZPF
               LDA #>RFORT
               LDX #<RFORTA
               LDY #>RFORTA
               JMP SETDSTR


SETDATP9A      LDA #<RFORTA
               STA ZPF
               LDA #>RFORTA
               LDX #<RFORT
               LDY #>RFORT
               JMP SETDSTR


SETDATPA       LDA #<THIRTSD
               STA ZPF
               LDA #>THIRTSD
               JMP SETDSTR


SETDATPAA      LDA #<SIDSLP
               STA ZPF
               LDA #>SIDSLP
               LDX #<SIDSLPA
               LDY #>SIDSLPA
               JMP SETDSTR


SETDATPB       LDA #<FORTSD
               STA ZPF
               LDA #>FORTSD
               JMP SETDSTR


SETDATPC       LDX #<RTHSHD
               LDY #>RTHSHD
SETZPFOY       STX ZPF
               STY ZPF+1
               RTS


SETDATPCC      LDA #<SIDSLO
               STA ZPF
               LDA #>SIDSLO
               LDX #<SIDSLOA
               LDY #>SIDSLOA
               JMP SETDSTR


SETDATPD       LDX #<RFTSHD
               LDY #>RFTSHD
               JMP SETZPFOY


SETDATPE       LDA #<THIRT1
               STA ZPF
               LDA #>THIRT1
               LDX #<THRT1A
               LDY #>THRT1A
               JMP SETDSTR


SETDATPF       LDA #<RTHIRT1
               STA ZPF
               LDA #>RTHIRT1
               LDX #<RTHRT1A
               LDY #>RTHRT1A
               JMP SETDSTR


SETDATP12      LDX #<ENDD45
               LDY #>ENDD45
               JMP SETZPFOY


SETDATP13      LDX #<END45
               LDY #>END45
               JMP SETZPFOY


SETDATP14      LDX #<BOKTOP
               LDY #>BOKTOP
               JMP SETZPFOY


SETDATP15      LDX #<BUNTOP
               LDY #>BUNTOP
               JMP SETZPFOY


SETDATP16      LDA #<BOKBOT
               STA ZPF
               STA ZPE
               LDA #>BOKBOT
               STA ZPF+1
               STA ZPE+1
               RTS


SETDATP17      LDA #<POLE
               STA ZPF
               STA ZPE
               LDA #>POLE
               STA ZPF+1
               STA ZPE+1
               RTS


SETDATP18      LDX #<DOOR0
               LDY #>DOOR0
               JMP SETZPFOY


SETDATP19      LDX #<DOOR1
               LDY #>DOOR1
               JMP SETZPFOY


SETDATP1A      LDX #<DOORT
               LDY #>DOORT
               JMP SETZPFOY


SETDATP1B      LDX #<EXIT
               LDY #>EXIT
               JMP SETZPFOY


;*----------------------
               ;Extra routine for sides of 30' slopes with shape numbers $8 & $C

               ;...makes slope look nicer by putting edge in right place

               ;this one for vertical strips...
XTRBSTARD      ;first print a black line
               AND #0
               TAY

               STA (STRIPDAT),Y
               INY

               STA (STRIPDAT),Y
               ;now fix it so that first line of the next 8*16 block is skipped
               LDA ZPB

               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS EXBPLSQTR ;branch if reached final tri (bot of plane strip)

               LDA POS

               AND #$20
               BNE EXBPMBYTEK
               ORA #4
               STA POS+1

               LDA POS

               ORA #4
               STA POS

               LDA #7 ;# of lines to print

               STA ROTATE ;counter

               LDX #$18 ;offset for STRIPDAT

               JSR TSSERIN

               JMP PRSDSQP1L ;continue in main print loop


EXBPMBYTEK     AND #0
               STA POS

               LDY #$24 ;data offset

               LDA #7 ;# of lines to print

               STA ROTATE ;counter

               LDX #$18 ;offset for STRIPDAT

               JSR BSTRDIN

               JMP PRSDSQP1L ;continue in main print loop


               ;print last triangle
EXBPLSQTR      LDY #$44 ;data offset
               LDA #7 ;# of lines to print

               STA ROTATE ;counter

               LDX #$18 ;offset for STRIPDAT

               JMP BSTRDIN

;*----------------------




FTPPLNCDS      JSR GETPORG
               JMP CONPTOQ



               ;find origin of plane

GETPORG        LDY #$B
               LDA (ZPG),Y ;xdim

               LSR ; = LSR A ;/2

               STA XSM

               INY ;=$C
               LDA (ZPG),Y ;ydim

               LSR ; = LSR A ;/2

               STA YSM

               CLC

               ADC XSM

               STA LINEC

               LDY #$2

               LDA (ZPG),Y ;x-co hi
               STA ZPA+1

               INY ;=$3
               LDA (ZPG),Y ;x-co lo
               STA ZPA

               INY ;=$4
               LDA (ZPG),Y ;y-co hi
               STA ZPB+1

               INY ;=$5
               LDA (ZPG),Y ;y-co lo
               STA ZPB

               INY ;=$6
               LDA (ZPG),Y ;z-co (lo)
               CLC

               LDY #$D

               ADC (ZPG),Y ;zdim

               STA ZPC ;no carry

               RTS



               ;convert P xyz to Q xy
CONPTOQ        JSR SIDSQSUB2
               LDA SRT1 ;Qy lo

               SEC

               SBC YSM ;ydim/2

               STA SRT1
               BCS DNTPRINT
               DEC SRT1+1 ;Qy hi



DNTPRINT       RTS



TOPPLANE       JSR FTPPLNCDS

               ;check for print...

               JSR HORCHCKSL

               LDX SRT1+1 ;Qy hi

               LDY #1
               LDA ZPA

               SEC

               SBC YSM ;ydim/2

               BCS FINTTYP

TOPTYP0        DEY
               LDA YSM

               SEC

               SBC ZPA

               SBC #8
FINTTYP        CLC
               ADC SRT1 ;Qy lo

               BCC LZ4163
               INX

LZ4163         STA ZPB

               STX ZPB+1

               STY DATA
               ;find bot type & so get bot pos

               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC XSM

               BCC BOTTYP0

               STA ZPC ;temp

               LDA LINEC

               ;sec
               SBC ZPC

               SBC #8
               CLC

               JMP FINBTYP

BOTTYP0        LDA ZPA
               ;clc
               ADC YSM

FINBTYP        ;clc

               ADC SRT1 ;Qy lo

               BCC LZ4164
               INX

LZ4164         STA ZPC

               STX ZPC+1

               ;check top/bottom of strip of plane in range
               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCS LZ417

DONTPRINT      RTS ;dont print
LZ417          LDA ZPB
               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS DONTPRINT

               ;set up data offset flags for printing
               LDA #$20
               STA POS

               AND #0
               STA POS+1

               ;now find how far down strip to start etc

LZ418          LDA ZPB
               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS TOPTRIINS

               ;top triangle is off top of strip

               LDA STRIPY
               SEC

               SBC ZPB

               LSR ; = LSR A

               LSR ; = LSR A

               LSR ; = LSR A ;/8

               AND #1
               EOR #1
               EOR DATA ;top type
               STA DATA
               LDA STRIPY
               STA ZPB

               LDA STRIPY+1
               STA ZPB+1

               JSR DOTRIDATMD
               JMP TPSTRIN

               ;print top triangle half
TOPTRIINS      JSR SKIPLINES
               JSR DOTRIDATMD
               LDY #0 ;data offset

               JSR DO1TRI


PRTRILOOP      LDA STRIPDAT ;move ptr in work area down 8 pixel lines

               CLC

               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ421

               INC STRIPDAT+1
               CLC

LZ421          LDA ZPB ;Ttopy - add 8 to vertical coord

               ADC #8
               STA ZPB

               BCC LZ420

               INC ZPB+1

LZ420          CMP ZPD ;Sy+h
               LDA ZPB+1

               SBC ZPD+1

               BCS EXPRTRI ;branch if bot of strip reached

TPSTRIN        LDA ZPB
               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS PRLSTTRI ;branch if reached final tri (bot of plane strip)
               JSR DO2TRIS

               JMP PRTRILOOP



PRLSTTRI       ;print bottom triangle half

               LDA POS+1

               AND #$20
               BEQ LZ422 ;branch if "1st" triangle is one that needs finishing

               JSR SWOPPTRS ;swop data ptrs for "1st" & "2nd" triangles
LZ422          LDY #$20 ;data offset
               JMP DO1TRI
EXPRTRI        RTS


DOTRIDATMD     LDA DATA
               BEQ DTDMDE
SWOPPTRS       LDX ZPE

               LDY ZPE+1

               LDA ZPF

               STA ZPE

               LDA ZPF+1

               STA ZPE+1

               STX ZPF

               STY ZPF+1

DTDMDE         RTS



               ;print one triangle

DO1TRI         LDA #8 ;# of lines to print

               STA ROTATE ;counter

               LDX #0 ;offset for STRIPDAT

               ;set up smc for printing in work area

BSTRDIN        LDA STRIPDAT

               STA STRMD1+1
               STA STRMD2+1
               STA STRMD3+1
               STA STRMD4+1
               LDA STRIPDAT+1
               STA STRMD1+2
               STA STRMD2+2
               STA STRMD3+2
               STA STRMD4+2
STRMD1         LDA $FFFF,X ;smc
               AND (ZPF),Y

               INY

               ORA (ZPF),Y

               INY

STRMD2         STA $FFFF,X ;smc
               INX

STRMD3         LDA $FFFF,X ;smc
               AND (ZPF),Y

               INY

               ORA (ZPF),Y

               INY

STRMD4         STA $FFFF,X ;smc
               TXA

               CLC

               ADC #$17
               TAX

               DEC ROTATE
               BNE STRMD1
               RTS



               ;print 2 triangles
DO2TRIS        LDA POS ;flip data offsets

               AND #$20
               STA POS

               LDA POS+1

               AND #$20
               STA POS+1

DO2SECS        LDA #8 ;# of lines to print
               STA ROTATE ;counter

               LDX #0 ;offset for work area block
TSSERIN        LDA STRIPDAT ;set up smc for printing in work area

               STA STRMD5+1
               STA STRMD6+1
               STA STRMD7+1
               STA STRMD8+1
               LDA STRIPDAT+1
               STA STRMD5+2
               STA STRMD6+2
               STA STRMD7+2
               STA STRMD8+2
PR2TRL

STRMD5         LDA $FFFF,X ;smc
               LDY POS

               AND (ZPF),Y

               INY

               ORA (ZPF),Y

               INY

               STY POS

               LDY POS+1

               AND (ZPE),Y

               INY

               ORA (ZPE),Y

               INY

               STY POS+1

STRMD6         STA $FFFF,X ;smc
               INX

STRMD7         LDA $FFFF,X ;smc
               LDY POS

               AND (ZPF),Y

               INY

               ORA (ZPF),Y

               INY

               STY POS

               LDY POS+1

               AND (ZPE),Y

               INY

               ORA (ZPE),Y

               INY

               STY POS+1

STRMD8         STA $FFFF,X ;smc
               TXA

               CLC

               ADC #$17 ;next line in work area
               TAX

               DEC ROTATE
               BNE PR2TRL
               RTS



SKIPLINES      STA PTR+1 ;=0
               TXA

               ASL ; = ASL A

               ;rol ptr+1 [<$80]

               ASL ; = ASL A

               ROL PTR+1

               ASL ; = ASL A

               ROL PTR+1 ;*8

               STA PTR

               ;clc ^
               ADC STRIPDAT
               STA STRIPDAT
               LDA PTR+1

               ADC STRIPDAT+1
               TAX

               LDA PTR

               ASL ; = ASL A

               ROL PTR+1 ;[*8]*2=*$10 overall
               ;clc
               ADC STRIPDAT
               STA STRIPDAT
               TXA

               ADC PTR+1

               STA STRIPDAT+1
               RTS

;
; File X7.BUB
; Starts Here
;
;

;
;
;  PDS Pc1.10 :000:  (C)  1985-7 P.D Systems
;
;
;       File  "ULTIMATE BUBBLER---"
;    Version          01.03
;         By  "JAKE AND MATTHEW..."
;
;     Created on Tue the 29th of Mar 1988
;        Last update 08:41 on 10/05/88
;

SIDSQSUB       ;find plane origin P xyz
               JSR GETPORG

               ;convert P xyz to Q xy
               ;Qx=Px+Py;Qy=$8000-Pz+Px/2-Py/2

SIDSQSUB2      LDA ZPA
               CLC

               ADC ZPB

               STA SRT0 ;Qx lo

               LDA ZPA+1

               ADC ZPB+1

               STA SRT0+1 ;Qx hi

               LSR ZPB+1

               ROR ZPB ;/2

               LDA ZPA+1

               LSR ; = LSR A

               ROR ZPA ;/2

               CLC

               ADC #$80
               TAX

               LDA ZPA

               SEC

               SBC ZPB

               TAY

               TXA

               SBC ZPB+1

               TAX

               TYA

               SEC

               SBC ZPC

               STA SRT1 ;Qy lo

               TXA

               SBC #0
               STA SRT1+1 ;Qy hi

               RTS



SIDSQP1        JSR SIDSQSUB

               ;check for print
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE DONTPRTIT

               BCS LZ447

DONTPRTIT      RTS ;dont print
LZ447          TXA
               LSR ; = LSR A ;/2

               CMP XSM ;xdim/2

               BCS DONTPRTIT

               STA ZPA

               LDX SRT1+1 ;Qy hi

               ADC SRT1 ;Qy lo

               BCC LZ448

               INX

DOVSQSTR       CLC

LZ448          STA ZPB
               STX ZPB+1

               LDY #$D

               ADC (ZPG),Y ;zdim

               BCC LZ4149
               INX

LZ4149         STA ZPC

               STX ZPC+1

               ;check top/bottom of strip of plane in range
               CMP STRIPY
               TXA

               SBC STRIPY+1
               BCC DONTPRTIT

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS DONTPRTIT

               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PRTOPBIT
               ;top section is off top of strip
               LDA STRIPY
               SEC

               SBC ZPB

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
TRSSQENT       AND #$20
               STA POS

               LDA STRIPY
               STA ZPB

               LDA STRIPY+1
               STA ZPB+1

               JMP MIDSTRENT

PRTOPBIT       JSR SKIPLINES ;fix STRIPDAT

FN2SSL1        LDY #0 ;data offset
               JSR DO1TRI
               LDA #$20
ANOTENT        STA POS


PRSDSQP1L      LDA STRIPDAT ;move ptr in work area down 8 pixel lines

               CLC

               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ450

               INC STRIPDAT+1
               CLC

LZ450          LDA ZPB ;Ttopy - add 8 to vertical coord

               ADC #8
               STA ZPB

               BCC LZ451

               INC ZPB+1

LZ451          CMP ZPD ;Sy+h
               LDA ZPB+1

               SBC ZPD+1

               BCS EXPRSQST ;branch if bot of strip reached
MIDSTRENT      LDA ZPB
               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS PLSTSQTRI ;branch if reached final tri (bot of plane strip)

               LDA POS

               AND #$20
               BNE PRMIBYTELK
               STA POS+1 ;=0

               JSR DO2SECS

               JMP PRSDSQP1L



PRMIBYTELK     AND #0
               STA POS

               LDY #$20 ;data offset

               JSR DO1TRI
               JMP PRSDSQP1L



PLSTSQTRI      LDY #$40 ;data offset
               JMP DO1TRI
EXPRSQST       RTS



RHSSLSUB       JSR GETPORG

               LDA XSM ;xdim/2

               ASL ; = ASL A ;*2=xdim
               ADC ZPA

               STA ZPA

               BCC LZ480

               INC ZPA+1

LZ480          JMP SIDSQSUB2 ;convert to Qxy


DSIDSQSB       JSR RHSSLSUB
               LDA SRT1 ;Qy lo

               SEC

               SBC YSM ;ydim/2

               STA SRT1
               BCS LZ481

               DEC SRT1+1 ;Qy hi

LZ481          RTS


SIDSQP2        JSR DSIDSQSB

               ;check for print
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE EXPRSQST ;dont print
               ;bcc exprsqst

               TXA

               LSR ; = LSR A ;/2

               CMP YSM ;ydim/2

               BCS EXPRSQST ;dont print
               STA ZPA

               LDX SRT1+1 ;Qy hi

               LDA SRT1 ;Qy lo

               CLC

               ADC YSM

               BCC LZ460

               INX

LZ460          SEC
               SBC ZPA

               BCS LZ462

               DEX

               SEC

LZ462          SBC #8

               BCS LZ463

               DEX

LZ463          JMP DOVSQSTR ;rest is same as for plane 4


SLPFCSB        JSR SIDSQSUB

               LDA SRT1 ;Qy lo

               SEC

               SBC YSM ;ydim/2

               STA SRT1
               BCS NOPLPRINT

               DEC SRT1+1 ;Qy hi

NOPLPRINT      RTS


SLOPFC1        JSR SLPFCSB
               JSR HORCHCKSL

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC YSM

               BCC THRPLNT0
               INY

               ASL ; = ASL A ;*2

               JMP CHTADD2

THRPLNT0       LDA YSM

               SEC

               SBC ZPA

               SEC

               SBC #8
CHTADD2        CLC
               ADC SRT1 ;Qy lo

               BCC FINTHTPLN

               INX

FINTHTPLN      STA ZPB
               STX ZPB+1

               STY DATA
               CMP ZPD

               TXA

               SBC ZPD+1

               BCS NOPLPRINT

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC XSM

               BCC THRTPLNB0

               INY

               STA ZPC ;temp

               LDA XSM

               ASL ; = ASL A ;*2

               ;clc
               CLC

               ADC YSM

               SEC

               SBC ZPC

               SEC

               SBC #8
               JMP CHTADD8

THRTPLNB0      LDA ZPA
               ASL ; = ASL A ;*2

               CLC

               ADC YSM

CHTADD8        CLC
               ADC SRT1 ;Qy lo

               BCC FINTHBPL
               INX

THRTPLSM

FINTHBPL       STA ZPC

               STX ZPC+1

               STY DATA+1
               LDA STRIPY
               SEC

               SBC ZPC

               TAX

               LDA STRIPY+1
               SBC ZPC+1

               BCC THPPRSTOK

               BNE NOPRNTPLN

               TXA

               BEQ THPPRSTOK

               CPX #8
               BNE NOPRNTPLN

               TYA

               BNE NOPRNTPLN

               LDY #$40 ;data offset

               JMP DO1TRI
NOPRNTPLN      RTS
THPPRSTOK      LDA ZPB
               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PTTRITHSL

               ;top triangle is off top of strip

               LDA STRIPY
               SEC

               SBC ZPB

               LDY STRIPY
               STY ZPB

               LDY STRIPY+1
               STY ZPB+1

               LDY DATA ;top type
               BEQ NSUBL0
               CMP #$08
               BNE NSUBL0
               LDY #$80
               JMP P2NDTHTTR

NSUBL0         LSR ; = LSR A

               LSR ; = LSR A

               LSR ; = LSR A ;/8

               TAX

               LDA AREM3,X

               DEY ;top type

               BNE NSUBL2
               ;clc
               ADC #$20
               CMP #$60
               BCC NSUBL2
               SBC #$60
NSUBL2         STA POS

               JMP THPMDENT
PTTRITHSL      JSR SKIPLINES ;fix STRIPDAT
               LDA DATA ;top type
               BEQ PRTTRITP0

               LDY #$60
               JSR DO1TRI
               JSR STRIPSUB ;YR not affected

               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXPTHRSL ;branch if bot of strip reached
P2NDTHTTR      JSR DO1TRI

               LDY #0
               JMP PREPTHLP
PRTTRITP0      LDY #0

               JSR DO1TRI
PREPTHLP       STY POS



PRTHSLLP       LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC

               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ492

               INC STRIPDAT+1
               CLC

LZ492          LDA ZPB ;Ttopy - add 8 to vertical coord

               ADC #8
               STA ZPB

               BCC LZ493

               INC ZPB+1

LZ493          CMP ZPD ;Sy+h
               LDA ZPB+1

               SBC ZPD+1

               BCS EXPTHRSL ;branch if bot of strip reached
THPMDENT       LDA ZPB

               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS PRLTHTRI ;branch if reached final tri (bot of plane strip)
               LDA POS

               CMP #$60
               BCC LZ497

               AND #0
               STA POS

               CLC

LZ497          ADC #$40

               CMP #$60
               BCC LZ498

               SBC #$60
LZ498          STA POS+1
               JSR DO2SECS

               JMP PRTHSLLP




PRLTHTRI       LDA DATA+1 ;bot type
               BEQ PBTBGTRI
               LDY #$A0 ;data offset

               JMP DO1TRI
EXPTHRSL       RTS

PBTBGTRI       LDY #$20 ;data offset

               JSR DO1TRI
               JSR STRIPSUB ;YR not affected

               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXPTHRSL ;branch if bot of strip reached
               JMP DO1TRI




NOPLNPRNT      RTS
SLOPFC2        JSR SLPFCSB
               JSR HORCHCKSL

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC YSM

               BCS RTHTTYP0
               INY

               LDA YSM

               SEC

               SBC ZPA

               SBC #8 ;$10/2

               ASL ; = ASL A ;*2

RTHTTYP0       CLC

               ADC SRT1 ;Qy lo

               BCC FINRTHTTP

               INX

FINRTHTTP      STA ZPB
               STX ZPB+1

               STY DATA
               CMP ZPD

               TXA

               SBC ZPD+1

               BCS NOPLNPRNT

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               CMP XSM

               BCC RTHBTYP1
               LDA LINEC

               SEC

               SBC ZPA

               ASL ; = ASL A

               CLC

               ADC XSM

               SEC

               SBC #$10
               JMP CHTADD1

RTHBTYP1       INY

               LDA YSM

               ASL ; = ASL A

               ;clc
               ADC ZPA

CHTADD1        CLC
               ADC SRT1 ;Qy lo

               BCC FNRTHBTP
               INX

FNRTHBTP       JMP THRTPLSM ;rest is same as for plane #6




NOMORPRT       RTS

FSLPFC1        JSR SLPFCSB
               JSR HORCHCKSL

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC YSM

               BCC VFTSTT0

               INY

               STA ZPB ;temp

               ASL ; = ASL A ;*2

               ADC ZPB

               JMP PFNFSLT

VFTSTT0        LDA YSM
               SEC

               SBC ZPA

               SBC #8
PFNFSLT        CLC
               ADC SRT1 ;Qy lo

               BCC FNVFTSL

               INX

FNVFTSL        STA ZPB
               STX ZPB+1

               STY DATA
               CMP ZPD

               TXA

               SBC ZPD+1

               BCS NOMORPRT
               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               CMP XSM

               BCC VFTSLBT1
               LDA XSM

               ASL ; = ASL A

               ASL ; = ASL A ;*4

               SEC

               SBC ZPA

               SEC

               SBC #8
               JMP PNBTFTSL
VFTSLBT1       INY

               LDA ZPA

               ASL ; = ASL A

               ADC ZPA

PNBTFTSL       CLC

               ADC YSM

               CLC

               ADC SRT1 ;Qy lo

               BCC FNBTFTSL
               INX

FNBTFTSL       STA ZPC

               STX ZPC+1

               STY DATA+1
               LDA STRIPY
               SEC

               SBC ZPC

               TAX

               LDA STRIPY+1
               SBC ZPC+1

               BCC FTPPRSTOK

               BNE NOMRPRNT
               TXA

               BEQ JPLFTTR

               CPX #$18
               BCS NOMRPRNT
               TYA

               BEQ NOMRPRNT
               CPX #$08
               BNE LZ4148
               LDY #$40
               JMP NLLSTTR1
LZ4148         LDY #$60
               JMP NLLSTTR2
JPLFTTR        JMP PRLFTTRI

NOMRPRNT       RTS

FTPPRSTOK      ;find how far down strip to start printing

               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS FTVPTPBIT

               ;plane strip starts above screen strip
               LDA STRIPY
               SEC

               SBC ZPB

               LDY STRIPY
               STY ZPB

               LDY STRIPY+1
               STY ZPB+1

               LDY DATA ;top type
               BEQ SUBL5

               CMP #$18
               BCS SUBL6

               CMP #$10
               BCC LZ4145
               LDY #$C0
               JMP P3RDFVSC
LZ4145         LDY #$A0
               JMP P2NDFVSC
SUBL6          CLC
               ADC #$08
SUBL5          AND #$1F

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               STA POS

               JMP FTMIDENT
FTVPTPBIT      JSR SKIPLINES
               LDA DATA ;top type
               BEQ FTVPPTT0
               LDY #$80
               JSR DO1TRI
               JSR STRIPSUB ;YR not affected

               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS NOMRPRNT ;branch if bot of strip reached
P2NDFVSC       JSR DO1TRI
               JSR STRIPSUB ;YR not affected

               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS NOMRPRNT ;branch if bot of strip reached
P3RDFVSC       JSR DO1TRI
               LDY #0
               JMP PREFTVPL
FTVPPTT0       LDY #0 ;data offset

               JSR DO1TRI
PREFTVPL       STY POS





FTVPRLP        LDA STRIPDAT ;move ptr in work area down 8 pixel lines

               CLC

               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ4146
               INC STRIPDAT+1
               CLC

LZ4146         LDA ZPB ;Ttopy - add 8 to vertical coord
               ADC #8
               STA ZPB

               BCC LZ4147
               INC ZPB+1

LZ4147         CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS FINIFTPR ;branch if bot of strip reached
FTMIDENT       LDA ZPB

               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS PRLFTTRI ;branch if reached final tri (bot of plane strip)
               LDA POS

               AND #$7F
               STA POS

               CLC

               ADC #$60
               AND #$7F
               STA POS+1

               JSR DO2SECS

               JMP FTVPRLP



FTSMLBT        JSR SWOPPTRS

               LDY #$60 ;data offset

               JMP DO1TRI


PRLFTTRI       LDA DATA+1 ;bot type
               BEQ FTSMLBT

               LDY #$20 ;data offset

               JSR DO1TRI
               JSR STRIPSUB ;YR not affected

               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS FINIFTPR ;branch if bot of strip reached
NLLSTTR1       JSR DO1TRI
               JSR STRIPSUB ;YR not affected

               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS FINIFTPR ;branch if bot of strip reached
NLLSTTR2       JMP DO1TRI
FINIFTPR       RTS





FSLPFC2        JSR SLPFCSB
               JSR HORCHCKSL

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC YSM

               BCS VFT2TT0

               INY

               LDA YSM

               SEC

               SBC ZPA

               STA ZPB ;temp

               ASL ; = ASL A

               ;clc
               ADC ZPB

               SEC

               SBC #$18
VFT2TT0        CLC
               ADC SRT1 ;Qy lo

               BCC FNVFT2TT
               INX

FNVFT2TT       STA ZPB

               STX ZPB+1

               STY DATA
               CMP ZPD

               TXA

               SBC ZPD+1

               BCS FINIFTPR
               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC XSM

               BCC VFT2BT0

               INY

               LDA LINEC

               SEC

               SBC ZPA

               STA ZPC ;temp

               ASL ; = ASL A ;*2

               CLC

               ADC ZPC ;=*3
               CLC

               ADC XSM

               SEC

               SBC #$18
               JMP CHTADDC

VFT2BT0        LDA YSM
               ASL ; = ASL A ;*2

               ;clc
               ADC YSM

               CLC

               ADC ZPA

CHTADDC        CLC
               ADC SRT1 ;Qy lo

               BCC FNVFT2BT
               INX

FNVFT2BT       JMP FNBTFTSL ;rest is nearly same as for plane 8




EXTRSL1        RTS
TRISL1         JSR SIDSQSUB
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE EXTRSL1 ;dont print

               ;bcc extrsl1
               TXA

               LSR ; = LSR A ;/2

               CMP XSM ;xdim/2

               BCS EXTRSL1 ;dont print

               STA ZPA

               TXA

               ;clc
               ADC SRT1 ;Qy lo

               STA ZPB

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPB+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS EXTRSL1 ;dont print

               LDA XSM

               CLC

               ADC ZPA ;no carry

               ADC SRT1 ;Qy lo

               STA ZPC

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC EXTRSL1 ;dont print

               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PRTSSL1

               LDA STRIPY
               SEC

               SBC ZPB

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               CMP #8
               BEQ PSTSSL1

               EOR ZPA

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               AND #$20
               STA POS

               JSR SETDATPAA

               JMP MIDSTRENT



PRTSSL1        JSR SKIPLINES ;fix STRIPDAT
               LDY #0
               JSR DO1TRI
               JSR STRIPSUB
               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EX2TRSL1
PSTSSL1        LDA ZPA
               AND #$08
               BNE NOVTTP
               LDY #$20
               JSR DO1TRI
               AND #0
               STA POS

               JSR SETDATPAA

               JMP PRSDSQP1L



NOVTTP         LDY #$40
NOVTTPA        JSR DO1TRI

               LDA ZPB

               CMP ZPC

               LDA ZPB+1

               SBC ZPC+1

               BCS EX2TRSL1
               JSR SETDATPAA

               JMP FN2SSL1



EX2TRSL1       RTS



TRISL2         JSR SIDSQSUB
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE EX2TRSL1 ;dont print
               ;bcc ex2trsl1

               TXA

               LSR ; = LSR A ;/2

               CMP XSM ;xdim/2

               BCS EX2TRSL1 ;dont print
               STA ZPA

               TXA

               ;clc
               ADC ZPA

               ;clc
               ADC SRT1 ;Qy lo

               STA ZPB

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPB+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS EX2TRSL1 ;dont print
               LDA XSM

               ASL ; = ASL A ;*2

               ;clc
               ADC ZPA ;no carry

               ADC SRT1 ;Qy lo

               STA ZPC

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC EX2TRSL1 ;dont print
               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PRTSSL2

               LDA STRIPY
               SEC

               SBC ZPB

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               CMP #$10
               BEQ PTTSSL2

               BCC PSTSSL2

FNFTLVEP       ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               AND #$20
               STA POS

               JSR SETDATPAA

               JMP MIDSTRENT



PRTSSL2        JSR SKIPLINES ;fix STRIPDAT
               LDY #0
               JSR DO1TRI
               JSR STRIPSUB
               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXTRSL2

PSTSSL2        LDY #$20

               JSR DO1TRI
               JSR STRIPSUB
               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXTRSL2

PTTSSL2        LDY #$40

               JMP NOVTTPA



EXTRSL2        RTS


RTRIPLS        JSR SIDSQSUB

               LDA XSM

               ASL ; = ASL A ;*2

               ;clc
               ADC SRT0 ;Qx lo

               STA SRT0
               BCC LZ4151
               INC SRT0+1 ;Qx hi

LZ4151         LDA XSM

               CLC

               ADC SRT1 ;Qy lo

               BCC LZ4153
               INC SRT1+1 ;Qy hi

LZ4153         SEC

               SBC YSM

               STA SRT1
               BCS EXRTP1
               DEC SRT1+1 ;Qy hi

EXRTP1         RTS





RTRISL1        JSR RTRIPLS
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE EXRTP1 ;dont print
               ;bcc exrtp1

               TXA

               LSR ; = LSR A ;/2

               CMP YSM ;ydim/2

               BCS EXRTP1 ;dont print
               STA ZPA

               LDA YSM

               SEC

               SBC ZPA

               ;sec
               SBC #8
               ASL ; = ASL A ;*2

               ;clc
               ADC SRT1 ;Qy lo

               STA ZPB

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPB+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS EXRTP1 ;dont print
               LDA YSM

               ASL ; = ASL A ;*2

               SEC

               SBC ZPA

               SBC #8
               CLC

               ADC SRT1 ;Qy lo

               STA ZPC

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC EXRTP1 ;dont print
               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PRTSSL3

               LDA STRIPY
               SEC

               SBC ZPB

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               CMP #8
               BEQ PSTSSL3

               EOR ZPA

HLFWVTST       EOR #8
               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               AND #$20
               STA POS

               JSR SETDATPCC

               JMP MIDSTRENT



PRTSSL3        JSR SKIPLINES ;fix STRIPDAT
               LDY #0
               JSR DO1TRI
               JSR STRIPSUB
               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EX2TRSL3
PSTSSL3        LDA ZPA
               AND #$08
               BEQ NOVTTP3

               LDY #$20 ;data offset

               JSR DO1TRI
               AND #0
               STA POS

               JSR SETDATPCC

               JMP PRSDSQP1L



NOVTTP3        LDY #$40 ;data offset
NOVTTP3A       JSR DO1TRI
               LDA ZPB

               CMP ZPC

               LDA ZPB+1

               SBC ZPC+1

               BCS EX2TRSL3
               JSR SETDATPCC

               JMP FN2SSL1



EX2TRSL3       RTS





RTRISL2        JSR RTRIPLS
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE EX2TRSL3 ;dont print
               ;bcc ex2trsl3

               TXA

               LSR ; = LSR A ;/2

               CMP YSM ;ydim/2

               BCS EX2TRSL3 ;dont print
               STA ZPA

               LDA YSM

               SEC

               SBC ZPA

               STA ZPB ;temp

               ASL ; = ASL A ;*2

               ;clc
               ADC ZPB ;=*3
               SEC

               SBC #$18
               CLC

               ADC SRT1 ;Qy lo

               STA ZPB

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPB+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS EX2TRSL3 ;dont print
               LDA YSM

               ASL ; = ASL A ;*2

               ;clc
               ADC YSM

               SEC

               SBC ZPA

               SBC #8
               CLC

               ADC SRT1 ;Qy lo

               STA ZPC

               LDA SRT1+1 ;Qy hi

               ADC #0
               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC EX2TRSL3 ;dont print
               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PRTSSL4

               LDA STRIPY
               SEC

               SBC ZPB

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               CMP #$10
               BEQ PTTSSL4

               BCC PSTSSL4

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               AND #$20
               STA POS

               JSR SETDATPCC

               JMP MIDSTRENT



PRTSSL4        JSR SKIPLINES ;fix STRIPDAT
               LDY #0 ;data offset

               JSR DO1TRI
               JSR STRIPSUB
               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXTRSL4

PSTSSL4        LDY #$20 ;data offset
               JSR DO1TRI
               JSR STRIPSUB
               CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXTRSL4

PTTSSL4        LDA #$40

               STA POS

               LDY #$40
               JMP NOVTTP3A


EXTRSL4        RTS


SMLSSFC        JSR SIDSQSUB

               JSR HORCHCKSL

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               SEC

               SBC YSM

               BCS SMLST1
               INY

               AND #0
SMLST1         CLC

               ADC SRT1 ;Qy lo

               BCC LZ4154
               INX

LZ4154         STA ZPB

               STX ZPB+1

               STY DATA
               CMP ZPD

               TXA

               SBC ZPD+1

               BCS EXTRSL4 ;dont print

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               CMP XSM

               BCC SMLSB1
               INY

               LDA XSM

               SEC

               SBC #8
               CLC

SMLSB1         ADC SRT1 ;Qy lo

               BCC LZ4155
               INX

LZ4155         STA ZPC

               STX ZPC+1

               STY DATA+1
               CMP STRIPY
               TXA

               SBC STRIPY+1
               BCC EXTRSL4 ;dont print

               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PTTSMTSP
               ;top triangle is off top of strip

               LDA STRIPY
               STA ZPB

               LDA STRIPY+1
               STA ZPB+1

               JMP SMTPMDIN
PTTSMTSP       JSR SKIPLINES ;fix STRIPDAT

               LDA DATA ;top type
               BNE SMTPMDIN
               LDY #$20
               JSR DO1TRI
               LDA ZPB

               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS EXSMSQ1



SSQTPLPL       LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC

               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ4156
               INC STRIPDAT+1
               CLC

LZ4156         LDA ZPB ;Ttopy - add 8 to vertical coord
               ADC #8
               STA ZPB

               BCC LZ4157
               INC ZPB+1

LZ4157         CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCS EXSMSQ1 ;branch if bot of strip reached

SMTPMDIN       LDY #0
               STY POS

               STY POS+1

               LDA ZPB

               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS PLSSQTTR ;branch if reached final tri (bot of plane strip)
               JSR DO2SECS

               JMP SSQTPLPL


PLSSQTTR       LDA DATA+1
               BEQ SSQPB1T

               JMP DO2SECS

SSQPB1T        JMP DO1TRI



EXSMSQ1        RTS




SMLSP2SB       JSR SIDSQSUB
               LDA SRT1 ;Qy lo

               CLC

               ADC XSM

               BCC LZ4158
               INC SRT1+1 ;Qy hi

LZ4158         SEC

               SBC YSM

               BCS LZ4159
               DEC SRT1+1 ;Qy hi

LZ4159         STA SRT1 ;Qy lo

               RTS



RSMLSSF        JSR SMLSP2SB

               JSR HORCHCKSL

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               CMP YSM

               BCC SMSP2T0

               INY

               LDA SRT1 ;Qy lo

               JMP CHTADDF

SMSP2T0        LDA YSM
               SEC

               SBC ZPA

               SBC #8
               CLC

               ADC SRT1 ;Qy lo

               BCC CHTADDF

               INX

CHTADDF        STA ZPB
               STX ZPB+1

               STY DATA
               CMP ZPD

               TXA

               SBC ZPD+1
               BCS EXSMSQ1 ;dont print

               LDY #0
               LDX SRT1+1 ;Qy hi

               LDA ZPA

               CMP XSM

               BCS SMSP2B0

               INY

               LDA YSM

               SEC

               JMP CHTADDG

SMSP2B0        LDA LINEC
               ;sec
               SBC ZPA

CHTADDG        ;sec

               SBC #8
               CLC

               JMP SMLSB1




NOPLPRN3       RTS

FCNGTR1        JSR RHSSLSUB

               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE NOPLPRN3 ;dont print
               ;bcc noplprn3

               TXA

               LSR ; = LSR A ;/2

               CMP YSM ;ydim/2

               BCS NOPLPRN3 ;dont print
               STA ZPA

               AND #$08
               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               EOR #$20
               STA POS

               LDA SRT1 ;Qy lo

               STA ZPB

               CMP ZPD

               LDA SRT1+1 ;Qy hi

               STA ZPB+1

               SBC ZPD+1

               BCS NOPLPRN3 ;dont print
               LDA YSM

               SEC

               SBC ZPA

               SBC #8
               CLC

FNVPFCNS       ADC SRT1 ;Qy lo

               STA ZPC

               LDA SRT1+1
               ADC #0
               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC NOPLPRN3
               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCC GSTTSSQR
               ;start at top of strip
               JSR SKIPLINES ;fix STRIPDAT

               JMP XTRBSTARD ;replaces: jmp MIDSTRENT
               ;start part-way down strip
GSTTSSQR       LDA STRIPY
               SEC

               SBC ZPB

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               EOR POS

               JMP TRSSQENT




NOPLPRN5       RTS

FCNGTR2        JSR SIDSQSUB

               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE NOPLPRN5 ;dont print
               ;bcc noplprn5

               TXA

               LSR ; = LSR A ;/2

               CMP XSM ;xdim/2

               BCS NOPLPRN5 ;dont print
               STA ZPA

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               AND #$20
               STA POS

               LDA SRT1 ;Qy lo

               CLC

               ADC XSM

               BCC LZ4162
               INC SRT1+1 ;Qy hi

LZ4162         STA SRT1 ;Qy lo

               STA ZPB

               CMP ZPD

               LDA SRT1+1
               STA ZPB+1

               SBC ZPD+1

               BCS NOPLPRN5 ;dont print
               LDA ZPA

               ;clc
               JMP FNVPFCNS




RFTRPLQS       JSR SIDSQSUB
               LDA XSM

               ASL ; = ASL A ;*2

               ;clc
               ADC SRT0 ;Qx lo

               STA SRT0
               BCC LZ4165
               INC SRT0+1 ;Qx hi

LZ4165         LDA XSM

               CLC

               ADC SRT1 ;Qy lo

               BCC LZ4166
               INC SRT1+1 ;Qy hi

LZ4166         STA SRT1
NOPLPRN7       RTS



FCNGTR3        JSR RFTRPLQS

               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE NOPLPRN7 ;dont print
               ;bcc noplprn7

               TXA

               LSR ; = LSR A ;/2

               CMP YSM ;ydim/2

               BCS NOPLPRN7 ;dont print
               STA ZPA

               ;clc
               ADC SRT1 ;Qy lo

               STA ZPB

               AND #0
               ADC SRT1+1 ;Qy hi

               STA ZPB+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS NOPLPRN7 ;dont print
               LDA YSM

               ASL ; = ASL A ;*2

               SEC

               SBC ZPA

               SBC #8
               CLC

               ADC SRT1 ;Qy lo

               STA ZPC

               AND #0
               ADC SRT1+1 ;Qy hi

               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC NOPLPRN7 ;dont print
               ;find how far down strip to start printing
               LDA ZPB

               ;sec
               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PTPTRFS1
               LDA STRIPY
               SEC

               SBC ZPB

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               JMP HLFWVTST


PTPTRFS1       JSR SKIPLINES ;fix STRIPDAT

               LDY #0
               JSR DO1TRI
               JSR SETDATPCC

               AND #0
               JMP ANOTENT





FCNG4SB        JSR SIDSQSUB

               LDA SRT1 ;Qy lo

               CLC

               ADC XSM

               BCC LZ4167
               INC SRT1+1 ;Qy hi

LZ4167         STA SRT1
NOPLPRN9       RTS



FCNGTR4        JSR FCNG4SB
               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE NOPLPRN9 ;dont print
               ;bcc noplprn9

               TXA

               LSR ; = LSR A ;/2

               CMP XSM ;xdim/2

               BCS NOPLPRN9 ;dont print
               STA ZPA

               ;clc
               ADC XSM

               ADC SRT1 ;Qy lo

               STA ZPC

               AND #0
               ADC SRT1+1 ;Qy hi

               STA ZPC+1

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC NOPLPRN9 ;dont print
               LDA XSM

               ;sec
               SBC ZPA

               SBC #8
               CLC

               ADC SRT1 ;Qy lo

               STA ZPB

               AND #0
               ADC SRT1+1 ;Qy hi

               STA ZPB+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS NOPLPRN9 ;dont print
               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PTPTRFS2
               LDA STRIPY
               SEC

               SBC ZPB

               EOR #8
               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               JMP FNFTLVEP


PTPTRFS2       JSR SKIPLINES ;fix STRIPDAT

               LDY #0
               JSR DO1TRI
               JSR SETDATPAA

               AND #0
               JMP ANOTENT





NOPLPRNB       RTS

PTPBOTL        JSR SIDSQSUB

               LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               STA POS

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE NOPLPRNB ;dont print
               ;bcc noplprnb

               LDA POS

               BEQ LZ4170
               CMP #$20
               BCS NOPLPRNB ;dont print
               LDA #$60
               STA POS



LZ4170         LDA SRT1 ;Qy lo

               STA ZPB

               CLC

               ADC #$18
               STA ZPC

               LDA SRT1+1 ;Qy hi

               STA ZPB+1

               ADC #0
               STA ZPC+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS NOPLPRNB ;dont print
               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC NOPLPRNB ;dont print
               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PTPTRFS3
               LDA STRIPY
               SEC

               SBC ZPB

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               CLC

               ADC POS

               TAY

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               JMP PRBTTLLP


PTPTRFS3       JSR SKIPLINES ;fix STRIPDAT

               LDY POS



PRBTTLLP       LDA ZPB

               CMP ZPC ;Tboty
               LDA ZPB+1

               SBC ZPC+1

               BCS ENDTBTP ;branch if reached final tri (bot of plane strip)

               JSR DO1TRI
               LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC

               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ4168
               INC STRIPDAT+1
               CLC

LZ4168         LDA ZPB ;Ttopy - add 8 to vertical coord
               ADC #8
               STA ZPB

               BCC LZ4169
               INC ZPB+1

LZ4169         CMP ZPD ;Sy+h

               LDA ZPB+1

               SBC ZPD+1

               BCC PRBTTLLP ;branch if bot of strip not reached


ENDTBTP        RTS


PMBYTEOTL      JSR SIDSQSUB

               LDY #$D

               LDA (ZPG),Y ;zdim

               SEC

               SBC #$10
               BEQ ENDTBTP

               STA XSM

               LDA SRT1 ;Qy lo

               CLC

               ADC #$10
               BCC LZ4171
               INC SRT1+1 ;Qy hi

LZ4171         STA SRT1 ;Qy lo

POLENT         LDA STRIPX
               SEC

               SBC SRT0 ;Qx lo

               STA POS

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE ENDTBTP ;dont print

               ;bcc endtbtp
               LDA POS

               BEQ LZ4172
               CMP #$20
               BCS ENDTBTP ;dont print

               LDA #$60
               STA POS



LZ4172         LDA SRT1 ;Qy lo

               STA ZPB

               CLC

               ADC XSM ;zdim

               STA ZPC

               LDA SRT1+1 ;Qy hi

               STA ZPB+1

               ADC #0
               STA ZPC+1

               LDA ZPB

               CMP ZPD

               LDA ZPB+1

               SBC ZPD+1

               BCS ENDTBTP ;dont print

               LDA ZPC

               CMP STRIPY
               LDA ZPC+1

               SBC STRIPY+1
               BCC ENDTBTP ;dont print

               ;find how far down strip to start printing
               LDA ZPB

               SEC

               SBC STRIPY
               TAX

               LDA ZPB+1

               SBC STRIPY+1
               BCS PTPMBYTETL
               LDA STRIPY
               SEC

               SBC ZPB

               AND #$08
               ASL ; = ASL A

               ASL ; = ASL A ;*4

               ADC POS

               STA POS

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               JMP PMBTLMIN


PTPMBYTETL     JSR SKIPLINES ;fix STRIPDAT

               LDY POS
               JSR DO1TRI
               STY POS

PMBTPLP        LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC
               ADC #$C0 ;=$18*8
               BCC LZ4173
               INC STRIPDAT+1
               CLC

LZ4173         STA STRIPDAT
               LDA ZPB ;Ttopy - add 8 to vertical coord
               ADC #8
               BCC LZ4174
               INC ZPB+1

LZ4174         STA ZPB

               CMP ZPD ;Sy+h
               LDA ZPB+1
               SBC ZPD+1
               BCS FNPTBTL ;branch if bot of strip reached

PMBTLMIN       LDA ZPB
               CMP ZPC ;Tboty
               LDA ZPB+1
               SBC ZPC+1
               BCS PLSTMBBT ;branch if reached final tri (bot of plane strip)

               LDA POS
               BEQ POLBFBTL
               CMP #$60
               BEQ POLBFBTL
               TAY

               JSR DO1TRI
               TYA
               SEC
               SBC #$40
               STA POS
               JMP PMBTPLP

POLBFBTL       CLC
               ADC #$40
               STA POS+1
               JSR DO2SECS

               JMP PMBTPLP


PLSTMBBT       LDA POS
               CLC
               ADC #$40
               TAY
               JMP DO1TRI

FNPTBTL        RTS


PRVPOLE        JSR SIDSQSUB

               LDY #$D
               LDA (ZPG),Y ;zdim
               STA XSM

               JMP POLENT


PRSDTDR        JSR SIDSQSUB

ENRRHLP        LDA STRIPX

               CMP SRT0 ;Qx lo

               BNE FNPTBTL ;dont print

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi

               BNE FNPTBTL ;dont print

               ;bcc fnptbtl
               LDA SRT1 ;Qy lo

               CMP ZPD

               LDA SRT1+1 ;Qy hi

               SBC ZPD+1

               BCS FNPTBTL ;dont print

               ;find how far down strip to start printing
               LDA SRT1 ;Qy lo

               SEC

               SBC STRIPY
               TAX

               LDA SRT1+1 ;Qy hi

               SBC STRIPY+1
               BCS PTPSDTD

               LDA STRIPY
               SEC

               SBC SRT1 ;Qy lo

               TAX

               LDA STRIPY+1
               SBC SRT1+1 ;Qy hi

               BNE FNPTBTL ;dont print

               CPX #$18
               BCS FNPTBTL ;dont print

               TXA

               ASL ; = ASL A

               ASL ; = ASL A ;[*8]*4=*$20
               TAY

               LDX STRIPY
               STX ZPB

               LDX STRIPY+1
               STX ZPB+1

               JMP PSDTPDL


PTPSDTD        JSR SKIPLINES ;fix STRIPDAT
               LDY #0


PSDTPDL        JSR DO1TRI

               CPY #$60
               BCS FNPTBTL ;finished printing
               LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC

               ADC #$C0 ;=$18*8
               BCC LZ40175

               INC STRIPDAT+1
               CLC

LZ40175        STA STRIPDAT

               LDA ZPB ;Ttopy - add 8 to vertical coord
               ADC #8
               BCC LZ40176

               INC ZPB+1

LZ40176        STA ZPB
               CMP ZPD ;Sy+h
               LDA ZPB+1
               SBC ZPD+1
               BCC PSDTPDL ;branch if bot of strip NOT reached

LVDNPRNT       RTS


RPSDTDR        JSR SIDSQSUB

               LDA SRT0 ;Qx lo
               CLC
               ADC #$10
               BCC LZ40177
               INC SRT0+1 ;Qx hi

LZ40177        STA SRT0 ;Qx lo
               JMP ENRRHLP


PTPTRDR        JSR SIDSQSUB

               LDA SRT1 ;Qy lo
               SEC
               SBC #8
               BCS LZ40178
               DEC SRT1+1 ;Qy hi

LZ40178        STA SRT1 ;Qy lo
               LDA STRIPX
               SEC
               SBC SRT0 ;Qx lo
               TAX

               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi
               BNE LVDNPRNT ;dont print
               ;bcc lvdnprnt

               TXA
               BEQ LZ40179

               CMP #$10
               BNE LVDNPRNT ;dont print
               LDA #$40
LZ40179        TAY

               LDA SRT1 ;Qy lo
               CMP ZPD
               LDA SRT1+1 ;Qy hi
               SBC ZPD+1
               BCS LVDNPRNT ;dont print

               ;find how far down strip to start printing
               LDA SRT1 ;Qy lo
               SEC
               SBC STRIPY
               TAX
               LDA SRT1+1 ;Qy hi
               SBC STRIPY+1
               BCS PTPTTRD

               LDA STRIPY
               SEC
               SBC SRT1 ;Qy lo
               TAX

               LDA STRIPY+1
               SBC SRT1+1 ;Qy hi
               BNE LVDNPRNT ;dont print

               CPX #$08
               BNE LVDNPRNT ;dont print
               TYA
               CLC
               ADC #$20
               TAY

               LDX STRIPY
               STX ZPB
               LDX STRIPY+1
               STX ZPB+1

               JMP PBTTTRD


PTPTTRD        JSR SKIPLINES ;fix STRIPDAT;YR not affected
               JSR DO1TRI
               LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC
               ADC #$C0 ;=$18*8
               BCC LZ40180
               INC STRIPDAT+1
LZ40180        STA STRIPDAT

PBTTTRD        LDA ZPD
               SEC
               SBC SRT1 ;Qy lo
               CMP #8
               BEQ VLVDNPRT ;finish printing
               JMP DO1TRI
VLVDNPRT       RTS


;***********

STRIPSUB       LDA STRIPDAT ;move ptr in work area down 8 pixel lines
               CLC
               ADC #$C0 ;=$18*8
               STA STRIPDAT
               BCC LZ495
               INC STRIPDAT+1
               CLC
LZ495          LDA ZPB ;Ttopy - add 8 to vertical coord
               ADC #8
               STA ZPB
               BCC LZ496
               INC ZPB+1
LZ496          RTS


HSTRIPSUB      LDA STRIPDAT ;move ptr in work area along 2 bytes
               CLC
               ADC #2
               STA STRIPDAT
               BCC LZ4109
               INC STRIPDAT+1
               CLC
LZ4109         LDA ZPB ;Tleftx - add $10 to horizontal coord
               ADC #$10
               STA ZPB
               BCC LZ4110
               INC ZPB+1
LZ4110         RTS


STRDALNG       TXA ;fix STRIPDAT
               LSR ; = LSR A
               LSR ; = LSR A
               LSR ; = LSR A ;/8
               CLC
               ADC STRIPDAT
               STA STRIPDAT
               BCC LZ456
               INC STRIPDAT+1
LZ456          RTS


HORCHCKSL      LDA STRIPX
               SEC
               SBC SRT0 ;Qx lo
               TAX
               LDA STRIPX+1
               SBC SRT0+1 ;Qx hi
               BNE EXTHSPLN
               ;bcc exthspln

               TXA

               LSR ; = LSR A ;/2

               CMP LINEC ;(xdim+ydim)/2
               BCC HORCHOK

EXTHSPLN       PLA

               PLA

HORCHOK        STA ZPA
               RTS


;*******************************************************************
;*  Print a horizontal strip (8 pixels high)
;*
;*  SETUP: Background objects to be printed in SPECAL table
;*
;*         Strip origin x,y (2 bytes each) in STRIPX,STRIPY
;*         STRIPX+strip width in ZPD (2 bytes)
;*
;*         Ptr to begining of strip in work area: STRIPDAT
;*
;*******************************************************************
PRHSTRIP       LDA #(SPECAL+1)&255
               STA ZPG

               LDA #(SPECAL+1)/256
               STA ZPG+1

               LDA SPECAL
               BEQ EXPRHSTRIP
               STA SRA

PRHSTL         LDY #$A
               LDA (ZPG),Y ;shape #
               TAX

               LDA OBJPTOFF,X
               STA COUNTER

PRPLHL         LDX COUNTER

               LDY O1,X
               BEQ LPHOBJS

               LDA INITSTRDAT
               STA STRIPDAT
               LDA INITSTRDAT+1
               STA STRIPDAT+1

               LDA HPLANESJL,Y
               STA HPLANESJSR+1
               LDA HPLANESJH,Y
               STA HPLANESJSR+2

HPLANESJSR     JSR $0000 ; smc - print a strip of current plane

               INC COUNTER

               BNE PRPLHL ;jmp

LPHOBJS        LDA ZPG ;next SPECAL object
               CLC
               ADC #SPECSIZ
               STA ZPG
               BCC LZ425
               INC ZPG+1
LZ425          DEC SRA
               BNE PRHSTL
EXPRHSTRIP     RTS ;printed all planes of all objs in strip


HPLANESJL      BYTE <0, <HPL1, <HPL2, <HPL3, <HPL4, <HPL5, <HPL6, <HPL7, <HPL8, <HPL9, <HPLA
               BYTE <HPLB, <HPLC, <HPLD, <HPLE, <HPLF, <HPL10, <HPL11, <HPL12, <HPL13
               BYTE <HPL14, <HPL15, <HPL16, <HPL17, <HPL18, <HPL19, <HPL1A, <HPL1B

HPLANESJH      BYTE >0, >HPL1, >HPL2, >HPL3, >HPL4, >HPL5, >HPL6, >HPL7, >HPL8, >HPL9, >HPLA
               BYTE >HPLB, >HPLC, >HPLD, >HPLE, >HPLF, >HPL10, >HPL11, >HPL12, >HPL13
               BYTE >HPL14, >HPL15, >HPL16, >HPL17, >HPL18, >HPL19, >HPL1A, >HPL1B


HPL1           JSR SETDATP1
               JMP HTPPLANE
HPL2           JSR SETDATP2
               JMP HTPPLANE
HPL3           JSR SETDATP3
               JMP HTPPLANE
HPL4           JSR SETDATP4
               JMP HSSIDSQP1
HPL5           JSR SETDATP5
               JMP HSSIDSQP2
HPL6           JSR SETDATP6
               JMP HSLOPFC1
HPL7           JSR SETDATP7A
               JMP HSLOPFC2
HPL8           JSR SETDATP8
               JMP HFSLPFC1
HPL9           JSR SETDATP9A
               JMP HFSLPFC2
HPLA           JSR SETDATPAA
               JMP HTRISL1
HPLB           JSR SETDATPAA
               JMP HTRISL2
HPLC           JSR SETDATPC
               JMP HRTRISL1
HPLD           JSR SETDATPD
               JMP HRTRISL2
HPLE           JSR SETDATPE
               JMP HSMLSSFC
HPLF           JSR SETDATPF
               JMP HRSMLSSF
HPL10          JSR SETDATPCC
               JMP HFCNGTR1
HPL11          JSR SETDATPAA
               JMP HFCNGTR2
HPL12          JSR SETDATPCC
               JMP HFCNGTR3
HPL13          JSR SETDATPAA
               JMP HFCNGTR4
HPL14          JSR SETDATP14
               JMP HPTPBOTL
HPL15          JSR SETDATP15
               JMP HPTPBOTL
HPL16          JSR SETDATP16
               JMP HPMBYTEOTL
HPL17          JSR SETDATP17
               JMP PRHPOLE

HPL18          JSR SETDATP18
               JMP HPRSDTDR
HPL19          JSR SETDATP19
               JMP RHPSDTDR
HPL1A          JSR SETDATP1A
               JMP HPTPTRDR
HPL1B          JSR SETDATP1B
               JMP HPTPTRDR


;****************************************
;*             Include files.
;****************************************

;*             XB - Upload Bits for colour screen
;;             INCLUDE BubblerUploadBits.asm


;*             XC - Matthew's screen print routines
               INCLUDE BubblerScreenPrint.asm

;timer definitions

;;             ORG $1000 Moved from here
CHARTAB        HEX 0000FFFF000000000000000000FFFF000000FFFF000000808080000000FFFF000000FFFF000080C0C0C0800000FFFF00
               HEX 0000FFFF0000C0E0E0E0C00000FFFF000000FFFF0080E0F0F0F0E08000FFFF000000FFFF00C0F0F8F8F8F0C000FFFF00
               HEX 0000FFFF00E0F8FCFCFCF8E000FFFF000000FFFF00F0FCFEFEFEFCF000FFFF000000FFFF00F8FEFFFFFFFEF800FFFF00
               HEX 0000FFFF00FCFFFFFFFFFFFC00FFFF000000FFFF00FEFFFFFFFFFFFE00FFFF000000FFFF00FFFFFFFFFFFFFF00FFFF00
               HEX 000080E0F038181C0C1C1838F0E08000000080E0F038189C8C9C1838F0E08000000080E0F03898DCCCDC9838F0E08000
               HEX FF80808080808080FE000101000000000000FFFF000000000000000000FFFF000000FFFF000000010101000000FFFF00
               HEX 0000FFFF000001030303010000FFFF000000FFFF000003070707030000FFFF000000FFFF0001070F0F0F070100FFFF00
               HEX 0000FFFF00030F1F1F1F0F0300FFFF000000FFFF00071F3F3F3F1F0700FFFF000000FFFF000F3F7F7F7F3F0F00FFFF00
               HEX 0000FFFF001F7FFFFFFF7F1F00FFFF000000FFFF003FFFFFFFFFFF3F00FFFF000000FFFF00BFFFFFFFFFFF7F00FFFF00
               HEX 0000FFFF00FFFFFFFFFFFFFF00FFFF00000000070F1C18383038181C0F070000000000070F1C18393139181C0F070000
               HEX 000000070F1C193B333B191C0F07000000000000000000000000000000000000
;number for lives and bottles definitions
               HEX 00001F3F7FF1E3E600000080C0E0E0E0ECF8F17F3F1F0000E0E0E0C0800000000000070F1F1F07070000808080808080
               HEX 070707070707000080808080808000000000FFFFFF03033F000080C0E0E0E0C07FF800FFFFFF0000800000E0E0E00000
               HEX 0000FFFFFF007FFF0000E0C0800080C07F0101FFFFFF0000E0E0E0E0C080000000000103070F1F3F0000C0C0C0C0C0C0
               HEX 7BFFFFFF03030000C0E0E0E0C0C000000000FFFFFFF0FFFF0000E0E0E00080C0FF0101FFFFFF0000E0E0E0E0C0800000
               HEX 00001E1E3C3C797B00000000000080C07BF1F1FF7F3F0000E0E0E0E0C08000000000FFFFFF0003030000E0E0E000C0C0
               HEX 07070F0F1E1E0000808000000000000000003F7FFFF1F17B000080C0E0E0E0C07BF1F1FF7F3F0000C0E0E0E0C0800000
               HEX 00003F7FFFF1F1FB000080C0E0E0E0E07B3307070F0F0000C0C080800000000000000103070F0E0E0000F0F8FC1E3E6E
               HEX 0E0F0F0703010000CE8E1EFCF8F000000000000001010000000078F8F8F8787800000000000000007878787878780000
               HEX 00000F0F0F0000030000F8FCFE3E3EFC070F000F0F0F0000F88000FEFEFE000000000F0F0F00070F0000FEFCF800F8FC
               HEX 0700000F0F0F0000FE1E1EFEFCF80000000000000000010300001C3C7CFCFCFC070F0F0F00000000BCFEFEFE3C3C0000
               HEX 00000F0F0F0F0F0F0000FEFEFE00F8FC0F00000F0F0F0000FE1E1EFEFCF8000000000101030307070000E0E0C0C098BC
               HEX 070F0F0F07030000BE1E1EFEFCF8000000000F0F0F0000000000FEFEFE003C3C00000000010100007878F0F0E0E00000
               HEX 000003070F0F0F070000F8FCFE1E1EBC070F0F0F07030000BC1E1EFEFCF80000000003070F0F0F0F0000F8FCFE1E1EBE
               HEX 0703000000000000BC3C7878F0F000000000000000000007000000000000FF8100000000000000E01C30606060707C67
               HEX 0000000000000081380C0606060E3EE6603030301818180CFF00000000000000060C0C0C181818300C0C060606030000
               HEX 000000000081FF183030606060C00000000000000000000000000000000000FF0000000000000000071F3F3F3F1F2730
               HEX FFFFFFFFFFFFFFFFE0F8FCFCFCF8E40833333118181C0C0C00FFFFFFFFFFFF7FC8D4A8D0A850A0500606060303010000
               HEX 7E7F7E7D3EBD3E00A040804080000000
;definitions for score numbers (2x3)

               HEX 000000020F1F3F3D0000000080C0E0E07879797B7B7E7E7CF0F0F0F0F0F0F0F07C783D3F1F0F0200F0F0E0E0C0800000
               HEX 000000070F1F1F1F00000080808080801F07070707070707808080808080808007070707070707008080808080808000
               HEX 000000020F1F3F3D0000000080C0E0E07800000103070F1FF0F0F0E0E0C080001F3E3E7C7F7F7F0000000000F0F0F000
               HEX 000000020F1F3F3D0000000080C0E0E0780001070F0F0701F0E0E0C08080C0E000783D3F1F0F0200E0F0E0E0C0800000
               HEX 0000000001010303000000E0E0E0E0E007070F0F1F1F3D3DE0E0E0E0E0E0E0E07F7F7F0101010100F0F0F0E0E0E0E000
               HEX 0000007F7F7F7F78000000F0F0F0F00078787F7F7F010000000080E0E0F0F0F000783D3F1F0F0200F0F0E0E0C0800000
               HEX 00000007070F0F1F00000080808000001E3E3C3F7F7F7D7800000000C0E0E0F078783D3F1F0F0200F0F0E0E0C0800000
               HEX 0000007F7F7F7F01000000F0F0F0F0F00103030303070707F0E0E0E0E0C0C0C0070F0F0F1F1F1F00C080808000000000
               HEX 000000020F1F3F3D0000000080C0E0E07878383D1F1F3D38F0F0E0E0C0C0E0E078783D3F1F0F0200F0F0E0E0C0800000
               HEX 000000020F1F3F3D0000000080C0E0E07878783D3F1F0701F0F0F0F0F0F0E0E0030307070F0F0F00E0C0C08080000000
               HEX FF80808080808080FF00000000000000FF80808080808080FF00000000000000FF80808080808080FF00000000000000
               HEX FF80808080808080FF00000000000000FF80808080808080FF00000000000000FF80808080808080FF00000000000000
               HEX 0F0F0F0F0F0F0F0FFEFFFFFFFFFFFEF00383C1C1C18040800F101F8F8FCFCFC7F00DFAFDFAF5FAF58040800080000000
               HEX 67676737331B0300EAF4E8D4E8D0E000000000000001030300000020F8FCFEDE07070707070707078F9F9FBFBFEFEFCF
               HEX 0707030301000000CF8FDEFEFCF82000000000000001010100000078F8F8F8F80100000000000000F878787878787878
               HEX 00000000000000007878787878787800000000000001030300000020F8FCFEDE07000000000000018F0F0F1E3E7CF8F0
               HEX 0103030707070700F0E0E0C0FFFFFF00000000000001030300000020F8FCFEDE07000000000000008F0E1E7CF8F87C1E
               HEX 00070303010000000E8FDEFEFCF8200000000000000000000000000E1E1E3E3E00000000010103037E7EFEFEFEFEDEDE
               HEX 0707070000000000FFFFFF1E1E1E1E000000000707070707000000FFFFFFFF8007070707070000008080F8FEFE1F0F0F
               HEX 00070303010000000F8FDEFEFCF8200000000000000000010000007878F8F0F00103030307070707E0E0C0F0FCFEDE8F
               HEX 07070303010000008F8FDEFEFCF820000000000707070700000000FFFFFFFF1F00000000000000001F3E3E3E3E7C7C7C
               HEX 00000000010101007CF8F8F8F0F0F000000000000001030300000020F8FCFEDE07070303010103038F8F8EDEFCFCDE8E
               HEX 07070303010000008F8FDEFEFCF82000000000000001030300000020F8FCFEDE07070703030100008F8F8FDFFFFF7E1E
               HEX 00000000000000003E3C7C78F8F0F0000F08080808080808FF00000000000000

;*******************************************
;*             MATTHEW'S TABLES,VARIABLES ETC
;*******************************************

EIGHTTAB       BYTE  0*8, 1*8,  2*8,  3*8,  4*8,  5*8,  6*8,  7*8,  8*8, 9*8, 10*8, 11*8
               BYTE 12*8, 13*8, 14*8, 15*8, 16*8, 17*8, 18*8, 19*8, 20*8, 21*8, 22*8, 23*8

WW             EQU WA_WIDTH_BYTES ;WORK AREA WIDTH

WKEDL          BYTE  <[WA+(WW*0)],  <[WA+(WW*1)],  <[WA+(WW*2)],  <[WA+(WW*3)],  <[WA+(WW*4)],  <[WA+(WW*5)],  <[WA+(WW*6)],  <[WA+(WW*7)],  <[WA+(WW*8)],  <[WA+(WW*9)], <[WA+(WW*10)]
               BYTE <[WA+(WW*11)], <[WA+(WW*12)], <[WA+(WW*13)], <[WA+(WW*14)], <[WA+(WW*15)], <[WA+(WW*16)], <[WA+(WW*17)], <[WA+(WW*18)], <[WA+(WW*19)], <[WA+(WW*20)], <[WA+(WW*21)]
               BYTE <[WA+(WW*22)], <[WA+(WW*23)], <[WA+(WW*24)], <[WA+(WW*25)], <[WA+(WW*26)], <[WA+(WW*27)], <[WA+(WW*28)], <[WA+(WW*29)], <[WA+(WW*30)], <[WA+(WW*31)], <[WA+(WW*32)]
               BYTE <[WA+(WW*33)], <[WA+(WW*34)], <[WA+(WW*35)], <[WA+(WW*36)], <[WA+(WW*37)], <[WA+(WW*38)], <[WA+(WW*39)], <[WA+(WW*40)], <[WA+(WW*41)], <[WA+(WW*42)], <[WA+(WW*43)]
               BYTE <[WA+(WW*44)], <[WA+(WW*45)], <[WA+(WW*46)], <[WA+(WW*47)], <[WA+(WW*48)], <[WA+(WW*49)], <[WA+(WW*50)], <[WA+(WW*51)], <[WA+(WW*52)], <[WA+(WW*53)], <[WA+(WW*54)]
               BYTE <[WA+(WW*55)], <[WA+(WW*56)], <[WA+(WW*57)], <[WA+(WW*58)], <[WA+(WW*59)], <[WA+(WW*60)], <[WA+(WW*61)], <[WA+(WW*62)], <[WA+(WW*63)], <[WA+(WW*64)], <[WA+(WW*65)]
               BYTE <[WA+(WW*66)], <[WA+(WW*67)], <[WA+(WW*68)], <[WA+(WW*69)], <[WA+(WW*70)], <[WA+(WW*71)], <[WA+(WW*72)], <[WA+(WW*73)], <[WA+(WW*74)], <[WA+(WW*75)], <[WA+(WW*76)]
               BYTE <[WA+(WW*77)], <[WA+(WW*78)], <[WA+(WW*79)], <[WA+(WW*80)], <[WA+(WW*81)], <[WA+(WW*82)], <[WA+(WW*83)], <[WA+(WW*84)], <[WA+(WW*85)], <[WA+(WW*86)], <[WA+(WW*87)]
               BYTE <[WA+(WW*88)], <[WA+(WW*89)], <[WA+(WW*90)], <[WA+(WW*91)], <[WA+(WW*92)], <[WA+(WW*93)], <[WA+(WW*94)], <[WA+(WW*95)], <[WA+(WW*96)], <[WA+(WW*97)], <[WA+(WW*98)]
               BYTE <[WA+(WW*99)], <[WA+(WW*100)], <[WA+(WW*101)], <[WA+(WW*102)], <[WA+(WW*103)], <[WA+(WW*104)], <[WA+(WW*105)], <[WA+(WW*106)], <[WA+(WW*107)], <[WA+(WW*108)]
               BYTE <[WA+(WW*109)], <[WA+(WW*110)], <[WA+(WW*111)], <[WA+(WW*112)], <[WA+(WW*113)], <[WA+(WW*114)], <[WA+(WW*115)], <[WA+(WW*116)], <[WA+(WW*117)], <[WA+(WW*118)]
               BYTE <[WA+(WW*119)], <[WA+(WW*120)], <[WA+(WW*121)], <[WA+(WW*122)], <[WA+(WW*123)], <[WA+(WW*124)], <[WA+(WW*125)], <[WA+(WW*126)], <[WA+(WW*127)]

WKEDH          BYTE  >[WA+(WW*0)],  >[WA+(WW*1)],  >[WA+(WW*2)],  >[WA+(WW*3)],  >[WA+(WW*4)],  >[WA+(WW*5)],  >[WA+(WW*6)],  >[WA+(WW*7)],  >[WA+(WW*8)],  >[WA+(WW*9)], >[WA+(WW*10)]
               BYTE >[WA+(WW*11)], >[WA+(WW*12)], >[WA+(WW*13)], >[WA+(WW*14)], >[WA+(WW*15)], >[WA+(WW*16)], >[WA+(WW*17)], >[WA+(WW*18)], >[WA+(WW*19)], >[WA+(WW*20)], >[WA+(WW*21)]
               BYTE >[WA+(WW*22)], >[WA+(WW*23)], >[WA+(WW*24)], >[WA+(WW*25)], >[WA+(WW*26)], >[WA+(WW*27)], >[WA+(WW*28)], >[WA+(WW*29)], >[WA+(WW*30)], >[WA+(WW*31)], >[WA+(WW*32)]
               BYTE >[WA+(WW*33)], >[WA+(WW*34)], >[WA+(WW*35)], >[WA+(WW*36)], >[WA+(WW*37)], >[WA+(WW*38)], >[WA+(WW*39)], >[WA+(WW*40)], >[WA+(WW*41)], >[WA+(WW*42)], >[WA+(WW*43)]
               BYTE >[WA+(WW*44)], >[WA+(WW*45)], >[WA+(WW*46)], >[WA+(WW*47)], >[WA+(WW*48)], >[WA+(WW*49)], >[WA+(WW*50)], >[WA+(WW*51)], >[WA+(WW*52)], >[WA+(WW*53)], >[WA+(WW*54)]
               BYTE >[WA+(WW*55)], >[WA+(WW*56)], >[WA+(WW*57)], >[WA+(WW*58)], >[WA+(WW*59)], >[WA+(WW*60)], >[WA+(WW*61)], >[WA+(WW*62)], >[WA+(WW*63)], >[WA+(WW*64)], >[WA+(WW*65)]
               BYTE >[WA+(WW*66)], >[WA+(WW*67)], >[WA+(WW*68)], >[WA+(WW*69)], >[WA+(WW*70)], >[WA+(WW*71)], >[WA+(WW*72)], >[WA+(WW*73)], >[WA+(WW*74)], >[WA+(WW*75)], >[WA+(WW*76)]
               BYTE >[WA+(WW*77)], >[WA+(WW*78)], >[WA+(WW*79)], >[WA+(WW*80)], >[WA+(WW*81)], >[WA+(WW*82)], >[WA+(WW*83)], >[WA+(WW*84)], >[WA+(WW*85)], >[WA+(WW*86)], >[WA+(WW*87)]
               BYTE >[WA+(WW*88)], >[WA+(WW*89)], >[WA+(WW*90)], >[WA+(WW*91)], >[WA+(WW*92)], >[WA+(WW*93)], >[WA+(WW*94)], >[WA+(WW*95)], >[WA+(WW*96)], >[WA+(WW*97)], >[WA+(WW*98)]
               BYTE >[WA+(WW*99)], >[WA+(WW*100)], >[WA+(WW*101)], >[WA+(WW*102)], >[WA+(WW*103)], >[WA+(WW*104)], >[WA+(WW*105)], >[WA+(WW*106)], >[WA+(WW*107)], >[WA+(WW*108)]
               BYTE >[WA+(WW*109)], >[WA+(WW*110)], >[WA+(WW*111)], >[WA+(WW*112)], >[WA+(WW*113)], >[WA+(WW*114)], >[WA+(WW*115)], >[WA+(WW*116)], >[WA+(WW*117)], >[WA+(WW*118)]
               BYTE >[WA+(WW*119)], >[WA+(WW*120)], >[WA+(WW*121)], >[WA+(WW*122)], >[WA+(WW*123)], >[WA+(WW*124)], >[WA+(WW*125)], >[WA+(WW*126)], >[WA+(WW*127)]

SHADO          ;**SHADOW TABLES**
               ; Each entry is $10 bytes with the same meanings as bytes
               ; $00-$0F of IXTAB entries below

               ;EACH TABLE IS SHADOW OF SHAPE IN SAME POS IN THE LIST
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
               HEX 00000000000000000000000303010000
LASTSHD        HEX 00000000000000000000000303010000

               ;blob

;;;
;;; Non-landscape objects (so-called "sprites" in the original Spectrum code,
;;; but not hardware sprites in C64 sense).
;;;
;;; IXTAB entries:
;;;   00 Shape: (See DATTABLO/DATTABHI for graphics)
;;;             $00 = Free           See DATTABLO/DATTABHI for graphics
;;;             $01 = Blob                (MAJOR)  - 5 animation frames
;;;             $02 = Shadow              (SHADOW) - 6 animation frames
;;;             $03 = Shadow              (SHADOW) - 6 animation frames
;;;             $04 = Shadow              (SHADOW) - 6 animation frames
;;;             $05 = Blob's Bullet       (BULLIT) - 2 animation frames
;;;             $06 = Lift                (LIFTG)  - 2 animation frames
;;;             $07 = Explosion           (EXPLOF) - 7 animation frames
;;;             $08 = Bubble              (BUBBLE) - 7 animation frames
;;;             $09 = Saucer              (SAUCE)  - 1 animation frame
;;;             $0A = Triple Bubble       (BALLY)  - 9 animation frames
;;;             $0B = Crab                (CRABG)  - 2 animation frames
;;;             $0C = Materialisation Pad (MATTER) - 3 animation frames
;;;             $0D = Bullet Explosion    (BULEX)  - 2 animation frames
;;;             $0E = Gun Tower's Bullet  (SAUZE)  - 4 animation frames
;;;             $0F = Mystery Bubble      (MYST)   - 8 animatino frames
;;;             $10 = Gun/Spikes          (SPIKY)  - 2 animation frames
;;;             $11 = Pyramid             (PYRAM)  - 7 animation frames
;;;
;;;   01 bits0-6 - data offset (which frame of animation); bit7 set means shape off screen - dont print
;;;   02 x-co hi
;;;   03 x-co lo
;;;   04 y-co hi
;;;   05 y-co lo
;;;   06 z-co (lo)
;;;   07 transfer address lo  (Used for saving backgrounds behind shapes?)
;;;   08 transfer address hi
;;;   09 top 4 bits - dissolve info; bottom 4 bits - chars across of shape - max 4
;;;   0A top 2 bits - spike shape (shadow); bits 0-5: pixels up of shape - max $1A
;;;   0B x-dim
;;;   0C y-dim
;;;   0D z-dim
;;;   0E on-screen x-co ($00-$AF)
;;;   0F on-screen y-co
;;;   10 bit 7 set if resting on an object ;bit6 set: shadow is spike
;;;   11 object height above its shadow (0 or $FF = no shadow)
;;;   12 velocity/direction in X - bit 7: 1=left/0=right;   bits 0-6: speed
;;;   13 velocity/direction in Y - bit 7: 1=forward/0=back; bits 0-6: speed
;;;   14 velocity/direction in Z - bit 7: 1=down/0=up;      bits 0-6: speed
;;;   15 IX table # (constant)
;;;   16 x-co accum
;;;   17 y-co accum
;;;   18 z-co accum
;;;   19 SPR_NO of object this sprite is sitting on if it is resting on a sprite
;;;   1A bit0 set if gravity not to act on object (for bubbles etc)
;;;
;;; NB: Velocities are stored as direction (bit 7) + speed (bits 0-6), NOT as
;;;     two's complement.  So, eg, velocity of -1 is stored as %10000001, NOT
;;;     %11111111.
;;;
IXTAB          BYTE $01,$00,$02,$8C,$01,$6B,$00,$00,$00,$03,$0C
               BYTE $0C,$0C,$05 ;x/y/z dimensions
               DS.B 7,$00
               BYTE $00 ;IX table # (constant)
               BYTE $02,$02,$02 ;x/y/z co accum
               BYTE $00,$00

               ;aliens (max 2)
ALTABS         BYTE $00,$00,$02,$C6,$01,$06,$10,$00,$00,$12,$14
               BYTE $10,$10,$08
               DS.B 7,$00
               BYTE $01 ;IX table # (constant)
               DS.B 5,$00

               IF [IXTAB+IXSIZ] != ALTABS
               ECHO "IXTAB WRONG SIZE!!!"
               ERR
               ENDIF

ALIEN2         BYTE $00,$00,$02,$E6,$01,$06,$10,$00,$00,$12,$14
               BYTE $10,$10,$08
               DS.B 7,$00
               BYTE $02 ;IX table # (constant)
               DS.B 5,$00

               IF [ALTABS+IXSIZ] != ALIEN2
               ECHO "ALTABS WRONG SIZE!!!"
               ERR
               ENDIF

SFIRE          BYTE $00,$80,$02,$C0,$02,$00,$00,$00,$00,$02,$14
               BYTE $08,$08,$08
               DS.B 7,$00
               BYTE $03 ;IX table # (constant)
               DS.B 5,$00

               IF [ALIEN2+IXSIZ] != SFIRE
               ECHO "ALIEN2 WRONG SIZE!!!"
               ERR
               ENDIF

               BYTE $00,$80,$02,$C0,$02,$00,$00,$00,$00,$02,$14
               BYTE $08,$08,$08
               DS.B 7,$00
               BYTE $04 ;IX table # (constant)
               DS.B 5,$00

               ;blob's bullets
MYBUL          BYTE $00,$A0,$00,$00,$00,$00,$00,$00,$00,$01,$06
               BYTE $02,$02,$02
               DS.B 7,$00
               BYTE $05 ;IX table # (constant)
               DS.B 5,$00

               IF [SFIRE + (2*IXSIZ)] != MYBUL
               ECHO "SFIRE WRONG SIZE!!!"
               ERR
               ENDIF

               BYTE $00,$A0,$00,$00,$00,$00,$00,$00,$00,$01,$06
               BYTE $02,$02,$02
               DS.B 7,$00
               BYTE $06 ;IX table # (constant)
               DS.B 5,$00

               ;lift
BACKS          DS.B $B,$00
               BYTE $0A,$0A,$0A
               DS.B 7,$00
               BYTE $07 ;IX table # (constant)
               DS.B 5,$00

               IF [MYBUL + (2*IXSIZ)] != BACKS
               ECHO "MYBUL WRONG SIZE!!!"
               ERR
               ENDIF

               ;lift 2
BACKS2         DS.B $B,$00
               BYTE $0A,$0A,$0A
               DS.B 7,$00
               BYTE $08 ;IX table # (constant)
               DS.B 5,$00

               IF [BACKS + IXSIZ] != BACKS2
               ECHO "BACKS WRONG SIZE!!!"
               ERR
               ENDIF

               ;static background objects (max 4)
STATOB         DS.B $B,$00
               BYTE $10,$10,$10
               DS.B 7,$00
               BYTE $09 ;IX table # (constant)
               DS.B 5,$00

               IF [BACKS2 + IXSIZ] != STATOB
               ECHO "BACKS2 WRONG SIZE!!!"
               ERR
               ENDIF

               DS.B $B,$00
               BYTE $10,$10,$10
               DS.B 7,$00
               BYTE $0A ;IX table # (constant)
               DS.B 5,$00

               DS.B $B,$00
               BYTE $10,$10,$10
               DS.B 7,$00
               BYTE $0B ;IX table # (constant)
               DS.B 5,$00

LASTIX         DS.B $B,$00
               BYTE $10,$10,$10
               DS.B 7,$00
               BYTE $0C ;IX table # (constant) ;IXNUM should be 1 more than this
               DS.B 5,$00

               IF [STATOB + (3*IXSIZ)] != LASTIX
               ECHO "STATOB WRONG SIZE!!!"
               ERR
               ENDIF

EM             ;End of Matt's vars

               IF [LASTIX + IXSIZ] != EM
               ECHO "LASTIX WRONG SIZE!!!"
               ERR
               ENDIF

NUM_IX_ENTRIES EQU [EM - IXTAB] / IXSIZ

               IF [IXTAB + (NUM_IX_ENTRIES * IXSIZ)] != EM
               ECHO "IXTAB WRONG SIZE!!!"
               ERR
               ENDIF


;*             X9 - Background graphics defintions
;              ORG EM
               INCLUDE BubblerBackgrounds.asm ;Graphics for background objects



;*             XA - Moving Graphics & Map Data
               INCLUDE BubblerMapEtcData.asm ;Moving graphics & Map Data


               ;area for saving backgounds behind moving shapes
SAVNUM         DS.B $0548
Z



TAB_END        EQU ENMUS
;              LIST OFF ; = PDS directive?
;              ELSE
;              ENDIF
;              .END

;
;
;              NOTE:          Moving shapes (blob,aliens etc) have dimension ranges:
;                             1 to 4 chars across
;                             $04 to $1A pixels up
;
;                             Therefore, max mem occupied by one moving shape definition
;
;                             is $0068 bytes [$68=104 decimal]
;
;                             So DISBUF must be (2*$68)+2 bytes long (remember mask + dims)
;                             Max number of shapes is $0D so MAX amount of mem needed
;                             for saved backgrounds is $0548 bytes
;
;
;
;              *****************************************
;              *              NOTE:          < now means LOW BYTE           *
;              *                             > now means HIGH BYTE *
;              *****************************************
;
;
;
;>>>>>>>>NO SKIP means overlapping sections<<<<<<<<<
;
;

;****************************************
;*             I wish this bloody hard disk
;*             would SHUT UP !!
;****************************************


;
; File XD.BUB
; Starts Here
;
;

;****************************************
;*             Music delay notation
;****************************************
D1             EQU 8          ;crochet
B1             EQU D1/2
A1             EQU B1/2
C1             EQU B1+A1
E1             EQU D1+A1
F1             EQU D1+B1
G1             EQU D1+C1
H1             EQU D1*2
I1             EQU D1*4


MUSIC1
;PHRASE1
;BAR1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;BAR2
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
;BAR3
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR4
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
;PHRASE2
;BAR1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;BAR2
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
;BAR3
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR4
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
;PHRASE3
;BAR1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;BAR2
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
;BAR3
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR4
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
;PHRASE4
;BAR1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;BAR2
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
               BYTE 3,35,D1,6,71,D1,3,35,D1,6,71,D1
;BAR3
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR4
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
               BYTE 3,134,D1,7,12,D1,3,134,D1,7,12,D1
;BAR5
               BYTE 3,134,D1,7,233,I1,0,H1
;DELAY
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,255

MUSIC2
               BYTE 18,209,D1,21,31,D1,42,62,D1,50,60,D1,42,62,D1
               BYTE 63,75,D1,42,62,D1,50,60,D1
               BYTE 23,181,D1,25,30,D1,37,162,D1,50,60,D1
               BYTE 37,162,D1,63,75,D1,37,162,D1,50,60,D1
               BYTE 17,195,D1,18,209,D1,37,162,D1,44,193,D1
               BYTE 37,162,D1,56,99,D1,37,162,D1,44,193,D1
               BYTE 25,30,D1,28,49,D1,33,135,D1,42,62,D1
               BYTE 33,135,D1,56,99,D1,33,135,D1,42,62,D1

               BYTE 18,209,D1,21,31,D1,42,62,D1,50,60,D1,42,62,D1
               BYTE 63,75,D1,42,62,D1,50,60,D1
               BYTE 23,181,D1,25,30,D1,37,162,D1,50,60,D1
               BYTE 37,162,D1,63,75,D1,37,162,D1,50,60,D1
               BYTE 17,195,D1,18,209,D1,37,162,D1,44,193,D1
               BYTE 37,162,D1,56,99,D1,37,162,D1,44,193,D1
               BYTE 25,30,D1,28,49,D1,67,15,D1,63,75,D1
               BYTE 56,99,D1,50,60,D1,63,75,D1,42,62,D1

               BYTE 18,209,D1,21,31,D1,42,62,D1,50,60,D1,42,62,D1
               BYTE 63,75,D1,42,62,D1,50,60,D1
               BYTE 23,181,D1,25,30,D1,37,162,D1,50,60,D1
               BYTE 37,162,D1,63,75,D1,37,162,D1,50,60,D1
               BYTE 17,195,D1,18,209,D1,37,162,D1,44,193,D1
               BYTE 37,162,D1,56,99,D1,37,162,D1,44,193,D1
               BYTE 25,30,D1,28,49,D1,33,135,D1,42,62,D1
               BYTE 33,135,D1,56,99,D1,33,135,D1,42,62,D1

               BYTE 18,209,D1,21,31,D1,42,62,D1,50,60,D1,42,62,D1
               BYTE 63,75,D1,42,62,D1,50,60,D1
               BYTE 23,181,D1,25,30,D1,37,162,D1,50,60,D1
               BYTE 37,162,D1,63,75,D1,37,162,D1,50,60,D1
               BYTE 17,195,D1,18,209,D1,37,162,D1,44,193,D1
               BYTE 37,162,D1,56,99,D1,37,162,D1,44,193,D1
               BYTE 25,30,D1,28,49,D1,67,15,D1,63,75,D1
               BYTE 56,99,D1,50,60,D1,63,75,D1,42,62,D1

               BYTE 37,162,D1,42,62,I1,0,H1
;DELAY
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,I1,0,I1,0,I1,0,I1

MUSIC3
;PHRASE1
;BAR1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
;BAR2
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR3
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
;BAR4
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;PHRASE2
;BAR1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
;BAR2
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR3
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
;BAR4
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;PHRASE3
;BAR1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
;BAR2
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR3
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
;BAR4
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;PHRASE4
;BAR1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
               BYTE 3,244,D1,7,233,D1,3,244,D1,7,233,D1
;BAR2
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
               BYTE 4,180,D1,9,104,D1,4,180,D1,9,104,D1
;BAR3
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
               BYTE 7,12,D1,14,24,D1,7,12,D1,14,24,D1
;BAR4
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
               BYTE 5,71,D1,10,143,D1,5,71,D1,10,143,D1
;BAR5
               BYTE 5,71,D1,10,143,I1,0,H1
;DELAY
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,I1,0,I1,0,I1,0,I1
               BYTE 0,I1,0,I1,0,I1,0,I1

;****************************************
;*             Tune delay notation
;****************************************
AD1            EQU 15         ;crochet
AB1            EQU AD1/2
AA1            EQU AB1/2
AC1            EQU AB1+AA1
AE1            EQU AD1+AA1
AF1            EQU AD1+AB1
AG1            EQU AD1+AC1
AH1            EQU AD1*2
AI1            EQU AD1*4

TUNE1          BYTE 0,AD1,25,30,AD1,25,30,AD1,28,49,AD1,33,135,AH1
               BYTE 42,62,AD1,33,135,AD1     ;bar 1
               BYTE 37,162,AH1,44,193,AD1,37,162,AD1,42,62,AD1
               BYTE 42,62,AD1,50,60,AD1,56,96,AD1                          ;bar 2
               BYTE 37,162,AD1,37,162,AD1,42,62,AD1,44,193,AD1
               BYTE 33,135,AH1,42,62,AD1,33,135,AD1                        ;bar 3
               BYTE 37,162,AH1,44,193,AD1,37,162,AD1,25,30,AD1
               BYTE 25,30,AD1,28,49,AD1,31,165,AD1                                        ;bar 4
               BYTE 33,135,AD1,31,165,AD1,33,135,AI1
               BYTE 0,AH1,0,255

TUNE2          BYTE 0,AI1,16,195,AI1,15,210,AI1,16,195,AI1
               BYTE 15,210,AI1,16,195,AI1,15,210,AI1,12,143,AI1
               BYTE 8,97,AI1,0,AI1,0,AI1

TUNE3          BYTE 0,AI1,21,31,AI1,18,209,AI1,21,31,AI1
               BYTE 18,209,AI1,21,31,AI1,18,209,AI1,18,209,AI1
               BYTE 16,195,AI1,0,AI1,0,AI1

BD1            EQU 15         ;crochet
BB1            EQU BD1/2
BA1            EQU BB1/2
BC1            EQU BB1+BA1
BE1            EQU BD1+BA1
BF1            EQU BD1+BB1
BG1            EQU BD1+BC1
BH1            EQU BD1*2
BI1            EQU BD1*4

SONG1          BYTE 25,30,BD1,18,209,BD1,15,210,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,16,195,BD1,14,24,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,16,195,BD1
               BYTE 25,30,BD1,18,209,BD1,15,210,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,16,195,BD1,14,24,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,21,31,BD1,25,30,BD1,18,209,BD1,25,30,BH1
               BYTE 0,255

SONG2          BYTE 6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1
               BYTE 6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1
               BYTE 6,71,BD1,3,35,BD1
               BYTE 6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1
               BYTE 6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1
               BYTE 6,71,BD1,3,35,BD1,6,71,BD1,3,35,BD1,6,71,BH1

SONG3          BYTE 25,30,BD1,18,209,BD1,15,210,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,16,195,BD1,14,24,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,16,195,BD1
               BYTE 25,30,BD1,18,209,BD1,15,210,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,16,195,BD1,14,24,BD1,25,30,BD1,21,31,BD1,16,195,BD1
               BYTE 25,30,BD1,21,31,BD1,25,30,BD1,18,209,BD1,25,30,BH1

XD1            EQU 20         ;crochet
XB1            EQU XD1/2
XA1            EQU XB1/2
XC1            EQU XB1+XA1
XE1            EQU XD1+XA1
XF1            EQU XD1+XB1
XG1            EQU XD1+XC1
XH1            EQU XD1*2
XI1            EQU XD1*4

MELODY1        BYTE 50,60,XH1,53,57,XD1,39,223,XD1,53,57,XD1,59,190,XD1
               BYTE 63,75,XD1,59,190,XD1
               BYTE 53,57,XD1,39,223,XD1,47,107,XD1,35,134,XD1
               BYTE 47,107,XD1,53,57,XD1,56,99,XD1,53,57,XD1
               BYTE 47,107,XD1,50,60,XD1,53,57,XD1,39,223,XD1
               BYTE 53,57,XD1,63,75,XD1,79,191,XD1,63,75,XD1
               BYTE 53,57,XD1,79,191,XD1,71,12,XD1,53,57,XD1
               BYTE 71,12,XD1,79,191,XD1,89,131,XD1,79,191,XD1
               BYTE 71,12,XD1,89,131,XD1,71,12,XD1,56,99,XD1
               BYTE 71,12,XD1,79,191,XD1,84,125,XD1,79,191,XD1
               BYTE 71,12,XD1,56,99,XD1,53,57,XH1
               BYTE 42,62,XH1,47,107,XH1
               BYTE 53,57,XD1,47,107,XD1,42,62,XI1,0,XH1
               BYTE 0,255

MELODY2        BYTE 0,XH1,13,78,XD1,15,210,XD1,19,239,XD1
               BYTE 13,78,XD1,15,210,XD1,19,239,XD1
               BYTE 13,78,XD1,15,210,XD1,11,218,XD1,14,24,XD1
               BYTE 17,195,XD1,11,218,XD1,14,24,XD1,17,195,XD1
               BYTE 11,218,XD1,14,24,XD1,13,78,XD1,15,210,XD1
               BYTE 19,239,XD1,13,78,XD1,15,210,XD1,19,239,XD1
               BYTE 13,78,XD1,15,210,XD1,13,78,XD1,17,195,XD1
               BYTE 22,96,XD1,13,78,XD1,17,195,XD1,22,96,XD1
               BYTE 13,78,XD1,17,195,XD1,14,24,XD1,17,195,XD1
               BYTE 21,31,XD1,14,24,XD1,17,195,XD1,21,31,XD1
               BYTE 14,24,XD1,17,195,XD1,13,78,XD1,15,210,XD1
               BYTE 21,31,XD1,13,78,XD1,11,218,XD1,15,210,XD1
               BYTE 19,239,XD1,11,218,XD1,13,78,XD1,15,210,XD1
               BYTE 21,31,XD1,13,78,XD1,15,210,XD1,21,31,XD1

MELODY3        BYTE 0,XH1,6,167,XH1,6,167,XD1,6,167,XH1,6,167,XD1
               BYTE 6,167,XH1,5,237,XH1,5,237,XD1,5,237,XH1,5,237,XD1
               BYTE 5,237,XH1,6,167,XH1,6,167,XD1,6,167,XH1,6,167,XD1
               BYTE 6,167,XH1,4,112,XH1,4,112,XD1,4,112,XH1,4,112,XD1
               BYTE 4,112,XH1,7,12,XH1,7,12,XD1,7,12,XH1,7,12,XD1
               BYTE 7,12,XH1,5,71,XH1,5,71,XD1,7,233,XH1,7,233,XD1
               BYTE 7,233,XH1,5,71,XD1,5,71,XH1,5,71,XH1,0,XI1


;              LIST ON ; = PDS directive?

PROG_END       ;EQU FIN ;FIN is in XC.BUB [@ end]

               IF PROG_END>$C200
               ECHO "ERROR: PROGRAM TOO LONG BY ",[PROG_END-$C200]," BYTES !!!"
               ERR
               ENDIF

;colour NYBBLES for the option screen
;
; To go in colour memory
;
              ORG $C200
OPTION_SCREEN_NYBBLES
               ; Column ->           1                   2                   3                   4      |
               ;   1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  Row v
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404 ; 1
               HEX 04040707070707070707070707070707070707070707070707070707070707070707070707070404
               HEX 04040101010101010101010101010101010101010101010101010101010101010101010101010404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040202020202020202020202020202020202020202020202020202020202020202020202020404 ; 5
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040202020202020202020202020202020202020202020202020202020202020202020202020404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040101010101010101010101010101010101010101010101010101010101010101010101010404
               HEX 04040707070707070707070707070707070707070707070707070707070707070707070704040404 ;10
               HEX 04040505050505050505050505050505050505050505050505050505050505050505050505050404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040303030303030303030303030303030303030303030303030303030303030303030303030404
               HEX 04040505050505050505050505050505050505050505050505050505050505050505050505050404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404 ;15
               HEX 04040707070707070707070707070707070707070707070707070707070707070707070707070404
               HEX 04040505050505050505050505050505050505050505050505050505050505050505050505050404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040303030303030303030303030303030303030303030303030303030303030303030303030404 ;20
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040101010101010401010101010101010101010101010101010101010101040101010101010404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404
               HEX 04040404040404040404040404040404040404040404040404040404040404040404040404040404 ;25
               HEX 040404040404040404040404040404040404040404040404


;more sprite defs:arrows and 'bubbler' symbol to go at $CE00
               ORG $CE00
;               ALIGN $100
SPRITES_ETC
               ; Sprite #1: Bubbler logo (options screen) part 1/4 ("BU")
               HEX 1FC405 ; Line 1
               HEX 3FEE0E
               HEX 7FF71F
               HEX FFFB1F
               HEX F9FB1F
               HEX F9F71F
               HEX FFEF1F ; Line 7
               HEX FFDF1F
               HEX FFEF1F
               HEX FFF71F
               HEX FFFBBF
               HEX FFFBFF
               HEX F8FBFF
               HEX F8FBFE ; Line 14
               HEX FFFBFE
               HEX FFFBFD
               HEX 7FF7FD
               HEX 1FC1F0
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Sprite #2: Bubbler logo (options screen) part 2/4 ("BB")
               HEX FF1FE7 ; Line 1
               HEX FFBFF7
               HEX 7FDFFB
               HEX 7FEFFD
               HEX 67ECFD
               HEX 67DCFB
               HEX 7FBFF7 ; Line 7
               HEX 7F7FEF
               HEX 7FBFF7
               HEX 7FDFFB
               HEX 7FEFFD
               HEX 7FEFFD
               HEX 63EC7D
               HEX E3EC7D ; Line 14
               HEX FFEFFD
               HEX FFEFFD
               HEX FFDFFB
               HEX 7F3FE7
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Sprite #3: Bubbler logo (options screen) part 3/4 ("LE")
               HEX 01FE7E ; Line 1
               HEX 83FF7F
               HEX C7FF7F
               HEX C7FF7B
               HEX C7FEF1
               HEX C7C1FB
               HEX C7C1FF ; Line 7
               HEX C7FDFF
               HEX C7FEFF
               HEX C7FEFF
               HEX C7FDF7
               HEX C7C1F7
               HEX C3C1F7
               HEX FCFEF3 ; Line 14
               HEX FF3F73
               HEX FFDF73
               HEX FFDF71
               HEX FFBEE0
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Sprite #4: Bubbler logo (options screen) part 4/4 ("R")
               HEX 000000 ; Line 1
               HEX 800000
               HEX E00000
               HEX F00000
               HEX F00000
               HEX F00000
               HEX F00000 ; Line 7
               HEX E00000
               HEX C00000
               HEX C00000
               HEX E00000
               HEX E00000
               HEX E00000
               HEX F00000 ; Line 14
               HEX F00000
               HEX F00000
               HEX F00000
               HEX E00000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Sprite #5: "P1" arrow on options screen
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000100
               HEX 000180
               HEX 0001C0
               HEX 0001E0
               HEX 1FFFF0 ; Line 7
               HEX 187BF8
               HEX 1BB3FC
               HEX 1BBBFE
               HEX 187BFF
               HEX 1BFBFE
               HEX 1BFBFC
               HEX 1BF1F8 ; Line 14
               HEX 1FFFF0
               HEX 0001E0
               HEX 0001C0
               HEX 000180
               HEX 000100
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Sprite #6: "P2" arrow on options screen
               HEX 000000 ; Line 1
               HEX 000000
               HEX 008000
               HEX 018000
               HEX 038000
               HEX 078000
               HEX 0FFFF8 ; Line 7
               HEX 1F8638
               HEX 3FBBD8
               HEX 7FBBD8
               HEX FF87B8
               HEX 7FBF78
               HEX 3FBEF8
               HEX 1FBE18 ; Line 14
               HEX 0FFFF8
               HEX 078000
               HEX 038000
               HEX 018000
               HEX 008000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; #7: unused
               HEX 000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000
               HEX 00

               ; #8: unused
               HEX 000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000
               HEX 00


;character set to go at $D000
               ORG $D000
CHARACTER_SET
               ;   @               A               B               C               D               E
               HEX 7CFE06DCC0FE7C007CFEE6F6F6E6E600FCFE06FCE6FEFC007CFC00E0E0FC7C00FCFE06E6E6FEFC00FEFE00F8E0FEFE00
               ;   F               G               H               I               J               K
               HEX FEFE00F8E0E0E0007EFE00EEE6FE7E00E6E6E6F6F6E6E60038380038383838001C1C001C1C7C7800E6EEFCF8FCEEE600
               ;   L               M               N               O               P               Q
               HEX E0E0E0E0E0FEFE00FCFEC6D6D6D6D600FCFEE6E6E6E6E6007CFE06E6E6FE7C00F8FC0CFCF8E0E0007CFEE6E6E67A3C00
               ;   R               S               T               U               V               W
               HEX FCFE0EFCFCEEE600E070381C1CFCF800FEFE00E0E0FE7E00E6E6E6E6E6FE7E00EEEEEE6C6C7C3800D6D6D6D6D6FE7C00
               ;   X               Y               Z               corner top lt   horiz line      corner top rt
               HEX E77E3C007EE7C300EEEEEE7C00383800FEFE003870FEFE0000071F3F3F7E7C7C00FFFFFFFF00000000E0F8FCFC7E3E3E
               ;   up arrow        down arrow      space           vert line       corner bot lt   horiz line
               HEX 10387CFE00383800383800FE7C38100000000000000000007C7C7C7C7C7C7C7C7C7E3F3F1F0700000000FFFFFFFF0000
               ;   corner bot rt   vert line       corner top lt   corner top rt   left arrow      right arrow
               HEX 3E7EFCFCF8E000003E3E3E3E3E3E3E3E0000E7DFBFBF7E7C0000E7FBFDFD7E3E103077F777301000080CEEEFEE0C0800
               ;   *               +               ,               -               .               /
               HEX 00006618FF1866000018187E7E1818000000001C1C3860000000007E7E00000000000000383838000007001C3870E000
               ;   0               1               2               3               4               5
               HEX 387CEEC6EE7C38003838783838383800FCFE067C00FEFE00FEFC003C06FEFC000C1C3C60FEFE0C00FEFE00FE06FE7C00
               ;   6               7               8               9               :               ;
               HEX 1C3870ECEEFE7C007E7E001C383838007CEEEE6CEEEE7C007CFEEE6E1C387000001C1C001C1C0000001C001C1C386000
               ;                  =               return symbol   ?               triangle (sp)   '
               HEX 1C3C00FC70FCFC00007E7E007E7E00000E2E6EFEFC6020007C7E0E1C380038000000183C7EFF00000F1E386000000000
               ;   `               <--- Copied from C64 ROM                                                    --->
               HEX F0781C0600000000000000FFFF0000000000FFFF0000000000FFFF000000000000000000FFFF00003030303030303030
               ;   <--- Copied from C64 ROM                                                                    --->
               HEX 0C0C0C0C0C0C0C0C000000E0F038181818181C0F07000000181838F0E0000000C0C0C0C0C0C0FFFFC0E070381C0E0703
               ;   <--- Copied from C64 ROM                                                                    --->
               HEX 03070E1C3870E0C0FFFFC0C0C0C0C0C0FFFF030303030303003C7E7E7E7E3C000000000000FFFF00367F7F7F3E1C0800
               ;   <--- Copied from C64 ROM                                                                    --->
               HEX 6060606060606060000000070F1C1818C3E77E3C3C7EE7C3003C7E66667E3C001818666618183C000606060606060606
               ;   <--- Copied from C64 ROM                                        pi                          --->
               HEX 081C3E7F3E1C0800181818FFFF181818C0C03030C0C0303018181818181818180000033E76363600FF7F3F1F0F070301
               ;   <--- Copied from C64 ROM                                                                    --->
               HEX 0000000000000000F0F0F0F0F0F0F0F000000000FFFFFFFFFF0000000000000000000000000000FFC0C0C0C0C0C0C0C0
               ;   <--- Copied from C64 ROM                                                                    --->
               HEX CCCC3333CCCC3333030303030303030300000000CCCC3333FFFEFCF8F0E0C08003030303030303031818181F1F181818
               ;   C64 ROM char    corner bot lt   horiz line      corner top lt   horiz line      corner top rt
               HEX 000000000F0F0F0F7C7E3F3F1F0700000000FFFFFFFF00000000E7DFBFBF7E7C0000FFFFFFFF00000000E7FBFDFD7E3E
               ;   horiz line      corner bot rt   vert line       vert line       vert line       horiz line thin
               HEX 0000FFFFFFFF00003E7EFCFCF8E00000C0C0C0C0C0C0C0C0E0E0E0E0E0E0E0E01010101010101010000000FF00000000
               ;   sq corn top rt  sq corn top lt  sq corn bot rt  sq corn bot lt  *               space
               HEX 000000F0101010100000001F10101010101010F0000000001010101F0000000000663CFF3C6600000000000000000000
               ;   box             box with X
               HEX FF818181818181FFFFC3A59999A5C3FF

;option screen
               ORG $D400
OPTION_SCREEN
               ; Column ->           1                   2                   3                   4      |
               ;   1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  Row v
               HEX 201B1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1D20 ; 1
               HEX 20212020203115102020202020202020202020202020202020202020202020202032151020202520
               HEX 20212030303030303020202020202020202020202020202020202020202020303030303030202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 202120202020202020202020200120202D20203120100C0119051220202020202020202020202520 ; 5
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 202120202020202020202020200220202D20203220100C0119051220202020202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 20212020202020202030200F122006091205202D2013140112142007010D05202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520 ;10
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 202120202020202020202020202020200B0519020F01120420202020202020202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520 ;15
               HEX 202120202020202020202020202020200A0F19131409030B20202020202020202020202020202520
               HEX 20212020202020202020202020202020100F12142014170F20202020202020202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 202120202020202020202020202020040506090E05200B0519132020202020202020202020202520 ;20
               HEX 20212020202020202020202020202020202020202020202020202020202020202020202020202520
               HEX 20222323232323232623232323232323232323232323232323232323232323272323232323232420
               HEX 20202020202020202120202020202020202020202020202020202020202020252020202020202020
               HEX 20202020202020202223232323232323232323232323232323232323232323242020202020202020
               HEX 20202020202020202020202020202020202020202020202020202020202020202020202020202020 ;25
               HEX 202020202020202020202020202020202020202020202020

;gauge direction sprites to go at $D800
               ORG $D800
GAUGE_SPRITES
               ; Gauge sprite #1
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 038000
               HEX 07C000
               HEX 07FFFF
               HEX 04FFFF
               HEX 038000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #2
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 001800
               HEX 003C00
               HEX 007C00 ; Line 14
               HEX 004C00
               HEX 003F00
               HEX 0007C0
               HEX 0001F0
               HEX 00007C
               HEX 00001F
               HEX 000007 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #3
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000180
               HEX 0003C0
               HEX 0003C0
               HEX 0002C0 ; Line 14
               HEX 0001E0
               HEX 000070
               HEX 000038
               HEX 00001C
               HEX 00000E
               HEX 000007
               HEX 000003 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #4
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000030
               HEX 000078
               HEX 000078
               HEX 000058
               HEX 000068 ; Line 14
               HEX 000038
               HEX 00000C
               HEX 00000E
               HEX 000006
               HEX 000007
               HEX 000003
               HEX 000003 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #5
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 600000
               HEX F00000
               HEX F00000
               HEX 900000
               HEX 600000 ; Line 14
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #6
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 0C0000
               HEX 1E0000
               HEX 1E0000
               HEX 1A0000
               HEX 160000 ; Line 14
               HEX 1C0000
               HEX 300000
               HEX 700000
               HEX 600000
               HEX E00000
               HEX C00000
               HEX C00000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #7
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 018000
               HEX 03C000
               HEX 03C000
               HEX 034000 ; Line 14
               HEX 078000
               HEX 0E0000
               HEX 1C0000
               HEX 380000
               HEX 700000
               HEX E00000
               HEX C00000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #8
               HEX 000000 ; Line 1
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 001800
               HEX 003C00
               HEX 003E00 ; Line 14
               HEX 003200
               HEX 00FC00
               HEX 03E000
               HEX 0F8000
               HEX 3E0000
               HEX F80000
               HEX E00000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #9
               HEX 0001C0 ; Line 1
               HEX 0003E0
               HEX FFFFE0
               HEX FFFF20
               HEX 0001C0
               HEX 000000
               HEX 000000 ; Line 7
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #10
               HEX E00000 ; Line 1
               HEX F80000
               HEX 3E0000
               HEX 0F8000
               HEX 03E000
               HEX 00FC00
               HEX 003E00 ; Line 7
               HEX 003E00
               HEX 002400
               HEX 001800
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #11
               HEX C00000 ; Line 1
               HEX E00000
               HEX 700000
               HEX 380000
               HEX 1C0000
               HEX 0E0000
               HEX 078000 ; Line 7
               HEX 03C000
               HEX 03C000
               HEX 02C000
               HEX 018000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #12
               HEX C00000 ; Line 1
               HEX C00000
               HEX E00000
               HEX 600000
               HEX 700000
               HEX 300000
               HEX 1C0000 ; Line 7
               HEX 1E0000
               HEX 1E0000
               HEX 160000
               HEX 1E0000
               HEX 0C0000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #13
               HEX 600000 ; Line 1
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000
               HEX 600000 ; Line 7
               HEX 600000
               HEX 900000
               HEX F00000
               HEX F00000
               HEX 600000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #14
               HEX 000003 ; Line 1
               HEX 000003
               HEX 000007
               HEX 000006
               HEX 00000E
               HEX 00000C
               HEX 000038 ; Line 7
               HEX 000078
               HEX 000078
               HEX 000068
               HEX 000078
               HEX 000030
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #15
               HEX 000003 ; Line 1
               HEX 000007
               HEX 00000E
               HEX 00001C
               HEX 000038
               HEX 000070
               HEX 0001E0 ; Line 7
               HEX 0003C0
               HEX 0003C0
               HEX 000340
               HEX 000180
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

               ; Gauge sprite #16
               HEX 000007 ; Line 1
               HEX 00001F
               HEX 00007C
               HEX 0001F0
               HEX 0007C0
               HEX 003F00
               HEX 007C00 ; Line 7
               HEX 007400
               HEX 006800
               HEX 003000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 14
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000
               HEX 000000 ; Line 21
               HEX 00     ; Filler

;colour screen nybbles to go at $DC00
               ORG $DC00
COLOUR_NYBBLES
               ; Column ->           1                   2                   3                   4      |
               ;   1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0  Row v
               HEX 70707070707070707036363636363636363636363636363636363636363636707070707070707070 ; 1
               HEX 70707070707070707036707070707070707070707070707070707070707036707070707070707070
               HEX 70707070707070707036707070707070707070707070707070707070707036707070707070707070
               HEX 10101010101010101036707070707070707070707070707070707070707036101010101010101010
               HEX 10101010101010101036707070707070707070707070707070707070707036101010101010101010 ; 5
               HEX 10101010101010101036707070707070707070707070707070707070707036101010101010101010
               HEX 70707070707010101036707070707070707070707070707070707070707036101010107070707070
               HEX 70707070707010101036707070707070707070707070707070707070707036101010107070707070
               HEX 70707050505010101036707070707070707070707070707070707070707036701070505050707070
               HEX 70707050505070107036707070707070707070707070707070707070707036701070505050707070 ;10
               HEX 70707050505010101036707070707070707070707070707070707070707036101010505050707070
               HEX 70707050505010101036707070707070707070707070707070707070707036101010505050707070
               HEX 70707050505070707036707070707070707070707070707070707070707036707070505050707070
               HEX 40404040404040407036707070707070707070707070707070707070707036704040404040404040
               HEX 40404040404040407036707070707070707070707070707070707070707036704040404040404040 ;15
               HEX 40404040404040407036363636363636363636363636363636363636363636704040404040404040
               HEX 10101010101010101010101010101036101010101010101036101010101010101010101010101010
               HEX 10101010101010101010101010101036101010101010101036101010101010101010101010101010
               HEX 50505007070707070707070750505036363636363636363636505050070707070707070707505050
               HEX 50101010101010101010101010105070202020202020202070501010101010101010101010101050 ;20
               HEX 501010101010C0C0C010101010105070202020202020202070501010101010C0C0C0101010101050
               HEX 50101010101010C0101010101010507020202020202020207050101010101010C010101010101050
               HEX 50101010101010101010101010105030303030303030303030501010101010101010101010101050
               HEX 50101010101010101010101010105030303030303030303030501010101010101010101010101050
               HEX 50505050500707070707505050505030303030303030303030505050505007070707075050505050 ;25
               HEX B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1626365665153574E


;*             X8 - Hires screen
;              INCLUDE BubblerHiResScreen.asm ;Hi-res screen


;
; File X8.BUB
; Starts Here
;
;; 2009-10: MY: The way the loader currently works, this main game screen is displayed during the final uncompress
;;              of the program.  (Once the program runs the Options screen is displayed.)  Tweaked the data below
;;              so it says "LOADING..." in the play area.  (This will get erased and overwritten when a game
;;              actually starts.)
;;
               ;hi-res screen
               ORG $E000
HI_RES_SCREEN
               HEX 0000FCFCFEFE7F7F000000000000010100001F7FFFFFFFFC0000F0F1F3F3F70700007EFFFFFFFFE700000F8FCFCFEFE0
               HEX 0000FCFFFFFFFF0F00001F1F9F9FDFC00000FFFFFFFFFF000000071F3F3F7E7C0000FFFFFFFF00000000FFFFFFFF0000
               HEX 0000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF0000
               HEX 0000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF0000
               HEX 0000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF0000
               HEX 0000E0F8FCFC7E3E0000FCFCFEFE7F7F000000000000010100001F7FFFFFFFFC0000F0F1F3F3F70700007EFFFFFFFFE7
               HEX 00000F8FCFCFEFE00000F8FEFFFFFF0F00001F1F1F9F9FC00000FFFFFFFFFF003F3F1F1F0F0F07078181C1C1E1E1F1F1
               HEX F8F8F8F8F8F8F8F80707070707070707C3C3C3C3C3C3C3C3E0E0EFEFEFEFEFEF070FFFFFFFFEBE9FC0C09F9F1F1F1F1F
               HEX 0000FEFEFEFE80807C7C7C7C7C7C7C7C0000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000000000000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E3F3F1F1F0F0F0707
               HEX 8181C1C1E1E1F1F1F8F8F8F8F8F8F8F80707070707070707C3C3C3C3C3C3C3C3E0E0EFEFEFEFEFEF070FFFFFFFFEBE9F
               HEX C0C09F9F1F1F1F1F0000FEFEFEFE80800303FFFFFFFFFFFFF9F9FDFDF8F8F0C0F8FCFFFFFFFF7F1F0707F7F7F3F3F1F0
               HEX C3E7FFFFFFFFFF7EEFEFEFEFCFCF8F0F9F9F8F8F8F8787871F1F9F9F9FDFDFDF808080FFFFFFFFFF7C7C7C7C7C7C7C7C
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000003E3E3E3E3E3E3E3E0303FFFFFFFFFFFFF9F9FDFDF8F8F0C0F8FCFFFFFFFF7F1F
               HEX 0707F7F7F3F3F1F0C3E7FFFFFFFFFF7EEFEFEFEFCFCF8F0F9F9F8F8F8F8787871F1F9F9F9FDFDFDF808080FFFFFFFFFF
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 0000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C00000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 3E3E3E3E3E3E3E3E00000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000007C7C7C7C7C7C7C7C0000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000000000000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E0000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000003E3E3E3E3E3E3E3E000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 0000000000000000000000000103070F00010F7FFEF9E3C700C0F8FFFFFFFFFF0000000040A050A80000000000000000
               HEX 0000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C00000000000000000000000000000000
;               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
;               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
;               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               ;   space           space           space           L               O               A
               HEX 000000000000000000000000000000000000000000000000E0E0E0E0E0FEFE007CFE06E6E6FE7C007CFEE6F6F6E6E600
               ;   D               I               N               G               .               .
               HEX FCFE06E6E6FEFC003838003838383800FCFEE6E6E6E6E6007EFE00EEE6FE7E0000000000383838000000000038383800
               ;   .               space           space           space           space           space
               HEX 000000003838380000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 3E3E3E3E3E3E3E3E00000000000000000000000000000000000000000000000000000000000000000000000003070F1F
               HEX 00031FFFFDF3C78F0080F0FEFEFFFEFF000000008040A050000000000000000000000000000000000F1F1F1F0F070100
               HEX CF8F9F9FFFFFFF1FFFFFFFFFFFFEFDF8D0A8D4A850A00000000000000000000000000000000000000000000000000000
               HEX 00000000000000007C7C7C7C7C7C7C7C0000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000000000000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E0000000000000000
               HEX 0000000000000000000000000000000000000000000000001F3F3F3F1F0F03009F1F3F3FFFFFFF3FFFFFFFFFFEFDFAF0
               HEX A050A850A040000000000000000000000000000000000007000000000000FF8100000000000000E00000000000010000
               HEX 00000000FC86FCFC00000000000000000000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000003E3E3E3E3E3E3E3E000000000000000000000000000000000000000000000000
               HEX 0000000000000000000000003F613F3F00000000008000000000000000000007000000000000FF8100000000000000E0
               HEX 1C30606060707C670000000000000081380C0606060E3EE6000102050A142A25B432797AFDFCFDFE0000008040A05090
               HEX 0000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C00000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 3E3E3E3E3E3E3E3E0000000000000000000000000000000000000000000000000000000102050A092D4C9E5EBF3FBF7F
               HEX 008040A0502854A41C30606060707C670000000000000081380C0606060E3EE6603030301818180CFF00000000000000
               HEX 060C0C0C181818301128152915281428FE00FEFE0200CCCC2050A050A050A05000000000000000000000000000000000
               HEX 00000000000000007C7C7C7C7C7C7C7C0000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000000000000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E0000000000000000
               HEX 00000000000000000000000000000000040A050A050A050A7F007F7F400033338814A894A8142814603030301818180C
               HEX FF00000000000000060C0C0C181818300C0C060606030000000000000081FF183030606060C000001429152914281528
               HEX 007A86FEFC30CE30A050A050A050A0500000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000003E3E3E3E3E3E3E3E000000000000000000000000000000000000000000000000
               HEX 050A050A050A050A005E617F3F0C730C2894A8942814A8140C0C060606030000000000000081FF183030606060C00000
               HEX 0000000000000000000000000000000000000000000000001409050000000000FCFEFE0000000000A040800000000000
               HEX 0000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C00000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 3E3E3E3E3E3E3E3E00000000000000000000000000000000000000000000000005020100000000003F7F7F0000000000
               HEX 2890A0000000000000000000000000000000000000000000000000000000000000FFFFFFFFFF000000FCFCFCFCFC0000
               HEX 00FCFCFCFCFC000000FFFFFFFFFFFCF800F8FFFFFFFF0F07000F0F8F8FCFC0C000FFFFFFFFFF000000C0C0C0C0C00000
               HEX 00000000000000007C7C7C7C7C7C7C7C0000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000000000000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E0000000000000000
               HEX 000303030303000000FFFFFFFFFF000000F3F3F3F3F3000000F3F3F3F3F3030300FFFFFFFFFFF0E000F0FCFEFEFF3F1F
               HEX 003F3F3F3F3F000000FFFFFFFFFF000000FEFEFEFEFEFEFE000000000000000000FCFCFCFCFCFCFCF8F9F9F9F9F9F9F9
               HEX C7E7E7E7E7E7E7E7C0CFCFCFCFCFCFCF00FFFFFFFFFFC0C0008080808080000000000000000000007C7C7C7C7C7C7C7C
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000003E3E3E3E3E3E3E3E0000000000000000000303030303030300F8F8F8F8F8F8F8
               HEX 000303030303030303F3F3F3F3F3F3F3E3E7E7E7E7E7E7E71F9F9F9F9F9F9F9F003F3F3F3F3F3F3F00FEFEFEFEFE0000
               HEX FEFFFF7F7F3F0F000000FCFCFCFCFC00FCFCFCFCFCFCFC00F9F9F9F9F9F9F900E7E7E7E7E7E7E700CFCFCFCFCFCFCF00
               HEX C0C0C0FFFFFFFF00000000C0C0C0C00000000000000000007C7E7F3F3F1F07000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 0000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000F3EFDFBFBF7E0000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 0000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 0000CFF7FBFDFD7E0000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 3E7EFEFCFCF8E00000000000000000000303030101000000F8FCFFFFFFFF3F000303F3F3F3F3F300F3F3F3F3F3F3F300
               HEX E7E7E7E7E7E7E7009F9F9F9F9F9F9F003F3F3F3F3F3F3F000000FFFFFFFFFF00000001070F1C193B0000FFFF0078FEFF
               HEX 0000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF00000000
               HEX 0000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF00000000
               HEX 000080E0F038181C7C7C7C7C7C7C7C7C0000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000000000000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E000001070F1C193B
               HEX 0000FFFF0078FEFF0000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF00000000
               HEX 0000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF00000000
               HEX 0000FFFF00000000000080E0F038181C333B191C0F070100FFFFFE7800FFFF000000000000FFFF000000000000FFFF00
               HEX 0000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF00
               HEX 0000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000C1C1838F0E080007C7C7C7C7C7C7C7C
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000003E3E3E3E3E3E3E3E333B191C0F070100FFFFFE7800FFFF000000000000FFFF00
               HEX 0000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF00
               HEX 0000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000000000000FFFF000C1C1838F0E08000
               HEX 00071F3F3F7F7E7C00FFFFFFFFFF000000FFFFFFFFFF000000786C66666C7800003131313131310000E1B1B1E1B19900
               HEX 00F18181E181F10000F180808080F00000F860606060600000C3C3C3C3C3C30000F030303030F00000FCCCCCCCCCCC00
               HEX 00FFFFFFFFFF000000FFFFFFFFFF000000E0F8FCFCFE7E3E7C7E7F3F3F1F07000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 0000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 3E7EFEFCFCF8E00000071F3F3F7F7E7C00FFFFFFFFFF000000FFFFFFFFFF000000786C66666C78000031313131313100
               HEX 00E1B1B1E1B1990000F18181E181F10000F180808080F00000F860606060600000C3C3C3C3C3C30000F030303030F000
               HEX 00FCCCCCCCCCCC0000FFFFFFFFFF000000FFFFFFFFFF000000E0F8FCFCFE7E3E7C7C7C7C7C7C7C7C0000000000000000
               HEX 000000000000000000000000010F3F7F0000073FF1F8FCFF003FFFFFFFFF7FE07FFFFFFFFFFFC000FFC7C7C7FFFF0000
               HEX FCFFFFFFFFFF070000F8FFFFFFFEFC0F0000C0F81F3F7FFF0000000000E0F8FC00000000000000000000000000000000
               HEX 3E3E3E3E3E3E3E3E00000000000000000000FCFCFCFCFCFC00003E3E3E3E3E3E00007E7E7E7E7E000000000000000000
               HEX 00003F3F3F3F1F1F000000008080C0C00000071F3F3F7F7F0000FCFCFCFCFC0000000000000000007C7C7C7C7C7C7C7C
               HEX 0000000000000000000000000000000000000000010F3F7F0000073FF1F8FCFF003FFFFFFFFF7FE07FFFFFFFFFFFC000
               HEX FFC7C7C7FFFF0000FCFFFFFFFFFF070000F8FFFFFFFEFC0F0000C0F81F3F7FFF0000000000E0F8FC0000000000000000
               HEX 00000000000000003E3E3E3E3E3E3E3E7C7C7C7C7C7C7C7C0000000000000000010303070F0F1F0EFFFFFFFFFEFCF818
               HEX FEF0C0000000000000000000000000000000000000000003000000000038FEFF00000000000000800000000000000000
               HEX FF1F070100000000FFFFFFFFFF7F3F300080C0E0F0F0F8F000000000000000003E3E3E3E3E3E3E3E0000000000000000
               HEX FCFCFFFFFFFFFFFC3E3EBEBEBEBEBE3E00007E7E7E7E7E7E00003F3F3F3F3F000F0F070703030101E0E0F0F0F8F8FCFC
               HEX 7E7E7E7E7E7E7E7E000000000000000000000000000000007C7C7C7C7C7C7C7C0000000000000000010303070F0F1F0E
               HEX FFFFFFFFFEFCF818FEF0C0000000000000000000000000000000000000000003000000000038FEFF0000000000000080
               HEX 0000000000000000FF1F070100000000FFFFFFFFFF7F3F300080C0E0F0F0F8F000000000000000003E3E3E3E3E3E3E3E
               HEX 7C7C7C7C7C7C7C7C00000000000000000E17171B1D1E1F1F18FCFFFFFF7FBF8F000000C0F0FEFFFC000000000000E07F
               HEX 00000000000000C0FE3800000000000000000000000000070000000000000FFC000001071FFFFF7F307FFFFFFFFCFAE5
               HEX F0E8E8C80848884800000000000000003E3E3E3E3E3E3E3E0000000000000000FCFCFCFCFCFCFC003E3E3E3E3E3E3E00
               HEX 7E7E7E7E7E7E7E00000000000000000000003F3F3F3F3F00FEFEFFFFFEFEFC007E7F7F7F3F3F1F000000FCFCFCFCFC00
               HEX 00000000000000007C7C7C7C7C7C7C7C00000000000000000E17171B1D1E1F1F18FCFFFFFF7FBF8F000000C0F0FEFFFC
               HEX 000000000000E07F00000000000000C0FE3800000000000000000000000000070000000000000FFC000001071FFFFF7F
               HEX 307FFFFFFFFCFAE5F0E8E8C80848884800000000000000003E3E3E3E3E3E3E3E7C7C7C7C7C7C7C7C0000000000000000
               HEX 1F1F0F0F070301008180808080808000F83D8750AB57AB57FFFFFF3FC0FFFFFFFFFFFFFF7F80FFFFFFC7C7C7FF00FFFF
               HEX FFFFFFFFFC03FFFFFEFFFFF807FFFFFF3F78C73FFFFFFFFF0AF5EAF5EAF5EAF488488040804000000000000000000000
               HEX 3E3E3E3E3E3E3E3E00000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C
               HEX 00000000000000001F1F0F0F070301008180808080808000F83D8750AB57AB57FFFFFF3FC0FFFFFFFFFFFFFF7F80FFFF
               HEX FFC7C7C7FF00FFFFFFFFFFFFFC03FFFFFEFFFFF807FFFFFF3F78C73FFFFFFFFF0AF5EAF5EAF5EAF48848804080400000
               HEX 00000000000000003E3E3E3E3E3E3E3E7C7C7C7C7C7C7C7C000000000000000000000000000000000000000000000000
               HEX AB57AB1703000000FFFFFFFFFF3F0000FFFFFFFFFFFF7F00FFFFFFFFFFFFFF00FFFFFFFFFFFFFC00FFFFFFFFFFF80000
               HEX FFFFFFF8C0000000E8E0000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E0000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 0000000000000000000000000000000000000000000000007C7C7C7C7C7C7C7C00000000000000000000000000000000
               HEX 0000000000000000AB57AB1703000000FFFFFFFFFF3F0000FFFFFFFFFFFF7F00FFFFFFFFFFFFFF00FFFFFFFFFFFFFC00
               HEX FFFFFFFFFFF80000FFFFFFF8C0000000E8E0000000000000000000000000000000000000000000003E3E3E3E3E3E3E3E
               HEX 7C7E7F3F3F1F07000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF00007E60606E667E00
               HEX 007E66667E6666000066666666667E00007E60606E667E00007C606078607C000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 0000FFFFFFFFFF000000FFFFFFFFFF003E7EFEFCFCF8E000000000000000000000000000000000000000000000000000
               HEX 000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
               HEX 00000000000000007C7E7F3F3F1F07000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF00
               HEX 007E60606E667E00007E66667E6666000066666666667E00007E60606E667E00007C606078607C000000FFFFFFFFFF00
               HEX 0000FFFFFFFFFF000000FFFFFFFFFF000000FFFFFFFFFF003E7EFEFCFCF8E000

HI_RES_SCREEN_END

